@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdal_alg.h,v 1.19 2004/12/26 16:12:21 fwarmerdam Exp $
+ * $Id: gdal_alg.h,v 1.22 2005/08/02 22:19:41 fwarmerdam Exp $
  *
  * Project:  GDAL Image Processing Algorithms
  * Purpose:  Prototypes, and definitions for various GDAL based algorithms.
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: gdal_alg.h,v $
+ * Revision 1.22  2005/08/02 22:19:41  fwarmerdam
+ * approx transformer now can own its subtransformer
+ *
+ * Revision 1.21  2005/04/11 17:36:53  fwarmerdam
+ * added GDALDestroyTransformer
+ *
+ * Revision 1.20  2005/04/04 15:24:16  fwarmerdam
+ * added CPL_STDCALL to some functions
+ *
  * Revision 1.19  2004/12/26 16:12:21  fwarmerdam
  * thin plate spline support now implemented
  *
@@ -100,7 +109,7 @@
 
 CPL_C_START
 
-int CPL_DLL GDALComputeMedianCutPCT( GDALRasterBandH hRed, 
+int CPL_DLL CPL_STDCALL GDALComputeMedianCutPCT( GDALRasterBandH hRed, 
                              GDALRasterBandH hGreen, 
                              GDALRasterBandH hBlue, 
                              int (*pfnIncludePixel)(int,int,void*),
@@ -109,7 +118,7 @@ int CPL_DLL GDALComputeMedianCutPCT( GDALRasterBandH hRed,
                              GDALProgressFunc pfnProgress, 
                              void * pProgressArg );
 
-int CPL_DLL GDALDitherRGB2PCT( GDALRasterBandH hRed, 
+int CPL_DLL CPL_STDCALL GDALDitherRGB2PCT( GDALRasterBandH hRed, 
                        GDALRasterBandH hGreen, 
                        GDALRasterBandH hBlue, 
                        GDALRasterBandH hTarget, 
@@ -117,7 +126,7 @@ int CPL_DLL GDALDitherRGB2PCT( GDALRasterBandH hRed,
                        GDALProgressFunc pfnProgress, 
                        void * pProgressArg );
 
-int CPL_DLL GDALChecksumImage( GDALRasterBandH hBand, 
+int CPL_DLL CPL_STDCALL GDALChecksumImage( GDALRasterBandH hBand, 
                                int nXOff, int nYOff, int nXSize, int nYSize );
                                
 
@@ -138,6 +147,8 @@ typedef struct {
     CPLXMLNode *(*pfnSerialize)( void * );
 } GDALTransformerInfo;
 
+void CPL_DLL GDALDestroyTransformer( void *pTransformerArg );
+
 
 /* High level transformer for going from image coordinates on one file
    to image coordiantes on another, potentially doing reprojection, 
@@ -197,6 +208,8 @@ int CPL_DLL GDALRPCTransform(
 void CPL_DLL *
 GDALCreateApproxTransformer( GDALTransformerFunc pfnRawTransformer, 
                              void *pRawTransformerArg, double dfMaxError );
+void CPL_DLL GDALApproxTransformerOwnsSubtransformer( void *pCBData, 
+                                                      int bOwnFlag );
 void CPL_DLL GDALDestroyApproxTransformer( void *pApproxArg );
 int  CPL_DLL GDALApproxTransform(
     void *pTransformArg, int bDstToSrc, int nPointCount,
@@ -205,16 +218,17 @@ int  CPL_DLL GDALApproxTransform(
                       
 
 
-int CPL_DLL GDALSimpleImageWarp( GDALDatasetH hSrcDS, 
-                                 GDALDatasetH hDstDS, 
-                                 int nBandCount, int *panBandList,
-                                 GDALTransformerFunc pfnTransform,
-                                 void *pTransformArg,
-                                 GDALProgressFunc pfnProgress, 
-                                 void *pProgressArg, 
-                                 char **papszWarpOptions );
+int CPL_DLL CPL_STDCALL
+GDALSimpleImageWarp( GDALDatasetH hSrcDS, 
+                     GDALDatasetH hDstDS, 
+                     int nBandCount, int *panBandList,
+                     GDALTransformerFunc pfnTransform,
+                     void *pTransformArg,
+                     GDALProgressFunc pfnProgress, 
+                     void *pProgressArg, 
+                     char **papszWarpOptions );
 
-CPLErr CPL_DLL
+CPLErr CPL_DLL CPL_STDCALL
 GDALSuggestedWarpOutput( GDALDatasetH hSrcDS, 
                          GDALTransformerFunc pfnTransformer,
                          void *pTransformArg,
