@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalsimplewarp.cpp,v 1.7 2003/07/08 15:33:04 warmerda Exp $
+ * $Id: gdalsimplewarp.cpp,v 1.8 2005/04/04 15:24:16 fwarmerdam Exp $
  *
  * Project:  Mapinfo Image Warper
  * Purpose:  Simple (source in memory) warp algorithm.
@@ -29,6 +29,9 @@
  ******************************************************************************
  *
  * $Log: gdalsimplewarp.cpp,v $
+ * Revision 1.8  2005/04/04 15:24:16  fwarmerdam
+ * added CPL_STDCALL to some functions
+ *
  * Revision 1.7  2003/07/08 15:33:04  warmerda
  * avoid warnings
  *
@@ -56,7 +59,7 @@
 #include "gdal_alg.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: gdalsimplewarp.cpp,v 1.7 2003/07/08 15:33:04 warmerda Exp $");
+CPL_CVSID("$Id: gdalsimplewarp.cpp,v 1.8 2005/04/04 15:24:16 fwarmerdam Exp $");
 
 static void 
 GDALSimpleWarpRemapping( int nBandCount, GByte **papabySrcData, 
@@ -100,12 +103,13 @@ GDALSimpleWarpRemapping( int nBandCount, GByte **papabySrcData,
  * @return TRUE if the operation completes, or FALSE if an error occurs. 
  */
 
-int GDALSimpleImageWarp( GDALDatasetH hSrcDS, GDALDatasetH hDstDS, 
-                         int nBandCount, int *panBandList, 
-                         GDALTransformerFunc pfnTransform, void *pTransformArg,
-                         GDALProgressFunc pfnProgress, void *pProgressArg, 
-                         char **papszWarpOptions )
-
+int CPL_STDCALL
+GDALSimpleImageWarp( GDALDatasetH hSrcDS, GDALDatasetH hDstDS, 
+                     int nBandCount, int *panBandList, 
+                     GDALTransformerFunc pfnTransform, void *pTransformArg,
+                     GDALProgressFunc pfnProgress, void *pProgressArg, 
+                     char **papszWarpOptions )
+    
 {
     int		iBand, bCancelled = FALSE;
 
