@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaltransformer.cpp,v 1.20 2004/12/26 16:12:21 fwarmerdam Exp $
+ * $Id: gdaltransformer.cpp,v 1.23 2005/08/02 22:21:06 fwarmerdam Exp $
  *
  * Project:  Mapinfo Image Warper
  * Purpose:  Implementation of one or more GDALTrasformerFunc types, including
@@ -30,6 +30,16 @@
  ******************************************************************************
  *
  * $Log: gdaltransformer.cpp,v $
+ * Revision 1.23  2005/08/02 22:21:06  fwarmerdam
+ * Approx transformer can now own and cleanup it's subtransformer.
+ * GenImg transformer now deserialized with property GTI signature.
+ *
+ * Revision 1.22  2005/04/11 17:36:53  fwarmerdam
+ * added GDALDestroyTransformer
+ *
+ * Revision 1.21  2005/04/04 15:24:16  fwarmerdam
+ * added CPL_STDCALL to some functions
+ *
  * Revision 1.20  2004/12/26 16:12:21  fwarmerdam
  * thin plate spline support now implemented
  *
@@ -104,7 +114,7 @@
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: gdaltransformer.cpp,v 1.20 2004/12/26 16:12:21 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdaltransformer.cpp,v 1.23 2005/08/02 22:21:06 fwarmerdam Exp $");
 CPL_C_START
 CPLXMLNode *GDALSerializeGCPTransformer( void *pTransformArg );
 void *GDALDeserializeGCPTransformer( CPLXMLNode *psTree );
@@ -212,11 +222,12 @@ points may have failed) or FALSE if the overall transformation fails.
  */
 
 
-CPLErr GDALSuggestedWarpOutput( GDALDatasetH hSrcDS, 
-                                GDALTransformerFunc pfnTransformer, 
-                                void *pTransformArg, 
-                                double *padfGeoTransformOut, 
-                                int *pnPixels, int *pnLines )
+CPLErr CPL_STDCALL
+GDALSuggestedWarpOutput( GDALDatasetH hSrcDS, 
+                         GDALTransformerFunc pfnTransformer, 
+                         void *pTransformArg, 
+                         double *padfGeoTransformOut, 
+                         int *pnPixels, int *pnLines )
 
 {
 /* -------------------------------------------------------------------- */
@@ -968,6 +979,12 @@ void *GDALDeserializeGenImgProjTransformer( CPLXMLNode *psTree )
     psInfo = (GDALGenImgProjTransformInfo *) 
         CPLCalloc(sizeof(GDALGenImgProjTransformInfo),1);
 
+    strcpy( psInfo->sTI.szSignature, "GTI" );
+    psInfo->sTI.pszClassName = "GDALGenImgProjTransformer";
+    psInfo->sTI.pfnTransform = GDALGenImgProjTransform;
+    psInfo->sTI.pfnCleanup = GDALDestroyGenImgProjTransformer;
+    psInfo->sTI.pfnSerialize = GDALSerializeGenImgProjTransformer;
+
 /* -------------------------------------------------------------------- */
 /*      SrcGeotransform                                                 */
 /* -------------------------------------------------------------------- */
@@ -1311,6 +1328,8 @@ typedef struct
     GDALTransformerFunc pfnBaseTransformer;
     void             *pBaseCBData;
     double	      dfMaxError;
+
+    int               bOwnSubtransformer;
 } ApproxTransformInfo;
 
 /************************************************************************/
@@ -1401,6 +1420,7 @@ void *GDALCreateApproxTransformer( GDALTransformerFunc pfnBaseTransformer,
     psATInfo->pfnBaseTransformer = pfnBaseTransformer;
     psATInfo->pBaseCBData = pBaseTransformArg;
     psATInfo->dfMaxError = dfMaxError;
+    psATInfo->bOwnSubtransformer = FALSE;
 
     strcpy( psATInfo->sTI.szSignature, "GTI" );
     psATInfo->sTI.pszClassName = "GDALApproxTransformer";
@@ -1411,6 +1431,18 @@ void *GDALCreateApproxTransformer( GDALTransformerFunc pfnBaseTransformer,
     return psATInfo;
 }
 
+/************************************************************************/
+/*              GDALApproxTransformerOwnsSubtransformer()               */
+/************************************************************************/
+
+void GDALApproxTransformerOwnsSubtransformer( void *pCBData, int bOwnFlag )
+
+{
+    ApproxTransformInfo	*psATInfo = (ApproxTransformInfo *) pCBData;
+
+    psATInfo->bOwnSubtransformer = bOwnFlag;
+}
+
 /************************************************************************/
 /*                    GDALDestroyApproxTransformer()                    */
 /************************************************************************/
@@ -1427,6 +1459,11 @@ void *GDALCreateApproxTransformer( GDALTransformerFunc pfnBaseTransformer,
 void GDALDestroyApproxTransformer( void * pCBData )
 
 {
+    ApproxTransformInfo	*psATInfo = (ApproxTransformInfo *) pCBData;
+
+    if( psATInfo->bOwnSubtransformer ) 
+        GDALDestroyTransformer( psATInfo->pBaseCBData );
+
     CPLFree( pCBData );
 }
 
@@ -1561,14 +1598,20 @@ GDALDeserializeApproxTransformer( CPLXMLNode *psTree )
         GDALDeserializeTransformer( psContainer->psChild, 
                                     &pfnBaseTransform, 
                                     &pBaseCBData );
+        
     }
     
     if( pfnBaseTransform == NULL )
         return NULL;
     else
-        return GDALCreateApproxTransformer( pfnBaseTransform,
-                                            pBaseCBData, 
-                                            dfMaxError );
+    {
+        void *pApproxCBData = GDALCreateApproxTransformer( pfnBaseTransform,
+                                                           pBaseCBData, 
+                                                           dfMaxError );
+        GDALApproxTransformerOwnsSubtransformer( pApproxCBData, TRUE );
+
+        return pApproxCBData;
+    }
 }
 
 /************************************************************************/
@@ -1593,7 +1636,7 @@ GDALDeserializeApproxTransformer( CPLXMLNode *psTree )
  * @param *pdfGeoY output location where GeoX (northing/latitude) location is placed.
  */
 
-void GDALApplyGeoTransform( double *padfGeoTransform, 
+void CPL_STDCALL GDALApplyGeoTransform( double *padfGeoTransform, 
                             double dfPixel, double dfLine, 
                             double *pdfGeoX, double *pdfGeoY )
 {
@@ -1619,7 +1662,7 @@ void GDALApplyGeoTransform( double *padfGeoTransform,
  * @return TRUE on success or FALSE if the equation is uninvertable. 
  */
 
-int GDALInvGeoTransform( double *gt_in, double *gt_out )
+int CPL_STDCALL GDALInvGeoTransform( double *gt_in, double *gt_out )
 
 {
     double	det, inv_det;
@@ -1718,3 +1761,22 @@ CPLErr GDALDeserializeTransformer( CPLXMLNode *psTree,
 
     return CPLGetLastErrorType();
 }
+
+/************************************************************************/
+/*                       GDALDestroyTransformer()                       */
+/************************************************************************/
+
+void GDALDestroyTransformer( void *pTransformArg )
+
+{
+    GDALTransformerInfo *psInfo = (GDALTransformerInfo *) pTransformArg;
+
+    if( psInfo == NULL || !EQUAL(psInfo->szSignature,"GTI") )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "Attempt to destroy non-GTI transformer." );
+    }
+    else
+        psInfo->pfnCleanup( pTransformArg );
+}
+
