@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalwarper.cpp,v 1.17 2004/12/31 02:11:20 fwarmerdam Exp $
+ * $Id: gdalwarper.cpp,v 1.19 2005/04/11 17:20:40 fwarmerdam Exp $
  *
  * Project:  High Performance Image Reprojector
  * Purpose:  Implementation of high level convenience APIs for warper.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: gdalwarper.cpp,v $
+ * Revision 1.19  2005/04/11 17:20:40  fwarmerdam
+ * ensure dstnodata is duplicate in clonewarpoptions: bug 821
+ *
+ * Revision 1.18  2005/04/04 15:24:16  fwarmerdam
+ * added CPL_STDCALL to some functions
+ *
  * Revision 1.17  2004/12/31 02:11:20  fwarmerdam
  * Avoid warning.
  *
@@ -88,7 +94,7 @@
 #include "cpl_string.h"
 #include "cpl_minixml.h"
 
-CPL_CVSID("$Id: gdalwarper.cpp,v 1.17 2004/12/31 02:11:20 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdalwarper.cpp,v 1.19 2005/04/11 17:20:40 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                         GDALReprojectImage()                         */
@@ -128,13 +134,14 @@ CPL_CVSID("$Id: gdalwarper.cpp,v 1.17 2004/12/31 02:11:20 fwarmerdam Exp $");
  * @return CE_None on success or CE_Failure if something goes wrong.
  */
 
-CPLErr GDALReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT, 
-                           GDALDatasetH hDstDS, const char *pszDstWKT,
-                           GDALResampleAlg eResampleAlg, 
-                           double dfWarpMemoryLimit, 
-                           double dfMaxError,
-                           GDALProgressFunc pfnProgress, void *pProgressArg, 
-                           GDALWarpOptions *psOptions )
+CPLErr CPL_STDCALL 
+GDALReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT, 
+                    GDALDatasetH hDstDS, const char *pszDstWKT,
+                    GDALResampleAlg eResampleAlg, 
+                    double dfWarpMemoryLimit, 
+                    double dfMaxError,
+                    GDALProgressFunc pfnProgress, void *pProgressArg, 
+                    GDALWarpOptions *psOptions )
 
 {
     GDALWarpOptions *psWOptions;
@@ -287,7 +294,7 @@ CPLErr GDALReprojectImage( GDALDatasetH hSrcDS, const char *pszSrcWKT,
 /*      This is a "quicky" reprojection API.                            */
 /************************************************************************/
 
-CPLErr GDALCreateAndReprojectImage( 
+CPLErr CPL_STDCALL GDALCreateAndReprojectImage( 
     GDALDatasetH hSrcDS, const char *pszSrcWKT, 
     const char *pszDstFilename, const char *pszDstWKT,
     GDALDriverH hDstDriver, char **papszCreateOptions,
@@ -718,7 +725,7 @@ GDALWarpDstAlphaMasker( void *pMaskFuncArg, int nBandCount, GDALDataType eType,
 /*                       GDALCreateWarpOptions()                        */
 /************************************************************************/
 
-GDALWarpOptions *GDALCreateWarpOptions()
+GDALWarpOptions * CPL_STDCALL GDALCreateWarpOptions()
 
 {
     GDALWarpOptions *psOptions;
@@ -736,7 +743,7 @@ GDALWarpOptions *GDALCreateWarpOptions()
 /*                       GDALDestroyWarpOptions()                       */
 /************************************************************************/
 
-void GDALDestroyWarpOptions( GDALWarpOptions *psOptions )
+void CPL_STDCALL GDALDestroyWarpOptions( GDALWarpOptions *psOptions )
 
 {
     CSLDestroy( psOptions->papszWarpOptions );
@@ -765,7 +772,8 @@ void GDALDestroyWarpOptions( GDALWarpOptions *psOptions )
 /*                        GDALCloneWarpOptions()                        */
 /************************************************************************/
 
-GDALWarpOptions *GDALCloneWarpOptions( const GDALWarpOptions *psSrcOptions )
+GDALWarpOptions * CPL_STDCALL
+GDALCloneWarpOptions( const GDALWarpOptions *psSrcOptions )
 
 {
     GDALWarpOptions *psDstOptions = GDALCreateWarpOptions();
@@ -780,6 +788,8 @@ GDALWarpOptions *GDALCloneWarpOptions( const GDALWarpOptions *psSrcOptions )
     COPY_MEM( panDstBands, int, psSrcOptions->nBandCount );
     COPY_MEM( padfSrcNoDataReal, double, psSrcOptions->nBandCount );
     COPY_MEM( padfSrcNoDataImag, double, psSrcOptions->nBandCount );
+    COPY_MEM( padfDstNoDataReal, double, psSrcOptions->nBandCount );
+    COPY_MEM( padfDstNoDataImag, double, psSrcOptions->nBandCount );
     COPY_MEM( papfnSrcPerBandValidityMaskFunc, GDALMaskFunc, 
               psSrcOptions->nBandCount );
 
@@ -790,7 +800,8 @@ GDALWarpOptions *GDALCloneWarpOptions( const GDALWarpOptions *psSrcOptions )
 /*                      GDALSerializeWarpOptions()                      */
 /************************************************************************/
 
-CPLXMLNode *GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
+CPLXMLNode * CPL_STDCALL 
+GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
 
 {
     CPLXMLNode *psTree;
@@ -934,7 +945,7 @@ CPLXMLNode *GDALSerializeWarpOptions( const GDALWarpOptions *psWO )
 /*                     GDALDeserializeWarpOptions()                     */
 /************************************************************************/
 
-GDALWarpOptions *GDALDeserializeWarpOptions( CPLXMLNode *psTree )
+GDALWarpOptions * CPL_STDCALL GDALDeserializeWarpOptions( CPLXMLNode *psTree )
 
 {
     CPLErrorReset();
