@@ -1,5 +1,5 @@
 /* ****************************************************************************
- * $Id: gdal_translate.cpp,v 1.28 2005/01/27 20:58:11 fwarmerdam Exp $
+ * $Id: gdal_translate.cpp,v 1.32 2005/05/17 19:04:47 fwarmerdam Exp $
  *
  * Project:  GDAL Utilities
  * Purpose:  GDAL Image Translator Program
@@ -28,6 +28,19 @@
  * ****************************************************************************
  *
  * $Log: gdal_translate.cpp,v $
+ * Revision 1.32  2005/05/17 19:04:47  fwarmerdam
+ * Make sure nodata value is set after copycommoninfo.
+ *
+ * Revision 1.31  2005/05/09 14:31:56  fwarmerdam
+ * Added optional elevation in -gcp usage message.
+ *
+ * Revision 1.30  2005/04/26 20:38:52  fwarmerdam
+ * Added main/ProxyMain distinction since calling main does not work on
+ * some platforms.
+ *
+ * Revision 1.29  2005/04/12 14:37:47  fwarmerdam
+ * Added the -a_nodata switch to assign nodata values to output bands.
+ *
  * Revision 1.28  2005/01/27 20:58:11  fwarmerdam
  * Added -a_ullr switch.
  *
@@ -156,7 +169,7 @@
 #include "ogr_spatialref.h"
 #include "vrt/vrtdataset.h"
 
-CPL_CVSID("$Id: gdal_translate.cpp,v 1.28 2005/01/27 20:58:11 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdal_translate.cpp,v 1.32 2005/05/17 19:04:47 fwarmerdam Exp $");
 
 static int ArgIsNumeric( const char * );
 static void AttachMetadata( GDALDatasetH, char ** );
@@ -177,8 +190,8 @@ static void Usage()
             "       [-of format] [-b band] [-outsize xsize[%%] ysize[%%]]\n"
             "       [-scale [src_min src_max [dst_min dst_max]]]\n"
             "       [-srcwin xoff yoff xsize ysize] [-projwin ulx uly lrx lry]\n"
-            "       [-a_srs srs_def] [-a_ullr ulx uly lrx lry]\n"
-            "       [-gcp pixel line easting northing]*\n" 
+            "       [-a_srs srs_def] [-a_ullr ulx uly lrx lry] [-a_nodata value]\n"
+            "       [-gcp pixel line easting northing [elevation]]*\n" 
             "       [-mo \"META-TAG=VALUE\"]* [-quiet] [-sds]\n"
             "       [-co \"NAME=VALUE\"]*\n"
             "       src_dataset dst_dataset\n\n" );
@@ -200,11 +213,11 @@ static void Usage()
     }
 }
 
-/* ******************************************************************** */
-/*                                main()                                */
-/* ******************************************************************** */
+/************************************************************************/
+/*                             ProxyMain()                              */
+/************************************************************************/
 
-int main( int argc, char ** argv )
+static int ProxyMain( int argc, char ** argv )
 
 {
     GDALDatasetH	hDataset, hOutDS;
@@ -233,6 +246,8 @@ int main( int argc, char ** argv )
     int                 iSrcFileArg = -1, iDstFileArg = -1;
     int                 bCopySubDatasets = FALSE;
     double              adfULLR[4];
+    int                 bSetNoData = FALSE;
+    double		dfNoDataReal = 0.0;
 
 
     anSrcWin[0] = 0;
@@ -332,6 +347,13 @@ int main( int argc, char ** argv )
             /* should set id and info? */
         }   
 
+        else if( EQUAL(argv[i],"-a_nodata") && i < argc - 1 )
+        {
+            bSetNoData = TRUE;
+            dfNoDataReal = atof(argv[i+1]);
+            i += 1;
+        }   
+
         else if( EQUAL(argv[i],"-a_ullr") && i < argc - 4 )
         {
             adfULLR[0] = atof(argv[i+1]);
@@ -489,7 +511,7 @@ int main( int argc, char ** argv )
             {
                 argv[iSrcFileArg] = strstr(papszSubdatasets[i],"=")+1;
                 sprintf( pszSubDest, "%s%d", pszDest, i/2 + 1 );
-                if( main( argc, argv ) != 0 )
+                if( ProxyMain( argc, argv ) != 0 )
                     break;
             }
 
@@ -670,7 +692,7 @@ int main( int argc, char ** argv )
         && anSrcWin[3] == GDALGetRasterYSize(hDataset) 
         && pszOXSize == NULL && pszOYSize == NULL 
         && nGCPCount == 0 && !bGotBounds
-        && pszOutputSRS == NULL )
+        && pszOutputSRS == NULL && !bSetNoData )
     {
         
         hOutDS = GDALCreateCopy( hDriver, pszDest, hDataset, 
@@ -874,6 +896,12 @@ int main( int argc, char ** argv )
 /*      copy over some other information of interest.                   */
 /* -------------------------------------------------------------------- */
         poVRTBand->CopyCommonInfoFrom( poSrcBand );
+
+/* -------------------------------------------------------------------- */
+/*      Set a forcable nodata value?                                    */
+/* -------------------------------------------------------------------- */
+        if( bSetNoData )
+            poVRTBand->SetNoDataValue( dfNoDataReal );
     }
 
 /* -------------------------------------------------------------------- */
@@ -951,3 +979,15 @@ static void AttachMetadata( GDALDatasetH hDS, char **papszMetadataOptions )
 
     CSLDestroy( papszMetadataOptions );
 }
+
+/************************************************************************/
+/*                                main()                                */
+/************************************************************************/
+
+int main( int argc, char ** argv )
+
+{
+    return ProxyMain( argc, argv );
+}
+
+
