@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaladdo.cpp,v 1.10 2004/07/28 17:56:00 warmerda Exp $
+ * $Id: gdaladdo.cpp,v 1.11 2005/09/15 18:28:47 fwarmerdam Exp $
  *
  * Project:  GDAL Utilities
  * Purpose:  Commandline application to build overviews. 
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: gdaladdo.cpp,v $
+ * Revision 1.11  2005/09/15 18:28:47  fwarmerdam
+ * Cleanup on exit more carefully.
+ *
  * Revision 1.10  2004/07/28 17:56:00  warmerda
  * use return instead of exit() to avoid lame warnings on windows
  *
@@ -61,8 +64,9 @@
  */
 
 #include "gdal_priv.h"
+#include "cpl_string.h"
 
-CPL_CVSID("$Id: gdaladdo.cpp,v 1.10 2004/07/28 17:56:00 warmerda Exp $");
+CPL_CVSID("$Id: gdaladdo.cpp,v 1.11 2005/09/15 18:28:47 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                               Usage()                                */
@@ -144,5 +148,8 @@ int main( int nArgc, char ** papszArgv )
 /* -------------------------------------------------------------------- */
     delete poDataset;
 
+    CSLDestroy( papszArgv );
+    GDALDestroyDriverManager();
+
     return nResultStatus;
 }
