@@ -26,6 +26,15 @@
  * serves as an early test harnass.
  *
  * $Log: gdalinfo.c,v $
+ * Revision 1.39  2005/09/11 17:18:33  fwarmerdam
+ * added CPLCleanupTLS()
+ *
+ * Revision 1.38  2005/07/29 04:27:42  fwarmerdam
+ * Don't crash if a "paletted" band has no color table object.
+ *
+ * Revision 1.37  2005/05/11 14:39:43  fwarmerdam
+ * added option to report stats
+ *
  * Revision 1.36  2004/08/24 20:13:28  warmerda
  * ensure -nomd works for bands too
  *
@@ -139,7 +148,7 @@
 #include "cpl_string.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: gdalinfo.c,v 1.36 2004/08/24 20:13:28 warmerda Exp $");
+CPL_CVSID("$Id: gdalinfo.c,v 1.39 2005/09/11 17:18:33 fwarmerdam Exp $");
 
 static int 
 GDALInfoReportCorner( GDALDatasetH hDataset, 
@@ -173,6 +182,7 @@ int main( int argc, char ** argv )
     char		**papszMetadata;
     int                 bComputeMinMax = FALSE, bSample = FALSE;
     int                 bShowGCPs = TRUE, bShowMetadata = TRUE ;
+    int                 bStats = FALSE;
     const char          *pszFilename = NULL;
 
     GDALAllRegister();
@@ -188,6 +198,8 @@ int main( int argc, char ** argv )
     {
         if( EQUAL(argv[i], "-mm") )
             bComputeMinMax = TRUE;
+        else if( EQUAL(argv[i], "-stats") )
+            bStats = TRUE;
         else if( EQUAL(argv[i], "-sample") )
             bSample = TRUE;
         else if( EQUAL(argv[i], "-nogcp") )
@@ -361,6 +373,9 @@ int main( int argc, char ** argv )
         double      dfMin, dfMax, adfCMinMax[2], dfNoData;
         int         bGotMin, bGotMax, bGotNodata, bSuccess;
         int         nBlockXSize, nBlockYSize;
+        double      dfMean, dfStdDev;
+        GDALColorTableH	hTable;
+        CPLErr      eErr;
 
         hBand = GDALGetRasterBand( hDataset, iBand+1 );
 
@@ -405,6 +420,14 @@ int main( int argc, char ** argv )
             printf( "\n" );
         }
 
+        eErr = GDALGetRasterStatistics( hBand, FALSE, bStats, 
+                                        &dfMin, &dfMax, &dfMean, &dfStdDev );
+        if( eErr == CE_None )
+        {
+            printf( "  Minimum=%.3f, Maximum=%.3f, Mean=%.3f, StdDev=%.3f\n",
+                    dfMin, dfMax, dfMean, dfStdDev );
+        }
+
         dfNoData = GDALGetRasterNoDataValue( hBand, &bGotNodata );
         if( bGotNodata )
         {
@@ -469,12 +492,11 @@ int main( int argc, char ** argv )
             }
         }
 
-        if( GDALGetRasterColorInterpretation(hBand) == GCI_PaletteIndex )
+        if( GDALGetRasterColorInterpretation(hBand) == GCI_PaletteIndex 
+            && (hTable = GDALGetRasterColorTable( hBand )) != NULL )
         {
-            GDALColorTableH	hTable;
             int			i;
 
-            hTable = GDALGetRasterColorTable( hBand );
             printf( "  Color Table (%s with %d entries)\n", 
                     GDALGetPaletteInterpretationName(
                         GDALGetPaletteInterpretation( hTable )), 
@@ -504,6 +526,7 @@ int main( int argc, char ** argv )
     GDALDestroyDriverManager();
 
     CPLDumpSharedList( NULL );
+    CPLCleanupTLS();
 
     exit( 0 );
 }
