@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalwarp.cpp,v 1.15 2004/12/26 16:14:53 fwarmerdam Exp $
+ * $Id: gdalwarp.cpp,v 1.18 2005/09/13 01:57:31 fwarmerdam Exp $
  *
  * Project:  High Performance Image Reprojector
  * Purpose:  Test program for high performance warper API.
@@ -29,6 +29,15 @@
  ******************************************************************************
  *
  * $Log: gdalwarp.cpp,v $
+ * Revision 1.18  2005/09/13 01:57:31  fwarmerdam
+ * Fixed usage message.
+ *
+ * Revision 1.17  2005/09/13 01:24:45  fwarmerdam
+ * Set UNIFIED_SRC_NODATA by default for -srcnodata switch.
+ *
+ * Revision 1.16  2005/09/12 18:06:25  fwarmerdam
+ * Added quotes around -srcnodata and -dstnodata in usage message.
+ *
  * Revision 1.15  2004/12/26 16:14:53  fwarmerdam
  * added -tps flag
  *
@@ -117,7 +126,7 @@
 #include "cpl_string.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: gdalwarp.cpp,v 1.15 2004/12/26 16:14:53 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdalwarp.cpp,v 1.18 2005/09/13 01:57:31 fwarmerdam Exp $");
 
 static GDALDatasetH 
 GDALWarpCreateOutput( GDALDatasetH hSrcDS, const char *pszFilename, 
@@ -144,7 +153,7 @@ static void Usage()
         "    [-s_srs srs_def] [-t_srs srs_def] [-order n] ] [-tps] [-et err_threshold]\n"
         "    [-te xmin ymin xmax ymax] [-tr xres yres] [-ts width height]\n"
         "    [-wo \"NAME=VALUE\"] [-ot Byte/Int16/...] [-wt Byte/Int16]\n"
-        "    [-srcnodata value [value...]] [-dstnodata value [value...]] -dstalpha\n" 
+        "    [-srcnodata \"value [value...]\"] [-dstnodata \"value [value...]\"] -dstalpha\n" 
         "    [-rn] [-rb] [-rc] [-rcs] [-wm memory_in_mb] [-multi] [-q]\n"
         "    [-of format] [-co \"NAME=VALUE\"]* srcfile dstfile\n" );
     exit( 1 );
@@ -579,6 +588,9 @@ int main( int argc, char ** argv )
         }
 
         CSLDestroy( papszTokens );
+
+        papszWarpOptions = CSLSetNameValue(papszWarpOptions,
+                                           "UNIFIED_SRC_NODATA", "YES" );
     }
 
 /* -------------------------------------------------------------------- */
