@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: aaigriddataset.cpp,v 1.28 2005/02/08 18:15:34 fwarmerdam Exp $
+ * $Id: aaigriddataset.cpp,v 1.31 2005/08/23 16:32:23 fwarmerdam Exp $
  *
  * Project:  GDAL
  * Purpose:  Implements Arc/Info ASCII Grid Format.
@@ -28,6 +28,15 @@
  *****************************************************************************
  *
  * $Log: aaigriddataset.cpp,v $
+ * Revision 1.31  2005/08/23 16:32:23  fwarmerdam
+ * Fixed to support tabs for white space too.
+ *
+ * Revision 1.30  2005/05/05 14:01:36  fwarmerdam
+ * PAM Enable
+ *
+ * Revision 1.29  2005/04/15 18:34:35  fwarmerdam
+ * Added AREA_OR_POINT metadata.
+ *
  * Revision 1.28  2005/02/08 18:15:34  fwarmerdam
  * Removed tail recursion in IReadBlock() to avoid stack overflows.
  *
@@ -116,12 +125,12 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include <ctype.h>
 #include "cpl_string.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: aaigriddataset.cpp,v 1.28 2005/02/08 18:15:34 fwarmerdam Exp $");
+CPL_CVSID("$Id: aaigriddataset.cpp,v 1.31 2005/08/23 16:32:23 fwarmerdam Exp $");
 
 CPL_C_START
 void    GDALRegister_AAIGrid(void);
@@ -135,7 +144,7 @@ CPL_C_END
 
 class AAIGRasterBand;
 
-class CPL_DLL AAIGDataset : public GDALDataset
+class CPL_DLL AAIGDataset : public GDALPamDataset
 {
     friend class AAIGRasterBand;
     
@@ -166,7 +175,7 @@ class CPL_DLL AAIGDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class AAIGRasterBand : public GDALRasterBand
+class AAIGRasterBand : public GDALPamRasterBand
 {
     friend class AAIGDataset;
 
@@ -366,6 +375,8 @@ AAIGDataset::AAIGDataset()
 AAIGDataset::~AAIGDataset()
 
 {
+    FlushCache();
+
     if( fp != NULL )
         VSIFClose( fp );
 
@@ -400,7 +411,7 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
 
     papszTokens =  
         CSLTokenizeString2( (const char *) poOpenInfo->pabyHeader,
-                                  " \n\r", 0 );
+                                  " \n\r\t", 0 );
 
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
@@ -409,9 +420,6 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
 
     poDS = new AAIGDataset();
 
-    poDS->fp = poOpenInfo->fp;
-    poOpenInfo->fp = NULL;
-    
 /* -------------------------------------------------------------------- */
 /*      Parse the header.                                               */
 /* -------------------------------------------------------------------- */
@@ -449,6 +457,8 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
     else if ((i = CSLFindString( papszTokens, "xllcenter" )) >= 0 &&
              (j = CSLFindString( papszTokens, "yllcenter" )) >= 0 )
     {
+        poDS->SetMetadataItem( GDALMD_AREA_OR_POINT, GDALMD_AOP_POINT );
+
         poDS->adfGeoTransform[0] = atof(papszTokens[i + 1]) - 0.5 * dfCellSize;
         poDS->adfGeoTransform[1] = dfCellSize;
         poDS->adfGeoTransform[2] = 0.0;
@@ -476,6 +486,9 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
     
     CSLDestroy( papszTokens );
 
+    poDS->fp = poOpenInfo->fp;
+    poOpenInfo->fp = NULL;
+    
 /* -------------------------------------------------------------------- */
 /*      Find the start of real data.                                    */
 /* -------------------------------------------------------------------- */
@@ -538,6 +551,12 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
     CPLFree( pszDirname );
     CPLFree( pszBasename );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
@@ -699,7 +718,16 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         CPLFree( pszESRIProjection );
     }
     
-    return (GDALDataset *) GDALOpen( pszFilename, GA_Update );
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxilary pam information.         */
+/* -------------------------------------------------------------------- */
+    GDALPamDataset *poDS = (GDALPamDataset *) 
+        GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
 }
 
 /************************************************************************/
