@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: aigdataset.cpp,v 1.23 2005/02/21 03:04:18 fwarmerdam Exp $
+ * $Id: aigdataset.cpp,v 1.25 2005/09/16 20:38:19 fwarmerdam Exp $
  *
  * Project:  Arc/Info Binary Grid Driver
  * Purpose:  Implements GDAL interface to underlying library.
@@ -28,6 +28,12 @@
  *****************************************************************************
  *
  * $Log: aigdataset.cpp,v $
+ * Revision 1.25  2005/09/16 20:38:19  fwarmerdam
+ * added overview support
+ *
+ * Revision 1.24  2005/05/05 15:52:48  fwarmerdam
+ * PAM Enabled
+ *
  * Revision 1.23  2005/02/21 03:04:18  fwarmerdam
  * fixed for files in arcseconds (bug 775)
  *
@@ -100,12 +106,12 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "cpl_string.h"
 #include "ogr_spatialref.h"
 #include "aigrid.h"
 
-CPL_CVSID("$Id: aigdataset.cpp,v 1.23 2005/02/21 03:04:18 fwarmerdam Exp $");
+CPL_CVSID("$Id: aigdataset.cpp,v 1.25 2005/09/16 20:38:19 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_AIGrid(void);
@@ -123,7 +129,7 @@ static const char*OSR_GDS( char **papszNV, const char * pszField,
 
 class AIGRasterBand;
 
-class CPL_DLL AIGDataset : public GDALDataset
+class CPL_DLL AIGDataset : public GDALPamDataset
 {
     friend class AIGRasterBand;
     
@@ -152,7 +158,7 @@ class CPL_DLL AIGDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class AIGRasterBand : public GDALRasterBand
+class AIGRasterBand : public GDALPamRasterBand
 {
     friend class AIGDataset;
 
@@ -365,6 +371,7 @@ AIGDataset::AIGDataset()
 AIGDataset::~AIGDataset()
 
 {
+    FlushCache();
     CPLFree( pszProjection );
     CSLDestroy( papszPrj );
     if( psInfo != NULL )
@@ -473,6 +480,17 @@ GDALDataset *AIGDataset::Open( GDALOpenInfo * poOpenInfo )
 
     }
 
+/* -------------------------------------------------------------------- */
+/*      Open overviews.                                                 */
+/* -------------------------------------------------------------------- */
+    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
+
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( psInfo->pszCoverName );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
