@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: bmpdataset.cpp,v 1.32 2004/09/28 14:18:06 fwarmerdam Exp $
+ * $Id: bmpdataset.cpp,v 1.35 2005/07/05 18:00:10 dron Exp $
  *
  * Project:  Microsoft Windows Bitmap
  * Purpose:  Read/write MS Windows Device Independent Bitmap (DIB) files
@@ -29,6 +29,16 @@
  ******************************************************************************
  *
  * $Log: bmpdataset.cpp,v $
+ * Revision 1.35  2005/07/05 18:00:10  dron
+ * Add support for reading external overviews.
+ *
+ * Revision 1.34  2005/05/17 15:12:24  fwarmerdam
+ * Cleanup out residual flushcache and projections support for
+ * better pam support.
+ *
+ * Revision 1.33  2005/05/05 14:01:36  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.32  2004/09/28 14:18:06  fwarmerdam
  * A few more additions to improve error handling.  The "create" write
  * error checks don't work (at least on Linux) because of the buffering.
@@ -95,46 +105,12 @@
  * Revision 1.12  2002/12/13 14:15:50  dron
  * IWriteBlock() fixed, CreateCopy() removed, added RLE4 decoding and OS/2 BMP
  * support.
- *
- * Revision 1.11  2002/12/11 22:09:13  warmerda
- * Re-enable createcopy.
- *
- * Revision 1.10  2002/12/11 22:08:20  warmerda
- * fixed multiband support for writing ... open wit wb+ in create
- *
- * Revision 1.9  2002/12/11 16:13:04  dron
- * Support for 16-bit RGB images (untested).
- *
- * Revision 1.8  2002/12/10 22:12:59  dron
- * RLE8 decoding added.
- *
- * Revision 1.7  2002/12/09 19:31:44  dron
- * Switched to CPL_LSBWORD32 macro.
- *
- * Revision 1.6  2002/12/07 15:20:23  dron
- * SetColorTable() added. Create() really works now.
- *
- * Revision 1.5  2002/12/06 20:50:13  warmerda
- * fixed type warning
- *
- * Revision 1.4  2002/12/06 18:37:05  dron
- * Create() method added, 1- and 4-bpp images readed now.
- *
- * Revision 1.3  2002/12/05 19:25:35  dron
- * Preliminary CreateCopy() function.
- *
- * Revision 1.2  2002/12/04 18:37:49  dron
- * Support 32-bit, 24-bit True Color images and 8-bit pseudocolor ones.
- *
- * Revision 1.1  2002/12/03 19:04:18  dron
- * Initial version.
- *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: bmpdataset.cpp,v 1.32 2004/09/28 14:18:06 fwarmerdam Exp $");
+CPL_CVSID("$Id: bmpdataset.cpp,v 1.35 2005/07/05 18:00:10 dron Exp $");
 
 CPL_C_START
 void    GDALRegister_BMP(void);
@@ -283,7 +259,7 @@ typedef struct
 /* ==================================================================== */
 /************************************************************************/
 
-class BMPDataset : public GDALDataset
+class BMPDataset : public GDALPamDataset
 {
     friend class BMPRasterBand;
     friend class BMPComprRasterBand;
@@ -295,7 +271,6 @@ class BMPDataset : public GDALDataset
     GDALColorTable      *poColorTable;
     double              adfGeoTransform[6];
     int                 bGeoTransformValid;
-    char                *pszProjection;
 
     const char          *pszFilename;
     FILE                *fp;
@@ -313,11 +288,9 @@ class BMPDataset : public GDALDataset
     static GDALDataset  *Create( const char * pszFilename,
                                 int nXSize, int nYSize, int nBands,
                                 GDALDataType eType, char ** papszParmList );
-    virtual void        FlushCache( void );
 
     CPLErr              GetGeoTransform( double * padfTransform );
     virtual CPLErr      SetGeoTransform( double * );
-    const char          *GetProjectionRef();
 };
 
 /************************************************************************/
@@ -326,7 +299,7 @@ class BMPDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class BMPRasterBand : public GDALRasterBand
+class BMPRasterBand : public GDALPamRasterBand
 {
     friend class BMPDataset;
 
@@ -857,7 +830,6 @@ BMPDataset::BMPDataset()
     pszFilename = NULL;
     fp = NULL;
     nBands = 0;
-    pszProjection = CPLStrdup( "" );
     bGeoTransformValid = FALSE;
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
@@ -879,8 +851,6 @@ BMPDataset::~BMPDataset()
 {
     FlushCache();
 
-    if ( pszProjection )
-        CPLFree( pszProjection );
     if ( pabyColorTable )
         CPLFree( pabyColorTable );
     if ( poColorTable != NULL )
@@ -900,7 +870,7 @@ CPLErr BMPDataset::GetGeoTransform( double * padfTransform )
     if( bGeoTransformValid )
         return CE_None;
     else
-        return CE_Failure;
+        return GDALPamDataset::GetGeoTransform( padfTransform );
 }
 
 /************************************************************************/
@@ -914,28 +884,18 @@ CPLErr BMPDataset::SetGeoTransform( double * padfTransform )
     memcpy( adfGeoTransform, padfTransform, sizeof(double) * 6 );
 
     if ( pszFilename && bGeoTransformValid )
+    {
         if ( GDALWriteWorldFile( pszFilename, "wld", adfGeoTransform )
              == FALSE )
         {
             CPLError( CE_Failure, CPLE_FileIO, "Can't write world file." );
             eErr = CE_Failure;
         }
+    }
 
     return eErr;
 }
 
-/************************************************************************/
-/*                          GetProjectionRef()                          */
-/************************************************************************/
-
-const char *BMPDataset::GetProjectionRef()
-{
-    if( pszProjection )
-        return pszProjection;
-    else
-        return "";
-}
-
 /************************************************************************/
 /*                             IRasterIO()                              */
 /*                                                                      */
@@ -966,16 +926,6 @@ CPLErr BMPDataset::IRasterIO( GDALRWFlag eRWFlag,
                                     nPixelSpace, nLineSpace, nBandSpace );
 }
 
-/************************************************************************/
-/*                             FlushCache()                             */
-/************************************************************************/
-
-void BMPDataset::FlushCache()
-
-{
-    GDALDataset::FlushCache();
-}
-
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -1195,6 +1145,17 @@ GDALDataset *BMPDataset::Open( GDALOpenInfo * poOpenInfo )
             GDALReadWorldFile( poOpenInfo->pszFilename, ".bmpw",
                                poDS->adfGeoTransform );
 
+/* -------------------------------------------------------------------- */
+/*      Check for overviews.                                            */
+/* -------------------------------------------------------------------- */
+    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
+
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
