@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: bsbdataset.cpp,v 1.14 2004/04/02 16:38:38 warmerda Exp $
+ * $Id: bsbdataset.cpp,v 1.15 2005/05/05 14:01:36 fwarmerdam Exp $
  *
  * Project:  BSB Reader
  * Purpose:  BSBDataset implementation for BSB format.
@@ -28,6 +28,9 @@
  ******************************************************************************
  * 
  * $Log: bsbdataset.cpp,v $
+ * Revision 1.15  2005/05/05 14:01:36  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.14  2004/04/02 16:38:38  warmerda
  * Also disable creation type metadata.
  *
@@ -74,11 +77,11 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "bsb_read.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: bsbdataset.cpp,v 1.14 2004/04/02 16:38:38 warmerda Exp $");
+CPL_CVSID("$Id: bsbdataset.cpp,v 1.15 2005/05/05 14:01:36 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_BSB(void);
@@ -95,7 +98,7 @@ CPL_C_END
 
 class BSBRasterBand;
 
-class BSBDataset : public GDALDataset
+class BSBDataset : public GDALPamDataset
 {
     int         nGCPCount;
     GDAL_GCP    *pasGCPList;
@@ -127,7 +130,7 @@ class BSBDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class BSBRasterBand : public GDALRasterBand
+class BSBRasterBand : public GDALPamRasterBand
 {
     GDALColorTable	oCT;
 
@@ -249,6 +252,8 @@ BSBDataset::BSBDataset()
 BSBDataset::~BSBDataset()
 
 {
+    FlushCache();
+
     GDALDeinitGCPs( nGCPCount, pasGCPList );
     CPLFree( pasGCPList );
 
@@ -417,6 +422,12 @@ GDALDataset *BSBDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->ScanForGCPs();
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
