@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ceosdataset.cpp,v 1.9 2002/09/04 06:50:36 warmerda Exp $
+ * $Id: ceosdataset.cpp,v 1.10 2005/05/05 14:01:36 fwarmerdam Exp $
  *
  * Project:  CEOS Translator
  * Purpose:  GDALDataset driver for CEOS translator.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ceosdataset.cpp,v $
+ * Revision 1.10  2005/05/05 14:01:36  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.9  2002/09/04 06:50:36  warmerda
  * avoid static driver pointers
  *
@@ -58,9 +61,9 @@
  */
 
 #include "ceosopen.h"
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 
-CPL_CVSID("$Id: ceosdataset.cpp,v 1.9 2002/09/04 06:50:36 warmerda Exp $");
+CPL_CVSID("$Id: ceosdataset.cpp,v 1.10 2005/05/05 14:01:36 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_CEOS(void);
@@ -74,13 +77,15 @@ CPL_C_END
 
 class CEOSRasterBand;
 
-class CEOSDataset : public GDALDataset
+class CEOSDataset : public GDALPamDataset
 {
     friend class CEOSRasterBand;
     
     CEOSImage	*psCEOS;
 
   public:
+                 CEOSDataset();
+                ~CEOSDataset();
     static GDALDataset *Open( GDALOpenInfo * );
 };
 
@@ -90,7 +95,7 @@ class CEOSDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class CEOSRasterBand : public GDALRasterBand
+class CEOSRasterBand : public GDALPamRasterBand
 {
     friend class CEOSDataset;
     
@@ -133,6 +138,34 @@ CPLErr CEOSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
     return( CEOSReadScanline(poCEOS_DS->psCEOS, nBand, nBlockYOff+1, pImage) );
 }
 
+/************************************************************************/
+/* ==================================================================== */
+/*                             CEOSDataset                              */
+/* ==================================================================== */
+/************************************************************************/
+
+/************************************************************************/
+/*                            CEOSDataset()                             */
+/************************************************************************/
+
+CEOSDataset::CEOSDataset()
+
+{
+    psCEOS = NULL;
+}
+
+/************************************************************************/
+/*                            ~CEOSDataset()                            */
+/************************************************************************/
+
+CEOSDataset::~CEOSDataset()
+
+{
+    FlushCache();
+    if( psCEOS )
+        CEOSClose( psCEOS );
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -192,6 +225,12 @@ GDALDataset *CEOSDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
