@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: dteddataset.cpp,v 1.14 2004/01/30 18:27:25 gwalter Exp $
+ * $Id: dteddataset.cpp,v 1.16 2005/05/05 14:01:36 fwarmerdam Exp $
  *
  * Project:  DTED Translator
  * Purpose:  GDALDataset driver for DTED translator.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: dteddataset.cpp,v $
+ * Revision 1.16  2005/05/05 14:01:36  fwarmerdam
+ * PAM Enable
+ *
+ * Revision 1.15  2005/04/15 19:28:57  fwarmerdam
+ * added AREA_OR_POINT=Point metadata
+ *
  * Revision 1.14  2004/01/30 18:27:25  gwalter
  * Fixed bug in tile sizing.
  *
@@ -74,9 +80,9 @@
  */
 
 #include "dted_api.h"
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 
-CPL_CVSID("$Id: dteddataset.cpp,v 1.14 2004/01/30 18:27:25 gwalter Exp $");
+CPL_CVSID("$Id: dteddataset.cpp,v 1.16 2005/05/05 14:01:36 fwarmerdam Exp $");
 
 CPL_C_START
 void    GDALRegister_DTED(void);
@@ -90,7 +96,7 @@ CPL_C_END
 
 class DTEDRasterBand;
 
-class DTEDDataset : public GDALDataset
+class DTEDDataset : public GDALPamDataset
 {
     friend class DTEDRasterBand;
 
@@ -111,7 +117,7 @@ class DTEDDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class DTEDRasterBand : public GDALRasterBand
+class DTEDRasterBand : public GDALPamRasterBand
 {
     friend class DTEDDataset;
 
@@ -204,6 +210,7 @@ double DTEDRasterBand::GetNoDataValue( int * pbSuccess )
 DTEDDataset::~DTEDDataset()
 
 {
+    FlushCache();
     if( psDTED != NULL )
         DTEDClose( psDTED );
 }
@@ -325,6 +332,14 @@ GDALDataset *DTEDDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->SetMetadataItem( "DTED_RelVerticalAccuracy", pszValue );
     CPLFree( pszValue );
     
+    poDS->SetMetadataItem( GDALMD_AREA_OR_POINT, GDALMD_AOP_POINT );
+
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
@@ -557,7 +572,16 @@ DTEDCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
     DTEDClose( psDTED );
 
-    return (GDALDataset *) GDALOpen( pszFilename, GA_ReadOnly );
+/* -------------------------------------------------------------------- */
+/*      Reopen and copy missing information into a PAM file.            */
+/* -------------------------------------------------------------------- */
+    GDALPamDataset *poDS = (GDALPamDataset *) 
+        GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
 }
 
 /************************************************************************/
