@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ecwcreatecopy.cpp,v 1.13 2005/03/02 14:40:33 fwarmerdam Exp $
+ * $Id: ecwcreatecopy.cpp,v 1.21 2005/07/30 03:06:33 fwarmerdam Exp $
  *
  * Project:  GDAL ECW Driver
  * Purpose:  ECW CreateCopy method implementation.
@@ -28,6 +28,30 @@
  *****************************************************************************
  *
  * $Log: ecwcreatecopy.cpp,v $
+ * Revision 1.21  2005/07/30 03:06:33  fwarmerdam
+ * added LARGE_OK create option, and much improved error reporting
+ *
+ * Revision 1.20  2005/05/17 20:13:28  fwarmerdam
+ * use gmljp2://xml format for uri
+ *
+ * Revision 1.19  2005/05/11 14:38:42  fwarmerdam
+ * added dictionary support, updated to latest urn and gml.root-instance plans
+ *
+ * Revision 1.18  2005/05/03 21:11:47  fwarmerdam
+ * upgraded to copy PAM information
+ *
+ * Revision 1.17  2005/04/12 03:58:34  fwarmerdam
+ * turn ownership of fpVSIL over to vsiiostream
+ *
+ * Revision 1.16  2005/04/02 22:05:21  fwarmerdam
+ * ifdef out some stuff on ECW_FW
+ *
+ * Revision 1.15  2005/04/02 21:26:34  fwarmerdam
+ * preliminary support for creating GML coverage boxes
+ *
+ * Revision 1.14  2005/03/22 04:37:47  fwarmerdam
+ * Integrated ECW_LARGE_OK option.
+ *
  * Revision 1.13  2005/03/02 14:40:33  fwarmerdam
  * Fixed up TARGET validation.
  *
@@ -70,13 +94,14 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 #include "cpl_conv.h"
 #include "vsiiostream.h"
+#include "jp2userbox.h"
 
-CPL_CVSID("$Id: ecwcreatecopy.cpp,v 1.13 2005/03/02 14:40:33 fwarmerdam Exp $");
+CPL_CVSID("$Id: ecwcreatecopy.cpp,v 1.21 2005/07/30 03:06:33 fwarmerdam Exp $");
 
 #if defined(FRMT_ecw) && defined(HAVE_COMPRESS)
 
@@ -97,12 +122,16 @@ public:
                         int bIsJPEG2000 );
     CPLErr  CloseDown();
 
+    CPLErr  PrepareCoverageBox( const char *pszWKT, double *padfGeoTransform );
+
+#ifdef ECW_FW
+    CNCSJP2File::CNCSJPXAssocBox  m_oGMLAssoc;
+#endif
+
     // Data
 
     GDALDataset *m_poSrcDS;
 
-    FILE *fpVSIL;
-
     VSIIOStream m_OStream;
     int m_nPercentComplete;
 
@@ -127,7 +156,6 @@ GDALECWCompressor::GDALECWCompressor()
     m_bCancelled = FALSE;
     pfnProgress = GDALDummyProgress;
     pProgressData = NULL;
-    fpVSIL = NULL;
 }
 
 /************************************************************************/
@@ -154,9 +182,6 @@ CPLErr GDALECWCompressor::CloseDown()
 
     Close( true );
 
-    if( fpVSIL != NULL )
-        VSIFCloseL( fpVSIL );
-
     return CE_None;
 }
 
@@ -224,6 +249,199 @@ bool GDALECWCompressor::WriteCancel()
     return (bool) m_bCancelled;
 }
 
+/************************************************************************/
+/*                         PrepareCoverageBox()                         */
+/************************************************************************/
+
+CPLErr  GDALECWCompressor::PrepareCoverageBox( const char *pszWKT, 
+                                               double *padfGeoTransform )
+
+{
+#ifndef ECW_FW
+    return CE_Failure;
+#else
+/* -------------------------------------------------------------------- */
+/*      Try do determine a PCS or GCS code we can use.                  */
+/* -------------------------------------------------------------------- */
+    OGRSpatialReference oSRS;
+    char *pszWKTCopy = (char *) pszWKT;
+    int nEPSGCode = 0;
+    char szSRSName[100];
+
+    if( oSRS.importFromWkt( &pszWKTCopy ) != OGRERR_NONE )
+        return CE_Failure;
+
+    if( oSRS.IsProjected() )
+    {
+        const char *pszAuthName = oSRS.GetAuthorityName( "PROJCS" );
+
+        if( pszAuthName != NULL && EQUAL(pszAuthName,"epsg") )
+        {
+            nEPSGCode = atoi(oSRS.GetAuthorityCode( "PROJCS" ));
+        }
+    }
+    else if( oSRS.IsGeographic() )
+    {
+        const char *pszAuthName = oSRS.GetAuthorityName( "GEOGCS" );
+
+        if( pszAuthName != NULL && EQUAL(pszAuthName,"epsg") )
+        {
+            nEPSGCode = atoi(oSRS.GetAuthorityCode( "GEOGCS" ));
+        }
+    }
+
+    if( nEPSGCode != 0 )
+        sprintf( szSRSName, "urn:ogc:def:crs:EPSG::%d", nEPSGCode );
+    else
+        strcpy( szSRSName, 
+                "gmljp2://xml/CRSDictionary.gml#ogrcrs1" );
+
+/* -------------------------------------------------------------------- */
+/*      For now we hardcode for a minimal instance format.              */
+/* -------------------------------------------------------------------- */
+    char szDoc[4000];
+
+    sprintf( szDoc, 
+"<gml:FeatureCollection\n"
+"   xmlns:gml=\"http://www.opengis.net/gml\"\n"
+"   xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
+"   xsi:schemaLocation=\"http://www.opengis.net/gml http://www.math.ubc.ca/~burggraf/gml/gml4jp2.xsd\">\n"
+"  <gml:boundedBy>\n"
+"    <gml:Null>withheld</gml:Null>\n"
+"  </gml:boundedBy>\n"
+"  <gml:featureMember>\n"
+"    <gml:FeatureCollection>\n"
+"      <gml:featureMember>\n"
+"        <gml:RectifiedGridCoverage dimension=\"2\" gml:id=\"RGC0001\">\n"
+"          <gml:rectifiedGridDomain>\n"
+"            <gml:RectifiedGrid dimension=\"2\">\n"
+"              <gml:limits>\n"
+"                <gml:GridEnvelope>\n"
+"                  <gml:low>0 0</gml:low>\n"
+"                  <gml:high>%d %d</gml:high>\n"
+"                </gml:GridEnvelope>\n"
+"              </gml:limits>\n"
+"              <gml:axisName>x</gml:axisName>\n"
+"              <gml:axisName>y</gml:axisName>\n"
+"              <gml:origin>\n"
+"                <gml:Point gml:id=\"P0001\" srsName=\"%s\">\n"
+"                  <gml:pos>%.15g %.15g</gml:pos>\n"
+"                </gml:Point>\n"
+"              </gml:origin>\n"
+"              <gml:offsetVector srsName=\"%s\">%.15g %.15g</gml:offsetVector>\n"
+"              <gml:offsetVector srsName=\"%s\">%.15g %.15g</gml:offsetVector>\n"
+"            </gml:RectifiedGrid>\n"
+"          </gml:rectifiedGridDomain>\n"
+"          <gml:rangeSet>\n"
+"            <gml:File>\n"
+"              <gml:fileName>urn:ogc:tc:gmljp2:codestream:0</gml:fileName>\n"
+"              <gml:fileStructure>Record Interleaved</gml:fileStructure>\n"
+"            </gml:File>\n"
+"          </gml:rangeSet>\n"
+"        </gml:RectifiedGridCoverage>\n"
+"      </gml:featureMember>\n"
+"    </gml:FeatureCollection>\n"
+"  </gml:featureMember>\n"
+"</gml:FeatureCollection>\n",
+             sFileInfo.nSizeX-1, sFileInfo.nSizeY-1, 
+             szSRSName,
+             padfGeoTransform[0] + padfGeoTransform[1] * 0.5
+                                 + padfGeoTransform[4] * 0.5, 
+             padfGeoTransform[3] + padfGeoTransform[2] * 0.5
+                                 + padfGeoTransform[5] * 0.5,
+             szSRSName, 
+             padfGeoTransform[1], padfGeoTransform[2],
+             szSRSName,
+             padfGeoTransform[4], padfGeoTransform[5] );
+
+/* -------------------------------------------------------------------- */
+/*      If we need a user defined CRSDictionary entry, prepare it       */
+/*      here.                                                           */
+/* -------------------------------------------------------------------- */
+    char *pszDictBox = NULL;
+
+    if( nEPSGCode == 0 )
+    {
+        char *pszGMLDef = NULL;
+
+        if( oSRS.exportToXML( &pszGMLDef, NULL ) == OGRERR_NONE )
+        {
+            pszDictBox = (char *) CPLMalloc(strlen(pszGMLDef) + 4000);
+            
+            sprintf( pszDictBox, 
+"<gml:Dictionary gml:id=\"CRSU1\" \n"
+"        xmlns:gml=\"http://www.opengis.net/gml\"\n"
+"        xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n"
+"        xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n"
+"  <gml:dictionaryEntry>\n"
+"%s\n"
+"  </gml:dictionaryEntry>\n"
+"</gml:Dictionary>\n",
+                     pszGMLDef );
+        }
+        CPLFree( pszGMLDef );
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Setup the various required boxes.                               */
+/* -------------------------------------------------------------------- */
+    JP2UserBox *poGMLData;
+    CNCSJP2File::CNCSJPXAssocBox *poAssoc;
+    CNCSJP2File::CNCSJPXLabelBox *poLabel;
+
+    poLabel = new CNCSJP2File::CNCSJPXLabelBox();
+    poLabel->SetLabel( "gml.data" );
+    poLabel->m_bValid = true;
+    m_oGMLAssoc.m_OtherBoxes.push_back( poLabel );
+    m_oGMLAssoc.m_OwnedBoxes.push_back( poLabel );
+    
+    poAssoc = new CNCSJP2File::CNCSJPXAssocBox();
+    m_oGMLAssoc.m_OtherBoxes.push_back( poAssoc );
+    m_oGMLAssoc.m_OwnedBoxes.push_back( poAssoc );
+    poAssoc->m_bValid = true;
+
+    poLabel = new CNCSJP2File::CNCSJPXLabelBox();
+    poLabel->SetLabel( "gml.root-instance" );
+    poLabel->m_bValid = true;
+    poAssoc->m_OtherBoxes.push_back( poLabel );
+    poAssoc->m_OwnedBoxes.push_back( poLabel );
+
+    poGMLData = new JP2UserBox();
+    poGMLData->m_nTBox = 'xml ';
+    poGMLData->SetData( strlen( szDoc ), (unsigned char *) szDoc );
+    poAssoc->m_OtherBoxes.push_back( poGMLData );
+    poAssoc->m_OwnedBoxes.push_back( poGMLData );
+
+    if( pszDictBox != NULL )
+    {
+        poAssoc = new CNCSJP2File::CNCSJPXAssocBox();
+        m_oGMLAssoc.m_OtherBoxes.push_back( poAssoc );
+        m_oGMLAssoc.m_OwnedBoxes.push_back( poAssoc );
+        poAssoc->m_bValid = true;
+        
+        poLabel = new CNCSJP2File::CNCSJPXLabelBox();
+        poLabel->SetLabel( "CRSDictionary.gml" );
+        poLabel->m_bValid = true;
+        poAssoc->m_OtherBoxes.push_back( poLabel );
+        poAssoc->m_OwnedBoxes.push_back( poLabel );
+
+        poGMLData = new JP2UserBox();
+        poGMLData->m_nTBox = 'xml ';
+        poGMLData->SetData( strlen(pszDictBox), 
+                            (unsigned char *) pszDictBox );
+        poAssoc->m_OtherBoxes.push_back( poGMLData );
+        poAssoc->m_OwnedBoxes.push_back( poGMLData );
+
+        CPLFree( pszDictBox );
+    }
+
+    m_oGMLAssoc.m_bValid = true;
+    AddBox( &m_oGMLAssoc );
+
+    return CE_None;
+#endif /* def ECW_FW */
+}
+
 /************************************************************************/
 /*                        ECWTranslateFromWKT()                         */
 /************************************************************************/
@@ -625,7 +843,23 @@ CPLErr GDALECWCompressor::Initialize(
             SetParameter( 
                 CNCSJP2FileView::JP2_COMPRESS_PROGRESSION_RPCL );
 
-        // JP2_GEODATA_USAGE? 
+#ifdef ECW_FW
+        pszOption = CSLFetchNameValue(papszOptions, "GEODATA_USAGE");
+        if( pszOption == NULL )
+            /* do nothing special */;
+        else if( EQUAL(pszOption,"NONE") )
+            SetGeodataUsage( JP2_GEODATA_USE_NONE );
+        else if( EQUAL(pszOption,"PCS_ONLY") )
+            SetGeodataUsage( JP2_GEODATA_USE_PCS_ONLY );
+        else if( EQUAL(pszOption,"GML_ONLY") )
+            SetGeodataUsage( JP2_GEODATA_USE_GML_ONLY );
+        else if( EQUAL(pszOption,"PCS_GML") )
+            SetGeodataUsage( JP2_GEODATA_USE_PCS_GML );
+        else if( EQUAL(pszOption,"GML_PCS") )
+            SetGeodataUsage( JP2_GEODATA_USE_GML_PCS );
+        else if( EQUAL(pszOption,"ALL") )
+            SetGeodataUsage( JP2_GEODATA_USE_GML_PCS_WLD );
+#endif
 
         pszOption = CSLFetchNameValue(papszOptions, "DECOMPRESS_LAYERS");
         if( pszOption != NULL )
@@ -694,10 +928,15 @@ CPLErr GDALECWCompressor::Initialize(
     CPLDebug( "ECW", "Writing with PROJ=%s, DATUM=%s", 
               szProjection, szDatum );
 
+/* -------------------------------------------------------------------- */
+/*      Setup GML information.                                          */
+/* -------------------------------------------------------------------- */
+    PrepareCoverageBox( pszWKT, padfGeoTransform );
+    
 /* -------------------------------------------------------------------- */
 /*      Handle special case of a JPEG2000 data stream in another file.  */
 /* -------------------------------------------------------------------- */
-    fpVSIL = NULL;
+    FILE *fpVSIL = NULL;
 
     if( EQUALN(pszFilename,"J2K_SUBFILE:",12) )
     {
@@ -736,6 +975,25 @@ CPLErr GDALECWCompressor::Initialize(
                           subfile_offset, subfile_size );
     }
 
+
+/* -------------------------------------------------------------------- */
+/*      Check if we can enable large files.  This option should only    */
+/*      be set when the application is adhering to one of the           */
+/*      ERMapper options for licensing larger than 500MB input          */
+/*      files.  See Bug 767.                                            */
+/* -------------------------------------------------------------------- */
+    const char *pszLargeOK = CSLFetchNameValue(papszOptions, "LARGE_OK");
+    if( pszLargeOK == NULL )
+        pszLargeOK = "NO";
+
+    pszLargeOK = CPLGetConfigOption( "ECW_LARGE_OK", pszLargeOK );
+
+    if( CSLTestBoolean(pszLargeOK) )
+    {
+        CNCSFile::SetKeySize();
+        CPLDebug( "ECW", "Large file generation enabled." );
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Set the file info.                                              */
 /* -------------------------------------------------------------------- */
@@ -751,8 +1009,20 @@ CPLErr GDALECWCompressor::Initialize(
             oError = CNCSJP2FileView::Open( &(m_OStream) );
     }
 
-
-    return CE_None;
+    if( oError.GetErrorNumber() == NCS_SUCCESS )
+        return CE_None;
+    else if( oError.GetErrorNumber() == NCS_INPUT_SIZE_EXCEEDED )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "ECW SDK 500MB compress limit exceeded." );
+        return CE_Failure;
+    }
+    else
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "%s", oError.GetErrorMessage() );
+        return CE_Failure;
+    }
 }
 
 /************************************************************************/
@@ -808,7 +1078,16 @@ ECWCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     oCompressor.CloseDown();
     pfnProgress( 1.001, NULL, pProgressData );
 
-    return (GDALDataset *) GDALOpen( pszFilename, GA_ReadOnly );
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxilary pam information.         */
+/* -------------------------------------------------------------------- */
+    GDALDataset *poDS = (GDALDataset *) 
+        GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        ((GDALPamDataset *) poDS)->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
 }
 
 /************************************************************************/
