@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ecwdataset.cpp,v 1.39 2005/03/03 17:05:40 fwarmerdam Exp $
+ * $Id: ecwdataset.cpp,v 1.53 2005/09/26 21:09:31 fwarmerdam Exp $
  *
  * Project:  GDAL 
  * Purpose:  ECW (ERMapper Wavelet Compression Format) Driver
@@ -28,6 +28,48 @@
  *****************************************************************************
  *
  * $Log: ecwdataset.cpp,v $
+ * Revision 1.53  2005/09/26 21:09:31  fwarmerdam
+ * Avoid problems in deletion of the iostream.
+ *
+ * Revision 1.52  2005/07/30 03:05:56  fwarmerdam
+ * Added LARGE_OK, frmt_jp2ecw.html link
+ *
+ * Revision 1.51  2005/07/05 22:09:23  fwarmerdam
+ * added ParseMSIG() call
+ *
+ * Revision 1.50  2005/06/28 14:17:06  fwarmerdam
+ * Added worldfile support (on read).
+ *
+ * Revision 1.49  2005/06/24 20:54:00  fwarmerdam
+ * added explicit init/shutdown calls
+ *
+ * Revision 1.48  2005/05/23 06:56:10  fwarmerdam
+ * disable pam support for subfile streams
+ *
+ * Revision 1.47  2005/05/03 21:12:14  fwarmerdam
+ * use GDALJP2Metadata facilities for GML and GeoTIFF
+ *
+ * Revision 1.46  2005/04/25 18:40:16  fwarmerdam
+ * It seems dynamic cast is bad karma on VC6 with /Ox
+ *
+ * Revision 1.45  2005/04/25 04:32:23  fwarmerdam
+ * dynamic cast seems to throw exception sometimes
+ *
+ * Revision 1.44  2005/04/12 03:58:34  fwarmerdam
+ * turn ownership of fpVSIL over to vsiiostream
+ *
+ * Revision 1.43  2005/04/11 13:56:27  fwarmerdam
+ * Added a bit of reformatting.
+ *
+ * Revision 1.42  2005/04/11 13:52:50  fwarmerdam
+ * added proper handling of iostream cleanup from David Carter
+ *
+ * Revision 1.41  2005/04/02 22:02:12  fwarmerdam
+ * initialize variables
+ *
+ * Revision 1.40  2005/04/02 21:22:21  fwarmerdam
+ * added GML and GeoTIFF box direct reading
+ *
  * Revision 1.39  2005/03/03 17:05:40  fwarmerdam
  * fixed up serious problems with non-8bit data in ECWDataset::IRasterIO
  *
@@ -98,67 +140,20 @@
  *
  * Revision 1.18  2004/03/30 17:00:07  warmerda
  * Allow "ecwp:" urls to be opened.
- *
- * Revision 1.17  2003/12/16 20:24:01  dron
- * Implemented support for supersampling in IRasterIO().
- *
- * Revision 1.16  2003/10/27 15:04:36  warmerda
- * fix small new memory leak
- *
- * Revision 1.15  2003/10/27 15:02:44  warmerda
- * Added ECWDataset::IRasterIO special case
- *
- * Revision 1.14  2003/09/23 13:47:47  warmerda
- * never return a NULL projection
- *
- * Revision 1.13  2003/06/19 18:50:20  warmerda
- * added projection reading support
- *
- * Revision 1.12  2003/06/18 17:26:35  warmerda
- * dont try ecwopen function if path isnt to a valid file
- *
- * Revision 1.11  2002/11/23 18:54:17  warmerda
- * added CREATIONDATATYPES metadata for drivers
- *
- * Revision 1.10  2002/10/01 19:34:27  warmerda
- * fixed problems with supersampling by forcing through fullres blocks
- *
- * Revision 1.9  2002/09/04 06:50:37  warmerda
- * avoid static driver pointers
- *
- * Revision 1.8  2002/07/23 14:40:41  warmerda
- * disable compress support on Unix
- *
- * Revision 1.7  2002/07/23 14:05:35  warmerda
- * Avoid warnings.
- *
- * Revision 1.6  2002/06/12 21:12:25  warmerda
- * update to metadata based driver info
- *
- * Revision 1.5  2002/01/22 14:48:19  warmerda
- * Fixed pabyWorkBuffer memory leak in IRasterIO().
- *
- * Revision 1.4  2001/11/11 23:50:59  warmerda
- * added required class keyword to friend declarations
- *
- * Revision 1.3  2001/07/18 04:51:56  warmerda
- * added CPL_CVSID
- *
- * Revision 1.2  2001/04/20 03:06:07  warmerda
- * addec compression support
- *
- * Revision 1.1  2001/04/02 17:06:46  warmerda
- * New
- *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
+#include "gdaljp2metadata.h"
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 #include "cpl_conv.h"
 #include "vsiiostream.h"
+#include "cpl_multiproc.h"
+#include "cpl_minixml.h"
+#include "ogr_api.h"
+#include "ogr_geometry.h"
 
-CPL_CVSID("$Id: ecwdataset.cpp,v 1.39 2005/03/03 17:05:40 fwarmerdam Exp $");
+CPL_CVSID("$Id: ecwdataset.cpp,v 1.53 2005/09/26 21:09:31 fwarmerdam Exp $");
 
 #ifdef FRMT_ecw
 
@@ -168,6 +163,17 @@ static unsigned char jp2_header[] =
 
 static int    gnTriedCSFile = FALSE;
 static char **gpapszCSLookup = NULL;
+static void *hECWDatasetMutex = NULL;
+
+CPL_C_START
+CPLErr CPL_DLL GTIFMemBufFromWkt( const char *pszWKT, 
+                                  const double *padfGeoTransform,
+                                  int nGCPCount, const GDAL_GCP *pasGCPList,
+                                  int *pnSize, unsigned char **ppabyBuffer );
+CPLErr CPL_DLL GTIFWktFromMemBuf( int nSize, unsigned char *pabyBuffer, 
+                          char **ppszWKT, double *padfGeoTransform,
+                          int *pnGCPCount, GDAL_GCP **ppasGCPList );
+CPL_C_END
 
 /************************************************************************/
 /* ==================================================================== */
@@ -177,18 +183,18 @@ static char **gpapszCSLookup = NULL;
 
 class ECWRasterBand;
 
-class CPL_DLL ECWDataset : public GDALDataset
+class CPL_DLL ECWDataset : public GDALPamDataset
 {
     friend class ECWRasterBand;
 
-    FILE        *fpVSIL;
-    
     CNCSJP2FileView *poFileView;
     NCSFileViewFileInfoEx *psFileInfo;
 
     GDALDataType eRasterDataType;
     NCSEcwCellType eNCSRequestDataType;
 
+    int         bUsingCustomStream;
+
     // Current view window. 
     int         bWinActive;
     int         nWinXOff, nWinYOff, nWinXSize, nWinYSize;
@@ -198,7 +204,13 @@ class CPL_DLL ECWDataset : public GDALDataset
     int         nWinBufLoaded;
     void        **papCurLineBuf;
 
+    int         bGeoTransformValid;
+    double      adfGeoTransform[6];
     char        *pszProjection;
+    int         nGCPCount;
+    GDAL_GCP    *pasGCPList;
+
+    char        **papszGMLMetadata;
 
     void        ECW2WKTProjection();
 
@@ -208,7 +220,6 @@ class CPL_DLL ECWDataset : public GDALDataset
                                 int, int *, int, int, int );
     CPLErr      LoadNextLine();
 
-    VSIIOStream *poIOStream;
 
   public:
     		ECWDataset();
@@ -225,6 +236,12 @@ class CPL_DLL ECWDataset : public GDALDataset
     virtual CPLErr GetGeoTransform( double * );
     virtual const char *GetProjectionRef();
 
+    virtual int    GetGCPCount();
+    virtual const char *GetGCPProjection();
+    virtual const GDAL_GCP *GetGCPs();
+
+    virtual char      **GetMetadata( const char * pszDomain = "" );
+
     virtual CPLErr AdviseRead( int nXOff, int nYOff, int nXSize, int nYSize,
                                int nBufXSize, int nBufYSize, 
                                GDALDataType eDT, 
@@ -238,7 +255,7 @@ class CPL_DLL ECWDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class ECWRasterBand : public GDALRasterBand
+class ECWRasterBand : public GDALPamRasterBand
 {
     friend class ECWDataset;
     
@@ -593,13 +610,23 @@ CPLErr ECWRasterBand::IReadBlock( int, int nBlockYOff, void * pImage )
 ECWDataset::ECWDataset()
 
 {
+    bUsingCustomStream = FALSE;
     pszProjection = NULL;
     poFileView = NULL;
-    fpVSIL = NULL;
     bWinActive = FALSE;
     panWinBandList = NULL;
     eRasterDataType = GDT_Byte;
-    poIOStream = NULL;
+    nGCPCount = 0;
+    pasGCPList = NULL;
+    papszGMLMetadata = NULL;
+    
+    bGeoTransformValid = FALSE;
+    adfGeoTransform[0] = 0.0;
+    adfGeoTransform[1] = 1.0;
+    adfGeoTransform[2] = 0.0;
+    adfGeoTransform[3] = 0.0;
+    adfGeoTransform[4] = 0.0;
+    adfGeoTransform[5] = 1.0;
 }
 
 /************************************************************************/
@@ -609,17 +636,55 @@ ECWDataset::ECWDataset()
 ECWDataset::~ECWDataset()
 
 {
+    FlushCache();
     CleanupWindow();
     CPLFree( pszProjection );
-    if( poFileView != NULL )
+    CSLDestroy( papszGMLMetadata );
+
+    if( nGCPCount > 0 )
     {
-        delete poFileView;
+        GDALDeinitGCPs( nGCPCount, pasGCPList );
+        CPLFree( pasGCPList );
     }
 
-    if( poIOStream != NULL )
-        delete poIOStream;
-    if( fpVSIL != NULL )
-        VSIFCloseL( fpVSIL );
+    if( hECWDatasetMutex == NULL )
+    {
+        hECWDatasetMutex = CPLCreateMutex();
+    }
+    else if( !CPLAcquireMutex( hECWDatasetMutex, 60.0 ) )
+    {
+        CPLDebug( "ECW", "Failed to acquire mutex in 60s." );
+    }
+    else
+    {
+        CPLDebug( "ECW", "Got mutex." );
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Release / dereference iostream.                                 */
+/* -------------------------------------------------------------------- */
+    // The underlying iostream of the CNCSJP2FileView (poFileView) object may 
+    // also be the underlying iostream of other CNCSJP2FileView (poFileView) 
+    // objects.  Consequently, when we delete the CNCSJP2FileView (poFileView) 
+    // object, we must decrement the nFileViewCount attribute of the underlying
+    // VSIIOStream object, and only delete the VSIIOStream object when 
+    // nFileViewCount is equal to zero.
+
+    if( poFileView != NULL )
+    {
+        VSIIOStream *poUnderlyingIOStream = (VSIIOStream *)NULL;
+
+        poUnderlyingIOStream = ((VSIIOStream *)(poFileView->GetStream()));
+        delete poFileView;
+
+        if( bUsingCustomStream )
+        {
+            if( --poUnderlyingIOStream->nFileViewCount == 0 )
+                delete poUnderlyingIOStream;
+        }
+    }
+
+    CPLReleaseMutex( hECWDatasetMutex );
 }
 
 /************************************************************************/
@@ -1004,9 +1069,6 @@ CPLErr ECWDataset::IRasterIO( GDALRWFlag eRWFlag,
         }
     }
 
-    CPLFree( papabyBIL );
-    CPLFree( pabyBILScanline );
-
     return CE_None;
 }
 
@@ -1109,12 +1171,50 @@ GDALDataset *ECWDataset::Open( GDALOpenInfo * poOpenInfo )
               return NULL;
           }
 
+          if( hECWDatasetMutex == NULL )
+          {
+              hECWDatasetMutex = CPLCreateMutex();
+          }
+          else if( !CPLAcquireMutex( hECWDatasetMutex, 60.0 ) )
+          {
+              CPLDebug( "ECW", "Failed to acquire mutex in 60s." );
+          }
+          else
+          {
+              CPLDebug( "ECW", "Got mutex." );
+          }
           poIOStream = new VSIIOStream();
           poIOStream->Access( fpVSIL, FALSE, real_filename,
                               subfile_offset, subfile_size );
 
           poFileView = new CNCSJP2FileView();
           oErr = poFileView->Open( poIOStream, false );
+
+          // The CNCSJP2FileView (poFileView) object may not use the iostream 
+          // (poIOStream) passed to the CNCSJP2FileView::Open() method if an 
+          // iostream is already available to the ECW JPEG 2000 SDK for a given
+          // file.  Consequently, if the iostream passed to 
+          // CNCSJP2FileView::Open() does not become the underlying iostream 
+          // of the CNCSJP2FileView object, then it should be deleted.
+          //
+          // In addition, the underlying iostream of the CNCSJP2FileView object
+          // should not be deleted until all CNCSJP2FileView objects using the 
+          // underlying iostream are deleted. Consequently, each time a 
+          // CNCSJP2FileView object is created, the nFileViewCount attribute 
+          // of the underlying VSIIOStream object must be incremented for use 
+          // in the ECWDataset destructor.
+		  
+          VSIIOStream * poUnderlyingIOStream = 
+              ((VSIIOStream *)(poFileView->GetStream()));
+          poUnderlyingIOStream->nFileViewCount++;
+
+          if ( poIOStream != poUnderlyingIOStream ) 
+          {
+              delete poIOStream;
+          } 		
+
+          CPLReleaseMutex( hECWDatasetMutex );
+
           if( oErr.GetErrorNumber() != NCS_SUCCESS )
           {
               CPLError( CE_Failure, CPLE_AppDefined, "%s",
@@ -1164,8 +1264,9 @@ GDALDataset *ECWDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS = new ECWDataset();
 
     poDS->poFileView = poFileView;
-    poDS->fpVSIL = fpVSIL;
-    poDS->poIOStream = poIOStream;
+
+    if( fpVSIL != NULL )
+        poDS->nPamFlags |= GPF_DISABLED;
 
 /* -------------------------------------------------------------------- */
 /*      Fetch general file information.                                 */
@@ -1237,11 +1338,95 @@ GDALDataset *ECWDataset::Open( GDALOpenInfo * poOpenInfo )
     for( i=0; i < poDS->psFileInfo->nBands; i++ )
         poDS->SetBand( i+1, new ECWRasterBand( poDS, i+1 ) );
 
-    poDS->ECW2WKTProjection();
+/* -------------------------------------------------------------------- */
+/*      Look for supporting coordinate system information.              */
+/* -------------------------------------------------------------------- */
+    if( fpVSIL == NULL )
+    {
+        GDALJP2Metadata oJP2Geo;
+        FILE *fpLL;
+        
+        fpLL = VSIFOpenL( poOpenInfo->pszFilename, "rb" );
+        
+        if( fpLL != NULL )
+        {
+            oJP2Geo.ReadBoxes( fpLL );
+            VSIFCloseL( fpLL );
+            
+            poDS->papszGMLMetadata = CSLDuplicate(oJP2Geo.papszGMLMetadata);
+        }
+        
+        if( oJP2Geo.ParseJP2GeoTIFF() 
+            || oJP2Geo.ParseGMLCoverageDesc() 
+            || oJP2Geo.ParseMSIG() )
+        {
+            poDS->pszProjection = CPLStrdup(oJP2Geo.pszProjection);
+            poDS->bGeoTransformValid = TRUE;
+            memcpy( poDS->adfGeoTransform, oJP2Geo.adfGeoTransform, 
+                    sizeof(double) * 6 );
+            poDS->nGCPCount = oJP2Geo.nGCPCount;
+            poDS->pasGCPList = oJP2Geo.pasGCPList;
+            oJP2Geo.pasGCPList = NULL;
+        }
+        else
+        {
+            poDS->ECW2WKTProjection();
+        }
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Check for world file.                                           */
+/* -------------------------------------------------------------------- */
+    poDS->bGeoTransformValid |= 
+        GDALReadWorldFile( poOpenInfo->pszFilename, ".eww", 
+                           poDS->adfGeoTransform )
+        || GDALReadWorldFile( poOpenInfo->pszFilename, ".ecww", 
+                              poDS->adfGeoTransform )
+        || GDALReadWorldFile( poOpenInfo->pszFilename, ".wld", 
+                              poDS->adfGeoTransform );
+
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
 
     return( poDS );
 }
 
+/************************************************************************/
+/*                            GetGCPCount()                             */
+/************************************************************************/
+
+int ECWDataset::GetGCPCount()
+
+{
+    return nGCPCount;
+}
+
+/************************************************************************/
+/*                          GetGCPProjection()                          */
+/************************************************************************/
+
+const char *ECWDataset::GetGCPProjection()
+
+{
+    if( nGCPCount > 0 )
+        return pszProjection;
+    else
+        return "";
+}
+
+/************************************************************************/
+/*                               GetGCP()                               */
+/************************************************************************/
+
+const GDAL_GCP *ECWDataset::GetGCPs()
+
+{
+    return pasGCPList;
+}
+
 /************************************************************************/
 /*                          GetProjectionRef()                          */
 /************************************************************************/
@@ -1250,7 +1435,7 @@ const char *ECWDataset::GetProjectionRef()
 
 {
     if( pszProjection == NULL )
-        return "";
+        return GDALPamDataset::GetProjectionRef();
     else
         return pszProjection;
 }
@@ -1262,15 +1447,13 @@ const char *ECWDataset::GetProjectionRef()
 CPLErr ECWDataset::GetGeoTransform( double * padfTransform )
 
 {
-    padfTransform[0] = psFileInfo->fOriginX;
-    padfTransform[1] = psFileInfo->fCellIncrementX;
-    padfTransform[2] = 0.0;
-
-    padfTransform[3] = psFileInfo->fOriginY;
-    padfTransform[4] = 0.0;
-    padfTransform[5] = psFileInfo->fCellIncrementY;
-
-    return( CE_None );
+    if( bGeoTransformValid )
+    {
+        memcpy( padfTransform, adfGeoTransform, sizeof(double) * 6 );
+        return( CE_None );
+    }
+    else
+        return GDALPamDataset::GetGeoTransform( padfTransform );
 }
 
 /************************************************************************/
@@ -1298,6 +1481,19 @@ char **ECWGetCSList()
     return gpapszCSLookup;
 }
 
+/************************************************************************/
+/*                            GetMetadata()                             */
+/************************************************************************/
+
+char **ECWDataset::GetMetadata( const char *pszDomain )
+
+{
+    if( pszDomain == NULL || !EQUAL(pszDomain,"GML") )
+        return GDALDataset::GetMetadata( pszDomain );
+    else
+        return papszGMLMetadata;
+}
+
 /************************************************************************/
 /*                         ECW2WKTProjection()                          */
 /*                                                                      */
@@ -1317,6 +1513,31 @@ void ECWDataset::ECW2WKTProjection()
     if( psFileInfo == NULL )
         return;
 
+/* -------------------------------------------------------------------- */
+/*      Capture Geotransform.                                           */
+/* -------------------------------------------------------------------- */
+
+    if( psFileInfo->fOriginX != 0.0 
+        || psFileInfo->fOriginY != 0.0 
+        && (psFileInfo->fCellIncrementX != 0.0 && 
+            psFileInfo->fCellIncrementX != 1.0)
+        && (psFileInfo->fCellIncrementY != 0.0 && 
+            psFileInfo->fCellIncrementY != 1.0) )
+    {
+        bGeoTransformValid = TRUE;
+        
+        adfGeoTransform[0] = psFileInfo->fOriginX;
+        adfGeoTransform[1] = psFileInfo->fCellIncrementX;
+        adfGeoTransform[2] = 0.0;
+        
+        adfGeoTransform[3] = psFileInfo->fOriginY;
+        adfGeoTransform[4] = 0.0;
+        adfGeoTransform[5] = psFileInfo->fCellIncrementY;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      do we have projection and datum?                                */
+/* -------------------------------------------------------------------- */
     CPLDebug( "ECW", "projection=%s, datum=%s",
               psFileInfo->szProjection, psFileInfo->szDatum );
 
@@ -1385,6 +1606,22 @@ void ECWDataset::ECW2WKTProjection()
 
 #endif /* def FRMT_ecw */
 
+/************************************************************************/
+/*                         GDALDeregister_ECW()                         */
+/************************************************************************/
+
+void GDALDeregister_ECW( GDALDriver * )
+
+{
+    if( gpapszCSLookup )
+    {
+        CSLDestroy( gpapszCSLookup );
+        gpapszCSLookup = NULL;
+        gnTriedCSFile = FALSE;
+        NCSecwShutdown();
+    }
+}
+
 /************************************************************************/
 /*                          GDALRegister_ECW()                        */
 /************************************************************************/
@@ -1399,6 +1636,8 @@ void GDALRegister_ECW()
     {
         poDriver = new GDALDriver();
         
+        NCSecwInit();
+
         poDriver->SetDescription( "ECW" );
         poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
                                    "ERMapper Compressed Wavelets" );
@@ -1407,6 +1646,7 @@ void GDALRegister_ECW()
         poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "ecw" );
         
         poDriver->pfnOpen = ECWDataset::OpenECW;
+        poDriver->pfnUnloadDriver = GDALDeregister_ECW;
 #ifdef HAVE_COMPRESS
 // The create method seems not to work properly.
 //        poDriver->pfnCreate = ECWCreateECW;  
@@ -1418,6 +1658,7 @@ void GDALRegister_ECW()
 "   <Option name='TARGET' type='float' description='Compression Percentage' />"
 "   <Option name='PROJ' type='string' description='ERMapper Projection Name'/>"
 "   <Option name='DATUM' type='string' description='ERMapper Datum Name' />"
+"   <Option name='LARGE_OK' type='boolean' description='Enable compressing 500+MB files'/>"
 "</CreationOptionList>" );
 #endif
 
@@ -1444,7 +1685,7 @@ void GDALRegister_JP2ECW()
         poDriver->SetMetadataItem( GDAL_DMD_LONGNAME, 
                                    "ERMapper JPEG2000" );
         poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, 
-                                   "frmt_ecw.html" );
+                                   "frmt_jp2ecw.html" );
         poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "jp2" );
         
         poDriver->pfnOpen = ECWDataset::OpenJPEG2000;
@@ -1458,6 +1699,7 @@ void GDALRegister_JP2ECW()
 "   <Option name='TARGET' type='float' description='Compression Percentage' />"
 "   <Option name='PROJ' type='string' description='ERMapper Projection Name'/>"
 "   <Option name='DATUM' type='string' description='ERMapper Datum Name' />"
+"   <Option name='LARGE_OK' type='boolean' description='Enable compressing 500+MB files'/>"
 "   <Option name='PROFILE' type='string-select'>"
 "       <Value>BASELINE_0</Value>"
 "       <Value>BASELINE_1</Value>"
@@ -1489,3 +1731,5 @@ void GDALRegister_JP2ECW()
 #endif /* def FRMT_ecw */
 }
 
+
+
