@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gifdataset.cpp,v 1.21 2004/08/20 17:33:06 warmerda Exp $
+ * $Id: gifdataset.cpp,v 1.23 2005/08/04 18:44:33 fwarmerdam Exp $
  *
  * Project:  GIF Driver
  * Purpose:  Implement GDAL GIF Support using libungif code.  
@@ -28,6 +28,13 @@
  ******************************************************************************
  * 
  * $Log: gifdataset.cpp,v $
+ * Revision 1.23  2005/08/04 18:44:33  fwarmerdam
+ * Report background as metadata rather than NODATA.  It was causing too
+ * many problems.
+ *
+ * Revision 1.22  2005/05/05 14:05:03  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.21  2004/08/20 17:33:06  warmerda
  * Support overviews.
  *
@@ -95,10 +102,10 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: gifdataset.cpp,v 1.21 2004/08/20 17:33:06 warmerda Exp $");
+CPL_CVSID("$Id: gifdataset.cpp,v 1.23 2005/08/04 18:44:33 fwarmerdam Exp $");
 
 CPL_C_START
 #include "gif_lib.h"
@@ -119,7 +126,7 @@ static int InterlacedJumps[] = { 8, 8, 4, 2 };
 
 class GIFRasterBand;
 
-class GIFDataset : public GDALDataset
+class GIFDataset : public GDALPamDataset
 {
     friend class GIFRasterBand;
 
@@ -142,7 +149,7 @@ class GIFDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class GIFRasterBand : public GDALRasterBand
+class GIFRasterBand : public GDALPamRasterBand
 {
     friend class GIFDataset;
 
@@ -251,19 +258,17 @@ GIFRasterBand::GIFRasterBand( GIFDataset *poDS, int nBand,
     }
 
 /* -------------------------------------------------------------------- */
-/*      If we have a known background color, but no identified          */
-/*      transparent color, then we use the background color as the      */
-/*      NODATA value, but we don't mark it in the colormap as           */
-/*      transparent.  "background" is a slightly different concept      */
-/*      than transparent, but we still want to be able to do it         */
-/*      transparent in some apps (like MapServer).                      */
+/*      If we have a background value, return it here.  Some            */
+/*      applications might want to treat this as transparent, but in    */
+/*      many uses this is inappropriate so we don't return it as        */
+/*      nodata or transparent.                                          */
 /* -------------------------------------------------------------------- */
-    if( nTransparentColor == -1 )
+    if( nBackground != 255 )
     {
-	if( nBackground == 255 )
-	    nTransparentColor = 0;
-	else
-	    nTransparentColor = nBackground;
+        char szBackground[10];
+        
+        sprintf( szBackground, "%d", nBackground );
+        SetMetadataItem( "GIF_BACKGROUND", szBackground );
     }
 }
 
@@ -363,6 +368,7 @@ GIFDataset::GIFDataset()
 GIFDataset::~GIFDataset()
 
 {
+    FlushCache();
     DGifCloseFile( hGifFile );
 }
 
@@ -479,6 +485,12 @@ GDALDataset *GIFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return poDS;
 }
 
@@ -665,7 +677,16 @@ GIFCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 	GDALWriteWorldFile( pszFilename, "wld", adfGeoTransform );
     }
 
-    return (GDALDataset *) GDALOpen( pszFilename, GA_ReadOnly );
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxilary pam information.         */
+/* -------------------------------------------------------------------- */
+    GDALPamDataset *poDS = (GDALPamDataset *) 
+        GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
 }
 
 /************************************************************************/
