@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: grass57dataset.cpp,v 1.3 2004/09/24 14:28:49 fwarmerdam Exp $
+ * $Id: grass57dataset.cpp,v 1.5 2005/09/30 21:07:57 fwarmerdam Exp $
  *
  * Project:  GRASS Driver
  * Purpose:  Implement GRASS raster read/write support
@@ -31,6 +31,12 @@
  ******************************************************************************
  *
  * $Log: grass57dataset.cpp,v $
+ * Revision 1.5  2005/09/30 21:07:57  fwarmerdam
+ * applied patches from bug 822, add georef support
+ *
+ * Revision 1.4  2005/05/06 18:21:14  fwarmerdam
+ * Applied patch from Radim.  Better logic to reset the region.
+ *
  * Revision 1.3  2004/09/24 14:28:49  fwarmerdam
  * fixed some typos with floating point (from Radim)
  *
@@ -64,7 +70,7 @@ extern "C" {
 
 #define GRASS_MAX_COLORS 100000  // what is the right value
 
-CPL_CVSID("$Id: grass57dataset.cpp,v 1.3 2004/09/24 14:28:49 fwarmerdam Exp $");
+CPL_CVSID("$Id: grass57dataset.cpp,v 1.5 2005/09/30 21:07:57 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_GRASS(void);
@@ -420,6 +426,24 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 	G_copy((void *) &sOpenWindow, (void *) sNewWindow, sizeof(struct Cell_head));
 	
     }
+    else
+    {
+        /* The windows are identical, check current window */
+        struct Cell_head sCurrentWindow;
+
+        G_get_window ( &sCurrentWindow );
+
+        if ( sNewWindow->north  != sCurrentWindow.north  || sNewWindow->south  != sCurrentWindow.south ||
+             sNewWindow->east   != sCurrentWindow.east   || sNewWindow->west   != sCurrentWindow.west ||
+             sNewWindow->ew_res != sCurrentWindow.ew_res || sNewWindow->ns_res != sCurrentWindow.ns_res ||
+             sNewWindow->rows   != sCurrentWindow.rows   || sNewWindow->cols   != sCurrentWindow.cols
+             )
+        {
+            /* Reset window */
+            G_set_window( sNewWindow );
+        }
+    }
+
 
     return CE_None;
 }
@@ -802,8 +826,10 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
     
     
     if ( !getenv( "GISBASE" ) ) {
-	strcpy( fake_gisbase, "GISBASE=/fake_gisbase" );
-	putenv( fake_gisbase );
+	// we are outside a GRASS session
+	// TODO: use function instead of hardcoded path
+	sprintf(fake_gisbase, "GISBASE=/usr/local/share/gdal/grass/" );	
+        putenv( fake_gisbase );
 	hasGisbase = false;
     } else {
 	hasGisbase = true;
@@ -916,8 +942,7 @@ GDALDataset *GRASSDataset::Open( GDALOpenInfo * poOpenInfo )
     if ( hasGisbase ) {
 	projinfo = G_get_projinfo();
 	projunits = G_get_projunits();
-	// TODO: disabled because creates circular reference GRASS <-> GDAL
-	//poDS->pszProjection = GPJ_grass_to_wkt ( projinfo, projunits, 0, 0);
+        poDS->pszProjection = GPJ_grass_to_wkt ( projinfo, projunits, 0, 0);
     }
 
 /* -------------------------------------------------------------------- */
