@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: geo_normalize.c,v 1.42 2005/02/17 01:21:38 fwarmerdam Exp $
+ * $Id: geo_normalize.c,v 1.45 2005/03/15 16:01:18 fwarmerdam Exp $
  *
  * Project:  libgeotiff
  * Purpose:  Code to normalize PCS and other composite codes in a GeoTIFF file.
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: geo_normalize.c,v $
+ * Revision 1.45  2005/03/15 16:01:18  fwarmerdam
+ * zero inv flattening interpreted as sphere
+ *
+ * Revision 1.44  2005/03/04 04:32:37  fwarmerdam
+ * added cylindricalequalarea support
+ *
+ * Revision 1.43  2005/03/04 04:02:40  fwarmerdam
+ * Fixed initialization of dfStdParallel2 for AEA and EC.
+ *
  * Revision 1.42  2005/02/17 01:21:38  fwarmerdam
  * fixed handling of ProjFalseOrigin{Easting,Northing}GeoKey
  *
@@ -1700,7 +1709,7 @@ static void GTIFFetchProjParms( GTIF * psGTIF, GTIFDefn * psDefn )
 
         if( GTIFKeyGet(psGTIF, ProjStdParallel2GeoKey, 
                        &dfStdParallel2, 0, 1 ) == 0 )
-            dfStdParallel1 = 0.0;
+            dfStdParallel2 = 0.0;
 
         if( GTIFKeyGet(psGTIF, ProjNatOriginLongGeoKey, 
                        &dfNatOriginLong, 0, 1 ) == 0
@@ -1733,6 +1742,35 @@ static void GTIFFetchProjParms( GTIF * psGTIF, GTIFDefn * psDefn )
         psDefn->ProjParm[6] = dfFalseNorthing;
         psDefn->ProjParmId[6] = ProjFalseNorthingGeoKey;
 
+        psDefn->nParms = 7;
+        break;
+
+/* -------------------------------------------------------------------- */
+      case CT_CylindricalEqualArea:
+/* -------------------------------------------------------------------- */
+        if( GTIFKeyGet(psGTIF, ProjStdParallel1GeoKey, 
+                       &dfStdParallel1, 0, 1 ) == 0 )
+            dfStdParallel1 = 0.0;
+
+        if( GTIFKeyGet(psGTIF, ProjNatOriginLongGeoKey, 
+                       &dfNatOriginLong, 0, 1 ) == 0
+            && GTIFKeyGet(psGTIF, ProjFalseOriginLongGeoKey, 
+                          &dfNatOriginLong, 0, 1 ) == 0
+            && GTIFKeyGet(psGTIF, ProjCenterLongGeoKey, 
+                          &dfNatOriginLong, 0, 1 ) == 0 )
+            dfNatOriginLong = 0.0;
+
+        /* notdef: should transform to decimal degrees at this point */
+
+        psDefn->ProjParm[0] = dfStdParallel1;
+        psDefn->ProjParmId[0] = ProjStdParallel1GeoKey;
+        psDefn->ProjParm[1] = dfNatOriginLong;
+        psDefn->ProjParmId[1] = ProjNatOriginLongGeoKey;
+        psDefn->ProjParm[5] = dfFalseEasting;
+        psDefn->ProjParmId[5] = ProjFalseEastingGeoKey;
+        psDefn->ProjParm[6] = dfFalseNorthing;
+        psDefn->ProjParmId[6] = ProjFalseNorthingGeoKey;
+
         psDefn->nParms = 7;
         break;
     }
@@ -2029,6 +2067,8 @@ int GTIFGetDefn( GTIF * psGTIF, GTIFDefn * psDefn )
         if( dfInvFlattening != 0.0 )
             psDefn->SemiMinor = 
                 psDefn->SemiMajor * (1 - 1.0/dfInvFlattening);
+        else
+            psDefn->SemiMinor = psDefn->SemiMajor;
     }
     
 /* -------------------------------------------------------------------- */
