@@ -1,5 +1,5 @@
 /******************************************************************************
- * tif_overview.c,v 1.1 2000/11/24 18:13:43 warmerda Exp
+ * $Id: $
  *
  * Project:  TIFF Overview Builder
  * Purpose:  Library function for building overviews in a TIFF file.
@@ -43,24 +43,6 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  ******************************************************************************
- *
- * tif_overview.c,v
- * Revision 1.6  2004/07/26 14:27:45  warmerda
- * updated from libtiff CVS
- *
- * Revision 1.5  2004/07/26 14:27:27  warmerda
- * improve error check for overview failure, fix warnings, fix header
- *
- * tif_overview.c,v
- * Revision 1.3  2000/04/18 22:48:31  warmerda
- * Added support for averaging resampling
- *
- * Revision 1.2  2000/01/28 15:36:38  warmerda
- * pass TIFF handle instead of filename to overview builder
- *
- * Revision 1.1  2000/01/28 15:04:03  warmerda
- * New
- *
  */
 
 #include <stdio.h>
@@ -98,7 +80,7 @@ void TIFFBuildOverviews( TIFF *, int, int *, int, const char *,
 /************************************************************************/
 
 uint32 TIFF_WriteOverview( TIFF *hTIFF, int nXSize, int nYSize,
-                           int nBitsPerPixel, int nSamples, 
+                           int nBitsPerPixel, int nPlanarConfig, int nSamples, 
                            int nBlockXSize, int nBlockYSize,
                            int bTiled, int nCompressFlag, int nPhotometric,
                            int nSampleFormat,
@@ -123,7 +105,7 @@ uint32 TIFF_WriteOverview( TIFF *hTIFF, int nXSize, int nYSize,
     if( nSamples == 1 )
         TIFFSetField( hTIFF, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG );
     else
-        TIFFSetField( hTIFF, TIFFTAG_PLANARCONFIG, PLANARCONFIG_SEPARATE );
+        TIFFSetField( hTIFF, TIFFTAG_PLANARCONFIG, nPlanarConfig );
 
     TIFFSetField( hTIFF, TIFFTAG_BITSPERSAMPLE, nBitsPerPixel );
     TIFFSetField( hTIFF, TIFFTAG_SAMPLESPERPIXEL, nSamples );
@@ -598,8 +580,8 @@ void TIFFBuildOverviews( TIFF *hTIFF, int nOverviews, int * panOvList,
         }
 
         nDirOffset = TIFF_WriteOverview( hTIFF, nOXSize, nOYSize,
-                                         nBitsPerPixel, nSamples,
-                                         nOBlockXSize, nOBlockYSize,
+                                         nBitsPerPixel, nPlanarConfig,
+                                         nSamples, nOBlockXSize, nOBlockYSize,
                                          bTiled, nCompressFlag, nPhotometric,
                                          nSampleFormat,
                                          panRedMap, panGreenMap, panBlueMap,
