@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gxf_ogcwkt.c,v 1.2 2001/07/18 04:51:57 warmerda Exp $
+ * $Id: gxf_ogcwkt.c,v 1.3 2004/04/07 02:08:11 warmerda Exp $
  *
  * Project:  GXF Reader
  * Purpose:  Handle GXF to OGC WKT projection transformation.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: gxf_ogcwkt.c,v $
+ * Revision 1.3  2004/04/07 02:08:11  warmerda
+ * SPHEROID belongs inside DATUM
+ *
  * Revision 1.2  2001/07/18 04:51:57  warmerda
  * added CPL_CVSID
  *
@@ -38,7 +41,7 @@
 
 #include "gxfopen.h"
 
-CPL_CVSID("$Id: gxf_ogcwkt.c,v 1.2 2001/07/18 04:51:57 warmerda Exp $");
+CPL_CVSID("$Id: gxf_ogcwkt.c,v 1.3 2004/04/07 02:08:11 warmerda Exp $");
 
 /* -------------------------------------------------------------------- */
 /* the following #defines come from ogr_spatialref.h in the GDAL/OGR	*/
@@ -266,8 +269,8 @@ static void OGCWKTSetProj( char * pszProjection, char ** papszMethods,
  * <pre>
    PROJCS["NAD83 / UTM zone 19N",
           GEOGCS["GRS 1980",
-                 DATUM["GRS_1980"],
-                 SPHEROID["GRS 1980",6378137,298.257222413684],
+                 DATUM["GRS_1980",
+                     SPHEROID["GRS 1980",6378137,298.257222413684]],
                  PRIMEM["unnamed",0],
                  UNIT["degree",0.0174532925199433]],
           PROJECTION["Transverse_Mercator"],
@@ -582,8 +585,8 @@ char *GXFGetMapProjectionAsOGCWKT( GXFHandle hGXF )
             
             sprintf( szGCS,
                      "GEOGCS[\"%s\","
-                       "DATUM[\"%s\"],"
-                       "SPHEROID[\"%s\",%s,%.15g],",
+                       "DATUM[\"%s\","
+                       "SPHEROID[\"%s\",%s,%.15g]],",
                      papszTokens[0],
                      pszOGCDatum,
                      papszTokens[0], /* this is datum, but should be ellipse*/
