@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hdf4dataset.h,v 1.14 2004/06/19 21:37:31 dron Exp $
+ * $Id: hdf4dataset.h,v 1.16 2005/04/25 20:04:43 dron Exp $
  *
  * Project:  Hierarchical Data Format Release 4 (HDF4)
  * Purpose:  Header file for HDF4 datasets reader.
@@ -28,6 +28,12 @@
  ******************************************************************************
  * 
  * $Log: hdf4dataset.h,v $
+ * Revision 1.16  2005/04/25 20:04:43  dron
+ * Make more functions constant.
+ *
+ * Revision 1.15  2005/03/29 12:42:53  dron
+ * Added AnyTypeToDouble() method.
+ *
  * Revision 1.14  2004/06/19 21:37:31  dron
  * Use HDF-EOS library for appropriate datasets; major cpde rewrite.
  *
@@ -109,9 +115,9 @@ class HDF4Dataset : public GDALDataset
 
   private:
 
-    char        **HDF4EOSTokenizeAttrs( const char *pszString );
+    char        **HDF4EOSTokenizeAttrs( const char *pszString ) const;
     char        **HDF4EOSGetObject( char **papszAttrList, char **ppszAttrName,
-                                    char **ppszAttrValue );
+                                    char **ppszAttrValue ) const;
      
   protected:
 
@@ -125,11 +131,14 @@ class HDF4Dataset : public GDALDataset
     char	**papszGlobalMetadata;
     char	**papszSubDatasets;
 
-    GDALDataType GetDataType( int32 );
-    const char  *GetDataTypeName( int32 );
-    int         GetDataTypeSize( int32 );
-    char        **TranslateHDF4Attributes( int32, int32, char *, int32, int32, char ** );
-    char        ** TranslateHDF4EOSAttributes( int32, int32, int32, char ** );
+    GDALDataType GetDataType( int32 ) const;
+    const char  *GetDataTypeName( int32 ) const;
+    int         GetDataTypeSize( int32 ) const;
+    double      AnyTypeToDouble( int32, void * ) const;
+    char        **TranslateHDF4Attributes( int32, int32, char *,
+                                           int32, int32, char ** ) const;
+    char        ** TranslateHDF4EOSAttributes( int32, int32, int32,
+                                               char ** ) const;
     CPLErr      ReadGlobalAttributes( int32 );
 
   public:
