@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hfa.h,v 1.11 2003/05/13 19:32:10 warmerda Exp $
+ * $Id: hfa.h,v 1.16 2005/08/19 02:14:11 fwarmerdam Exp $
  *
  * Project:  Erdas Imagine (.img) Translator
  * Purpose:  Public (C callable) interface for the Erdas Imagine reading
@@ -30,6 +30,21 @@
  ******************************************************************************
  *
  * $Log: hfa.h,v $
+ * Revision 1.16  2005/08/19 02:14:11  fwarmerdam
+ * bug 857: add ability to set layer names
+ *
+ * Revision 1.15  2005/05/13 04:57:28  fwarmerdam
+ * fix handling of large offsets in ige file for HFACreateLayer()
+ *
+ * Revision 1.14  2005/05/13 02:05:54  fwarmerdam
+ * adjusted CreateLayer
+ *
+ * Revision 1.13  2005/05/10 00:57:40  fwarmerdam
+ * added HFACreateLayer, HFACreateOverview
+ *
+ * Revision 1.12  2005/04/04 13:14:00  fwarmerdam
+ * Added HFAGetClassNames() (bug 819).
+ *
  * Revision 1.11  2003/05/13 19:32:10  warmerda
  * support for reading and writing opacity provided by Diana Esch-Mosher
  *
@@ -170,6 +185,7 @@ HFAHandle CPL_DLL HFACreateLL( const char *pszFilename );
 HFAHandle CPL_DLL HFACreate( const char *pszFilename, int nXSize, int nYSize, 
                              int nBands, int nDataType, char ** papszOptions );
 CPLErr  CPL_DLL HFAFlush( HFAHandle );
+int CPL_DLL HFACreateOverview( HFAHandle hHFA, int nBand, int nOverviewLevel);
 
 const Eprj_MapInfo CPL_DLL *HFAGetMapInfo( HFAHandle );
 CPLErr CPL_DLL HFASetMapInfo( HFAHandle, const Eprj_MapInfo * );
@@ -194,6 +210,11 @@ CPLErr CPL_DLL HFAGetOverviewRasterBlock( HFAHandle hHFA, int nBand,
 CPLErr CPL_DLL HFASetRasterBlock( HFAHandle hHFA, int nBand, 
                                   int nXBlock, int nYBlock,
                                   void * pData );
+CPLErr CPL_DLL HFASetOverviewRasterBlock( 
+    HFAHandle hHFA, int nBand, int iOverview,int nXBlock, int nYBlock, 
+    void * pData );
+const char * HFAGetBandName( HFAHandle hHFA, int nBand );
+void HFASetBandName( HFAHandle hHFA, int nBand, const char *pszName );
 int     CPL_DLL HFAGetDataTypeBits( int );
 CPLErr	CPL_DLL HFAGetPCT( HFAHandle, int, int *, 
                            double **, double **, double ** , double **);
@@ -203,6 +224,19 @@ void    CPL_DLL HFADumpDictionary( HFAHandle, FILE * );
 CPLErr  CPL_DLL HFAGetDataRange( HFAHandle, int, double *, double * );
 char  CPL_DLL **HFAGetMetadata( HFAHandle hHFA, int nBand );
 CPLErr  CPL_DLL HFASetMetadata( HFAHandle hHFA, int nBand, char ** );
+char  CPL_DLL **HFAGetClassNames( HFAHandle hHFA, int nBand );
+int CPL_DLL 
+HFACreateLayer( HFAHandle psInfo, HFAEntry *poParent,
+                const char *pszLayerName,
+                int bOverview, int nBlockSize, 
+                int bCreateCompressed, int bCreateLargeRaster,
+                int nXSize, int nYSize, int nDataType, 
+                char **papszOptions,
+                
+                // these are only related to external (large) files
+                GIntBig nStackValidFlagsOffset, 
+                GIntBig nStackDataOffset,
+                int nStackCount, int nStackIndex );
 
 /* -------------------------------------------------------------------- */
 /*      data types.                                                     */
