@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hfafield.cpp,v 1.15 2004/02/13 15:58:11 warmerda Exp $
+ * $Id: hfafield.cpp,v 1.18 2005/10/02 15:14:48 fwarmerdam Exp $
  *
  * Project:  Erdas Imagine (.img) Translator
  * Purpose:  Implementation of the HFAField class for managing information
@@ -29,6 +29,15 @@
  ******************************************************************************
  *
  * $Log: hfafield.cpp,v $
+ * Revision 1.18  2005/10/02 15:14:48  fwarmerdam
+ * fixed size for <8bit basedata items
+ *
+ * Revision 1.17  2005/09/28 19:38:07  fwarmerdam
+ * Added partial support for inline defined types.
+ *
+ * Revision 1.16  2005/05/10 00:56:17  fwarmerdam
+ * fixed bug with setting entries in an array (with count setting)
+ *
  * Revision 1.15  2004/02/13 15:58:11  warmerda
  * Fixed serious bug with GetInstBytes() for BASEDATA * fields with
  * a count of zero.  Such as the Excluded field of most stats nodes!
@@ -79,7 +88,7 @@
 
 #include "hfa_p.h"
 
-CPL_CVSID("$Id: hfafield.cpp,v 1.15 2004/02/13 15:58:11 warmerda Exp $");
+CPL_CVSID("$Id: hfafield.cpp,v 1.18 2005/10/02 15:14:48 fwarmerdam Exp $");
 
 #define MAX_ENTRY_REPORT   16
                            
@@ -174,6 +183,42 @@ const char *HFAField::Initialize( const char * pszInput )
         pszInput += i+1;
     }
 
+/* -------------------------------------------------------------------- */
+/*      If this is an inline object, we need to skip past the           */
+/*      definition, and then extract the object class name.             */
+/*                                                                      */
+/*      We ignore the actual definition, so if the object type isn't    */
+/*      already defined, things will not work properly.  See the        */
+/*      file lceugr250_00_pct.aux for an example of inline defs.        */
+/* -------------------------------------------------------------------- */
+    if( chItemType == 'x' && *pszInput == '{' )
+    {
+        int nBraceDepth = 1;
+        pszInput++;
+
+        // Skip past the definition.
+        while( nBraceDepth > 0 && *pszInput != '\0' )
+        {
+            if( *pszInput == '{' )
+                nBraceDepth++;
+            else if( *pszInput == '}' )
+                nBraceDepth--;
+            
+            pszInput++;
+        }
+
+        chItemType = 'o';
+
+        // find the comma terminating the type name.
+        for( i = 0; pszInput[i] != '\0' && pszInput[i] != ','; i++ ) {}
+
+        pszItemObjectType = (char *) CPLMalloc(i+1);
+        strncpy( pszItemObjectType, pszInput, i );
+        pszItemObjectType[i] = '\0';
+
+        pszInput += i+1;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      If this is an enumeration we have to extract all the            */
 /*      enumeration values.                                             */
@@ -388,15 +433,20 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
         GUInt32		nCount;
         GUInt32		nOffset;
 
+         /* set the count for fixed sized arrays */
         if( nBytes > -1 )
             nCount = nItemCount;
-        else if( chReqType == 's' )
+
+        /* Set the size from string length */
+        else if( chReqType == 's' && (chItemType == 'c' || chItemType == 'C'))
         {
             if( pValue == NULL )
                 nCount = 0;
             else
                 nCount = strlen((char *) pValue) + 1;
         }
+
+        /* set size based on index ... assumes in-order setting of array */
         else
             nCount = nIndexValue+1;
 
@@ -860,7 +910,7 @@ int HFAField::GetInstBytes( GByte * pabyData )
         nInstBytes += 12;
 
         nInstBytes += 
-            (HFAGetDataTypeBits( nBaseItemType ) / 8) * nRows * nColumns;
+            ((HFAGetDataTypeBits(nBaseItemType) + 7) / 8) * nRows * nColumns;
     }
     else if( poItemObjectType == NULL )
     {
