@@ -23,7 +23,16 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************/
+ ******************************************************************************
+ *
+ * $Log: ilwiscoordinatesystem.cpp,v $
+ * Revision 1.5  2005/09/14 13:37:18  dron
+ * Avoid warnings.
+ *
+ * Revision 1.4  2005/08/04 15:26:53  fwarmerdam
+ * added log headers
+ *
+ */
 #include "cpl_conv.h"
 #include "ilwisdataset.h"
 
@@ -361,7 +370,7 @@ static int scaleFromLATTS( string sEllips, double phits, double &scale )
 		else
 		{
 			IlwisEllips *piwEllips =  iwEllips;
-			double e2;
+			double e2 = 0.0;
 			while ( piwEllips->pszIlwisEllips )
 			{
 					if( EQUALN( sEllips.c_str(), piwEllips->pszIlwisEllips, strlen(piwEllips->pszIlwisEllips) ) )
