@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: jp2kakdataset.cpp,v 1.27 2004/10/07 20:18:27 fwarmerdam Exp $
+ * $Id: jp2kakdataset.cpp,v 1.29 2005/07/05 22:09:50 fwarmerdam Exp $
  *
  * Project:  JPEG-2000
  * Purpose:  Implementation of the ISO/IEC 15444-1 standard based on Kakadu.
@@ -28,6 +28,12 @@
  ******************************************************************************
  * 
  * $Log: jp2kakdataset.cpp,v $
+ * Revision 1.29  2005/07/05 22:09:50  fwarmerdam
+ * use GDALJP2Metadata for geoinfo, better pam support
+ *
+ * Revision 1.28  2005/05/05 15:54:48  fwarmerdam
+ * PAM Enabled
+ *
  * Revision 1.27  2004/10/07 20:18:27  fwarmerdam
  * added support for greyscale+alpha
  *
@@ -140,7 +146,8 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
+#include "gdaljp2metadata.h"
 #include "cpl_string.h"
 #include "cpl_multiproc.h"
 #include "jp2_local.h"
@@ -184,7 +191,7 @@
 #  define kdu_client void
 #endif
 
-CPL_CVSID("$Id: jp2kakdataset.cpp,v 1.27 2004/10/07 20:18:27 fwarmerdam Exp $");
+CPL_CVSID("$Id: jp2kakdataset.cpp,v 1.29 2005/07/05 22:09:50 fwarmerdam Exp $");
 
 CPL_C_START
 CPLErr CPL_DLL GTIFMemBufFromWkt( const char *pszWKT, 
@@ -221,7 +228,7 @@ static unsigned char jpc_header[] =
 /* ==================================================================== */
 /************************************************************************/
 
-class JP2KAKDataset : public GDALDataset
+class JP2KAKDataset : public GDALPamDataset
 {
     kdu_codestream oCodeStream;
     kdu_compressed_source *poInput;
@@ -234,6 +241,8 @@ class JP2KAKDataset : public GDALDataset
 
     char	   *pszProjection;
     double	   adfGeoTransform[6];
+    int            bGeoTransformValid;
+
     int		   nGCPCount;
     GDAL_GCP       *pasGCPList;
 
@@ -257,7 +266,7 @@ class JP2KAKDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class JP2KAKRasterBand : public GDALRasterBand
+class JP2KAKRasterBand : public GDALPamRasterBand
 {
     friend class JP2KAKDataset;
 
@@ -929,6 +938,7 @@ JP2KAKDataset::JP2KAKDataset()
     family = NULL;
 #endif
 
+    bGeoTransformValid = FALSE;
     adfGeoTransform[0] = 0.0;
     adfGeoTransform[1] = 1.0;
     adfGeoTransform[2] = 0.0;
@@ -944,6 +954,8 @@ JP2KAKDataset::JP2KAKDataset()
 JP2KAKDataset::~JP2KAKDataset()
 
 {
+    FlushCache();
+
     CPLFree( pszProjection );
     
     if( poInput != NULL )
@@ -978,7 +990,10 @@ JP2KAKDataset::~JP2KAKDataset()
 const char *JP2KAKDataset::GetProjectionRef()
 
 {
-    return( pszProjection );
+    if( pszProjection && *pszProjection )
+        return( pszProjection );
+    else
+        return GDALPamDataset::GetProjectionRef();
 }
 
 /************************************************************************/
@@ -988,9 +1003,14 @@ const char *JP2KAKDataset::GetProjectionRef()
 CPLErr JP2KAKDataset::GetGeoTransform( double * padfTransform )
 
 {
-    memcpy( padfTransform, adfGeoTransform, sizeof(double)*6 );
-
-    return CE_None;
+    if( bGeoTransformValid )
+    {
+        memcpy( padfTransform, adfGeoTransform, sizeof(double)*6 );
+    
+        return CE_None;
+    }
+    else
+        return GDALPamDataset::GetGeoTransform( padfTransform );
 }
 
 /************************************************************************/
@@ -1332,61 +1352,50 @@ GDALDataset *JP2KAKDataset::Open( GDALOpenInfo * poOpenInfo )
         }
 
 /* -------------------------------------------------------------------- */
-/*      Try to read any available georeferencing info in a "geotiff"    */
-/*      box".                                                           */
+/*      Look for supporting coordinate system information.              */
 /* -------------------------------------------------------------------- */
-        unsigned char  *pabyGTData = NULL;
-        int		nGTDataSize = 0;
-        jp2_input_box   oBox;
-
-        if( !bIsJPIP )
+        if( poOpenInfo->fp != NULL )
         {
-#ifdef KAKADU4
-            for( family == NULL || oBox.open( family ); 
-                 oBox.exists(); oBox.open_next() )
-#else
-            VSIFSeek( poOpenInfo->fp, 0, SEEK_SET );
-            while( oBox.open(poOpenInfo->fp).exists() 
-                   && oBox.get_remaining_bytes() != -1 )
-#endif
+            GDALJP2Metadata oJP2Geo;
+            FILE *fpLL;
+        
+            fpLL = VSIFOpenL( poOpenInfo->pszFilename, "rb" );
+        
+            if( fpLL != NULL )
             {
-                if( oBox.get_box_type() == 0x75756964 /* UUID */ )
-                {
-                    unsigned char uuid2[16];
-                    oBox.read( uuid2, 16 );
-                    if( memcmp( uuid2, msi_uuid2, 16 ) == 0 )
-                    {
-                        nGTDataSize = (int) oBox.get_remaining_bytes();
-                        
-                        pabyGTData = (unsigned char *) CPLMalloc(nGTDataSize);
-                        oBox.read( pabyGTData, nGTDataSize );
-                    }
-                }
-                oBox.close();
-            }
+                oJP2Geo.ReadBoxes( fpLL );
+                VSIFCloseL( fpLL );
             
-            oBox.close();
+                //poDS->papszGMLMetadata = CSLDuplicate(oJP2Geo.papszGMLMetadata);
+            }
+        
+            if( oJP2Geo.ParseJP2GeoTIFF() 
+                || oJP2Geo.ParseGMLCoverageDesc() 
+                || oJP2Geo.ParseMSIG() )
+            {
+                poDS->pszProjection = CPLStrdup(oJP2Geo.pszProjection);
+                poDS->bGeoTransformValid = TRUE;
+                memcpy( poDS->adfGeoTransform, oJP2Geo.adfGeoTransform, 
+                        sizeof(double) * 6 );
+                poDS->nGCPCount = oJP2Geo.nGCPCount;
+                poDS->pasGCPList = oJP2Geo.pasGCPList;
+                oJP2Geo.pasGCPList = NULL;
+            }
         }
 
 /* -------------------------------------------------------------------- */
-/*      Try to turn the geotiff block into projection and geotransform. */
+/*      Check for world file.                                           */
 /* -------------------------------------------------------------------- */
-        if( pabyGTData != NULL )
-        {
-            GTIFWktFromMemBuf( nGTDataSize, pabyGTData, 
-                               &(poDS->pszProjection), poDS->adfGeoTransform,
-                               &(poDS->nGCPCount), &(poDS->pasGCPList) );
-            CPLDebug("GDAL", "Got projection: %s", poDS->pszProjection );
-            CPLFree( pabyGTData );
-            pabyGTData = NULL;
-        }
-        else
+        poDS->bGeoTransformValid |= 
             GDALReadWorldFile( poOpenInfo->pszFilename, ".wld", 
                                poDS->adfGeoTransform );
 
-        if( poDS->pszProjection == NULL )
-            poDS->pszProjection = CPLStrdup("");
-        
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+        poDS->SetDescription( poOpenInfo->pszFilename );
+        poDS->TryLoadXML();
+
         return( poDS );
     }
 
@@ -2181,7 +2190,16 @@ JP2KAKCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     if( !pfnProgress( 1.0, NULL, pProgressData ) )
         return NULL;
 
-    return (GDALDataset *) GDALOpen( pszFilename, GA_ReadOnly );;
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxilary pam information.         */
+/* -------------------------------------------------------------------- */
+    GDALPamDataset *poDS = (GDALPamDataset *) 
+        GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
 }
 
 /************************************************************************/
