@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: jpeg2000dataset.cpp,v 1.19 2004/01/20 16:23:44 dron Exp $
+ * $Id: jpeg2000dataset.cpp,v 1.20 2005/05/05 15:54:48 fwarmerdam Exp $
  *
  * Project:  JPEG-2000
  * Purpose:  Partial implementation of the ISO/IEC 15444-1 standard
@@ -28,6 +28,9 @@
  ******************************************************************************
  * 
  * $Log: jpeg2000dataset.cpp,v $
+ * Revision 1.20  2005/05/05 15:54:48  fwarmerdam
+ * PAM Enabled
+ *
  * Revision 1.19  2004/01/20 16:23:44  dron
  * Don't close streams before deleting the JPEG2000Dataset object.
  *
@@ -87,12 +90,12 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "cpl_string.h"
 
 #include <jasper/jasper.h>
 
-CPL_CVSID("$Id: jpeg2000dataset.cpp,v 1.19 2004/01/20 16:23:44 dron Exp $");
+CPL_CVSID("$Id: jpeg2000dataset.cpp,v 1.20 2005/05/05 15:54:48 fwarmerdam Exp $");
 
 CPL_C_START
 void    GDALRegister_JPEG2000(void);
@@ -238,7 +241,7 @@ static unsigned char msi_uuid2[16] =
 /* ==================================================================== */
 /************************************************************************/
 
-class JPEG2000Dataset : public GDALDataset
+class JPEG2000Dataset : public GDALPamDataset
 {
     friend class JPEG2000RasterBand;
 
@@ -272,7 +275,7 @@ class JPEG2000Dataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class JPEG2000RasterBand : public GDALRasterBand
+class JPEG2000RasterBand : public GDALPamRasterBand
 {
     friend class JPEG2000Dataset;
 
@@ -497,6 +500,8 @@ JPEG2000Dataset::JPEG2000Dataset()
 JPEG2000Dataset::~JPEG2000Dataset()
 
 {
+    FlushCache();
+
     if ( psStream )
         jas_stream_close( psStream );
     if ( psImage )
@@ -791,6 +796,12 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
                                poDS->adfGeoTransform );
     }
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
@@ -1134,7 +1145,16 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         GDALWriteWorldFile( pszFilename, "wld", adfGeoTransform );
     }
 
-    return (GDALDataset *) GDALOpen( pszFilename, GA_Update );
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxilary pam information.         */
+/* -------------------------------------------------------------------- */
+    GDALPamDataset *poDS = (GDALPamDataset *) 
+        GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
 }
 
 /************************************************************************/
