@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: l1bdataset.cpp,v 1.24 2005/02/07 20:19:11 fwarmerdam Exp $
+ * $Id: l1bdataset.cpp,v 1.26 2005/07/31 01:00:08 fwarmerdam Exp $
  *
  * Project:  NOAA Polar Orbiter Level 1b Dataset Reader (AVHRR)
  * Purpose:  Can read NOAA-9(F)-NOAA-17(M) AVHRR datasets
@@ -30,6 +30,12 @@
  ******************************************************************************
  * 
  * $Log: l1bdataset.cpp,v $
+ * Revision 1.26  2005/07/31 01:00:08  fwarmerdam
+ * Moved CPL_LSB variables into #ifdef.
+ *
+ * Revision 1.25  2005/05/05 15:54:48  fwarmerdam
+ * PAM Enabled
+ *
  * Revision 1.24  2005/02/07 20:19:11  fwarmerdam
  * Parametrized sampling rates.
  *
@@ -107,10 +113,10 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: l1bdataset.cpp,v 1.24 2005/02/07 20:19:11 fwarmerdam Exp $");
+CPL_CVSID("$Id: l1bdataset.cpp,v 1.26 2005/07/31 01:00:08 fwarmerdam Exp $");
 
 CPL_C_START
 void    GDALRegister_L1B(void);
@@ -225,7 +231,7 @@ class TimeCode {
 /* ==================================================================== */
 /************************************************************************/
 
-class L1BDataset : public GDALDataset
+class L1BDataset : public GDALPamDataset
 {
     friend class L1BRasterBand;
 
@@ -283,7 +289,7 @@ class L1BDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class L1BRasterBand : public GDALRasterBand
+class L1BRasterBand : public GDALPamRasterBand
 {
     friend class L1BDataset;
 
@@ -429,6 +435,8 @@ L1BDataset::L1BDataset()
 L1BDataset::~L1BDataset()
 
 {
+    FlushCache();
+
     if( nGCPCount > 0 )
     {
         for( int i = 0; i < nGCPCount; i++ )
@@ -504,10 +512,10 @@ void L1BDataset::FetchNOAA9TimeCode( TimeCode *psTime, GByte *piRecordHeader,
 void L1BDataset::FetchNOAA15TimeCode( TimeCode *psTime,
                                       GUInt16 *piRecordHeader, int *iLocInd )
 {
+#ifdef CPL_LSB
     GUInt16 iTemp;
     GUInt32 lTemp;
 
-#ifdef CPL_LSB
     iTemp = piRecordHeader[1];
     psTime->SetYear(CPL_SWAP16(iTemp));
     iTemp = piRecordHeader[2];
@@ -1350,6 +1358,12 @@ GDALDataset *L1BDataset::Open( GDALOpenInfo * poOpenInfo )
         break;
     }
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 bad:
     delete poDS;
