@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: memdataset.h,v 1.10 2004/04/15 18:54:10 warmerda Exp $
+ * $Id: memdataset.h,v 1.12 2005/05/19 20:39:54 dron Exp $
  *
  * Project:  Memory Array Translator
  * Purpose:  Declaration of MEMDataset, and MEMRasterBand.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: memdataset.h,v $
+ * Revision 1.12  2005/05/19 20:39:54  dron
+ * Derive the MEMRasterBand from GDALPamRasterBand.
+ *
+ * Revision 1.11  2005/04/19 19:56:11  fwarmerdam
+ * Explicitly mark destructor as virtual.
+ *
  * Revision 1.10  2004/04/15 18:54:10  warmerda
  * added UnitType, Offset, Scale and CategoryNames support
  *
@@ -63,6 +69,7 @@
 #ifndef MEMDATASET_H_INCLUDED
 #define MEMDATASET_H_INCLUDED
 
+#include "gdal_pam.h"
 #include "gdal_priv.h"
 
 CPL_C_START
@@ -86,7 +93,7 @@ class CPL_DLL MEMDataset : public GDALDataset
 
   public:
                  MEMDataset();
-                 ~MEMDataset();
+    virtual      ~MEMDataset();
 
     virtual const char *GetProjectionRef(void);
     virtual CPLErr SetProjection( const char * );
@@ -107,7 +114,7 @@ class CPL_DLL MEMDataset : public GDALDataset
 /*                            MEMRasterBand                             */
 /************************************************************************/
 
-class CPL_DLL MEMRasterBand : public GDALRasterBand
+class CPL_DLL MEMRasterBand : public GDALPamRasterBand
 {
   protected:
 
