@@ -1,8 +1,8 @@
 /******************************************************************************
- * $Id: mrsiddataset.cpp,v 1.23 2005/02/17 22:17:36 fwarmerdam Exp $
+ * $Id: mrsiddataset.cpp,v 1.44 2005/09/16 22:09:18 fwarmerdam Exp $
  *
  * Project:  Multi-resolution Seamless Image Database (MrSID)
- * Purpose:  Read/write LizardTech's MrSID file format - Version 4 SDK.
+ * Purpose:  Read/write LizardTech's MrSID file format - Version 4+ SDK.
  * Author:   Andrey Kiselev, dron@remotesensing.org
  *
  ******************************************************************************
@@ -28,6 +28,74 @@
  ******************************************************************************
  *
  * $Log: mrsiddataset.cpp,v $
+ * Revision 1.44  2005/09/16 22:09:18  fwarmerdam
+ * Ensure SetDescription() is called before OpenZoom.
+ *
+ * Revision 1.43  2005/09/15 02:17:34  fwarmerdam
+ * note MRSID_HAVE_GETWKT macro.
+ *
+ * Revision 1.42  2005/09/15 00:46:21  fwarmerdam
+ * Fixed help topic link.
+ *
+ * Revision 1.41  2005/09/15 00:31:27  fwarmerdam
+ * added JP2MrSID driver, fixedup ESDK 5.x support
+ *
+ * Revision 1.40  2005/09/09 02:27:28  fwarmerdam
+ * Fallback to PAM if projection string is empty.
+ *
+ * Revision 1.39  2005/08/26 14:48:33  fwarmerdam
+ * Do not return nodata on bands since MrSID NODATA semantics are
+ * across all bands.   Note that IMAGE__TRANSPARENT_DATA_VALUE metadata
+ * on the dataset holds the set of values for application use if needed.
+ *
+ * Revision 1.38  2005/07/22 15:50:10  fwarmerdam
+ * Use blocked io for very small request buffers as well as small windows.
+ *
+ * Revision 1.37  2005/07/21 17:30:13  fwarmerdam
+ * Added preliminary worldfile support.
+ *
+ * Revision 1.36  2005/07/14 23:40:31  fwarmerdam
+ * getNumLevels() is the number of overviews
+ *
+ * Revision 1.35  2005/06/24 15:54:30  dron
+ * Properly fetch NODATA taking in account data type of the value.
+ *
+ * Revision 1.34  2005/05/13 01:12:25  fwarmerdam
+ * Fixed bug with block oriented access to 16bit data.
+ *
+ * Revision 1.33  2005/05/05 15:54:49  fwarmerdam
+ * PAM Enabled
+ *
+ * Revision 1.32  2005/05/04 17:21:00  fwarmerdam
+ * Avoid 32bit overflow when testing xsize*ysize.
+ *
+ * Revision 1.31  2005/04/21 16:43:26  fwarmerdam
+ * Fixed bug with RasterIO() at overview levels higher than is available
+ * in the file.  Now the code limits itself to available overview levels.
+ *
+ * Revision 1.30  2005/04/11 17:03:07  fwarmerdam
+ * Improved initial open checking to ensure it works properly with no header
+ * data at all.
+ *
+ * Revision 1.29  2005/04/11 14:41:14  fwarmerdam
+ * Only call getWKT() if HAVE_MRSID_GETWKT is defined.
+ *
+ * Revision 1.28  2005/04/06 22:03:17  kmelero
+ * Updated for CPLStrdup and mod to pointers for large file support per
+ * LizardTech Mrsid Team.  (kmelero@sanz.com)
+ *
+ * Revision 1.27  2005/04/02 19:26:08  kmelero
+ * Added read support for WKT.  (kmelero@sanz.com)
+ *
+ * Revision 1.26  2005/04/02 19:04:00  kmelero
+ * Added setting WKT in MrSID file.  (kmelero@sanz.com)
+ *
+ * Revision 1.25  2005/03/26 19:29:21  kmelero
+ * Added flag to set 64-bit fwrite in support of large MrSID files.  (kmelero@sanz.com)
+ *
+ * Revision 1.24  2005/03/25 15:02:22  kmelero
+ * Removed misplaced < character.  Was causing compiler error. (kmelero@sanz.com)
+ *
  * Revision 1.23  2005/02/17 22:17:36  fwarmerdam
  * some adjustments to deciding whether to use direct io or not
  *
@@ -45,75 +113,27 @@
  *
  * Revision 1.18  2004/11/14 21:18:22  dron
  * Initial support for MrSID Encoding SDK.
- *
- * Revision 1.17  2004/09/09 19:20:18  dron
- * Use VSIFClose instead of VSIFCloseL in MrSIDDataset::Open().
- *
- * Revision 1.16  2004/09/09 18:13:27  dron
- * Fixed bug when getting NoData value in MrSIDRasterBand().
- *
- * Revision 1.15  2004/08/29 10:11:43  dron
- * Implement caching for the case of sequental block reading.
- *
- * Revision 1.14  2004/08/20 19:03:58  dron
- * Fixed problem with wrong sign in georeference calculation.
- *
- * Revision 1.13  2004/08/07 10:25:12  dron
- * Added support for overviews in new version; fixed problem with the last
- * strip decoding; support for DSDK 4.0.9.713.
- *
- * Revision 1.12  2004/07/04 06:26:30  dron
- * Added support for decoding JPEG2000 format using MrSID DSDK.
- *
- * Revision 1.11  2004/05/02 15:47:26  dron
- * Updated to DSDK 4.0.7.646.
- *
- * Revision 1.10  2004/04/27 08:45:59  dron
- * Read strip height from image and use it as BlockYSize.
- *
- * Revision 1.9  2004/04/23 09:49:38  dron
- * Added support for DSDK v.4.0.x; significant changes in the driver structure.
- *
- * Revision 1.8  2004/03/19 12:19:48  dron
- * Few improvements; several memory leaks fixed.
- *
- * Revision 1.7  2004/02/08 14:29:43  dron
- * Support for DSDK v.3.2.x.
- *
- * Revision 1.6  2004/02/03 20:56:45  dron
- * Fixes in coordinate handling.
- *
- * Revision 1.5  2003/07/24 09:33:07  dron
- * Added MrSIDRasterBand::IRasterIO().
- *
- * Revision 1.4  2003/05/27 21:08:38  warmerda
- * avoid int/short casting warnings
- *
- * Revision 1.3  2003/05/25 15:24:49  dron
- * Added georeferencing.
- *
- * Revision 1.2  2003/05/07 10:26:02  dron
- * Added multidimensional metadata handling.
- *
- * Revision 1.1  2003/04/23 12:21:56  dron
- * New.
- *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
+#include <string>
 
 #include <geo_normalize.h>
 #include <geovalues.h>
 
-CPL_CVSID("$Id: mrsiddataset.cpp,v 1.23 2005/02/17 22:17:36 fwarmerdam Exp $");
+CPL_CVSID("$Id: mrsiddataset.cpp,v 1.44 2005/09/16 22:09:18 fwarmerdam Exp $");
 
 CPL_C_START
-void    GDALRegister_MrSID(void);
 double GTIFAngleToDD( double dfAngle, int nUOMAngle );
 CPL_C_END
 
+// Key Macros from Makefile:
+//   MRSID_ESDK: Means we have the encoding SDK (version 5 or newer required)
+//   MRSID_J2K: Means we are enabling MrSID SDK JPEG2000 support. 
+//   MRSID_HAVE_GETWKT: 
+
 #include "lt_types.h"
 #include "lt_base.h"
 #include "lt_fileSpec.h"
@@ -127,14 +147,21 @@ CPL_C_END
 #include "lti_metadataRecord.h"
 #include "lti_utils.h"
 #include "MrSIDImageReader.h"
-//#include "J2KImageReader.h"
 
-#ifdef MRSID_ESDK_VERSION_40
+#ifdef MRSID_J2K
+#  include "J2KImageReader.h"
+#endif
+
+#ifdef MRSID_ESDK
 # include "MG3ImageWriter.h"
 # include "MG3WriterParams.h"
 # include "MG2ImageWriter.h"
 # include "MG2WriterParams.h"
-#endif /* MRSID_ESDK_VERSION_40 */
+# ifdef MRSID_J2K
+#   include "J2KImageWriter.h"
+#   include "J2KWriterParams.h"
+# endif
+#endif /* MRSID_ESDK */
 
 LT_USE_NAMESPACE(LizardTech)
 
@@ -144,14 +171,16 @@ LT_USE_NAMESPACE(LizardTech)
 /* ==================================================================== */
 /************************************************************************/
 
-class MrSIDDataset : public GDALDataset
+class MrSIDDataset : public GDALPamDataset
 {
     friend class MrSIDRasterBand;
 
     LTIImageReader      *poImageReader;
-#ifdef MRSID_ESDK_VERSION_40
-    LTIMeteredImageWriter *poImageWriter;
-#endif /* MRSID_ESDK_VERSION_40 */
+
+#ifdef MRSID_ESDK
+    LTIGeoFileImageWriter *poImageWriter;
+#endif
+
     LTINavigator        *poLTINav;
     LTIMetadataDatabase *poMetadata;
     const LTIPixel      *poNDPixel;
@@ -195,14 +224,12 @@ class MrSIDDataset : public GDALDataset
     virtual CPLErr      GetGeoTransform( double * padfTransform );
     const char          *GetProjectionRef();
 
-#ifdef MRSID_ESDK_VERSION_40
+#ifdef MRSID_ESDK
     static GDALDataset  *Create( const char * pszFilename,
                                  int nXSize, int nYSize, int nBands,
                                  GDALDataType eType, char ** papszParmList );
     virtual void        FlushCache( void );
-    /*virtual CPLErr      SetGeoTransform( double * );
-    virtual CPLErr      SetProjectionRef( const char * );*/
-#endif /* MRSID_ESDK_VERSION_40 */
+#endif
 };
 
 #include "mrsidcommon.h"
@@ -213,7 +240,7 @@ class MrSIDDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class MrSIDRasterBand : public GDALRasterBand
+class MrSIDRasterBand : public GDALPamRasterBand
 {
     friend class MrSIDDataset;
 
@@ -239,9 +266,9 @@ class MrSIDRasterBand : public GDALRasterBand
     virtual int             GetOverviewCount();
     virtual GDALRasterBand  *GetOverview( int );
 
-#ifdef MRSID_ESDK_VERSION_40
+#ifdef MRSID_ESDK
     virtual CPLErr          IWriteBlock( int, int, void * );
-#endif /* MRSID_ESDK_VERSION_40 */
+#endif
 };
 
 /************************************************************************/
@@ -275,13 +302,55 @@ MrSIDRasterBand::MrSIDRasterBand( MrSIDDataset *poDS, int nBand )
 
 /* -------------------------------------------------------------------- */
 /*      Set NoData values.                                              */
+/*                                                                      */
+/*      This logic is disabled for now since the MrSID nodata           */
+/*      semantics are different than GDAL.  In MrSID all bands must     */
+/*      match the nodata value for that band in order for the pixel     */
+/*      to be considered nodata, otherwise all values are valid.        */
 /* -------------------------------------------------------------------- */
+#ifdef notdef
      if ( poDS->poNDPixel )
      {
-	 dfNoDataValue = poDS->poNDPixel->getSampleValueFloat32( nBand - 1 );
+         switch( poDS->eSampleType )
+         {
+             case LTI_DATATYPE_UINT8:
+             case LTI_DATATYPE_SINT8:
+                 dfNoDataValue = (double)
+                     poDS->poNDPixel->getSampleValueUint8( nBand - 1 );
+                 break;
+             case LTI_DATATYPE_UINT16:
+                 dfNoDataValue = (double)
+                     poDS->poNDPixel->getSampleValueUint16( nBand - 1 );
+                 break;
+             case LTI_DATATYPE_FLOAT32:
+                 dfNoDataValue =
+                     poDS->poNDPixel->getSampleValueFloat32( nBand - 1 );
+                 break;
+             case LTI_DATATYPE_SINT16:
+                 dfNoDataValue = (double)
+                     *(GInt16 *)poDS->poNDPixel->getSampleValueAddr( nBand - 1 );
+                 break;
+             case LTI_DATATYPE_UINT32:
+                 dfNoDataValue = (double)
+                     *(GUInt32 *)poDS->poNDPixel->getSampleValueAddr( nBand - 1 );
+                 break;
+             case LTI_DATATYPE_SINT32:
+                 dfNoDataValue = (double)
+                     *(GInt32 *)poDS->poNDPixel->getSampleValueAddr( nBand - 1 );
+                 break;
+             case LTI_DATATYPE_FLOAT64:
+                 dfNoDataValue =
+                     *(double *)poDS->poNDPixel->getSampleValueAddr( nBand - 1 );
+                 break;
+
+             case LTI_DATATYPE_INVALID:
+                 CPLAssert( FALSE );
+                 break;
+         }
 	 bNoDataSet = TRUE;
      }
      else
+#endif
      {
 	dfNoDataValue = 0.0;
         bNoDataSet = FALSE;
@@ -307,15 +376,15 @@ CPLErr MrSIDRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 {
     MrSIDDataset    *poGDS = (MrSIDDataset *)poDS;
 
-#ifdef MRSID_ESDK_VERSION_40
+#ifdef MRSID_ESDK
     if( poGDS->eAccess == GA_Update )
     {
         CPLDebug( "MrSID", 
-                  "IReadBlock() - DSDK 4.0 - read on updatable file fails." );
+                  "IReadBlock() - DSDK - read on updatable file fails." );
         memset( pImage, 0, nBlockSize * GDALGetDataTypeSize(eDataType) / 8 );
         return CE_None;
     }
-#endif /* MRSID_ESDK_VERSION_40 */
+#endif /* MRSID_ESDK */
 
     CPLDebug( "MrSID", "IReadBlock(%d,%d)", nBlockXOff, nBlockYOff );
 
@@ -366,12 +435,13 @@ CPLErr MrSIDRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
         poGDS->nPrevBlockYOff = nBlockYOff;
     }
 
-    memcpy( pImage, poGDS->poBuffer->getTotalBandData(nBand - 1), nBlockSize );
+    memcpy( pImage, poGDS->poBuffer->getTotalBandData(nBand - 1), 
+            nBlockSize * (GDALGetDataTypeSize(poGDS->eDataType) / 8) );
 
     return CE_None;
 }
 
-#ifdef MRSID_ESDK_VERSION_40
+#ifdef MRSID_ESDK
 
 /************************************************************************/
 /*                            IWriteBlock()                             */
@@ -422,7 +492,7 @@ CPLErr MrSIDRasterBand::IWriteBlock( int nBlockXOff, int nBlockYOff,
     return CE_None;
 }
 
-#endif /* MRSID_ESDK_VERSION_40 */
+#endif /* MRSID_ESDK */
 
 /************************************************************************/
 /*                             IRasterIO()                              */
@@ -555,9 +625,9 @@ GDALRasterBand *MrSIDRasterBand::GetOverview( int i )
 MrSIDDataset::MrSIDDataset()
 {
     poImageReader = NULL;
-#ifdef MRSID_ESDK_VERSION_40
+#ifdef MRSID_ESDK
     poImageWriter = NULL;
-#endif /* MRSID_ESDK_VERSION_40 */
+#endif
     poLTINav = NULL;
     poMetadata = NULL;
     poNDPixel = NULL;
@@ -592,12 +662,12 @@ MrSIDDataset::MrSIDDataset()
 
 MrSIDDataset::~MrSIDDataset()
 {
-#ifdef MRSID_ESDK_VERSION_40
     FlushCache();
 
+#ifdef MRSID_ESDK
     if ( poImageWriter )
         delete poImageWriter;
-#endif /* MRSID_ESDK_VERSION_40 */
+#endif
 
     if ( poImageReader && !bIsOverview )
         delete poImageReader;
@@ -636,7 +706,10 @@ CPLErr MrSIDDataset::IRasterIO( GDALRWFlag eRWFlag,
 /* -------------------------------------------------------------------- */
     int bUseBlockedIO = bForceCachedIO;
 
-    if( nYSize == 1 || nXSize * nYSize < 100 )
+    if( nYSize == 1 || nXSize * ((double) nYSize) < 100.0 )
+        bUseBlockedIO = TRUE;
+
+    if( nBufYSize == 1 || nBufXSize * ((double) nBufYSize) < 100.0 )
         bUseBlockedIO = TRUE;
 
     if( CSLTestBoolean( CPLGetConfigOption( "GDAL_ONE_BIG_READ", "NO") ) )
@@ -668,10 +741,13 @@ CPLErr MrSIDDataset::IRasterIO( GDALRWFlag eRWFlag,
 /*      request just very, very slightly larger than a zoom level do    */
 /*      not force us to the next level.                                 */
 /* -------------------------------------------------------------------- */
+    int iOverview = 0;
     double dfZoomMag = MIN((nXSize / (double)nBufXSize), 
                            (nYSize / (double)nBufYSize));
 
-    for( nZoomMag = 1; nZoomMag * 2 < (dfZoomMag + 0.1); nZoomMag *= 2 ) {}
+    for( nZoomMag = 1; 
+         nZoomMag * 2 < (dfZoomMag + 0.1) && iOverview < nOverviewCount; 
+         nZoomMag *= 2, iOverview++ ) {}
 
 /* -------------------------------------------------------------------- */
 /*      Work out the size of the temporary buffer and allocate it.      */
@@ -780,12 +856,13 @@ CPLErr MrSIDDataset::IRasterIO( GDALRWFlag eRWFlag,
 
 CPLErr MrSIDDataset::GetGeoTransform( double * padfTransform )
 {
-    memcpy( padfTransform, adfGeoTransform, sizeof(adfGeoTransform[0]) * 6 );
-
-    if ( !bHasGeoTransform )
-        return CE_Failure;
-
-    return CE_None;
+    if( bHasGeoTransform )
+    {
+        memcpy( padfTransform, adfGeoTransform, sizeof(adfGeoTransform[0]) * 6 );
+        return CE_None;
+    }
+    else
+        return GDALPamDataset::GetGeoTransform( padfTransform );
 }
 
 /************************************************************************/
@@ -794,10 +871,10 @@ CPLErr MrSIDDataset::GetGeoTransform( double * padfTransform )
 
 const char *MrSIDDataset::GetProjectionRef()
 {
-    if( pszProjection )
+    if( pszProjection && strlen(pszProjection) > 0 )
         return pszProjection;
     else
-        return "";
+        return GDALPamDataset::GetProjectionRef();
 }
 
 /************************************************************************/
@@ -965,29 +1042,29 @@ CPLErr MrSIDDataset::OpenZoomLevel( lt_int32 iZoom )
     eSampleType = poImageReader->getDataType();
     switch ( eSampleType )
     {
-        case LTI_DATATYPE_UINT16:
-            eDataType = GDT_UInt16;
-            break;
-        case LTI_DATATYPE_SINT16:
-            eDataType = GDT_Int16;
-            break;
-        case LTI_DATATYPE_UINT32:
-            eDataType = GDT_UInt32;
-            break;
-        case LTI_DATATYPE_SINT32:
-            eDataType = GDT_Int32;
-            break;
-        case LTI_DATATYPE_FLOAT32:
-            eDataType = GDT_Float32;
-            break;
-        case LTI_DATATYPE_FLOAT64:
-            eDataType = GDT_Float64;
-            break;
-        case LTI_DATATYPE_UINT8:
-        case LTI_DATATYPE_SINT8:
-        default:
-            eDataType = GDT_Byte;
-            break;
+      case LTI_DATATYPE_UINT16:
+        eDataType = GDT_UInt16;
+        break;
+      case LTI_DATATYPE_SINT16:
+        eDataType = GDT_Int16;
+        break;
+      case LTI_DATATYPE_UINT32:
+        eDataType = GDT_UInt32;
+        break;
+      case LTI_DATATYPE_SINT32:
+        eDataType = GDT_Int32;
+        break;
+      case LTI_DATATYPE_FLOAT32:
+        eDataType = GDT_Float32;
+        break;
+      case LTI_DATATYPE_FLOAT64:
+        eDataType = GDT_Float64;
+        break;
+      case LTI_DATATYPE_UINT8:
+      case LTI_DATATYPE_SINT8:
+      default:
+        eDataType = GDT_Byte;
+        break;
     }
 
 /* -------------------------------------------------------------------- */
@@ -1004,11 +1081,34 @@ CPLErr MrSIDDataset::OpenZoomLevel( lt_int32 iZoom )
         adfGeoTransform[3] = adfGeoTransform[3] - adfGeoTransform[5] / 2;
 	bHasGeoTransform = TRUE;
     }
+    else if( iZoom == 0 )
+    {
+        bHasGeoTransform = 
+            GDALReadWorldFile( GetDescription(), ".sdw",  
+                               adfGeoTransform )
+            || GDALReadWorldFile( GetDescription(), ".sidw", 
+                                  adfGeoTransform )
+            || GDALReadWorldFile( GetDescription(), ".wld", 
+                                  adfGeoTransform );
+    }
+    
+/* -------------------------------------------------------------------- */
+/*      Read wkt.                                                       */
+/* -------------------------------------------------------------------- */
+#ifdef MRSID_HAVE_GETWKT
+    if( !poImageReader->isGeoCoordImplicit() )
+    {
+	const LTIGeoCoord& oGeo = poImageReader->getGeoCoord();
+	
+	if( oGeo.getWKT() )
+            pszProjection =  CPLStrdup( oGeo.getWKT() );
+    }
+#endif // HAVE_MRSID_GETWKT
 
 /* -------------------------------------------------------------------- */
 /*      Read NoData value.                                              */
 /* -------------------------------------------------------------------- */
-     poNDPixel = poImageReader->getNoDataPixel();
+    poNDPixel = poImageReader->getNoDataPixel();
 
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
@@ -1021,6 +1121,43 @@ CPLErr MrSIDDataset::OpenZoomLevel( lt_int32 iZoom )
     return CE_None;
 }
 
+/************************************************************************/
+/*                             MrSIDOpen()                              */
+/*                                                                      */
+/*      This is just a jacket to verify that the file is MrSID.         */
+/************************************************************************/
+
+static GDALDataset *MrSIDOpen( GDALOpenInfo * poOpenInfo )
+{
+    if( poOpenInfo->nHeaderBytes < 32 )
+        return NULL;
+
+    if ( !EQUALN((const char *) poOpenInfo->pabyHeader, "msid", 4) )
+        return NULL;
+
+    return MrSIDDataset::Open( poOpenInfo );
+}
+
+/************************************************************************/
+/*                              JP2Open()                               */
+/*                                                                      */
+/*      This is just a jacket to verify that the file is JPEG2000.      */
+/************************************************************************/
+
+#ifdef MRSID_J2K
+static GDALDataset *JP2Open( GDALOpenInfo *poOpenInfo )
+
+{
+    if( poOpenInfo->nHeaderBytes < 32 )
+        return NULL;
+
+    if( !EQUALN((const char *) poOpenInfo->pabyHeader + 4, "jP  ", 4) )
+        return NULL;
+
+    return MrSIDDataset::Open( poOpenInfo );
+}
+#endif /* def MRSID_J2K */
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -1029,16 +1166,22 @@ GDALDataset *MrSIDDataset::Open( GDALOpenInfo * poOpenInfo )
 {
     int     bIsJP2 = FALSE;
 
-    if ( poOpenInfo->fp == NULL )
+/* -------------------------------------------------------------------- */
+/*      Is this a mrsid or jpeg 2000 file?                              */
+/* -------------------------------------------------------------------- */
+    if( poOpenInfo->nHeaderBytes < 32 )
         return NULL;
 
-    if ( EQUALN((const char *) poOpenInfo->pabyHeader + 4, "jP  ", 4) )
+    if( EQUALN((const char *) poOpenInfo->pabyHeader + 4, "jP  ", 4) )
         bIsJP2 = TRUE;
     else if ( !EQUALN((const char *) poOpenInfo->pabyHeader, "msid", 4) )
         return NULL;
 
-    VSIFClose( poOpenInfo->fp );
-    poOpenInfo->fp = NULL;
+    if(poOpenInfo->fp)
+    {
+        VSIFClose( poOpenInfo->fp );
+        poOpenInfo->fp = NULL;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
@@ -1047,10 +1190,13 @@ GDALDataset *MrSIDDataset::Open( GDALOpenInfo * poOpenInfo )
     const LTFileSpec    oFileSpec( poOpenInfo->pszFilename );
 
     poDS = new MrSIDDataset();
-    //    if ( bIsJP2 )
-    //    poDS->poImageReader = new J2KImageReader( oFileSpec, true );
-    //else
+#ifdef MRSID_J2K
+    if ( bIsJP2 )
+        poDS->poImageReader = new J2KImageReader( oFileSpec, true );
+    else
+#endif
         poDS->poImageReader = new MrSIDImageReader( oFileSpec );
+
     if ( !LT_SUCCESS( poDS->poImageReader->initialize() ) )
     {
         delete poDS;
@@ -1098,12 +1244,14 @@ GDALDataset *MrSIDDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Get number of resolution levels (we will use them as overviews).*/
 /* -------------------------------------------------------------------- */
-//    if ( bIsJP2 )
-//        poDS->nOverviewCount
-//	  = ((J2KImageReader *) (poDS->poImageReader))->getNumLevels() - 1;
-//    else
+#ifdef MRSID_J2K
+    if( bIsJP2 )
         poDS->nOverviewCount
-	  = ((MrSIDImageReader *) (poDS->poImageReader))->getNumLevels() - 1;
+            = ((J2KImageReader *) (poDS->poImageReader))->getNumLevels();
+    else
+#endif
+        poDS->nOverviewCount
+            = ((MrSIDImageReader *) (poDS->poImageReader))->getNumLevels();
 
     if ( poDS->nOverviewCount > 0 )
     {
@@ -1125,16 +1273,22 @@ GDALDataset *MrSIDDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Create object for the whole image.                              */
 /* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
     poDS->OpenZoomLevel( 0 );
 
     CPLDebug( "MrSID",
               "Opened image: width %d, height %d, bands %d",
               poDS->nRasterXSize, poDS->nRasterYSize, poDS->nBands );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
-#ifdef MRSID_ESDK_VERSION_40
+#ifdef MRSID_ESDK
 
 /************************************************************************/
 /* ==================================================================== */
@@ -1249,10 +1403,19 @@ LT_STATUS MrSIDDummyImageReader::initialize()
 
     if ( poDS->GetGeoTransform( adfGeoTransform ) == CE_None )
     {
+#ifdef MRSID_SDK_40
         LTIGeoCoord oGeo( adfGeoTransform[0] + adfGeoTransform[1] / 2,
                           adfGeoTransform[3] + adfGeoTransform[5] / 2,
                           adfGeoTransform[1], adfGeoTransform[5],
-                          adfGeoTransform[2], adfGeoTransform[4] );
+                          adfGeoTransform[2], adfGeoTransform[4], NULL,
+                          poDS->GetProjectionRef() );
+#else
+        LTIGeoCoord oGeo( adfGeoTransform[0] + adfGeoTransform[1] / 2,
+                          adfGeoTransform[3] + adfGeoTransform[5] / 2,
+                          adfGeoTransform[1], adfGeoTransform[5],
+                          adfGeoTransform[2], adfGeoTransform[4], 
+                          poDS->GetProjectionRef() );
+#endif
         if ( !LT_SUCCESS(setGeoCoord( oGeo )) )
             return LT_STS_Failure;
     }
@@ -1278,7 +1441,7 @@ LT_STATUS MrSIDDummyImageReader::initialize()
 
 /************************************************************************/
 /*                             decodeStrip()                            */
-</************************************************************************/
+/************************************************************************/
 
 LT_STATUS MrSIDDummyImageReader::decodeStrip(LTISceneBuffer& stripData,
                                              const LTIScene& stripScene)
@@ -1320,109 +1483,186 @@ void MrSIDDataset::FlushCache()
 }
 
 /************************************************************************/
-/*                             CreateCopy()                             */
+/*                          MrSIDCreateCopy()                           */
 /************************************************************************/
 
 static GDALDataset *
-CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
-            int bStrict, char ** papszOptions, 
-            GDALProgressFunc pfnProgress, void * pProgressData )
+MrSIDCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
+                 int bStrict, char ** papszOptions, 
+                 GDALProgressFunc pfnProgress, void * pProgressData )
 
 { 
-  const char* pszVerison = CSLFetchNameValue(papszOptions, "VERSION");
+    const char* pszVersion = CSLFetchNameValue(papszOptions, "VERSION");
+#ifdef DEBUG
+    bool bMeter = false;
+#else
+    bool bMeter = true;
+#endif    
 
-  // Output Mrsid Version 2 file.
-  if( pszVerison && atoi(pszVerison) == 2 )
+    // Output Mrsid Version 2 file.
+    if( pszVersion && atoi(pszVersion) == 2 )
     {
-      int nXSize = poSrcDS->GetRasterXSize();
-      int nYSize = poSrcDS->GetRasterYSize();
+        int nXSize = poSrcDS->GetRasterXSize();
+        int nYSize = poSrcDS->GetRasterYSize();
       
-      if( !pfnProgress( 0.0, NULL, pProgressData ) )
-        return NULL;
+        if( !pfnProgress( 0.0, NULL, pProgressData ) )
+            return NULL;
       
-      // Create the file.                                               
-      MrSIDDummyImageReader oImageReader( poSrcDS );
-      oImageReader.initialize();
+        // Create the file.                                               
+        MrSIDDummyImageReader oImageReader( poSrcDS );
+        oImageReader.initialize();
       
-      MG2ImageWriter oImageWriter(&oImageReader);
-      oImageWriter.initialize();
+        MG2ImageWriter oImageWriter(&oImageReader);
+        oImageWriter.initialize();
 
-      oImageWriter.setUsageMeterEnabled( true );
+        oImageWriter.setUsageMeterEnabled(bMeter);
    
-      // set output filename
-      oImageWriter.setOutputFileSpec( pszFilename );
+        // set output filename
+        oImageWriter.setOutputFileSpec( pszFilename );
 
-      // Set defaults
-      oImageWriter.params().setBlockSize(oImageWriter.params().getBlockSize());
-      oImageWriter.setStripHeight(oImageWriter.getStripHeight());
+        // Set defaults
+        oImageWriter.params().setBlockSize(oImageWriter.params().getBlockSize());
+        oImageWriter.setStripHeight(oImageWriter.getStripHeight());
 
-      // check for compression option
-      const char* pszValue = CSLFetchNameValue(papszOptions, "COMPRESSION");
-      if( pszValue != NULL )
-	oImageWriter.params().setCompressionRatio( atof(pszValue) );
+        // check for compression option
+        const char* pszValue = CSLFetchNameValue(papszOptions, "COMPRESSION");
+        if( pszValue != NULL )
+            oImageWriter.params().setCompressionRatio( atof(pszValue) );
 
-      // set MrSID world file
-      if( CSLFetchNameValue(papszOptions, "WORLDFILE") != NULL )
-	oImageWriter.setWorldFileSupport( true );
+        // set MrSID world file
+        if( CSLFetchNameValue(papszOptions, "WORLDFILE") != NULL )
+            oImageWriter.setWorldFileSupport( true );
       
-      // write the scene
-      const LTIScene oScene( 0, 0, nXSize, nYSize, 1.0 );
-      oImageWriter.write( oScene );
+        // write the scene
+        const LTIScene oScene( 0, 0, nXSize, nYSize, 1.0 );
+        oImageWriter.write( oScene );
     }
-  // Output Mrsid Version 3 file.
-  else
+    // Output Mrsid Version 3 file.
+    else
     {
-      int nXSize = poSrcDS->GetRasterXSize();
-      int nYSize = poSrcDS->GetRasterYSize();
+        int nXSize = poSrcDS->GetRasterXSize();
+        int nYSize = poSrcDS->GetRasterYSize();
       
-      if( !pfnProgress( 0.0, NULL, pProgressData ) )
+        if( !pfnProgress( 0.0, NULL, pProgressData ) )
+            return NULL;
+      
+        // Create the file.   
+        MrSIDDummyImageReader oImageReader( poSrcDS );
+        oImageReader.initialize();
+      
+        MG3ImageWriter oImageWriter(&oImageReader);
+        oImageWriter.initialize();
+      
+        // Set 64-bit Interface for large files.
+        oImageWriter.setFileStream64(true);
+
+        oImageWriter.setUsageMeterEnabled(bMeter);
+      
+        // set output filename
+        oImageWriter.setOutputFileSpec( pszFilename );
+
+        // Set defaults
+        oImageWriter.setStripHeight(oImageWriter.getStripHeight());
+
+        // set 2 pass optimizer option
+        if( CSLFetchNameValue(papszOptions, "TWOPASS") != NULL )
+            oImageWriter.params().setTwoPassOptimizer( true );
+
+        // set MrSID world file
+        if( CSLFetchNameValue(papszOptions, "WORLDFILE") != NULL )
+            oImageWriter.setWorldFileSupport( true );
+      
+        const char* pszValue;
+      
+        // set filesize in KB
+        pszValue = CSLFetchNameValue(papszOptions, "FILESIZE");
+        if( pszValue != NULL )
+            oImageWriter.params().setTargetFilesize( atoi(pszValue) );
+	
+        // write the scene
+        const LTIScene oScene( 0, 0, nXSize, nYSize, 1.0 );
+        oImageWriter.write( oScene );
+    }
+  
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxilary pam information.         */
+/* -------------------------------------------------------------------- */
+    GDALPamDataset *poDS = (GDALPamDataset *) 
+        GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
+}
+
+#ifdef MRSID_J2K
+/************************************************************************/
+/*                           JP2CreateCopy()                            */
+/************************************************************************/
+
+static GDALDataset *
+JP2CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
+                 int bStrict, char ** papszOptions, 
+                 GDALProgressFunc pfnProgress, void * pProgressData )
+
+{ 
+#ifdef DEBUG
+    bool bMeter = false;
+#else
+    bool bMeter = true;
+#endif    
+
+    int nXSize = poSrcDS->GetRasterXSize();
+    int nYSize = poSrcDS->GetRasterYSize();
+      
+    if( !pfnProgress( 0.0, NULL, pProgressData ) )
         return NULL;
       
-      // Create the file.                                               
-      MrSIDDummyImageReader oImageReader( poSrcDS );
-      oImageReader.initialize();
+    // Create the file.   
+    MrSIDDummyImageReader oImageReader( poSrcDS );
+    oImageReader.initialize();
       
-      MG3ImageWriter oImageWriter(&oImageReader);
-      oImageWriter.initialize();
+    J2KImageWriter oImageWriter(&oImageReader);
+    oImageWriter.initialize();
       
-      oImageWriter.setUsageMeterEnabled( true );
-      
-      // set output filename
-      oImageWriter.setOutputFileSpec( pszFilename );
+    // Set 64-bit Interface for large files.
+    oImageWriter.setFileStream64(true);
 
-      // Set defaults
-      oImageWriter.setStripHeight(oImageWriter.getStripHeight());
-
-      // set 2 pass optimizer option
-      if( CSLFetchNameValue(papszOptions, "TWOPASS") != NULL )
-	oImageWriter.params().setTwoPassOptimizer( true );
-
-      // set MrSID world file
-      if( CSLFetchNameValue(papszOptions, "WORLDFILE") != NULL )
-	oImageWriter.setWorldFileSupport( true );
+    oImageWriter.setUsageMeterEnabled(bMeter);
       
-      const char* pszValue;
+    // set output filename
+    oImageWriter.setOutputFileSpec( pszFilename );
+
+    // Set defaults
+    //oImageWriter.setStripHeight(oImageWriter.getStripHeight());
+
+    // set MrSID world file
+    if( CSLFetchNameValue(papszOptions, "WORLDFILE") != NULL )
+        oImageWriter.setWorldFileSupport( true );
       
-      // set filesize in KB
-      pszValue = CSLFetchNameValue(papszOptions, "FILESIZE");
-      if( pszValue != NULL )
-	oImageWriter.params().setTargetFilesize( atoi(pszValue) );
+    // check for compression option
+    const char* pszValue = CSLFetchNameValue(papszOptions, "COMPRESSION");
+    if( pszValue != NULL )
+        oImageWriter.params().setCompressionRatio( atof(pszValue) );
 	
-      // write the scene
-      const LTIScene oScene( 0, 0, nXSize, nYSize, 1.0 );
-      oImageWriter.write( oScene );
-    }
+    // write the scene
+    const LTIScene oScene( 0, 0, nXSize, nYSize, 1.0 );
+    oImageWriter.write( oScene );
   
-  // Return pointer to created dataset.                                  
-  GDALDataset *poDS;
-  
-  poDS = (GDALDataset *) GDALOpen( pszFilename, GA_Update );
-  if ( poDS == NULL )
-    poDS = (GDALDataset *) GDALOpen( pszFilename, GA_ReadOnly );
-  
-  return poDS;
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxilary pam information.         */
+/* -------------------------------------------------------------------- */
+    GDALPamDataset *poDS = (GDALPamDataset *) 
+        GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
 }
-#endif /* MRSID_ESDK_VERSION_40 */
+#endif /* MRSID_J2K */
+#endif /* MRSID_ESDK */
 
 /************************************************************************/
 /*                        GDALRegister_MrSID()                          */
@@ -1433,6 +1673,9 @@ void GDALRegister_MrSID()
 {
     GDALDriver  *poDriver;
 
+/* -------------------------------------------------------------------- */
+/*      MrSID driver.                                                   */
+/* -------------------------------------------------------------------- */
     if( GDALGetDriverByName( "MrSID" ) == NULL )
     {
         poDriver = new GDALDriver();
@@ -1443,8 +1686,9 @@ void GDALRegister_MrSID()
         poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_mrsid.html" );
         poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "sid" );
 
-#ifdef MRSID_ESDK_VERSION_40
-        poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, "Byte" );
+#ifdef MRSID_ESDK
+        poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
+                                   "Byte Int16 UInt16 Int32 UInt32 Float32 Float64" );
         poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST,
 "<CreationOptionList>"
 // Version 2 Options
@@ -1458,12 +1702,42 @@ void GDALRegister_MrSID()
 "   <Option name='VERSION' type='int' description='Valid versions are 2 and 3, default = 3'/>"
 "</CreationOptionList>" );
 
-        poDriver->pfnCreateCopy = CreateCopy;
-#endif /* MRSID_ESDK_VERSION_40 */
+        poDriver->pfnCreateCopy = MrSIDCreateCopy;
+#endif
 
-        poDriver->pfnOpen = MrSIDDataset::Open;
+        poDriver->pfnOpen = MrSIDOpen;
 
         GetGDALDriverManager()->RegisterDriver( poDriver );
     }
-}
 
+/* -------------------------------------------------------------------- */
+/*      JP2MRSID driver.                                                */
+/* -------------------------------------------------------------------- */
+#ifdef MRSID_J2K
+    if( GDALGetDriverByName( "JP2MrSID" ) == NULL )
+    {
+        poDriver = new GDALDriver();
+
+        poDriver->SetDescription( "JP2MrSID" );
+        poDriver->SetMetadataItem( GDAL_DMD_LONGNAME,
+                        "MrSID JPEG2000" );
+        poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_jp2mrsid.html" );
+        poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "jp2" );
+
+#ifdef MRSID_ESDK
+        poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
+                                   "Byte Int16 UInt16" );
+        poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST,
+"<CreationOptionList>"
+"   <Option name='COMPRESSION' type='double' description='Set compression ratio (0.0 default is meant to be lossless)'/>"
+"   <Option name='WORLDFILE' type='boolean' description='Write out world file'/>"
+"</CreationOptionList>" );
+
+        poDriver->pfnCreateCopy = JP2CreateCopy;
+#endif
+        poDriver->pfnOpen = JP2Open;
+
+        GetGDALDriverManager()->RegisterDriver( poDriver );
+    }
+#endif /* def MRSID_J2K */
+}
