@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: nitfdataset.cpp,v 1.37 2005/03/01 14:28:06 fwarmerdam Exp $
+ * $Id: nitfdataset.cpp,v 1.49 2005/10/03 17:39:45 fwarmerdam Exp $
  *
  * Project:  NITF Read/Write Translator
  * Purpose:  GDALDataset/GDALRasterBand implementation on top of "nitflib".
@@ -28,6 +28,48 @@
  ******************************************************************************
  *
  * $Log: nitfdataset.cpp,v $
+ * Revision 1.49  2005/10/03 17:39:45  fwarmerdam
+ * fixed memory leak of GCPs as per bug 920
+ *
+ * Revision 1.48  2005/09/19 21:55:24  gwalter
+ * Avoid creating gcps when geotransform already found.
+ *
+ * Revision 1.47  2005/09/14 12:54:40  dron
+ * Avoid possible using of uninitialized value.
+ *
+ * Revision 1.46  2005/09/14 01:12:52  fwarmerdam
+ * Removed complex types from creatable formats, apparently not
+ * currently supported.
+ *
+ * Revision 1.45  2005/07/28 20:00:57  fwarmerdam
+ * upgrade to support 2-4GB files, use large file api
+ *
+ * Revision 1.44  2005/05/23 06:57:36  fwarmerdam
+ * fix flushing to go through pam
+ *
+ * Revision 1.43  2005/05/05 15:54:49  fwarmerdam
+ * PAM Enabled
+ *
+ * Revision 1.42  2005/04/18 17:58:12  fwarmerdam
+ * fixed up ICORDS=' ' case to mean no geotransform
+ *
+ * Revision 1.41  2005/04/18 15:45:17  gwalter
+ * Check for Space Imaging style .hdr + .nfw, and
+ * default to these rather than in-file geocoding
+ * information if both files are present and have
+ * recognizable information.
+ *
+ * Revision 1.40  2005/04/18 13:40:22  fwarmerdam
+ * Remove the default geotransform if we don't have a valid geotransform.
+ *
+ * Revision 1.39  2005/03/21 16:25:41  fwarmerdam
+ * When writing IGEOLO adjust for center of corner pixels instead of
+ * outer image corners.  Also support generating geotransform from
+ * rotated IGEOLO values.
+ *
+ * Revision 1.38  2005/03/17 19:34:31  fwarmerdam
+ * Added IC and IMODE metadata.
+ *
  * Revision 1.37  2005/03/01 14:28:06  fwarmerdam
  * Added support for overriding JPEG2000 profile.
  *
@@ -142,12 +184,12 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "nitflib.h"
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: nitfdataset.cpp,v 1.37 2005/03/01 14:28:06 fwarmerdam Exp $");
+CPL_CVSID("$Id: nitfdataset.cpp,v 1.49 2005/10/03 17:39:45 fwarmerdam Exp $");
 
 static void NITFPatchImageLength( const char *pszFilename,
                                   long nImageOffset, 
@@ -166,7 +208,7 @@ static CPLErr NITFSetColorInterpretation( NITFImage *psImage,
 
 class NITFRasterBand;
 
-class NITFDataset : public GDALDataset
+class NITFDataset : public GDALPamDataset
 {
     friend class NITFRasterBand;
 
@@ -223,7 +265,7 @@ class NITFDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class NITFRasterBand : public GDALRasterBand
+class NITFRasterBand : public GDALPamRasterBand
 {
     friend class NITFDataset;
 
@@ -516,8 +558,8 @@ static CPLErr NITFSetColorInterpretation( NITFImage *psImage,
     
     if( nOffset != 0 )
     {
-        if( VSIFSeek( psImage->psFile->fp, nOffset, SEEK_SET ) != 0 
-            || VSIFWrite( (void *) szPadded, 1, 2, psImage->psFile->fp ) != 2 )
+        if( VSIFSeekL( psImage->psFile->fp, nOffset, SEEK_SET ) != 0 
+            || VSIFWriteL( (void *) szPadded, 1, 2, psImage->psFile->fp ) != 2 )
         {
             CPLError( CE_Failure, CPLE_AppDefined, 
                       "IO failure writing new IREPBAND value to NITF file." );
@@ -599,6 +641,13 @@ NITFDataset::NITFDataset()
     nGCPCount = 0;
     pasGCPList = NULL;
     pszGCPProjection = NULL;
+
+    adfGeoTransform[0] = 0.0;
+    adfGeoTransform[1] = 1.0;
+    adfGeoTransform[2] = 0.0;
+    adfGeoTransform[3] = 0.0;
+    adfGeoTransform[4] = 0.0;
+    adfGeoTransform[5] = 1.0;
 }
 
 /************************************************************************/
@@ -687,7 +736,7 @@ void NITFDataset::FlushCache()
     if( poJ2KDataset != NULL && bJP2Writing)
         poJ2KDataset->FlushCache();
 
-    GDALDataset::FlushCache();
+    GDALPamDataset::FlushCache();
 }
 
 /************************************************************************/
@@ -905,31 +954,12 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
         oSRSWork.exportToWkt( &(poDS->pszProjection) );
     }
 
+
 /* -------------------------------------------------------------------- */
-/*      Do we have IGEOLO data that can be treated as a geotransform?   */
+/*      Try looking for a .nfw file.                                    */
 /* -------------------------------------------------------------------- */
-    if( psImage->dfULX == psImage->dfLLX 
-        && psImage->dfURX == psImage->dfLRX
-        && psImage->dfULY == psImage->dfURY
-        && psImage->dfLLY == psImage->dfLRY
-        && psImage->dfULX != psImage->dfLRX
-        && psImage->dfULY != psImage->dfLRY )
-    {
-        poDS->bGotGeoTransform = TRUE;
-        poDS->adfGeoTransform[0] = psImage->dfULX;
-        poDS->adfGeoTransform[1] = 
-            (psImage->dfLRX - psImage->dfULX) / poDS->nRasterXSize;
-        poDS->adfGeoTransform[2] = 0.0;
-        poDS->adfGeoTransform[3] = psImage->dfULY;
-        poDS->adfGeoTransform[4] = 0.0;
-        poDS->adfGeoTransform[5] = 
-            (psImage->dfLRY - psImage->dfULY) / poDS->nRasterYSize;
-    }
-/* -------------------------------------------------------------------- */
-/*      Otherwise try looking for a .nfw file.                          */
-/* -------------------------------------------------------------------- */
-    else if( GDALReadWorldFile( poOpenInfo->pszFilename, "nfw", 
-                                poDS->adfGeoTransform ) )
+    if( GDALReadWorldFile( poOpenInfo->pszFilename, "nfw", 
+                           poDS->adfGeoTransform ) )
     {
         const char *pszHDR;
         FILE *fpHDR;
@@ -983,18 +1013,80 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
                     oSRSWork.SetWellKnownGeogCS( "WGS84" );
                     oSRSWork.exportToWkt( &(poDS->pszProjection) );
                 }
+                else
+                {
+                    /* Couldn't find associated projection info.
+                       Go back to original file for geotransform.
+                    */
+                    poDS->bGotGeoTransform = FALSE;
+                }
             }
+            else
+                poDS->bGotGeoTransform = FALSE;
             CSLDestroy(papszLines);
         }
-
+        else
+            poDS->bGotGeoTransform = FALSE;
     }
+
+/* -------------------------------------------------------------------- */
+/*      Do we have IGEOLO data that can be treated as a                 */
+/*      geotransform?  Our approach should support images in an         */
+/*      affine rotated frame of reference.                              */
+/* -------------------------------------------------------------------- */
+    int nGCPCount = 0;
+    GDAL_GCP    *psGCPs = NULL;
+
+    if( ( poDS->bGotGeoTransform == FALSE ) && ( psImage->chICORDS != ' ' ) )
+    {
+        nGCPCount = 4;
+
+        psGCPs = (GDAL_GCP *) CPLMalloc(sizeof(GDAL_GCP) * nGCPCount);
+        GDALInitGCPs( nGCPCount, psGCPs );
+
+        psGCPs[0].dfGCPPixel	= 0.0;
+        psGCPs[0].dfGCPLine		= 0.0;
+        psGCPs[0].dfGCPX		= psImage->dfULX;
+        psGCPs[0].dfGCPY		= psImage->dfULY;
+
+        psGCPs[1].dfGCPPixel = poDS->nRasterXSize;
+        psGCPs[1].dfGCPLine = 0.0;
+        psGCPs[1].dfGCPX		= psImage->dfURX;
+        psGCPs[1].dfGCPY		= psImage->dfURY;
+
+        psGCPs[2].dfGCPPixel = poDS->nRasterXSize;
+        psGCPs[2].dfGCPLine = poDS->nRasterYSize;
+        psGCPs[2].dfGCPX		= psImage->dfLRX;
+        psGCPs[2].dfGCPY		= psImage->dfLRY;
+
+        psGCPs[3].dfGCPPixel = 0.0;
+        psGCPs[3].dfGCPLine = poDS->nRasterYSize;
+        psGCPs[3].dfGCPX		= psImage->dfLLX;
+        psGCPs[3].dfGCPY		= psImage->dfLLY;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Convert the GCPs into a geotransform definition, if possible.   */
+/* -------------------------------------------------------------------- */
+    if(  ( poDS->bGotGeoTransform == FALSE ) && nGCPCount > 0 && 
+         GDALGCPsToGeoTransform( nGCPCount, psGCPs, 
+                                 poDS->adfGeoTransform, TRUE ) )
+    {	
+        poDS->bGotGeoTransform = TRUE;
+    } 
+
 /* -------------------------------------------------------------------- */
 /*      If we have IGEOLO that isn't north up, return it as GCPs.       */
 /* -------------------------------------------------------------------- */
     else if( (psImage->dfULX != 0 || psImage->dfURX != 0 
               || psImage->dfLRX != 0 || psImage->dfLLX != 0)
-             && psImage->chICORDS != 'N' )
+             && psImage->chICORDS != ' ' && 
+             ( poDS->bGotGeoTransform == FALSE ) )
     {
+        CPLDebug( "GDAL", 
+                  "NITFDataset::Open() wasn't able to derive a first order\n"
+                  "geotransform.  It will be returned as GCPs.");
+
         poDS->nGCPCount = 4;
         poDS->pasGCPList = (GDAL_GCP *) CPLCalloc(sizeof(GDAL_GCP),
                                                   poDS->nGCPCount);
@@ -1030,22 +1122,43 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
 
         poDS->pszGCPProjection = CPLStrdup( poDS->pszProjection );
     }
-                 
+
+    // This cleans up the original copy of the GCPs used to test if 
+    // this IGEOLO could be used for a geotransform.
+    if( nGCPCount > 0 )
+    {
+        GDALDeinitGCPs( nGCPCount, psGCPs );
+        CPLFree( psGCPs );
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Do we have metadata.                                            */
 /* -------------------------------------------------------------------- */
     char **papszMergedMD;
     char **papszUSE00A_MD;
 
+    // File and Image level metadata.
     papszMergedMD = CSLDuplicate( poDS->psFile->papszMetadata );
     papszMergedMD = CSLInsertStrings( papszMergedMD, 
                                       CSLCount( papszMergedMD ),
                                       psImage->papszMetadata );
 
+    // Comments.
     if( psImage->pszComments != NULL && strlen(psImage->pszComments) != 0 )
         papszMergedMD = CSLSetNameValue( 
             papszMergedMD, "NITF_IMAGE_COMMENTS", psImage->pszComments );
 
+    // Compression code. 
+    papszMergedMD = CSLSetNameValue( papszMergedMD, "NITF_IC", 
+                                     psImage->szIC );
+
+    // IMODE
+    char szIMODE[2];
+    szIMODE[0] = psImage->chIMODE;
+    szIMODE[1] = '\0';
+    papszMergedMD = CSLSetNameValue( papszMergedMD, "NITF_IMODE", szIMODE );
+
+    // USE00A 
     papszUSE00A_MD = NITFReadUSE00A( psImage );
     if( papszUSE00A_MD != NULL )
     {
@@ -1063,7 +1176,7 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
                                           papszUSE00A_MD );
         CSLDestroy( papszUSE00A_MD );
     }
-    
+
     poDS->SetMetadata( papszMergedMD );
     CSLDestroy( papszMergedMD );
     
@@ -1227,6 +1340,12 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
@@ -1285,11 +1404,10 @@ CPLErr NITFDataset::IRasterIO( GDALRWFlag eRWFlag,
 CPLErr NITFDataset::GetGeoTransform( double *padfGeoTransform )
 
 {
+    memcpy( padfGeoTransform, adfGeoTransform, sizeof(double) * 6 );
+
     if( bGotGeoTransform )
-    {
-        memcpy( padfGeoTransform, adfGeoTransform, sizeof(double) * 6 );
         return CE_None;
-    }
     else
         return CE_Failure;
 }
@@ -1301,23 +1419,27 @@ CPLErr NITFDataset::GetGeoTransform( double *padfGeoTransform )
 CPLErr NITFDataset::SetGeoTransform( double *padfGeoTransform )
 
 {
-    double dfULX, dfULY, dfURX, dfURY, dfLRX, dfLRY, dfLLX, dfLLY;
+    double dfIGEOLOULX, dfIGEOLOULY, dfIGEOLOURX, dfIGEOLOURY, 
+           dfIGEOLOLRX, dfIGEOLOLRY, dfIGEOLOLLX, dfIGEOLOLLY;
 
-    dfULX = padfGeoTransform[0];
-    dfULY = padfGeoTransform[3];
-    dfURX = dfULX + padfGeoTransform[1] * nRasterXSize;
-    dfURY = dfULY + padfGeoTransform[4] * nRasterXSize;
-    dfLRX = dfULX + padfGeoTransform[1] * nRasterXSize
-        + padfGeoTransform[2] * nRasterYSize;
-    dfLRY = dfULY + padfGeoTransform[4] * nRasterXSize
-        + padfGeoTransform[5] * nRasterYSize;
-    dfLLX = dfULX + padfGeoTransform[2] * nRasterYSize;
-    dfLLY = dfULY + padfGeoTransform[5] * nRasterYSize;
+
+    dfIGEOLOULX = padfGeoTransform[0] + 0.5 * padfGeoTransform[1] 
+                                      + 0.5 * padfGeoTransform[2];
+    dfIGEOLOULY = padfGeoTransform[3] + 0.5 * padfGeoTransform[4] 
+                                      + 0.5 * padfGeoTransform[5];
+    dfIGEOLOURX = dfIGEOLOULX + padfGeoTransform[1] * (nRasterXSize - 1);
+    dfIGEOLOURY = dfIGEOLOULY + padfGeoTransform[4] * (nRasterXSize - 1);
+    dfIGEOLOLRX = dfIGEOLOULX + padfGeoTransform[1] * (nRasterXSize - 1)
+                              + padfGeoTransform[2] * (nRasterYSize - 1);
+    dfIGEOLOLRY = dfIGEOLOULY + padfGeoTransform[4] * (nRasterXSize - 1)
+                              + padfGeoTransform[5] * (nRasterYSize - 1);
+    dfIGEOLOLLX = dfIGEOLOULX + padfGeoTransform[2] * (nRasterYSize - 1);
+    dfIGEOLOLLY = dfIGEOLOULY + padfGeoTransform[5] * (nRasterYSize - 1);
 
     if( NITFWriteIGEOLO( psImage, psImage->chICORDS, 
                          psImage->nZone, 
-                         dfULX, dfULY, dfURX, dfURY, 
-                         dfLRX, dfLRY, dfLLX, dfLLY ) )
+                         dfIGEOLOULX, dfIGEOLOULY, dfIGEOLOURX, dfIGEOLOURY, 
+                         dfIGEOLOLRX, dfIGEOLOLRY, dfIGEOLOLLX, dfIGEOLOLLY ) )
         return CE_None;
     else
         return CE_Failure;
@@ -1464,7 +1586,7 @@ NITFDatasetCreate( const char *pszFilename, int nXSize, int nYSize, int nBands,
 /* -------------------------------------------------------------------- */
 /*      We disallow any IC value except NC when creating this way.      */
 /* -------------------------------------------------------------------- */
-    GDALDriver *poJ2KDriver;
+    GDALDriver *poJ2KDriver = NULL;
 
     if( pszIC != NULL && EQUAL(pszIC,"C8") )
     {
@@ -1504,7 +1626,7 @@ NITFDatasetCreate( const char *pszFilename, int nXSize, int nYSize, int nBands,
 /* -------------------------------------------------------------------- */
 /*      Various special hacks related to JPEG2000 encoded files.        */
 /* -------------------------------------------------------------------- */
-    if( pszIC != NULL && EQUAL(pszIC,"C8") )
+    if( poJ2KDriver )
     {
         NITFFile *psFile = NITFOpen( pszFilename, TRUE );
         int nImageOffset = psFile->pasSegmentInfo[0].nSegmentStart;
@@ -1545,7 +1667,7 @@ NITFDataset::NITFCreateCopy(
     GDALRasterBand *poBand1 = poSrcDS->GetRasterBand(1);
     char  **papszFullOptions = CSLDuplicate( papszOptions );
     int   bJPEG2000 = FALSE;
-    GDALDataset *poDstDS = NULL;
+    NITFDataset *poDstDS = NULL;
     GDALDriver *poJ2KDriver = NULL;
 
     if( poBand1 == NULL )
@@ -1676,7 +1798,7 @@ NITFDataset::NITFCreateCopy(
 /* ==================================================================== */
     if( !bJPEG2000 )
     {
-        poDstDS = (GDALDataset *) GDALOpen( pszFilename, GA_Update );
+        poDstDS = (NITFDataset *) GDALOpen( pszFilename, GA_Update );
         if( poDstDS == NULL )
             return NULL;
         
@@ -1766,7 +1888,7 @@ NITFDataset::NITFCreateCopy(
 
         NITFPatchImageLength( pszFilename, nImageOffset, nPixelCount );
 
-        poDstDS = (GDALDataset *) GDALOpen( pszFilename, GA_Update );
+        poDstDS = (NITFDataset *) GDALOpen( pszFilename, GA_Update );
 
         if( poDstDS == NULL )
             return NULL;
@@ -1777,10 +1899,12 @@ NITFDataset::NITFCreateCopy(
 /* -------------------------------------------------------------------- */
     if( bWriteGeoTransform )
     {
-        ((NITFDataset *)poDstDS)->psImage->nZone = nZone;
+        poDstDS->psImage->nZone = nZone;
         poDstDS->SetGeoTransform( adfGeoTransform );
     }
 
+    poDstDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
     return poDstDS;
 }
 
@@ -1878,7 +2002,7 @@ void GDALRegister_NITF()
         poDriver->SetMetadataItem( GDAL_DMD_HELPTOPIC, "frmt_nitf.html" );
         poDriver->SetMetadataItem( GDAL_DMD_EXTENSION, "ntf" );
         poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
-                                   "Byte UInt16 Int16 UInt32 Int32 Float32 CFloat32 CFloat64" );
+                                   "Byte UInt16 Int16 UInt32 Int32 Float32" );
 
         GetGDALDriverManager()->RegisterDriver( poDriver );
     }
