@@ -8,6 +8,16 @@
 #define INCLUDED_CASSERT
 #endif
 
+#ifndef INCLUDED_ALGORITHM
+#include <algorithm>
+#define INCLUDED_ALGORITHM
+#endif
+
+#ifndef INCLUDED_LIMITS
+#include <limits>
+#define INCLUDED_LIMITS
+#endif
+
 #ifndef INCLUDED_PCRTYPES
 #include "pcrtypes.h"
 #define INCLUDED_PCRTYPES
@@ -463,3 +473,4 @@ CSF_VS fitValueScale(CSF_VS valueScale, CSF_CR cellRepresentation)
 
   return result;
 }
+
