@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: pngdataset.cpp,v 1.29 2004/08/26 21:20:08 warmerda Exp $
+ * $Id: pngdataset.cpp,v 1.34 2005/09/15 02:37:48 fwarmerdam Exp $
  *
  * Project:  PNG Driver
  * Purpose:  Implement GDAL PNG Support
@@ -43,6 +43,22 @@
  *    application termination. 
  * 
  * $Log: pngdataset.cpp,v $
+ * Revision 1.34  2005/09/15 02:37:48  fwarmerdam
+ * added support NODATA_VALUES for RGB images (read and write)
+ *
+ * Revision 1.33  2005/09/12 18:55:10  fwarmerdam
+ * Fixed serious bug with reading multi-band 16bit files, and with
+ * writing 16bit files on LSB systems.
+ *
+ * Revision 1.32  2005/09/11 19:11:59  fwarmerdam
+ * Redirect IO through VSI.
+ *
+ * Revision 1.31  2005/05/17 19:05:52  fwarmerdam
+ * Allow flowthrough to pam for geotransform & nodata.
+ *
+ * Revision 1.30  2005/04/27 16:35:45  fwarmerdam
+ * PAM enable
+ *
  * Revision 1.29  2004/08/26 21:20:08  warmerda
  * Adjusted so png_access_version_number() is not called for old libpng
  * versions (pre 1.2).
@@ -63,90 +79,25 @@
  *
  * Revision 1.24  2003/09/15 20:45:00  warmerda
  * add pngw and pgw support
- *
- * Revision 1.23  2003/07/08 21:27:34  warmerda
- * avoid warnings
- *
- * Revision 1.22  2003/01/31 18:08:24  warmerda
- * Added test for broken png_get_channels().  I don't know what is *really*
- * going on here.
- *
- * Revision 1.21  2003/01/25 22:28:18  warmerda
- * improved data type error message a bit
- *
- * Revision 1.20  2003/01/25 22:26:29  warmerda
- * fixed a read, and a write bug with 16bit png files
- *
- * Revision 1.19  2002/11/23 18:54:17  warmerda
- * added CREATIONDATATYPES metadata for drivers
- *
- * Revision 1.18  2002/09/04 06:50:37  warmerda
- * avoid static driver pointers
- *
- * Revision 1.17  2002/07/13 04:16:39  warmerda
- * added WORLDFILE support
- *
- * Revision 1.16  2002/06/18 02:49:23  warmerda
- * fixed multiline string constants
- *
- * Revision 1.15  2002/06/12 21:12:25  warmerda
- * update to metadata based driver info
- *
- * Revision 1.14  2002/04/20 10:12:05  dron
- * Added support for GDALWriteWolrldFile()
- * New option WORLDFILE=YES
- *
- * Revision 1.13  2002/04/20 09:51:03  dron
- * *** empty log message ***
- *
- * Revision 1.12  2001/11/11 23:51:00  warmerda
- * added required class keyword to friend declarations
- *
- * Revision 1.11  2001/09/28 14:30:19  warmerda
- * Ensure num_trans is initialized to zero in case png_get_tRNS fails.
- *
- * Revision 1.10  2001/08/23 03:45:15  warmerda
- * If there is only one transparent color in the color table, also consider
- * it to be the nodata value.
- *
- * Revision 1.9  2001/08/23 03:32:37  warmerda
- * implemented read/write support for transparency (colortable/nodata)
- *
- * Revision 1.8  2001/08/22 17:12:07  warmerda
- * added world file support
- *
- * Revision 1.7  2001/07/18 04:51:57  warmerda
- * added CPL_CVSID
- *
- * Revision 1.6  2000/08/15 19:28:26  warmerda
- * added help topic
- *
- * Revision 1.5  2000/06/05 13:04:15  warmerda
- * Fixed case where png_get_text fails due to no text blocks existing.
- *
- * Revision 1.4  2000/05/15 22:26:42  warmerda
- * Avoid warning.
- *
- * Revision 1.3  2000/04/28 20:57:07  warmerda
- * Removed some unused code.
- *
- * Revision 1.2  2000/04/28 20:56:01  warmerda
- * converted to use CreateCopy() instead of Create()
- *
- * Revision 1.1  2000/04/27 19:39:51  warmerda
- * New
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "png.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: pngdataset.cpp,v 1.29 2004/08/26 21:20:08 warmerda Exp $");
+CPL_CVSID("$Id: pngdataset.cpp,v 1.34 2005/09/15 02:37:48 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_PNG(void);
 CPL_C_END
 
+static void
+png_vsi_read_data(png_structp png_ptr, png_bytep data, png_size_t length);
+
+static void
+png_vsi_write_data(png_structp png_ptr, png_bytep data, png_size_t length);
+
+static void png_vsi_flush(png_structp png_ptr);
 
 /************************************************************************/
 /* ==================================================================== */
@@ -156,7 +107,7 @@ CPL_C_END
 
 class PNGRasterBand;
 
-class PNGDataset : public GDALDataset
+class PNGDataset : public GDALPamDataset
 {
     friend class PNGRasterBand;
 
@@ -200,7 +151,7 @@ class PNGDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class PNGRasterBand : public GDALRasterBand
+class PNGRasterBand : public GDALPamRasterBand
 {
     friend class PNGDataset;
 
@@ -353,10 +304,16 @@ double PNGRasterBand::GetNoDataValue( int *pbSuccess )
 {
     PNGDataset *poPDS = (PNGDataset *) poDS;
 
-    if( pbSuccess != NULL )
-        *pbSuccess = poPDS->bHaveNoData;
-
-    return poPDS->dfNoDataValue;
+    if( poPDS->bHaveNoData )
+    {
+        if( pbSuccess != NULL )
+            *pbSuccess = poPDS->bHaveNoData;
+        return poPDS->dfNoDataValue;
+    }
+    else
+    {
+        return GDALPamRasterBand::GetNoDataValue( pbSuccess );
+    }
 }
 
 /************************************************************************/
@@ -404,7 +361,8 @@ PNGDataset::~PNGDataset()
 
     png_destroy_read_struct( &hPNG, &psPNGInfo, NULL );
 
-    VSIFClose( fpImage );
+    if( fpImage )
+        VSIFCloseL( fpImage );
 
     if( poColorTable != NULL )
         delete poColorTable;
@@ -424,7 +382,7 @@ CPLErr PNGDataset::GetGeoTransform( double * padfTransform )
         return CE_None;
     }
     else
-        return CE_Failure;
+        return GDALPamDataset::GetGeoTransform( padfTransform );
 }
 
 /************************************************************************/
@@ -437,7 +395,7 @@ CPLErr PNGDataset::GetGeoTransform( double * padfTransform )
 void PNGDataset::FlushCache()
 
 {
-    GDALDataset::FlushCache();
+    GDALPamDataset::FlushCache();
 
     if( pabyBuffer != NULL )
     {
@@ -459,13 +417,12 @@ void PNGDataset::Restart()
 {
     png_destroy_read_struct( &hPNG, &psPNGInfo, NULL );
 
-    VSIRewind( fpImage );
-
     hPNG = png_create_read_struct( PNG_LIBPNG_VER_STRING, this, NULL, NULL );
 
     psPNGInfo = png_create_info_struct( hPNG );
 
-    png_init_io( hPNG, fpImage );
+    VSIFSeekL( fpImage, 0, SEEK_SET );
+    png_set_read_fn( hPNG, fpImage, png_vsi_read_data );
     png_read_info( hPNG, psPNGInfo );
 
     if( nBitDepth < 8 )
@@ -569,7 +526,7 @@ CPLErr PNGDataset::LoadScanline( int nLine )
 /* -------------------------------------------------------------------- */
 #ifdef CPL_LSB
     if( nBitDepth == 16 )
-        GDALSwapWords( row, 2, GetRasterXSize(), 2 );
+        GDALSwapWords( row, 2, GetRasterXSize() * GetRasterCount(), 2 );
 #endif
 
     return CE_None;
@@ -636,6 +593,18 @@ GDALDataset *PNGDataset::Open( GDALOpenInfo * poOpenInfo )
         return NULL;
     }
 
+/* -------------------------------------------------------------------- */
+/*      Open a file handle using large file API.                        */
+/* -------------------------------------------------------------------- */
+    FILE *fp = VSIFOpenL( poOpenInfo->pszFilename, "rb" );
+    if( fp == NULL )
+    {
+        CPLError( CE_Failure, CPLE_OpenFailed, 
+                  "Unexpected failure of VSIFOpenL(%s) in PNG Open()", 
+                  poOpenInfo->pszFilename );
+        return NULL;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Create a corresponding GDALDataset.                             */
 /* -------------------------------------------------------------------- */
@@ -643,6 +612,7 @@ GDALDataset *PNGDataset::Open( GDALOpenInfo * poOpenInfo )
 
     poDS = new PNGDataset();
 
+    poDS->fpImage = fp;
     poDS->eAccess = poOpenInfo->eAccess;
     
     poDS->hPNG = png_create_read_struct( PNG_LIBPNG_VER_STRING, poDS, 
@@ -671,9 +641,7 @@ GDALDataset *PNGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     /* we should likely do a setjmp() here */
 
-    VSIRewind( poOpenInfo->fp );
-    
-    png_init_io( poDS->hPNG, poOpenInfo->fp );
+    png_set_read_fn( poDS->hPNG, poDS->fpImage, png_vsi_read_data );
     png_read_info( poDS->hPNG, poDS->psPNGInfo );
 
 /* -------------------------------------------------------------------- */
@@ -712,12 +680,6 @@ GDALDataset *PNGDataset::Open( GDALOpenInfo * poOpenInfo )
     for( int iBand = 0; iBand < poDS->nBands; iBand++ )
         poDS->SetBand( iBand+1, new PNGRasterBand( poDS, iBand+1 ) );
 
-/* -------------------------------------------------------------------- */
-/*      Adopt the file pointer.                                         */
-/* -------------------------------------------------------------------- */
-    poDS->fpImage = poOpenInfo->fp;
-    poOpenInfo->fp = NULL;
-
 /* -------------------------------------------------------------------- */
 /*      Is there a palette?  Note: we should also read back and         */
 /*      apply transparency values if available.                         */
@@ -778,8 +740,7 @@ GDALDataset *PNGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Check for transparency values in greyscale images.              */
 /* -------------------------------------------------------------------- */
-    if( poDS->nColorType == PNG_COLOR_TYPE_GRAY 
-        || poDS->nColorType == PNG_COLOR_TYPE_GRAY_ALPHA )
+    if( poDS->nColorType == PNG_COLOR_TYPE_GRAY )
     {
         png_color_16 *trans_values = NULL;
         unsigned char *trans;
@@ -794,6 +755,29 @@ GDALDataset *PNGDataset::Open( GDALOpenInfo * poOpenInfo )
         }
     }
 
+/* -------------------------------------------------------------------- */
+/*      Check for nodata color for RGB images.                          */
+/* -------------------------------------------------------------------- */
+    if( poDS->nColorType == PNG_COLOR_TYPE_RGB ) 
+    {
+        png_color_16 *trans_values = NULL;
+        unsigned char *trans;
+        int num_trans;
+
+        if( png_get_tRNS( poDS->hPNG, poDS->psPNGInfo, 
+                          &trans, &num_trans, &trans_values ) != 0 
+            && trans_values != NULL )
+        {
+            CPLString oNDValue;
+
+            oNDValue.Printf( "%d %d %d", 
+                    trans_values->red, 
+                    trans_values->green, 
+                    trans_values->blue );
+            poDS->SetMetadataItem( "NODATA_VALUES", oNDValue.c_str() );
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Extract any text chunks as "metadata".                          */
 /* -------------------------------------------------------------------- */
@@ -804,6 +788,12 @@ GDALDataset *PNGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
 /* -------------------------------------------------------------------- */
 /*      Check for world file.                                           */
 /* -------------------------------------------------------------------- */
@@ -901,7 +891,7 @@ PNGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
     FILE	*fpImage;
 
-    fpImage = VSIFOpen( pszFilename, "wb" );
+    fpImage = VSIFOpenL( pszFilename, "wb" );
     if( fpImage == NULL )
     {
         CPLError( CE_Failure, CPLE_OpenFailed, 
@@ -920,7 +910,7 @@ PNGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                                     NULL, NULL, NULL );
     psPNGInfo = png_create_info_struct( hPNG );
     
-    png_init_io( hPNG, fpImage );
+    png_set_write_fn( hPNG, fpImage, png_vsi_write_data, png_vsi_flush );
 
     png_set_IHDR( hPNG, psPNGInfo, nXSize, nYSize, 
                   nBitDepth, nColorType, PNG_INTERLACE_NONE, 
@@ -928,9 +918,8 @@ PNGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 
 /* -------------------------------------------------------------------- */
 /*      Try to handle nodata values as a tRNS block (note for           */
-/*      paletted images, we safe the effect to apply as part of         */
-/*      palette).  We don't try to address a nodata value for RGB       */
-/*      images.                                                         */
+/*      paletted images, we save the effect to apply as part of         */
+/*      palette).                                                       */
 /* -------------------------------------------------------------------- */
     int		bHaveNoData = FALSE;
     double	dfNoDataValue = -1;
@@ -938,14 +927,31 @@ PNGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 
     dfNoDataValue = poSrcDS->GetRasterBand(1)->GetNoDataValue( &bHaveNoData );
 
-    if( (nColorType == PNG_COLOR_TYPE_GRAY 
-         || nColorType == PNG_COLOR_TYPE_GRAY_ALPHA)
+    if( (nColorType == PNG_COLOR_TYPE_GRAY )
         && dfNoDataValue > 0 && dfNoDataValue < 65536 )
     {
         sTRNSColor.gray = (png_uint_16) dfNoDataValue;
         png_set_tRNS( hPNG, psPNGInfo, NULL, 0, &sTRNSColor );
     }
 
+    // RGB case.
+    if( nColorType == PNG_COLOR_TYPE_RGB 
+        && poSrcDS->GetMetadataItem( "NODATA_VALUES" ) != NULL )
+    {
+        char **papszValues = CSLTokenizeString(
+            poSrcDS->GetMetadataItem( "NODATA_VALUES" ) );
+        
+        if( CSLCount(papszValues) >= 3 )
+        {
+            sTRNSColor.red   = (png_uint_16) atoi(papszValues[0]);
+            sTRNSColor.green = (png_uint_16) atoi(papszValues[1]);
+            sTRNSColor.blue  = (png_uint_16) atoi(papszValues[2]);
+            png_set_tRNS( hPNG, psPNGInfo, NULL, 0, &sTRNSColor );
+        }
+
+        CSLDestroy( papszValues );
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Write palette if there is one.  Technically, I think it is      */
 /*      possible to write 16bit palettes for PNG, but we will omit      */
@@ -1027,6 +1033,10 @@ PNGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                                      nBands * nXSize * nWordSize );
         }
 
+#ifdef CPL_LSB
+        if( nBitDepth == 16 )
+            GDALSwapWords( row, 2, nXSize * nBands, 2 );
+#endif
         png_write_rows( hPNG, &row, 1 );
     }
 
@@ -1035,7 +1045,7 @@ PNGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     png_write_end( hPNG, psPNGInfo );
     png_destroy_write_struct( &hPNG, &psPNGInfo );
 
-    VSIFClose( fpImage );
+    VSIFCloseL( fpImage );
 
     CPLFree( pabyAlpha );
     CPLFree( pasPNGColors );
@@ -1051,7 +1061,59 @@ PNGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 	GDALWriteWorldFile( pszFilename, "wld", adfGeoTransform );
     }
 
-    return (GDALDataset *) GDALOpen( pszFilename, GA_ReadOnly );
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxilary pam information.         */
+/* -------------------------------------------------------------------- */
+    PNGDataset *poDS = (PNGDataset *) GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
+}
+
+/************************************************************************/
+/*                         png_vsi_read_data()                          */
+/*                                                                      */
+/*      Read data callback through VSI.                                 */
+/************************************************************************/
+static void
+png_vsi_read_data(png_structp png_ptr, png_bytep data, png_size_t length)
+
+{
+   png_size_t check;
+
+   /* fread() returns 0 on error, so it is OK to store this in a png_size_t
+    * instead of an int, which is what fread() actually returns.
+    */
+   check = (png_size_t)VSIFReadL(data, (png_size_t)1, length,
+                                 (png_FILE_p)png_ptr->io_ptr);
+
+   if (check != length)
+      png_error(png_ptr, "Read Error");
+}
+
+/************************************************************************/
+/*                         png_vsi_write_data()                         */
+/************************************************************************/
+
+static void
+png_vsi_write_data(png_structp png_ptr, png_bytep data, png_size_t length)
+{
+   png_uint_32 check;
+
+   check = VSIFWriteL(data, 1, length, (png_FILE_p)(png_ptr->io_ptr));
+
+   if (check != length)
+      png_error(png_ptr, "Write Error");
+}
+
+/************************************************************************/
+/*                           png_vsi_flush()                            */
+/************************************************************************/
+static void png_vsi_flush(png_structp png_ptr)
+{
+    VSIFFlushL( (png_FILE_p)(png_ptr->io_ptr) );
 }
 
 /************************************************************************/
