@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: btdataset.cpp,v 1.6 2004/05/11 17:27:33 warmerda Exp $
+ * $Id: btdataset.cpp,v 1.7 2005/05/05 13:55:41 fwarmerdam Exp $
  *
  * Project:  VTP .bt Driver
  * Purpose:  Implementation of VTP .bt elevation format read/write support.
@@ -29,6 +29,9 @@
  ******************************************************************************
  *
  * $Log: btdataset.cpp,v $
+ * Revision 1.7  2005/05/05 13:55:41  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.6  2004/05/11 17:27:33  warmerda
  * Set eAccess properly in Open().
  *
@@ -54,7 +57,7 @@
 #include "rawdataset.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: btdataset.cpp,v 1.6 2004/05/11 17:27:33 warmerda Exp $");
+CPL_CVSID("$Id: btdataset.cpp,v 1.7 2005/05/05 13:55:41 fwarmerdam Exp $");
 
 CPL_C_START
 void    GDALRegister_BT(void);
@@ -230,7 +233,7 @@ CPLErr BTRasterBand::IWriteBlock( int nBlockXOff, int nBlockYOff,
 /* ==================================================================== */
 /************************************************************************/
 
-class BTDataset : public GDALDataset
+class BTDataset : public GDALPamDataset
 {
     FILE        *fpImage;       // image data file.
 
@@ -715,6 +718,12 @@ GDALDataset *BTDataset::Open( GDALOpenInfo * poOpenInfo )
                            poDS->adfGeoTransform );
 #endif
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
