@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpgdataset.cpp,v 1.6 2004/12/17 22:45:55 gwalter Exp $
+ * $Id: cpgdataset.cpp,v 1.7 2005/05/05 13:55:41 fwarmerdam Exp $
  *
  * Project:  Polarimetric Workstation
  * Purpose:  Convair PolGASP data (.img/.hdr format). 
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: cpgdataset.cpp,v $
+ * Revision 1.7  2005/05/05 13:55:41  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.6  2004/12/17 22:45:55  gwalter
  * Added support for sirc-style convair flat
  * binary datasets.
@@ -57,7 +60,7 @@
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: cpgdataset.cpp,v 1.6 2004/12/17 22:45:55 gwalter Exp $");
+CPL_CVSID("$Id: cpgdataset.cpp,v 1.7 2005/05/05 13:55:41 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_CPG(void);
@@ -594,6 +597,12 @@ GDALDataset *CPGDataset::Open( GDALOpenInfo * poOpenInfo )
     // Need to think about this. 
     // poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
