@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ehdrdataset.cpp,v 1.25 2005/02/25 19:40:27 fwarmerdam Exp $
+ * $Id: ehdrdataset.cpp,v 1.27 2005/10/04 05:25:13 fwarmerdam Exp $
  *
  * Project:  ESRI .hdr Driver
  * Purpose:  Implementation of EHdrDataset
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ehdrdataset.cpp,v $
+ * Revision 1.27  2005/10/04 05:25:13  fwarmerdam
+ * Added support for XLLCENTER/YLLCENTER.
+ *
+ * Revision 1.26  2005/05/05 13:55:41  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.25  2005/02/25 19:40:27  fwarmerdam
  * Support lower case byteorder values too.
  *
@@ -112,7 +118,7 @@
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ehdrdataset.cpp,v 1.25 2005/02/25 19:40:27 fwarmerdam Exp $");
+CPL_CVSID("$Id: ehdrdataset.cpp,v 1.27 2005/10/04 05:25:13 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_EHdr(void);
@@ -287,7 +293,8 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
             nSkipBytes = atoi(papszTokens[1]);
         }
         else if( EQUAL(papszTokens[0],"ulxmap") 
-                 || EQUAL(papszTokens[0],"xllcorner") )
+                 || EQUAL(papszTokens[0],"xllcorner") 
+                 || EQUAL(papszTokens[0],"xllcenter") )
         {
             dfULXMap = atof(papszTokens[1]);
         }
@@ -295,7 +302,8 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
         {
             dfULYMap = atof(papszTokens[1]);
         }
-        else if( EQUAL(papszTokens[0],"yllcorner") )
+        else if( EQUAL(papszTokens[0],"yllcorner") 
+                 || EQUAL(papszTokens[0],"yllcenter") )
         {
             dfYLLCorner = atof(papszTokens[1]);
         }
@@ -564,12 +572,18 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Check for overviews.                                            */
 /* -------------------------------------------------------------------- */
-        poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
+    poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
-        CPLFree( pszName );
-        CPLFree( pszPath );
-
-        return( poDS );
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+    
+    CPLFree( pszName );
+    CPLFree( pszPath );
+    
+    return( poDS );
 }
 
 /************************************************************************/
