@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: fastdataset.cpp,v 1.16 2004/07/12 18:24:23 gwalter Exp $
+ * $Id: fastdataset.cpp,v 1.18 2005/09/14 13:18:32 dron Exp $
  *
  * Project:  EOSAT FAST Format reader
  * Purpose:  Reads Landsat FAST-L7A, IRS 1C/1D
@@ -28,6 +28,12 @@
  ******************************************************************************
  * 
  * $Log: fastdataset.cpp,v $
+ * Revision 1.18  2005/09/14 13:18:32  dron
+ * Avoid warnings.
+ *
+ * Revision 1.17  2005/05/05 13:55:42  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.16  2004/07/12 18:24:23  gwalter
  * Fixed geotransform calculation.
  *
@@ -94,13 +100,12 @@
  *
  */
 
-#include "gdal_priv.h"
 #include "cpl_string.h"
 #include "cpl_conv.h"
 #include "ogr_spatialref.h"
 #include "rawdataset.h"
 
-CPL_CVSID("$Id: fastdataset.cpp,v 1.16 2004/07/12 18:24:23 gwalter Exp $");
+CPL_CVSID("$Id: fastdataset.cpp,v 1.18 2005/09/14 13:18:32 dron Exp $");
 
 CPL_C_START
 void	GDALRegister_FAST(void);
@@ -165,7 +170,7 @@ enum FASTSatellite  // Satellites:
 /* ==================================================================== */
 /************************************************************************/
 
-class FASTDataset : public GDALDataset
+class FASTDataset : public GDALPamDataset
 {
     friend class FASTRasterBand;
 
@@ -254,6 +259,9 @@ FASTDataset::~FASTDataset()
 
 {
     int i;
+
+    FlushCache();
+
     if ( pszDirname )
 	CPLFree( pszDirname );
     if ( pszProjection )
@@ -613,7 +621,7 @@ GDALDataset *FASTDataset::Open( GDALOpenInfo * poOpenInfo )
     // Now search for the first number occurance after that string
     for ( i = 1; i <= poDS->nBands; i++ )
     {
-        char *pszValue;
+        char *pszValue = NULL;
 
         pszTemp = strpbrk( pszTemp, "-.0123456789" );
         if ( pszTemp )
@@ -821,6 +829,12 @@ GDALDataset *FASTDataset::Open( GDALOpenInfo * poOpenInfo )
 
     CPLFree( pszHeader );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+    
     return( poDS );
 }
 
