@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hkvdataset.cpp,v 1.35 2004/01/23 18:55:01 gwalter Exp $
+ * $Id: hkvdataset.cpp,v 1.36 2005/05/05 13:55:42 fwarmerdam Exp $
  *
  * Project:  GView
  * Purpose:  Implementation of Atlantis HKV labelled blob support
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: hkvdataset.cpp,v $
+ * Revision 1.36  2005/05/05 13:55:42  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.35  2004/01/23 18:55:01  gwalter
  * Remove unnecessary print statement.
  *
@@ -142,7 +145,7 @@
 #include "ogr_spatialref.h"
 #include "atlsci_spheroid.h"
 
-CPL_CVSID("$Id: hkvdataset.cpp,v 1.35 2004/01/23 18:55:01 gwalter Exp $");
+CPL_CVSID("$Id: hkvdataset.cpp,v 1.36 2005/05/05 13:55:42 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_HKV(void);
@@ -1485,6 +1488,12 @@ GDALDataset *HKVDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, pszOvrFilename, TRUE );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( pszOvrFilename );
+    poDS->TryLoadXML();
+    
     CPLFree( pszOvrFilename );
 
     return( poDS );
@@ -1717,10 +1726,10 @@ HKVDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     /* check that other bands match type- sets type */
     /* to unknown if they differ.                  */
     for( iBand = 1; iBand < poSrcDS->GetRasterCount(); iBand++ )
-     {
-         GDALRasterBand *poBand = poSrcDS->GetRasterBand( iBand+1 );
-         eType = GDALDataTypeUnion( eType, poBand->GetRasterDataType() );
-     }
+    {
+        GDALRasterBand *poBand = poSrcDS->GetRasterBand( iBand+1 );
+        eType = GDALDataTypeUnion( eType, poBand->GetRasterDataType() );
+    }
 
     poDS = (HKVDataset *) Create( pszFilename, 
                                   poSrcDS->GetRasterXSize(), 
@@ -1856,6 +1865,8 @@ HKVDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         return NULL;
     }
 
+    poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
     return poDS;
 }
 
