@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: landataset.cpp,v 1.5 2004/05/28 18:15:58 warmerda Exp $
+ * $Id: landataset.cpp,v 1.9 2005/09/27 18:13:04 fwarmerdam Exp $
  *
  * Project:  eCognition
  * Purpose:  Implementation of Erdas .LAN / .GIS format.
@@ -28,6 +28,19 @@
  ******************************************************************************
  *
  * $Log: landataset.cpp,v $
+ * Revision 1.9  2005/09/27 18:13:04  fwarmerdam
+ * Only return 16 colors for 4bit files.
+ *
+ * Revision 1.8  2005/09/27 17:40:05  fwarmerdam
+ * Make sure we notify RawRasterBand we are using large file api.
+ *
+ * Revision 1.7  2005/09/27 17:37:20  fwarmerdam
+ * Derived 4bit band from GDALPamRasterBand, and added colortable support.
+ * Also use large file API consistently for file access.
+ *
+ * Revision 1.6  2005/05/05 13:55:42  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.5  2004/05/28 18:15:58  warmerda
  * added .TRL colortable support
  *
@@ -48,7 +61,7 @@
 #include "rawdataset.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: landataset.cpp,v 1.5 2004/05/28 18:15:58 warmerda Exp $");
+CPL_CVSID("$Id: landataset.cpp,v 1.9 2005/09/27 18:13:04 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_LAN(void);
@@ -119,7 +132,7 @@ if there are more than 255 bands.
 
 class LANDataset;
 
-class LAN4BitRasterBand : public GDALRasterBand
+class LAN4BitRasterBand : public GDALPamRasterBand
 {
     GDALColorTable *poCT;
     GDALColorInterp eInterp;
@@ -277,7 +290,10 @@ CPLErr LAN4BitRasterBand::SetColorTable( GDALColorTable *poNewCT )
 GDALColorTable *LAN4BitRasterBand::GetColorTable()
 
 {
-    return poCT;
+    if( poCT != NULL )
+        return poCT;
+    else
+        return GDALPamRasterBand::GetColorTable();
 }
 
 /************************************************************************/
@@ -325,8 +341,10 @@ LANDataset::LANDataset()
 LANDataset::~LANDataset()
 
 {
+    FlushCache();
+
     if( fpImage != NULL )
-        VSIFClose( fpImage );
+        VSIFCloseL( fpImage );
 
     CPLFree( pszProjection );
 }
@@ -359,8 +377,12 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Adopt the openinfo file pointer for use with this file.         */
 /* -------------------------------------------------------------------- */
-    poDS->fpImage = poOpenInfo->fp;
-    poOpenInfo->fp = NULL;
+    if( poOpenInfo->eAccess == GA_ReadOnly )
+        poDS->fpImage = VSIFOpenL( poOpenInfo->pszFilename, "rb" );
+    else
+        poDS->fpImage = VSIFOpenL( poOpenInfo->pszFilename, "rb+" );
+    if( poDS->fpImage == NULL )
+        return NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Do we need to byte swap the headers to local machine order?     */
@@ -457,7 +479,7 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
                                    * nPixelOffset * poDS->nRasterXSize,
                                    nPixelOffset, 
                                    poDS->nRasterXSize*nPixelOffset*nBandCount,
-                                   eDataType, FALSE ));
+                                   eDataType, FALSE, TRUE ));
     }
 
 /* -------------------------------------------------------------------- */
@@ -465,6 +487,12 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
 /* -------------------------------------------------------------------- */
 /*      Try to interprete georeferencing.                               */
 /* -------------------------------------------------------------------- */
@@ -525,15 +553,15 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
         CPLFormCIFilename( pszPath, pszBasename, "trl" );
     FILE *fpTRL;
 
-    fpTRL = VSIFOpen( pszTRLFilename, "rb" );
+    fpTRL = VSIFOpenL( pszTRLFilename, "rb" );
     if( fpTRL != NULL )
     {
         char szTRLData[896];
         int iColor;
         GDALColorTable *poCT;
 
-        VSIFRead( szTRLData, 1, 896, fpTRL );
-        VSIFClose( fpTRL );
+        VSIFReadL( szTRLData, 1, 896, fpTRL );
+        VSIFCloseL( fpTRL );
         
         poCT = new GDALColorTable();
         for( iColor = 0; iColor < 256; iColor++ )
@@ -545,6 +573,10 @@ GDALDataset *LANDataset::Open( GDALOpenInfo * poOpenInfo )
             sEntry.c3 = ((GByte *) szTRLData)[iColor+128+512];
             sEntry.c4 = 255;
             poCT->SetColorEntry( iColor, &sEntry );
+
+            // only 16 colors in 4bit files.
+            if( nPixelOffset == -1 && iColor == 15 )
+                break;
         }
 
         poDS->GetRasterBand(1)->SetColorTable( poCT );
