@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: mffdataset.cpp,v 1.27 2003/04/02 19:05:59 dron Exp $
+ * $Id: mffdataset.cpp,v 1.28 2005/05/05 13:55:42 fwarmerdam Exp $
  *
  * Project:  GView
  * Purpose:  Implementation of Atlantis MFF Support
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: mffdataset.cpp,v $
+ * Revision 1.28  2005/05/05 13:55:42  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.27  2003/04/02 19:05:59  dron
  * Large file support.
  *
@@ -115,7 +118,7 @@
 #include "ogr_spatialref.h"
 #include "atlsci_spheroid.h"
 
-CPL_CVSID("$Id: mffdataset.cpp,v 1.27 2003/04/02 19:05:59 dron Exp $");
+CPL_CVSID("$Id: mffdataset.cpp,v 1.28 2005/05/05 13:55:42 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_MFF(void);
@@ -1030,6 +1033,12 @@ GDALDataset *MFFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, poOpenInfo->pszFilename );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
 /* -------------------------------------------------------------------- */
 /*      Set all information from the .hdr that isn't well know to be    */
 /*      metadata.                                                       */
@@ -1630,6 +1639,8 @@ MFFDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         return NULL;
     }
 
+    poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
     return poDS;
 }
 
