@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: pauxdataset.cpp,v 1.29 2004/04/23 22:23:59 warmerda Exp $
+ * $Id: pauxdataset.cpp,v 1.30 2005/05/05 13:55:42 fwarmerdam Exp $
  *
  * Project:  PCI .aux Driver
  * Purpose:  Implementation of PAuxDataset
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: pauxdataset.cpp,v $
+ * Revision 1.30  2005/05/05 13:55:42  fwarmerdam
+ * PAM Enable
+ *
  * Revision 1.29  2004/04/23 22:23:59  warmerda
  * Fixed memory leak of pszAuxFilename.
  *
@@ -121,7 +124,7 @@
 #include "cpl_string.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: pauxdataset.cpp,v 1.29 2004/04/23 22:23:59 warmerda Exp $");
+CPL_CVSID("$Id: pauxdataset.cpp,v 1.30 2005/05/05 13:55:42 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_PAux(void);
@@ -655,7 +658,10 @@ const GDAL_GCP *PAuxDataset::GetGCPs()
 const char *PAuxDataset::GetProjectionRef()
 
 {
-    return pszProjection;
+    if( pszProjection )
+        return pszProjection;
+    else
+        return GDALPamDataset::GetProjectionRef();
 }
 
 /************************************************************************/
@@ -954,14 +960,18 @@ GDALDataset *PAuxDataset::Open( GDALOpenInfo * poOpenInfo )
     
     if( pszMapUnits != NULL )
         poDS->pszProjection = poDS->PCI2WKT( pszMapUnits, pszProjParms );
-    else
-        poDS->pszProjection = CPLStrdup("");
     
 /* -------------------------------------------------------------------- */
 /*      Check for overviews.                                            */
 /* -------------------------------------------------------------------- */
     poDS->oOvManager.Initialize( poDS, pszTarget );
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( pszTarget );
+    poDS->TryLoadXML();
+
     poDS->ScanForGCPs();
 
     CPLFree( pszTarget );
