@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: sdtsdataset.cpp,v 1.15 2003/02/06 03:18:49 warmerda Exp $
+ * $Id: sdtsdataset.cpp,v 1.16 2005/05/05 15:54:49 fwarmerdam Exp $
  *
  * Project:  SDTS Translator
  * Purpose:  GDALDataset driver for SDTS Raster translator.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: sdtsdataset.cpp,v $
+ * Revision 1.16  2005/05/05 15:54:49  fwarmerdam
+ * PAM Enabled
+ *
  * Revision 1.15  2003/02/06 03:18:49  warmerda
  * use Fixup() on SRS to set linear units
  *
@@ -76,10 +79,10 @@
  */
 
 #include "sdts_al.h"
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: sdtsdataset.cpp,v 1.15 2003/02/06 03:18:49 warmerda Exp $");
+CPL_CVSID("$Id: sdtsdataset.cpp,v 1.16 2005/05/05 15:54:49 fwarmerdam Exp $");
 
 /**
  \file sdtsdataset.cpp
@@ -99,7 +102,7 @@ CPL_C_END
 
 class SDTSRasterBand;
 
-class SDTSDataset : public GDALDataset
+class SDTSDataset : public GDALPamDataset
 {
     friend class SDTSRasterBand;
     
@@ -117,7 +120,7 @@ class SDTSDataset : public GDALDataset
     virtual CPLErr GetGeoTransform( double * );
 };
 
-class SDTSRasterBand : public GDALRasterBand
+class SDTSRasterBand : public GDALPamRasterBand
 {
     friend class SDTSDataset;
 
@@ -141,6 +144,8 @@ class SDTSRasterBand : public GDALRasterBand
 SDTSDataset::~SDTSDataset()
 
 {
+    FlushCache();
+
     if( poTransfer != NULL )
         delete poTransfer;
 
@@ -151,7 +156,6 @@ SDTSDataset::~SDTSDataset()
         CPLFree( pszProjection );
 }
 
-
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -277,6 +281,12 @@ GDALDataset *SDTSDataset::Open( GDALOpenInfo * poOpenInfo )
     if( oSRS.exportToWkt( &poDS->pszProjection ) != OGRERR_NONE )
         poDS->pszProjection = CPLStrdup("");
 
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
