@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: usgsdem_create.cpp,v 1.18 2004/12/17 19:50:03 fwarmerdam Exp $
+ * $Id: usgsdem_create.cpp,v 1.22 2005/05/05 15:54:49 fwarmerdam Exp $
  *
  * Project:  USGS DEM Driver
  * Purpose:  CreateCopy() implementation.
@@ -31,6 +31,19 @@
  ******************************************************************************
  * 
  * $Log: usgsdem_create.cpp,v $
+ * Revision 1.22  2005/05/05 15:54:49  fwarmerdam
+ * PAM Enabled
+ *
+ * Revision 1.21  2005/03/29 20:56:32  fwarmerdam
+ * Ensure that DataSpecVersion can be overridden even if CDED50K product
+ * is selected.
+ *
+ * Revision 1.20  2005/03/23 20:30:57  fwarmerdam
+ * Fixed "east" check for NTS option case.
+ *
+ * Revision 1.19  2005/03/23 17:41:42  fwarmerdam
+ * added NTS and INTERNALNAME options for Matt
+ *
  * Revision 1.18  2004/12/17 19:50:03  fwarmerdam
  * Fixed last bug reference.
  *
@@ -89,13 +102,13 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 #include "gdalwarper.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: usgsdem_create.cpp,v 1.18 2004/12/17 19:50:03 fwarmerdam Exp $");
+CPL_CVSID("$Id: usgsdem_create.cpp,v 1.22 2005/05/05 15:54:49 fwarmerdam Exp $");
 
 typedef struct 
 {
@@ -1011,13 +1024,30 @@ static int USGSDEMProductSetup_CDED50K( USGSDEMWriteInfo *psWInfo )
 /*      Fetch TOPLEFT location so we know what cell we are dealing      */
 /*      with.                                                           */
 /* -------------------------------------------------------------------- */
+    const char *pszNTS = 
+        CSLFetchNameValue( psWInfo->papszOptions, "NTS" );
     const char *pszTOPLEFT = CSLFetchNameValue( psWInfo->papszOptions, 
                                                 "TOPLEFT" );
     double dfULX = (psWInfo->dfULX+psWInfo->dfURX)*0.5;
     double dfULY = (psWInfo->dfULY+psWInfo->dfURY)*0.5;
 
+    // Have we been given an explicit NTS mapsheet name? 
+    if( pszNTS != NULL )
+    {
+        char szTrimmedTile[7];
+
+        strncpy( szTrimmedTile, pszNTS, 6 );
+        szTrimmedTile[6] = '\0';
+
+        if( !USGSDEM_LookupNTSByTile( szTrimmedTile, NULL, &dfULX, &dfULY ) )
+            return FALSE;
+
+        if( EQUALN(pszNTS+6,"e",1) )
+            dfULX += 0.25;
+    }
+
     // Try looking up TOPLEFT as a NTS mapsheet name.
-    if( pszTOPLEFT != NULL && strstr(pszTOPLEFT,",") == NULL
+    else if( pszTOPLEFT != NULL && strstr(pszTOPLEFT,",") == NULL
         && (strlen(pszTOPLEFT) == 6 || strlen(pszTOPLEFT) == 7) )
     {
         char szTrimmedTile[7];
@@ -1072,6 +1102,22 @@ static int USGSDEMProductSetup_CDED50K( USGSDEMWriteInfo *psWInfo )
         if( EQUALN(psWInfo->pszFilename+7,"e",1) )
             dfULX += 0.25;
     }
+             
+    else if( strlen(psWInfo->pszFilename) == 14 
+             && EQUALN(psWInfo->pszFilename+6,"DEM",3)
+             && EQUAL(psWInfo->pszFilename+10,".dem") )
+    {
+        char szTrimmedTile[7];
+
+        strncpy( szTrimmedTile, psWInfo->pszFilename, 6 );
+        szTrimmedTile[6] = '\0';
+
+        if( !USGSDEM_LookupNTSByTile( szTrimmedTile, NULL, &dfULX, &dfULY ) )
+            return FALSE;
+
+        if( EQUALN(psWInfo->pszFilename+9,"e",1) )
+            dfULX += 0.25;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Set resolution and size information.                            */
@@ -1121,6 +1167,8 @@ static int USGSDEMProductSetup_CDED50K( USGSDEMWriteInfo *psWInfo )
 /*      Can we find the NTS 50k tile name that corresponds with         */
 /*      this?                                                           */
 /* -------------------------------------------------------------------- */
+    const char *pszINTERNAL = 
+        CSLFetchNameValue( psWInfo->papszOptions, "INTERNALNAME" );
     char szTile[10];
     char chEWFlag = ' ';
 
@@ -1133,7 +1181,12 @@ static int USGSDEMProductSetup_CDED50K( USGSDEMWriteInfo *psWInfo )
         chEWFlag = 'e';
     }
 
-    if( chEWFlag != ' ' )
+    if( pszINTERNAL != NULL )
+    {
+        CPLFree( psWInfo->pszFilename );
+        psWInfo->pszFilename = CPLStrdup( pszINTERNAL );
+    }
+    else if( chEWFlag != ' ' )
     {
         CPLFree( psWInfo->pszFilename );
         psWInfo->pszFilename = 
@@ -1156,8 +1209,11 @@ static int USGSDEMProductSetup_CDED50K( USGSDEMWriteInfo *psWInfo )
 /* -------------------------------------------------------------------- */
     psWInfo->papszOptions = 
         CSLSetNameValue( psWInfo->papszOptions, "DEMLevelCode", "1" );
-    psWInfo->papszOptions = 
-        CSLSetNameValue( psWInfo->papszOptions, "DataSpecVersion", "1020" );
+
+    if( CSLFetchNameValue( psWInfo->papszOptions, "DataSpecVersion" ) == NULL )
+        psWInfo->papszOptions = 
+            CSLSetNameValue( psWInfo->papszOptions, "DataSpecVersion", 
+                             "1020" );
 
 /* -------------------------------------------------------------------- */
 /*      Set the destination coordinate system.                          */
@@ -1550,5 +1606,14 @@ USGSDEMCreateCopy( const char *pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
     USGSDEMWriteCleanup( &sWInfo );
 
-    return (GDALDataset *) GDALOpen( pszFilename, GA_ReadOnly );
+/* -------------------------------------------------------------------- */
+/*      Re-open dataset, and copy any auxilary pam information.         */
+/* -------------------------------------------------------------------- */
+    GDALPamDataset *poDS = (GDALPamDataset *) 
+        GDALOpen( pszFilename, GA_ReadOnly );
+
+    if( poDS )
+        poDS->CloneInfo( poSrcDS, GCIF_PAM_DEFAULT );
+
+    return poDS;
 }
