@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: usgsdemdataset.cpp,v 1.15 2004/04/08 12:59:10 warmerda Exp $
+ * $Id: usgsdemdataset.cpp,v 1.17 2005/05/05 15:54:49 fwarmerdam Exp $
  *
  * Project:  USGS DEM Driver
  * Purpose:  All reader for USGS DEM Reader
@@ -31,6 +31,12 @@
  ******************************************************************************
  * 
  * $Log: usgsdemdataset.cpp,v $
+ * Revision 1.17  2005/05/05 15:54:49  fwarmerdam
+ * PAM Enabled
+ *
+ * Revision 1.16  2005/04/15 19:28:57  fwarmerdam
+ * added AREA_OR_POINT=Point metadata
+ *
  * Revision 1.15  2004/04/08 12:59:10  warmerda
  * fixed sides of polygon fixed value
  *
@@ -78,10 +84,10 @@
  *
  */
 
-#include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: usgsdemdataset.cpp,v 1.15 2004/04/08 12:59:10 warmerda Exp $");
+CPL_CVSID("$Id: usgsdemdataset.cpp,v 1.17 2005/05/05 15:54:49 fwarmerdam Exp $");
 
 CPL_C_START
 void	GDALRegister_USGSDEM(void);
@@ -128,7 +134,7 @@ static double DConvert( FILE *fp, int nCharCount )
 
 class USGSDEMRasterBand;
 
-class USGSDEMDataset : public GDALDataset
+class USGSDEMDataset : public GDALPamDataset
 {
     friend class USGSDEMRasterBand;
 
@@ -161,7 +167,7 @@ class USGSDEMDataset : public GDALDataset
 /* ==================================================================== */
 /************************************************************************/
 
-class USGSDEMRasterBand : public GDALRasterBand
+class USGSDEMRasterBand : public GDALPamRasterBand
 {
     friend class USGSDEMDataset;
 
@@ -325,6 +331,8 @@ USGSDEMDataset::USGSDEMDataset()
 USGSDEMDataset::~USGSDEMDataset()
 
 {
+    FlushCache();
+
     CPLFree( pszProjection );
     if( fp != NULL )
         VSIFClose( fp );
@@ -611,6 +619,14 @@ GDALDataset *USGSDEMDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     poDS->SetBand( 1, new USGSDEMRasterBand( poDS ));
 
+    poDS->SetMetadataItem( GDALMD_AREA_OR_POINT, GDALMD_AOP_POINT );
+
+/* -------------------------------------------------------------------- */
+/*      Initialize any PAM information.                                 */
+/* -------------------------------------------------------------------- */
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->TryLoadXML();
+
     return( poDS );
 }
 
