@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vrtdataset.cpp,v 1.19 2004/10/03 16:52:45 fwarmerdam Exp $
+ * $Id: vrtdataset.cpp,v 1.22 2005/06/28 14:50:44 fwarmerdam Exp $
  *
  * Project:  Virtual GDAL Datasets
  * Purpose:  Implementation of VRTDataset
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: vrtdataset.cpp,v $
+ * Revision 1.22  2005/06/28 14:50:44  fwarmerdam
+ * avoid destroy psTree in case of failure - done at higher level
+ *
+ * Revision 1.21  2005/05/05 13:56:34  fwarmerdam
+ * moved metadata handling to PAM
+ *
+ * Revision 1.20  2005/04/11 17:08:36  fwarmerdam
+ * Fixed leak of xml tree when XMLInit() fails.
+ *
  * Revision 1.19  2004/10/03 16:52:45  fwarmerdam
  * ensure overviews initialized in Create()
  *
@@ -92,7 +101,7 @@
 #include "cpl_minixml.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: vrtdataset.cpp,v 1.19 2004/10/03 16:52:45 fwarmerdam Exp $");
+CPL_CVSID("$Id: vrtdataset.cpp,v 1.22 2005/06/28 14:50:44 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                            VRTDataset()                             */
@@ -244,7 +253,7 @@ CPLXMLNode *VRTDataset::SerializeToXML( const char *pszVRTPath )
     /* -------------------------------------------------------------------- */
     /*      Metadata                                                        */
     /* -------------------------------------------------------------------- */
-    psMD = VRTSerializeMetadata( this );
+    psMD = PamSerializeMetadata( this );
     if( psMD != NULL )
         CPLAddXMLChild( psDSTree, psMD );
 
@@ -412,7 +421,7 @@ CPLErr VRTDataset::XMLInit( CPLXMLNode *psTree, const char *pszVRTPath )
 /* -------------------------------------------------------------------- */
 /*      Apply any dataset level metadata.                               */
 /* -------------------------------------------------------------------- */
-    VRTApplyMetadata( psTree, this );
+    PamApplyMetadata( psTree, this );
 
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
@@ -447,7 +456,6 @@ CPLErr VRTDataset::XMLInit( CPLXMLNode *psTree, const char *pszVRTPath )
             }
             else
             {
-                CPLDestroyXMLNode( psTree );
                 if( poBand )
                     delete poBand; 
                 return CE_Failure;
@@ -703,7 +711,7 @@ GDALDataset *VRTDataset::OpenXML( const char *pszXML, const char *pszVRTPath )
     if( poDS->XMLInit( psTree, pszVRTPath ) != CE_None )
     {
         delete poDS;
-        return NULL;
+        poDS = NULL;
     }
 
 /* -------------------------------------------------------------------- */
@@ -885,70 +893,3 @@ VRTDataset::Create( const char * pszName,
     }
 }
 
-/************************************************************************/
-/*                          VRTApplyMetadata()                          */
-/************************************************************************/
-int VRTApplyMetadata( CPLXMLNode *psTree, GDALMajorObject *poMO )
-
-{
-    char **papszMD = NULL;
-    CPLXMLNode *psMetadata = CPLGetXMLNode( psTree, "Metadata" );
-    CPLXMLNode *psMDI;
-
-    if( psMetadata == NULL )
-        return FALSE;
-
-    /* Format is <MDI key="...">value_Text</MDI> */
-
-    for( psMDI = psMetadata->psChild; psMDI != NULL; psMDI = psMDI->psNext )
-    {
-        if( !EQUAL(psMDI->pszValue,"MDI") || psMDI->eType != CXT_Element 
-            || psMDI->psChild == NULL || psMDI->psChild->psNext == NULL 
-            || psMDI->psChild->eType != CXT_Attribute
-            || psMDI->psChild->psChild == NULL )
-            continue;
-
-        papszMD = 
-            CSLSetNameValue( papszMD, psMDI->psChild->psChild->pszValue, 
-                             psMDI->psChild->psNext->pszValue );
-    }
-
-    poMO->SetMetadata( papszMD );
-    CSLDestroy( papszMD );
-
-    return papszMD != NULL;
-}
-
-/************************************************************************/
-/*                        VRTSerializeMetadata()                        */
-/************************************************************************/
-
-CPLXMLNode *VRTSerializeMetadata( GDALMajorObject *poMO )
-
-{
-    char **papszMD = poMO->GetMetadata();
-
-    if( papszMD == NULL || CSLCount(papszMD) == 0 )
-        return NULL;
-
-    CPLXMLNode *psMD;
-
-    psMD = CPLCreateXMLNode( NULL, CXT_Element, "Metadata" );
-
-    for( int i = 0; papszMD[i] != NULL; i++ )
-    {
-        const char *pszRawValue;
-        char *pszKey;
-        CPLXMLNode *psMDI;
-
-        pszRawValue = CPLParseNameValue( papszMD[i], &pszKey );
-
-        psMDI = CPLCreateXMLNode( psMD, CXT_Element, "MDI" );
-        CPLSetXMLValue( psMDI, "#key", pszKey );
-        CPLCreateXMLNode( psMDI, CXT_Text, pszRawValue );
-
-        CPLFree( pszKey );
-    }
-
-    return psMD;
-}
