@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vrtdataset.h,v 1.18 2004/08/12 08:24:26 warmerda Exp $
+ * $Id: vrtdataset.h,v 1.20 2005/05/05 13:56:53 fwarmerdam Exp $
  *
  * Project:  Virtual GDAL Datasets
  * Purpose:  Declaration of virtual gdal dataset classes.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: vrtdataset.h,v $
+ * Revision 1.20  2005/05/05 13:56:53  fwarmerdam
+ * include gdal_pam.h
+ *
+ * Revision 1.19  2005/04/07 17:27:33  fwarmerdam
+ * Default path in OpenXML to NULL.
+ *
  * Revision 1.18  2004/08/12 08:24:26  warmerda
  * added overview support
  *
@@ -92,6 +98,7 @@
 #define VIRTUALDATASET_H_INCLUDED
 
 #include "gdal_priv.h"
+#include "gdal_pam.h"
 #include "cpl_minixml.h"
 
 CPL_C_START
@@ -172,7 +179,7 @@ class CPL_DLL VRTDataset : public GDALDataset
     virtual CPLErr      XMLInit( CPLXMLNode *, const char * );
  
     static GDALDataset *Open( GDALOpenInfo * );
-    static GDALDataset *OpenXML( const char *, const char * );
+    static GDALDataset *OpenXML( const char *, const char * = NULL );
     static GDALDataset *Create( const char * pszName,
                                 int nXSize, int nYSize, int nBands,
                                 GDALDataType eType, char ** papszOptions );
