@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdal_frmts.h,v 1.35 2005/01/06 20:27:51 fwarmerdam Exp $
+ * $Id: gdal_frmts.h,v 1.39 2005/08/17 15:33:46 fwarmerdam Exp $
  *
  * Project:  GDAL
  * Purpose:  Prototypes for all format specific driver initializations.
@@ -28,6 +28,18 @@
  ******************************************************************************
  *
  * $Log: gdal_frmts.h,v $
+ * Revision 1.39  2005/08/17 15:33:46  fwarmerdam
+ * added rik driver
+ *
+ * Revision 1.38  2005/07/12 16:58:06  denad21
+ * added hdf5 support
+ *
+ * Revision 1.37  2005/07/07 23:36:14  fwarmerdam
+ * added msgn support
+ *
+ * Revision 1.36  2005/05/19 20:43:08  dron
+ * Added RMF format.
+ *
  * Revision 1.35  2005/01/06 20:27:51  fwarmerdam
  * added NDF driver
  *
@@ -202,6 +214,11 @@ void CPL_DLL GDALRegister_ILWIS(void);
 void CPL_DLL GDALRegister_PCRaster(void);
 void CPL_DLL GDALRegister_IDA(void);
 void CPL_DLL GDALRegister_NDF(void);
+void CPL_DLL GDALRegister_RMF(void);
+void CPL_DLL GDALRegister_HDF5(void);
+void CPL_DLL GDALRegister_HDF5Image(void);
+void CPL_DLL GDALRegister_MSGN(void);
+void CPL_DLL GDALRegister_RIK(void);
 CPL_C_END
 
 #endif /* ndef GDAL_FRMTS_H_INCLUDED */
