@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdal_misc.cpp,v 1.63 2005/02/10 04:30:29 fwarmerdam Exp $
+ * $Id: gdal_misc.cpp,v 1.73 2005/09/26 15:52:03 fwarmerdam Exp $
  *
  * Project:  GDAL Core
  * Purpose:  Free standing functions for GDAL.
@@ -28,6 +28,38 @@
  ******************************************************************************
  *
  * $Log: gdal_misc.cpp,v $
+ * Revision 1.73  2005/09/26 15:52:03  fwarmerdam
+ * centralized .aux opening logic
+ *
+ * Revision 1.72  2005/09/11 21:07:54  fwarmerdam
+ * Changed worldfile reading to use large file API.
+ *
+ * Revision 1.71  2005/09/11 19:16:31  fwarmerdam
+ * Use CPLString for safer string manipulation.
+ *
+ * Revision 1.70  2005/07/15 13:28:00  fwarmerdam
+ * Fixed another big raster int overflow problem in GDALGetRasterSampleOverview
+ *
+ * Revision 1.69  2005/07/13 17:21:10  fwarmerdam
+ * Avoiding 32bit overflow in GDALGetRasterSampleOverview().
+ *
+ * Revision 1.68  2005/07/08 18:59:54  fwarmerdam
+ * Don't use thread local storage for GDALTermProgress() last complete, or
+ * the GDALVersionInfo() static buffer.
+ *
+ * Revision 1.67  2005/05/23 06:44:31  fwarmerdam
+ * Updated for locking of block refs
+ *
+ * Revision 1.66  2005/04/04 15:24:48  fwarmerdam
+ * Most C entry points now CPL_STDCALL
+ *
+ * Revision 1.65  2005/03/21 16:12:08  fwarmerdam
+ * Trimmed log.
+ *
+ * Revision 1.64  2005/03/21 16:11:32  fwarmerdam
+ * Added special case for simple 4 corners, non-rotated case in
+ * GDALGCPsToGeoTransform().
+ *
  * Revision 1.63  2005/02/10 04:30:29  fwarmerdam
  * added support for YCbCr color space
  *
@@ -69,100 +101,15 @@
  *
  * Revision 1.50  2004/01/18 16:43:37  dron
  * Added GDALGetDataTypeByName() function.
- *
- * Revision 1.49  2003/08/18 12:43:47  warmerda
- * always include ogr_spatialref.h
- *
- * Revision 1.48  2003/08/12 22:13:32  warmerda
- * Changed GDALReadTabFile(0 so that if the CoordSys results in a PROJCS, but
- * the UNITS keyword is set to "degrees" we just use the GEOGCS portion.  It
- * seems that the projection is "just for display", and the GCPs will actually
- * be in lat/long.
- *
- * Revision 1.47  2003/06/03 19:44:00  warmerda
- * added GDALRPCInfo support
- *
- * Revision 1.46  2003/05/23 15:52:54  warmerda
- * Cosmetic changes made.
- *
- * Revision 1.45  2003/05/21 04:31:53  warmerda
- * avoid warnings
- *
- * Revision 1.44  2003/04/30 17:13:48  warmerda
- * added docs for many C functions
- *
- * Revision 1.43  2003/03/13 14:37:17  warmerda
- * better end of range checking in GDALTermProgress
- *
- * Revision 1.42  2003/02/15 20:22:14  warmerda
- * GDALReadTabFile() returns true if it gets GCPs but cant make geotransform
- *
- * Revision 1.41  2003/01/27 21:55:52  warmerda
- * various documentation improvements
- *
- * Revision 1.40  2002/12/11 21:21:46  warmerda
- * fixed debug format problem
- *
- * Revision 1.39  2002/12/09 20:05:31  warmerda
- * fixed return flag from GDALReadTabFile
- *
- * Revision 1.38  2002/12/09 18:53:25  warmerda
- * GDALDecToDMS() now calls CPLDecToDMS()
- *
- * Revision 1.37  2002/12/05 17:55:30  warmerda
- * gdalreadtabfile should not be static
- *
- * Revision 1.36  2002/12/05 15:46:38  warmerda
- * added GDALReadTabFile()
- *
- * Revision 1.35  2002/07/09 20:33:12  warmerda
- * expand tabs
- *
- * Revision 1.34  2002/05/06 21:37:29  warmerda
- * added GDALGCPsToGeoTransform
- *
- * Revision 1.33  2002/04/25 16:18:41  warmerda
- * added extra checking
- *
- * Revision 1.32  2002/04/24 19:21:26  warmerda
- * Include <ctype.h> for toupper(), tolower().
- *
- * Revision 1.31  2002/04/19 12:22:05  dron
- * added GDALWriteWorldFile()
- *
- * Revision 1.30  2002/04/16 13:59:33  warmerda
- * added GDALVersionInfo
- *
- * Revision 1.29  2001/12/07 20:04:21  warmerda
- * fixed serious bug in random sampler
- *
- * Revision 1.28  2001/11/30 03:41:26  warmerda
- * Fixed bug with the block sampling rate being too low to satisfy large
- * sample count values.  Fixed bug with tiled images including some uninitialized
- * or zero data in the sample set on partial edge tiles.
- *
- * Revision 1.27  2001/11/26 20:14:01  warmerda
- * added GDALProjDef stubs for old 'bridges'
- *
- * Revision 1.26  2001/11/19 16:03:16  warmerda
- * moved GDALDectoDMS here
- *
- * Revision 1.25  2001/08/15 15:05:44  warmerda
- * return magnitude for complex samples in random sampler
- *
- * Revision 1.24  2001/07/18 04:04:30  warmerda
- * added CPL_CVSID
- *
- * Revision 1.23  2001/05/01 18:09:25  warmerda
- * added GDALReadWorldFile()
  */
 
 #include "gdal_priv.h"
 #include "cpl_string.h"
 #include "cpl_minixml.h"
 #include <ctype.h>
+#include <string>
 
-CPL_CVSID("$Id: gdal_misc.cpp,v 1.63 2005/02/10 04:30:29 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdal_misc.cpp,v 1.73 2005/09/26 15:52:03 fwarmerdam Exp $");
 
 #include "ogr_spatialref.h"
 
@@ -202,7 +149,8 @@ void __pure_virtual()
  * @return a data type able to express eType1 and eType2.
  */
 
-GDALDataType GDALDataTypeUnion( GDALDataType eType1, GDALDataType eType2 )
+GDALDataType CPL_STDCALL 
+GDALDataTypeUnion( GDALDataType eType1, GDALDataType eType2 )
 
 {
     int         bFloating, bComplex, nBits, bSigned;
@@ -343,7 +291,7 @@ GDALDataType GDALDataTypeUnion( GDALDataType eType1, GDALDataType eType2 )
  * @return the number of bits or zero if it is not recognised.
  */
 
-int GDALGetDataTypeSize( GDALDataType eDataType )
+int CPL_STDCALL GDALGetDataTypeSize( GDALDataType eDataType )
 
 {
     switch( eDataType )
@@ -387,7 +335,7 @@ int GDALGetDataTypeSize( GDALDataType eDataType )
  * component. 
  */
 
-int GDALDataTypeIsComplex( GDALDataType eDataType )
+int CPL_STDCALL GDALDataTypeIsComplex( GDALDataType eDataType )
 
 {
     switch( eDataType )
@@ -420,7 +368,7 @@ int GDALDataTypeIsComplex( GDALDataType eDataType )
  * @return string corresponding to type.
  */
 
-const char *GDALGetDataTypeName( GDALDataType eDataType )
+const char * CPL_STDCALL GDALGetDataTypeName( GDALDataType eDataType )
 
 {
     switch( eDataType )
@@ -481,7 +429,7 @@ const char *GDALGetDataTypeName( GDALDataType eDataType )
  * @return GDAL data type.
  */
 
-GDALDataType GDALGetDataTypeByName( const char *pszName )
+GDALDataType CPL_STDCALL GDALGetDataTypeByName( const char *pszName )
 
 {
     int	iType;
@@ -611,8 +559,9 @@ const char *GDALGetColorInterpretationName( GDALColorInterp eInterp )
  * maximum (adfMinMax[1]) are returned.
  */
 
-void GDALComputeRasterMinMax( GDALRasterBandH hBand, int bApproxOK, 
-                              double adfMinMax[2] )
+void CPL_STDCALL 
+GDALComputeRasterMinMax( GDALRasterBandH hBand, int bApproxOK, 
+                         double adfMinMax[2] )
 
 {
     double       dfMin=0.0, dfMax=0.0;
@@ -677,7 +626,7 @@ void GDALComputeRasterMinMax( GDALRasterBandH hBand, int bApproxOK,
         iYBlock = iSampleBlock / nBlocksPerRow;
         iXBlock = iSampleBlock - nBlocksPerRow * iYBlock;
         
-        poBlock = poBand->GetBlockRef( iXBlock, iYBlock );
+        poBlock = poBand->GetLockedBlockRef( iXBlock, iYBlock );
         if( poBlock == NULL )
             continue;
         
@@ -752,6 +701,8 @@ void GDALComputeRasterMinMax( GDALRasterBandH hBand, int bApproxOK,
                 }
             }
         }
+
+        poBlock->DropLock();
     }
 
     adfMinMax[0] = dfMin;
@@ -771,7 +722,7 @@ void GDALComputeRasterMinMax( GDALRasterBandH hBand, int bApproxOK,
  * to use one of the other progress functions that actually do something.
  */
 
-int GDALDummyProgress( double, const char *, void * )
+int CPL_STDCALL GDALDummyProgress( double, const char *, void * )
 
 {
     return TRUE;
@@ -794,8 +745,8 @@ typedef struct {
  * callback data returned by GDALCreateScaledProgress().
  */
 
-int GDALScaledProgress( double dfComplete, const char *pszMessage, 
-                        void *pData )
+int CPL_STDCALL GDALScaledProgress( double dfComplete, const char *pszMessage, 
+                                    void *pData )
 
 {
     GDALScaledProgressInfo *psInfo = (GDALScaledProgressInfo *) pData;
@@ -853,7 +804,7 @@ int GDALScaledProgress( double dfComplete, const char *pszMessage,
  * \endcode
  */
 
-void *GDALCreateScaledProgress( double dfMin, double dfMax, 
+void * CPL_STDCALL GDALCreateScaledProgress( double dfMin, double dfMax, 
                                 GDALProgressFunc pfnProgress, 
                                 void * pData )
 
@@ -887,7 +838,7 @@ void *GDALCreateScaledProgress( double dfMin, double dfMax,
  * @param pData scaled progress handle returned by GDALCreateScaledProgress().
  */
 
-void GDALDestroyScaledProgress( void * pData )
+void CPL_STDCALL GDALDestroyScaledProgress( void * pData )
 
 {
     CPLFree( pData );
@@ -913,6 +864,11 @@ void GDALDestroyScaledProgress( void * pData )
  * percentage reported and will get confused if two terminal based progress
  * reportings are active at the same time.
  *
+ * The GDALTermProgress() function maintains an internal memory of the 
+ * last percentage complete reported in a static variable, and this makes
+ * it unsuitable to have multiple GDALTermProgress()'s active eithin a 
+ * single thread or across multiple threads.
+ *
  * @param dfComplete completion ratio from 0.0 to 1.0.
  * @param pszMessage optional message.
  * @param pProgressArg ignored callback data argument. 
@@ -920,7 +876,7 @@ void GDALDestroyScaledProgress( void * pData )
  * @return Always returns TRUE indicating the process should continue.
  */
 
-int GDALTermProgress( double dfComplete, const char *pszMessage, 
+int CPL_STDCALL GDALTermProgress( double dfComplete, const char *pszMessage, 
                       void * pProgressArg )
 
 {
@@ -981,29 +937,30 @@ int GDALTermProgress( double dfComplete, const char *pszMessage,
  * @return optimal overview or hBand itself. 
  */
 
-GDALRasterBandH GDALGetRasterSampleOverview( GDALRasterBandH hBand, 
-                                             int nDesiredSamples )
+GDALRasterBandH CPL_STDCALL 
+GDALGetRasterSampleOverview( GDALRasterBandH hBand, 
+                             int nDesiredSamples )
 
 {
-    int     nBestSamples; 
+    double dfBestSamples; 
     GDALRasterBandH hBestBand = hBand;
 
-    nBestSamples = GDALGetRasterBandXSize(hBand) 
-        * GDALGetRasterBandYSize(hBand);
+    dfBestSamples = GDALGetRasterBandXSize(hBand) 
+        * (double) GDALGetRasterBandYSize(hBand);
 
     for( int iOverview = 0; 
          iOverview < GDALGetOverviewCount( hBand );
          iOverview++ )
     {
         GDALRasterBandH hOBand = GDALGetOverview( hBand, iOverview );
-        int    nOSamples;
+        double    dfOSamples;
 
-        nOSamples = GDALGetRasterBandXSize(hOBand) 
-            * GDALGetRasterBandYSize(hOBand);
+        dfOSamples = GDALGetRasterBandXSize(hOBand) 
+            * (double) GDALGetRasterBandYSize(hOBand);
 
-        if( nOSamples < nBestSamples && nOSamples > nDesiredSamples )
+        if( dfOSamples < dfBestSamples && dfOSamples > nDesiredSamples )
         {
-            nBestSamples = nOSamples;
+            dfBestSamples = dfOSamples;
             hBestBand = hOBand;
         }
     }
@@ -1015,8 +972,9 @@ GDALRasterBandH GDALGetRasterSampleOverview( GDALRasterBandH hBand,
 /*                     GDALGetRandomRasterSample()                      */
 /************************************************************************/
 
-int GDALGetRandomRasterSample( GDALRasterBandH hBand, int nSamples, 
-                               float *pafSampleBuf )
+int CPL_STDCALL 
+GDALGetRandomRasterSample( GDALRasterBandH hBand, int nSamples, 
+                           float *pafSampleBuf )
 
 {
     GDALRasterBand *poBand;
@@ -1079,7 +1037,7 @@ int GDALGetRandomRasterSample( GDALRasterBandH hBand, int nSamples,
         iYBlock = iSampleBlock / nBlocksPerRow;
         iXBlock = iSampleBlock - nBlocksPerRow * iYBlock;
 
-        poBlock = poBand->GetBlockRef( iXBlock, iYBlock );
+        poBlock = poBand->GetLockedBlockRef( iXBlock, iYBlock );
         if( poBlock == NULL )
             continue;
 
@@ -1156,6 +1114,8 @@ int GDALGetRandomRasterSample( GDALRasterBandH hBand, int nSamples,
 
             iRemainder = iX - iXValid;
         }
+
+        poBlock->DropLock();
     }
 
     return nActualSamples;
@@ -1165,7 +1125,7 @@ int GDALGetRandomRasterSample( GDALRasterBandH hBand, int nSamples,
 /*                            GDALInitGCPs()                            */
 /************************************************************************/
 
-void GDALInitGCPs( int nCount, GDAL_GCP * psGCP )
+void CPL_STDCALL GDALInitGCPs( int nCount, GDAL_GCP * psGCP )
 
 {
     for( int iGCP = 0; iGCP < nCount; iGCP++ )
@@ -1181,7 +1141,7 @@ void GDALInitGCPs( int nCount, GDAL_GCP * psGCP )
 /*                           GDALDeinitGCPs()                           */
 /************************************************************************/
 
-void GDALDeinitGCPs( int nCount, GDAL_GCP * psGCP )
+void CPL_STDCALL GDALDeinitGCPs( int nCount, GDAL_GCP * psGCP )
 
 {
     for( int iGCP = 0; iGCP < nCount; iGCP++ )
@@ -1196,7 +1156,8 @@ void GDALDeinitGCPs( int nCount, GDAL_GCP * psGCP )
 /*                         GDALDuplicateGCPs()                          */
 /************************************************************************/
 
-GDAL_GCP *GDALDuplicateGCPs( int nCount, const GDAL_GCP *pasGCPList )
+GDAL_GCP * CPL_STDCALL 
+GDALDuplicateGCPs( int nCount, const GDAL_GCP *pasGCPList )
 
 {
     GDAL_GCP    *pasReturn;
@@ -1231,9 +1192,9 @@ GDAL_GCP *GDALDuplicateGCPs( int nCount, const GDAL_GCP *pasGCPList )
 
 #define MAX_GCP 256
  
-int GDALReadTabFile( const char * pszBaseFilename, 
-                     double *padfGeoTransform, char **ppszWKT, 
-                     int *pnGCPCount, GDAL_GCP **ppasGCPs )
+int CPL_STDCALL GDALReadTabFile( const char * pszBaseFilename, 
+                                 double *padfGeoTransform, char **ppszWKT, 
+                                 int *pnGCPCount, GDAL_GCP **ppasGCPs )
 
 
 {
@@ -1434,14 +1395,14 @@ int GDALReadTabFile( const char * pszBaseFilename,
  * @return TRUE on success or FALSE on failure.
  */
 
-int GDALReadWorldFile( const char * pszBaseFilename, const char *pszExtension,
-                       double *padfGeoTransform )
+int CPL_STDCALL 
+GDALReadWorldFile( const char * pszBaseFilename, const char *pszExtension,
+                   double *padfGeoTransform )
 
 {
     const char  *pszTFW;
     char        szExtUpper[32], szExtLower[32];
     int         i;
-    FILE        *fpTFW;
     char        **papszLines;
 
 /* -------------------------------------------------------------------- */
@@ -1451,14 +1412,14 @@ int GDALReadWorldFile( const char * pszBaseFilename, const char *pszExtension,
     if( pszExtension == NULL )
     {
         char szDerivedExtension[100];
-        const char *pszBaseExt = CPLGetExtension( pszBaseFilename );
+        std::string  oBaseExt = CPLGetExtension( pszBaseFilename );
 
-        if( strlen(pszBaseExt) < 2 )
+        if( oBaseExt.length() < 2 )
             return FALSE;
 
         // windows version - first + last + 'w'
-        szDerivedExtension[0] = pszBaseExt[0];
-        szDerivedExtension[1] = pszBaseExt[strlen(pszBaseExt)-1];
+        szDerivedExtension[0] = oBaseExt[0];
+        szDerivedExtension[1] = oBaseExt[oBaseExt.length()-1];
         szDerivedExtension[2] = 'w';
         szDerivedExtension[3] = '\0';
         
@@ -1467,10 +1428,10 @@ int GDALReadWorldFile( const char * pszBaseFilename, const char *pszExtension,
             return TRUE;
 
         // unix version - extension + 'w'
-        if( strlen(pszBaseExt) > sizeof(szDerivedExtension)-2 )
+        if( oBaseExt.length() > sizeof(szDerivedExtension)-2 )
             return FALSE;
 
-        strcpy( szDerivedExtension, pszBaseExt );
+        strcpy( szDerivedExtension, oBaseExt.c_str() );
         strcat( szDerivedExtension, "w" );
         return GDALReadWorldFile( pszBaseFilename, szDerivedExtension, 
                                   padfGeoTransform );
@@ -1497,27 +1458,29 @@ int GDALReadWorldFile( const char * pszBaseFilename, const char *pszExtension,
 /* -------------------------------------------------------------------- */
 /*      Try lower case, then upper case.                                */
 /* -------------------------------------------------------------------- */
+    VSIStatBufL sStatBuf;
+    int bGotTFW;
+
     pszTFW = CPLResetExtension( pszBaseFilename, szExtLower );
 
-    fpTFW = VSIFOpen( pszTFW, "rt" );
+    bGotTFW = VSIStatL( pszTFW, &sStatBuf ) == 0;
 
 #ifndef WIN32
-    if( fpTFW == NULL )
+    if( !bGotTFW )
     {
         pszTFW = CPLResetExtension( pszBaseFilename, szExtUpper );
-        fpTFW = VSIFOpen( pszTFW, "rt" );
+        bGotTFW = VSIStatL( pszTFW, &sStatBuf ) == 0;
     }
 #endif
     
-    if( fpTFW == NULL )
+    if( !bGotTFW )
         return FALSE;
 
-    VSIFClose( fpTFW );
-
 /* -------------------------------------------------------------------- */
 /*      We found the file, now load and parse it.                       */
 /* -------------------------------------------------------------------- */
     papszLines = CSLLoad( pszTFW );
+
     if( CSLCount(papszLines) >= 6 
         && atof(papszLines[0]) != 0.0
         && atof(papszLines[3]) != 0.0 )
@@ -1556,33 +1519,43 @@ int GDALReadWorldFile( const char * pszBaseFilename, const char *pszExtension,
 /*      support for ESRI world files.                                   */
 /************************************************************************/
 
-int GDALWriteWorldFile( const char * pszBaseFilename, const char *pszExtension,
-                       double *padfGeoTransform )
+int CPL_STDCALL 
+GDALWriteWorldFile( const char * pszBaseFilename, const char *pszExtension,
+                    double *padfGeoTransform )
 
 {
+
+/* -------------------------------------------------------------------- */
+/*      Prepare the text to write to the file.                          */
+/* -------------------------------------------------------------------- */
+    CPLString osTFWText;
+
+    osTFWText.Printf( "%.10f\n%.10f\n%.10f\n%.10f\n%.10f\n%.10f\n", 
+                      padfGeoTransform[1],
+                      padfGeoTransform[4],
+                      padfGeoTransform[2],
+                      padfGeoTransform[5],
+                      padfGeoTransform[0] 
+                      + 0.5 * padfGeoTransform[1]
+                      + 0.5 * padfGeoTransform[2],
+                      padfGeoTransform[3]
+                      + 0.5 * padfGeoTransform[4]
+                      + 0.5 * padfGeoTransform[5] );
+
+/* -------------------------------------------------------------------- */
+/*      Update extention, and write to disk.                            */
+/* -------------------------------------------------------------------- */
     const char  *pszTFW;
     FILE    *fpTFW;
 
     pszTFW = CPLResetExtension( pszBaseFilename, pszExtension );
-    fpTFW = VSIFOpen( pszTFW, "wt" );
+    fpTFW = VSIFOpenL( pszTFW, "wt" );
     if( fpTFW == NULL )
         return FALSE;
 
-/* -------------------------------------------------------------------- */
-/*      We open the file, now fill it with the world data.              */
-/* -------------------------------------------------------------------- */
-    fprintf( fpTFW, "%.10f\n", padfGeoTransform[1] );
-    fprintf( fpTFW, "%.10f\n", padfGeoTransform[4] );
-    fprintf( fpTFW, "%.10f\n", padfGeoTransform[2] );
-    fprintf( fpTFW, "%.10f\n", padfGeoTransform[5] );
-    fprintf( fpTFW, "%.10f\n", padfGeoTransform[0] 
-             + 0.5 * padfGeoTransform[1]
-             + 0.5 * padfGeoTransform[2] );
-    fprintf( fpTFW, "%.10f\n", padfGeoTransform[3]
-             + 0.5 * padfGeoTransform[4]
-             + 0.5 * padfGeoTransform[5] );
+    VSIFWriteL( (void *) osTFWText.c_str(), 1, osTFWText.size(), fpTFW );
+    VSIFCloseL( fpTFW );
 
-    VSIFClose( fpTFW );
     return TRUE;
 }
 
@@ -1608,11 +1581,15 @@ int GDALWriteWorldFile( const char * pszBaseFilename, const char *pszExtension,
  * @return an internal string containing the requested information.
  */
 
-const char *GDALVersionInfo( const char *pszRequest )
+const char * CPL_STDCALL GDALVersionInfo( const char *pszRequest )
 
 {
-    static char szResult[128];
+    // NOTE: There is a slight risk of a multithreaded race condition if
+    // one thread is in the process of sprintf()ing into this buffer while
+    // another is using it but that seems pretty low risk.  All threads
+    // want the same value in the buffer.
 
+    static char szResult[128];
     
     if( pszRequest == NULL || EQUAL(pszRequest,"VERSION_NUM") )
         sprintf( szResult, "%d", GDAL_VERSION_NUM );
@@ -1637,7 +1614,7 @@ const char *GDALVersionInfo( const char *pszRequest )
 /*      hemisphere.                                                     */
 /************************************************************************/
 
-const char *GDALDecToDMS( double dfAngle, const char * pszAxis,
+const char * CPL_STDCALL GDALDecToDMS( double dfAngle, const char * pszAxis,
                           int nPrecision )
 
 {
@@ -1654,7 +1631,7 @@ const char *GDALDecToDMS( double dfAngle, const char * pszAxis,
  * See CPLPackedDMSToDec().
  */
 
-double GDALPackedDMSToDec( double dfPacked )
+double CPL_STDCALL GDALPackedDMSToDec( double dfPacked )
 
 {
     return CPLPackedDMSToDec( dfPacked );
@@ -1670,7 +1647,7 @@ double GDALPackedDMSToDec( double dfPacked )
  * See CPLDecToPackedDMS().
  */
 
-double GDALDecToPackedDMS( double dfDec )
+double CPL_STDCALL GDALDecToPackedDMS( double dfDec )
 
 {
     return CPLDecToPackedDMS( dfDec );
@@ -1684,6 +1661,12 @@ double GDALDecToPackedDMS( double dfDec )
  * Generate Geotransform from GCPs. 
  *
  * Given a set of GCPs perform first order fit as a geotransform. 
+ *
+ * Due to imprecision in the calculations the fit algorithm will often 
+ * return non-zero rotational coefficients even if given perfectly non-rotated
+ * inputs.  A special case has been implemented for corner corner coordinates
+ * given in TL, TR, BR, BL order.  So when using this to get a geotransform
+ * from 4 corner coordinates, pass them in this order. 
  * 
  * @param nGCPCount the number of GCPs being passed in.
  * @param pasGCPs the list of GCP structures. 
@@ -1697,8 +1680,9 @@ double GDALDecToPackedDMS( double dfDec )
  * and the fit is poor.
  */
 
-int GDALGCPsToGeoTransform( int nGCPCount, const GDAL_GCP *pasGCPs,
-                            double *padfGeoTransform, int bApproxOK )
+int CPL_STDCALL 
+GDALGCPsToGeoTransform( int nGCPCount, const GDAL_GCP *pasGCPs,
+                        double *padfGeoTransform, int bApproxOK )
 
 {
     int    i;
@@ -1734,9 +1718,43 @@ int GDALGCPsToGeoTransform( int nGCPCount, const GDAL_GCP *pasGCPs,
         return TRUE;
     }
 
+/* -------------------------------------------------------------------- */
+/*      Special case of 4 corner coordinates of a non-rotated           */
+/*      image.  The points must be in TL-TR-BR-BL order for now.        */
+/*      This case helps avoid some imprecision in the general           */
+/*      calcuations.                                                    */
+/* -------------------------------------------------------------------- */
+    if( nGCPCount == 4 
+        && pasGCPs[0].dfGCPLine == pasGCPs[1].dfGCPLine
+        && pasGCPs[2].dfGCPLine == pasGCPs[3].dfGCPLine
+        && pasGCPs[0].dfGCPPixel == pasGCPs[3].dfGCPPixel
+        && pasGCPs[1].dfGCPPixel == pasGCPs[2].dfGCPPixel
+        && pasGCPs[0].dfGCPLine != pasGCPs[2].dfGCPLine
+        && pasGCPs[0].dfGCPPixel != pasGCPs[1].dfGCPPixel 
+        && pasGCPs[0].dfGCPY == pasGCPs[1].dfGCPY
+        && pasGCPs[2].dfGCPY == pasGCPs[3].dfGCPY
+        && pasGCPs[0].dfGCPX == pasGCPs[3].dfGCPX
+        && pasGCPs[1].dfGCPX == pasGCPs[2].dfGCPX
+        && pasGCPs[0].dfGCPY != pasGCPs[2].dfGCPY
+        && pasGCPs[0].dfGCPX != pasGCPs[1].dfGCPX )
+    {
+        padfGeoTransform[1] = (pasGCPs[1].dfGCPX - pasGCPs[0].dfGCPX)
+            / (pasGCPs[1].dfGCPPixel - pasGCPs[0].dfGCPPixel);
+        padfGeoTransform[2] = 0.0;
+        padfGeoTransform[4] = 0.0;
+        padfGeoTransform[5] = (pasGCPs[2].dfGCPY - pasGCPs[1].dfGCPY)
+            / (pasGCPs[2].dfGCPLine - pasGCPs[1].dfGCPLine);
+
+        padfGeoTransform[0] = 
+            pasGCPs[0].dfGCPX - pasGCPs[0].dfGCPPixel * padfGeoTransform[1];
+        padfGeoTransform[3] = 
+            pasGCPs[0].dfGCPY - pasGCPs[0].dfGCPLine * padfGeoTransform[5];
+        return TRUE;
+    }
+
 /* -------------------------------------------------------------------- */
 /* In the general case, do a least squares error approximation by       */
-/* solving the equation Sum[(A - B*x + C*y - Lon)^2] = minimum			*/
+/* solving the equation Sum[(A - B*x + C*y - Lon)^2] = minimum		*/
 /* -------------------------------------------------------------------- */
 	
     double sum_x = 0.0, sum_y = 0.0, sum_xy = 0.0, sum_xx = 0.0, sum_yy = 0.0;
@@ -1881,7 +1899,8 @@ int GDALGCPsToGeoTransform( int nGCPCount, const GDAL_GCP *pasGCPs,
  * without error, return of -1 requests exit with error code.
  */
 
-int GDALGeneralCmdLineProcessor( int nArgc, char ***ppapszArgv, int nOptions )
+int CPL_STDCALL 
+GDALGeneralCmdLineProcessor( int nArgc, char ***ppapszArgv, int nOptions )
 
 {
     char **papszReturn = NULL;
@@ -2173,7 +2192,7 @@ static int _FetchDblFromMD( char **papszMD, const char *pszKey,
 /*                         GDALExtractRPCInfo()                         */
 /************************************************************************/
 
-int GDALExtractRPCInfo( char **papszMD, GDALRPCInfo *psRPC )
+int CPL_STDCALL GDALExtractRPCInfo( char **papszMD, GDALRPCInfo *psRPC )
 
 {
     if( CSLFetchNameValue( papszMD, "RPC_LINE_NUM_COEFF" ) == NULL )
@@ -2217,6 +2236,68 @@ int GDALExtractRPCInfo( char **papszMD, GDALRPCInfo *psRPC )
     return TRUE;
 }
 
+/************************************************************************/
+/*                     GDALFindAssociatedAuxFile()                      */
+/************************************************************************/
+
+GDALDataset *GDALFindAssociatedAuxFile( const char *pszBasename,
+                                        GDALAccess eAccess )
+
+{
+    if( EQUAL(CPLGetExtension(pszBasename),"aux") )
+        return NULL;
+
+/* -------------------------------------------------------------------- */
+/*      We didn't find that, so try and find a corresponding aux        */
+/*      file.  Check that we are the dependent file of the aux          */
+/*      file.                                                           */
+/* -------------------------------------------------------------------- */
+    VSIStatBufL sStatBuf;
+    CPLString oAuxFilename = CPLResetExtension(pszBasename,"aux");
+    CPLString oJustFile = CPLGetFilename(pszBasename); // without dir
+    GDALDataset *poODS = NULL;
+        
+    if( VSIStatL( oAuxFilename, &sStatBuf ) == 0 )
+    {
+        poODS = (GDALDataset *) GDALOpenShared( oAuxFilename, eAccess );
+    }
+
+    if( poODS != NULL )
+    {
+        const char *pszDep
+            = poODS->GetMetadataItem( "HFA_DEPENDENT_FILE", "HFA" );
+        if( pszDep == NULL || !EQUAL(pszDep,oJustFile) )
+        {
+            GDALClose( poODS );
+            poODS = NULL;
+        }
+    }
+        
+    if( poODS == NULL )
+    {
+        oAuxFilename = pszBasename;
+        oAuxFilename += ".aux";
+
+        if( VSIStatL( oAuxFilename, &sStatBuf ) == 0 )
+        {
+            poODS = (GDALDataset *) GDALOpen( oAuxFilename, eAccess );
+        }
+
+        if( poODS != NULL )
+        {
+            const char *pszDep
+                = poODS->GetMetadataItem( "HFA_DEPENDENT_FILE", "HFA" );
+            if( pszDep == NULL || !EQUAL(pszDep,oJustFile) )
+            {
+                GDALClose( poODS );
+                poODS = NULL;
+            }
+        }
+    }
+
+    return poODS;
+}
+
 /************************************************************************/
 /* -------------------------------------------------------------------- */
 /*      The following stubs are present to ensure that older GDAL       */
@@ -2226,25 +2307,25 @@ int GDALExtractRPCInfo( char **papszMD, GDALRPCInfo *psRPC )
 
 CPL_C_START
 
-void *GDALCreateProjDef( const char * )
+void * CPL_STDCALL GDALCreateProjDef( const char * )
 {
     CPLDebug( "GDAL", "GDALCreateProjDef no longer supported." );
     return NULL;
 }
 
-CPLErr GDALReprojectToLongLat( void *, double *, double * )
+CPLErr CPL_STDCALL GDALReprojectToLongLat( void *, double *, double * )
 {
     CPLDebug( "GDAL", "GDALReprojectToLatLong no longer supported." );
     return CE_Failure;
 }
 
-CPLErr GDALReprojectFromLongLat( void *, double *, double * )
+CPLErr CPL_STDCALL GDALReprojectFromLongLat( void *, double *, double * )
 {
     CPLDebug( "GDAL", "GDALReprojectFromLatLong no longer supported." );
     return CE_Failure;
 }
 
-void GDALDestroyProjDef( void * )
+void CPL_STDCALL GDALDestroyProjDef( void * )
 
 {
     CPLDebug( "GDAL", "GDALDestroyProjDef no longer supported." );
