@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdal_priv.h,v 1.51 2005/02/17 22:16:12 fwarmerdam Exp $
+ * $Id: gdal_priv.h,v 1.68 2005/09/26 15:52:03 fwarmerdam Exp $
  *
  * Name:     gdal_priv.h
  * Project:  GDAL Core
@@ -29,6 +29,59 @@
  ******************************************************************************
  *
  * $Log: gdal_priv.h,v $
+ * Revision 1.68  2005/09/26 15:52:03  fwarmerdam
+ * centralized .aux opening logic
+ *
+ * Revision 1.67  2005/09/23 20:52:21  fwarmerdam
+ * added the GMO flags on GDALMajorObject
+ *
+ * Revision 1.66  2005/09/23 16:55:39  fwarmerdam
+ * use CPLString instead of std::string
+ *
+ * Revision 1.65  2005/09/17 04:04:16  fwarmerdam
+ * provide default implementation for RAT functions
+ *
+ * Revision 1.64  2005/09/17 03:46:37  fwarmerdam
+ * added HFAAuxBuildOverviews
+ *
+ * Revision 1.63  2005/09/16 20:31:15  fwarmerdam
+ * added bOvrIsAux, and RAT methods
+ *
+ * Revision 1.62  2005/07/25 21:24:28  ssoule
+ * Changed GDALColorTable's "GDALColorEntry *paoEntries" to
+ * "std::vector<GDALColorEntry> aoEntries".
+ *
+ * Revision 1.61  2005/07/25 19:52:43  ssoule
+ * Changed GDALMajorObject's char *pszDescription to std::string sDescription.
+ *
+ * Revision 1.60  2005/07/11 21:08:17  fwarmerdam
+ * Removed GetAge() method.
+ *
+ * Revision 1.59  2005/07/11 19:07:02  fwarmerdam
+ * removed GDALRasterBlock nAge.
+ *
+ * Revision 1.58  2005/05/23 06:44:48  fwarmerdam
+ * blockrefs now fetched locked
+ *
+ * Revision 1.57  2005/05/13 18:19:15  fwarmerdam
+ * Added SetDefaultHistogram
+ *
+ * Revision 1.56  2005/05/10 15:30:28  fwarmerdam
+ * export some overview functions
+ *
+ * Revision 1.55  2005/05/10 04:49:24  fwarmerdam
+ * added getdefaulthistogram and GDALOvLevelAdjust
+ *
+ * Revision 1.54  2005/04/27 16:29:33  fwarmerdam
+ * GetHistogram() made virtual.  Added methods for getting and setting
+ * statistics.  Added SetUnitType method.
+ *
+ * Revision 1.53  2005/04/07 17:31:00  fwarmerdam
+ * added some brief descriptions of classes
+ *
+ * Revision 1.52  2005/04/04 15:24:48  fwarmerdam
+ * Most C entry points now CPL_STDCALL
+ *
  * Revision 1.51  2005/02/17 22:16:12  fwarmerdam
  * changed to use two level block cache
  *
@@ -192,6 +245,7 @@ class GDALMajorObject;
 class GDALDataset;
 class GDALRasterBand;
 class GDALDriver;
+class GDALRasterAttributeTable;
 
 /* -------------------------------------------------------------------- */
 /*      Pull in the public declarations.  This gets the C apis, and     */
@@ -203,6 +257,14 @@ class GDALDriver;
 #include "gdal_frmts.h"
 #include "cpl_vsi.h"
 #include "cpl_conv.h"
+#include "cpl_string.h"
+#include <vector>
+
+#define GMO_VALID                0x0001
+#define GMO_IGNORE_UNIMPLEMENTED 0x0002
+#define GMO_SUPPORT_MD           0x0004
+#define GMO_SUPPORT_MDMD         0x0008
+#define GMO_MD_DIRTY             0x0010
 
 /* ******************************************************************** */
 /*                           GDALMajorObject                            */
@@ -211,15 +273,21 @@ class GDALDriver;
 /*      services shared by major objects.                               */
 /* ******************************************************************** */
 
+//! Object with metadata.
+
 class CPL_DLL GDALMajorObject
 {
   protected:
-    char             *pszDescription;
-    char            **papszMetadata;
+    int                 nFlags; // GMO_* flags. 
+    CPLString           sDescription;
+    char              **papszMetadata;
     
   public:
                         GDALMajorObject();
     virtual            ~GDALMajorObject();
+
+    int                 GetMOFlags();
+    void                SetMOFlags(int nFlags);
                         
     virtual const char *GetDescription() const;
     virtual void        SetDescription( const char * );
@@ -243,6 +311,8 @@ class CPL_DLL GDALDefaultOverviews
     GDALDataset *poODS;
     
     char        *pszOvrFilename;
+
+    int        bOvrIsAux;
     
   public:
                GDALDefaultOverviews();
@@ -267,10 +337,12 @@ class CPL_DLL GDALDefaultOverviews
 /*                             GDALDataset                              */
 /* ******************************************************************** */
 
+//! A set of associated raster bands, usually from one file.
+
 class CPL_DLL GDALDataset : public GDALMajorObject
 {
-    friend GDALDatasetH GDALOpen( const char *, GDALAccess);
-    friend GDALDatasetH GDALOpenShared( const char *, GDALAccess);
+    friend GDALDatasetH CPL_STDCALL GDALOpen( const char *, GDALAccess);
+    friend GDALDatasetH CPL_STDCALL GDALOpenShared( const char *, GDALAccess);
     friend class GDALDriver;
 
   protected:
@@ -369,7 +441,6 @@ class CPL_DLL GDALRasterBlock
 {
     GDALDataType        eType;
     
-    int                 nAge;
     int                 bDirty;
     int                 nLockCount;
 
@@ -396,6 +467,7 @@ class CPL_DLL GDALRasterBlock
     void        MarkClean( void );
     void        AddLock( void ) { nLockCount++; }
     void        DropLock( void ) { nLockCount--; }
+    void        Detach();
 
     CPLErr      Write();
 
@@ -404,7 +476,6 @@ class CPL_DLL GDALRasterBlock
     int         GetYOff() { return nYOff; }
     int         GetXSize() { return nXSize; }
     int         GetYSize() { return nYSize; }
-    int         GetAge() { return nAge; }
     int         GetDirty() { return bDirty; }
     int         GetLockCount() { return nLockCount; }
 
@@ -414,8 +485,9 @@ class CPL_DLL GDALRasterBlock
 
     static int  FlushCacheBlock();
     static void Verify();
-};
 
+    static int  SafeLockBlock( GDALRasterBlock ** );
+};
 
 /* ******************************************************************** */
 /*                             GDALColorTable                           */
@@ -425,8 +497,7 @@ class CPL_DLL GDALColorTable
 {
     GDALPaletteInterp eInterp;
 
-    int         nEntryCount;
-    GDALColorEntry *paoEntries;
+    std::vector<GDALColorEntry> aoEntries;
 
 public:
                 GDALColorTable( GDALPaletteInterp = GPI_RGB );
@@ -490,9 +561,7 @@ class CPL_DLL GDALRasterBand : public GDALMajorObject
     void           InitBlockInfo();
 
     CPLErr         AdoptBlock( int, int, GDALRasterBlock * );
-    GDALRasterBlock *TryGetBlockRef( int nXBlockOff, int nYBlockYOff );
-    int            IsBlockCached( int, int );
-
+    GDALRasterBlock *TryGetLockedBlockRef( int nXBlockOff, int nYBlockYOff );
 
   public:
                 GDALRasterBand();
@@ -515,8 +584,8 @@ class CPL_DLL GDALRasterBand : public GDALMajorObject
 
     CPLErr      WriteBlock( int, int, void * );
 
-    GDALRasterBlock *GetBlockRef( int nXBlockOff, int nYBlockOff, 
-                                  int bJustInitialize = FALSE );
+    GDALRasterBlock *GetLockedBlockRef( int nXBlockOff, int nYBlockOff, 
+                                        int bJustInitialize = FALSE );
     CPLErr      FlushBlock( int = -1, int = -1 );
 
     // New OpengIS CV_SampleDimension stuff.
@@ -539,6 +608,13 @@ class CPL_DLL GDALRasterBand : public GDALMajorObject
     virtual CPLErr SetColorInterpretation( GDALColorInterp );
     virtual CPLErr SetOffset( double );
     virtual CPLErr SetScale( double );
+    virtual CPLErr SetUnitType( const char * );
+
+    virtual CPLErr GetStatistics( int bApproxOK, int bForce,
+                                  double *pdfMin, double *pdfMax, 
+                                  double *pdfMean, double *padfStdDev );
+    virtual CPLErr SetStatistics( double dfMin, double dfMax, 
+                                  double dfMean, double dfStdDev );
 
     virtual int HasArbitraryOverviews();
     virtual int GetOverviewCount();
@@ -550,10 +626,20 @@ class CPL_DLL GDALRasterBand : public GDALMajorObject
                                int nBufXSize, int nBufYSize, 
                                GDALDataType eDT, char **papszOptions );
 
-    CPLErr  GetHistogram( double dfMin, double dfMax,
+    virtual CPLErr  GetHistogram( double dfMin, double dfMax,
                           int nBuckets, int * panHistogram,
                           int bIncludeOutOfRange, int bApproxOK,
                           GDALProgressFunc, void *pProgressData );
+
+    virtual CPLErr GetDefaultHistogram( double *pdfMin, double *pdfMax,
+                                        int *pnBuckets, int ** ppanHistogram,
+                                        int bForce,
+                                        GDALProgressFunc, void *pProgressData);
+    virtual CPLErr SetDefaultHistogram( double dfMin, double dfMax,
+                                        int nBuckets, int *panHistogram );
+
+    virtual const GDALRasterAttributeTable *GetDefaultRAT();
+    virtual CPLErr SetDefaultRAT( const GDALRasterAttributeTable * );
 };
 
 /* ******************************************************************** */
@@ -587,7 +673,10 @@ class CPL_DLL GDALOpenInfo
 /*                              GDALDriver                              */
 /* ******************************************************************** */
 
+
 /**
+ * \brief Format specific driver.
+ *
  * An instance of this class is created for each supported format, and
  * manages information about the format.
  * 
@@ -687,14 +776,22 @@ CPL_C_END
 
 CPL_C_START
 
-CPLErr 
+CPLErr CPL_DLL
+HFAAuxBuildOverviews( const char *pszOvrFilename, GDALDataset *poParentDS,
+                      GDALDataset **ppoDS,
+                      int nBands, int *panBandList,
+                      int nNewOverviews, int *panNewOverviewList, 
+                      const char *pszResampling, 
+                      GDALProgressFunc pfnProgress, 
+                      void *pProgressData );
+CPLErr CPL_DLL 
 GTIFFBuildOverviews( const char * pszFilename,
                      int nBands, GDALRasterBand **papoBandList, 
                      int nOverviews, int * panOverviewList,
                      const char * pszResampling, 
                      GDALProgressFunc pfnProgress, void * pProgressData );
 
-CPLErr
+CPLErr CPL_DLL
 GDALDefaultBuildOverviews( GDALDataset *hSrcDS, const char * pszBasename,
                            const char * pszResampling, 
                            int nOverviews, int * panOverviewList,
@@ -702,10 +799,15 @@ GDALDefaultBuildOverviews( GDALDataset *hSrcDS, const char * pszBasename,
                            GDALProgressFunc pfnProgress, void * pProgressData);
                            
 
-CPLErr
+CPLErr CPL_DLL 
 GDALRegenerateOverviews( GDALRasterBand *, int, GDALRasterBand **,
                          const char *, GDALProgressFunc, void * );
 
+int CPL_DLL GDALOvLevelAdjust( int nOvLevel, int nXSize );
+
+GDALDataset CPL_DLL *
+GDALFindAssociatedAuxFile( const char *pszBasefile, GDALAccess eAccess );
+
 CPL_C_END
 
 #endif /* ndef GDAL_PRIV_H_INCLUDED */
