@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalcolortable.cpp,v 1.2 2001/07/18 04:04:30 warmerda Exp $
+ * $Id: gdalcolortable.cpp,v 1.5 2005/09/05 19:29:29 fwarmerdam Exp $
  *
  * Project:  GDAL Core
  * Purpose:  Color table implementation.
@@ -27,6 +27,16 @@
  * DEALINGS IN THE SOFTWARE.
  ******************************************************************************
  * $Log: gdalcolortable.cpp,v $
+ * Revision 1.5  2005/09/05 19:29:29  fwarmerdam
+ * minor formatting fix
+ *
+ * Revision 1.4  2005/07/25 21:24:28  ssoule
+ * Changed GDALColorTable's "GDALColorEntry *paoEntries" to
+ * "std::vector<GDALColorEntry> aoEntries".
+ *
+ * Revision 1.3  2005/04/04 15:24:48  fwarmerdam
+ * Most C entry points now CPL_STDCALL
+ *
  * Revision 1.2  2001/07/18 04:04:30  warmerda
  * added CPL_CVSID
  *
@@ -37,7 +47,7 @@
 
 #include "gdal_priv.h"
 
-CPL_CVSID("$Id: gdalcolortable.cpp,v 1.2 2001/07/18 04:04:30 warmerda Exp $");
+CPL_CVSID("$Id: gdalcolortable.cpp,v 1.5 2005/09/05 19:29:29 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           GDALColorTable()                           */
@@ -56,16 +66,13 @@ GDALColorTable::GDALColorTable( GDALPaletteInterp eInterpIn )
 
 {
     eInterp = eInterpIn;
-
-    nEntryCount = 0;
-    paoEntries = NULL;
 }
 
 /************************************************************************/
 /*                        GDALCreateColorTable()                        */
 /************************************************************************/
 
-GDALColorTableH GDALCreateColorTable( GDALPaletteInterp eInterp )
+GDALColorTableH CPL_STDCALL GDALCreateColorTable( GDALPaletteInterp eInterp )
 
 {
     return (GDALColorTableH) (new GDALColorTable( eInterp ));
@@ -85,15 +92,13 @@ GDALColorTableH GDALCreateColorTable( GDALPaletteInterp eInterp )
 GDALColorTable::~GDALColorTable()
 
 {
-    CPLFree( paoEntries );
-    paoEntries = NULL;
 }
 
 /************************************************************************/
 /*                       GDALDestroyColorTable()                        */
 /************************************************************************/
 
-void GDALDestroyColorTable( GDALColorTableH hTable )
+void CPL_STDCALL GDALDestroyColorTable( GDALColorTableH hTable )
 
 {
     delete (GDALColorTable *) hTable;
@@ -116,17 +121,18 @@ void GDALDestroyColorTable( GDALColorTableH hTable )
 const GDALColorEntry *GDALColorTable::GetColorEntry( int i ) const
 
 {
-    if( i < 0 || i >= nEntryCount )
+    if( i < 0 || i >= static_cast<int>(aoEntries.size()) )
         return NULL;
     else
-        return paoEntries + i;
+        return &aoEntries[i];
 }
 
 /************************************************************************/
 /*                         GDALGetColorEntry()                          */
 /************************************************************************/
 
-const GDALColorEntry *GDALGetColorEntry( GDALColorTableH hTable, int i )
+const GDALColorEntry * CPL_STDCALL 
+GDALGetColorEntry( GDALColorTableH hTable, int i )
 
 {
     return ((GDALColorTable *) hTable)->GetColorEntry( i );
@@ -157,10 +163,10 @@ const GDALColorEntry *GDALGetColorEntry( GDALColorTableH hTable, int i )
 int GDALColorTable::GetColorEntryAsRGB( int i, GDALColorEntry *poEntry ) const
 
 {
-    if( eInterp != GPI_RGB || i < 0 || i >= nEntryCount )
+    if( eInterp != GPI_RGB || i < 0 || i >= static_cast<int>(aoEntries.size()) )
         return FALSE;
     
-    *poEntry = paoEntries[i];
+    *poEntry = aoEntries[i];
     return TRUE;
 }
 
@@ -168,7 +174,7 @@ int GDALColorTable::GetColorEntryAsRGB( int i, GDALColorEntry *poEntry ) const
 /*                       GDALGetColorEntryAsRGB()                       */
 /************************************************************************/
 
-int GDALGetColorEntryAsRGB( GDALColorTableH hTable, int i, 
+int CPL_STDCALL GDALGetColorEntryAsRGB( GDALColorTableH hTable, int i, 
                             GDALColorEntry *poEntry )
 
 {
@@ -200,24 +206,21 @@ void GDALColorTable::SetColorEntry( int i, const GDALColorEntry * poEntry )
     if( i < 0 )
         return;
     
-    if( i >= nEntryCount )
+    if( i >= static_cast<int>(aoEntries.size()) )
     {
-        paoEntries = (GDALColorEntry *) 
-            CPLRealloc(paoEntries, sizeof(GDALColorEntry) * (i+1));
-        memset( paoEntries + nEntryCount, 0, 
-                sizeof(GDALColorEntry) * (i + 1 - nEntryCount) );
-        
-        nEntryCount = i+1;
+        GDALColorEntry oBlack;
+        oBlack.c1 = oBlack.c2 = oBlack.c3 = oBlack.c4 = 0;
+        aoEntries.resize(i+1, oBlack);
     }
 
-    paoEntries[i] = *poEntry;
+    aoEntries[i] = *poEntry;
 }
 
 /************************************************************************/
 /*                         GDALSetColorEntry()                          */
 /************************************************************************/
 
-void GDALSetColorEntry( GDALColorTableH hTable, int i, 
+void CPL_STDCALL GDALSetColorEntry( GDALColorTableH hTable, int i, 
                         const GDALColorEntry * poEntry )
 
 {
@@ -238,22 +241,14 @@ void GDALSetColorEntry( GDALColorTableH hTable, int i,
 GDALColorTable *GDALColorTable::Clone() const
 
 {
-    GDALColorTable *poNew;
-
-    poNew = new GDALColorTable(eInterp);
-    poNew->nEntryCount = nEntryCount;
-    poNew->paoEntries = (GDALColorEntry *) 
-        CPLMalloc(sizeof(GDALColorEntry)*nEntryCount);
-    memcpy( poNew->paoEntries, paoEntries, sizeof(GDALColorEntry)*nEntryCount);
-
-    return poNew;
+	return new GDALColorTable(*this);
 }
 
 /************************************************************************/
 /*                        GDALCloneColorTable()                         */
 /************************************************************************/
 
-GDALColorTableH GDALCloneColorTable( GDALColorTableH hTable )
+GDALColorTableH CPL_STDCALL GDALCloneColorTable( GDALColorTableH hTable )
 
 {
     return (GDALColorTableH) ((GDALColorTable *) hTable)->Clone();
@@ -274,14 +269,14 @@ GDALColorTableH GDALCloneColorTable( GDALColorTableH hTable )
 int GDALColorTable::GetColorEntryCount() const
 
 {
-    return nEntryCount;
+    return aoEntries.size();
 }
 
 /************************************************************************/
 /*                       GDALGetColorEntryCount()                       */
 /************************************************************************/
 
-int GDALGetColorEntryCount( GDALColorTableH hTable )
+int CPL_STDCALL GDALGetColorEntryCount( GDALColorTableH hTable )
 
 {
     return ((GDALColorTable *) hTable)->GetColorEntryCount();
@@ -311,7 +306,8 @@ GDALPaletteInterp GDALColorTable::GetPaletteInterpretation() const
 /*                    GDALGetPaltteInterpretation()                     */
 /************************************************************************/
 
-GDALPaletteInterp GDALGetPaletteInterpretation( GDALColorTableH hTable )
+GDALPaletteInterp CPL_STDCALL 
+GDALGetPaletteInterpretation( GDALColorTableH hTable )
 
 {
     return ((GDALColorTable *) hTable)->GetPaletteInterpretation();
