@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaldataset.cpp,v 1.46 2005/03/09 17:06:48 fwarmerdam Exp $
+ * $Id: gdaldataset.cpp,v 1.56 2005/09/23 20:52:55 fwarmerdam Exp $
  *
  * Project:  GDAL Core
  * Purpose:  Base class for raster file formats.  
@@ -28,6 +28,36 @@
  ******************************************************************************
  *
  * $Log: gdaldataset.cpp,v $
+ * Revision 1.56  2005/09/23 20:52:55  fwarmerdam
+ * avoid issuing NotSupported errors if GMO_IGNORE_UNIMPLEMENTED is set
+ *
+ * Revision 1.55  2005/07/25 23:15:31  fwarmerdam
+ * Fixed another typo.
+ *
+ * Revision 1.54  2005/07/25 23:13:38  fwarmerdam
+ * Fixed typo in BuildOverviews docs.
+ *
+ * Revision 1.53  2005/07/11 17:10:02  fwarmerdam
+ * moved in GDALOpen, GDALClose - make shared list thread safer
+ *
+ * Revision 1.52  2005/06/20 01:52:47  fwarmerdam
+ * an illegal band in GetRasterBand() no longer fatal
+ *
+ * Revision 1.51  2005/05/23 06:42:16  fwarmerdam
+ * added mutex to protect dataset list
+ *
+ * Revision 1.50  2005/04/20 16:39:04  fwarmerdam
+ * Fixed last fix.
+ *
+ * Revision 1.49  2005/04/20 16:38:10  fwarmerdam
+ * Undo last change, error is fatal.
+ *
+ * Revision 1.48  2005/04/20 16:32:06  fwarmerdam
+ * Ensure GetRasterBand() returns NULL on illegal band.
+ *
+ * Revision 1.47  2005/04/04 15:24:48  fwarmerdam
+ * Most C entry points now CPL_STDCALL
+ *
  * Revision 1.46  2005/03/09 17:06:48  fwarmerdam
  * Added use of BlockBasedRasterIO() if bForceCachedIO turned on.  untested.
  *
@@ -75,79 +105,17 @@
  *
  * Revision 1.31  2003/01/28 16:07:31  warmerda
  * improved documentation
- *
- * Revision 1.30  2002/12/18 15:17:05  warmerda
- * added errors in some unimplemented methods
- *
- * Revision 1.29  2002/09/06 01:29:55  warmerda
- * added C entry points for GetAccess() and GetOpenDatasets()
- *
- * Revision 1.28  2002/07/09 20:33:12  warmerda
- * expand tabs
- *
- * Revision 1.27  2002/05/28 18:56:22  warmerda
- * added shared dataset concept
- *
- * Revision 1.26  2001/11/16 21:36:01  warmerda
- * added the AddBand() method on GDALDataset
- *
- * Revision 1.25  2001/10/18 14:35:22  warmerda
- * avoid conflicts between parameters and member data
- *
- * Revision 1.24  2001/10/17 21:47:02  warmerda
- * added SetGCPs() on GDALDataset
- *
- * Revision 1.23  2001/07/18 04:04:30  warmerda
- * added CPL_CVSID
- *
- * Revision 1.22  2001/01/10 22:24:37  warmerda
- * Patched GDALDataset::FlushCache() to recover gracefully if papoBands
- * doesn't exist yet matching nBands.
- *
- * Revision 1.21  2000/10/06 15:27:13  warmerda
- * default bands to same access as dataset in SetBand()
- *
- * Revision 1.20  2000/08/09 16:26:00  warmerda
- * debug message on dataset cleanup
- *
- * Revision 1.19  2000/07/11 14:35:43  warmerda
- * added documentation
- *
- * Revision 1.18  2000/06/27 16:46:56  warmerda
- * default to using dummy progress func
- *
- * Revision 1.17  2000/06/26 21:44:50  warmerda
- * make progress func save for overviews
- *
- * Revision 1.16  2000/06/26 18:47:31  warmerda
- * added GDALBuildOverviews
- *
- * Revision 1.15  2000/04/21 21:56:23  warmerda
- * move metadata to GDALMajorObject, added BuildOverviews
- *
- * Revision 1.14  2000/03/31 13:42:06  warmerda
- * added gcp support methods
- *
- * Revision 1.13  2000/03/23 16:53:55  warmerda
- * default geotransform is 0,1,0,0,0,1
- *
- * Revision 1.12  2000/03/06 21:50:10  warmerda
- * fixed bug with setting nBands
- *
- * Revision 1.11  2000/03/06 02:20:56  warmerda
- * added reference counting
- *
- * Revision 1.10  2000/02/28 16:34:49  warmerda
- * set the nRasterX/YSize in bands
  */
 
 #include "gdal_priv.h"
 #include "cpl_string.h"
+#include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gdaldataset.cpp,v 1.46 2005/03/09 17:06:48 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdaldataset.cpp,v 1.56 2005/09/23 20:52:55 fwarmerdam Exp $");
 
-static int nGDALDatasetCount = 0;
-static GDALDataset **papoGDALDatasetList = NULL;
+static volatile int nGDALDatasetCount = 0;
+static GDALDataset ** volatile papoGDALDatasetList = NULL;
+static void *hDLMutex = NULL;
 
 /************************************************************************/
 /* ==================================================================== */
@@ -186,10 +154,14 @@ GDALDataset::GDALDataset()
 /* -------------------------------------------------------------------- */
 /*      Add this dataset to the open dataset list.                      */
 /* -------------------------------------------------------------------- */
-    nGDALDatasetCount++;
-    papoGDALDatasetList = (GDALDataset **) 
-        CPLRealloc( papoGDALDatasetList, sizeof(void *) * nGDALDatasetCount );
-    papoGDALDatasetList[nGDALDatasetCount-1] = this;
+    {
+        CPLMutexHolderD( &hDLMutex );
+        
+        nGDALDatasetCount++;
+        papoGDALDatasetList = (GDALDataset ** volatile) 
+            CPLRealloc( papoGDALDatasetList, sizeof(void *)*nGDALDatasetCount);
+        papoGDALDatasetList[nGDALDatasetCount-1] = this;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Set forced caching flag.                                        */
@@ -223,18 +195,23 @@ GDALDataset::~GDALDataset()
 /* -------------------------------------------------------------------- */
 /*      Remove dataset from the "open" dataset list.                    */
 /* -------------------------------------------------------------------- */
-    for( i = 0; i < nGDALDatasetCount; i++ )
     {
-        if( papoGDALDatasetList[i] == this )
+        CPLMutexHolderD( &hDLMutex );
+
+        for( i = 0; i < nGDALDatasetCount; i++ )
         {
-            papoGDALDatasetList[i] = papoGDALDatasetList[nGDALDatasetCount-1];
-            nGDALDatasetCount--;
-            if( nGDALDatasetCount == 0 )
+            if( papoGDALDatasetList[i] == this )
             {
-                CPLFree( papoGDALDatasetList );
-                papoGDALDatasetList = NULL;
+                papoGDALDatasetList[i] = 
+                    papoGDALDatasetList[nGDALDatasetCount-1];
+                nGDALDatasetCount--;
+                if( nGDALDatasetCount == 0 )
+                {
+                    CPLFree( papoGDALDatasetList );
+                    papoGDALDatasetList = NULL;
+                }
+                break;
             }
-            break;
         }
     }
 
@@ -289,7 +266,7 @@ void GDALDataset::FlushCache()
  * @see GDALDataset::FlushCache().
  */
 
-void GDALFlushCache( GDALDatasetH hDS )
+void CPL_STDCALL GDALFlushCache( GDALDatasetH hDS )
 
 {
     ((GDALDataset *) hDS)->FlushCache();
@@ -420,7 +397,7 @@ CPLErr GDALDataset::AddBand( GDALDataType eType, char ** papszOptions )
  * @see GDALDataset::AddBand().
  */
 
-CPLErr GDALAddBand( GDALDatasetH hDataset, 
+CPLErr CPL_STDCALL GDALAddBand( GDALDatasetH hDataset, 
                     GDALDataType eType, char **papszOptions )
 
 {
@@ -504,7 +481,7 @@ int GDALDataset::GetRasterXSize()
  * @see GDALDataset::GetRasterXSize().
  */
 
-int GDALGetRasterXSize( GDALDatasetH hDataset )
+int CPL_STDCALL GDALGetRasterXSize( GDALDatasetH hDataset )
 
 {
     return ((GDALDataset *) hDataset)->GetRasterXSize();
@@ -539,7 +516,7 @@ int GDALDataset::GetRasterYSize()
  * @see GDALDataset::GetRasterYSize().
  */
 
-int GDALGetRasterYSize( GDALDatasetH hDataset )
+int CPL_STDCALL GDALGetRasterYSize( GDALDatasetH hDataset )
 
 {
     return ((GDALDataset *) hDataset)->GetRasterYSize();
@@ -567,12 +544,13 @@ GDALRasterBand * GDALDataset::GetRasterBand( int nBandId )
 {
     if( nBandId < 1 || nBandId > nBands )
     {
-        CPLError( CE_Fatal, CPLE_IllegalArg,
+        CPLError( CE_Failure, CPLE_IllegalArg,
                   "GDALDataset::GetRasterBand(%d) - Illegal band #\n",
                   nBandId );
+        return NULL;
     }
-
-    return( papoBands[nBandId-1] );
+    else
+        return( papoBands[nBandId-1] );
 }
 
 /************************************************************************/
@@ -583,7 +561,7 @@ GDALRasterBand * GDALDataset::GetRasterBand( int nBandId )
  * @see GDALDataset::GetRasterBand().
  */
 
-GDALRasterBandH GDALGetRasterBand( GDALDatasetH hDS, int nBandId )
+GDALRasterBandH CPL_STDCALL GDALGetRasterBand( GDALDatasetH hDS, int nBandId )
 
 {
     return( (GDALRasterBandH) ((GDALDataset *) hDS)->GetRasterBand(nBandId) );
@@ -615,7 +593,7 @@ int GDALDataset::GetRasterCount()
  * @see GDALDataset::GetRasterCount().
  */
 
-int GDALGetRasterCount( GDALDatasetH hDS )
+int CPL_STDCALL GDALGetRasterCount( GDALDatasetH hDS )
 
 {
     return( ((GDALDataset *) hDS)->GetRasterCount() );
@@ -657,7 +635,7 @@ const char *GDALDataset::GetProjectionRef()
  * @see GDALDataset::GetProjectionRef()
  */
 
-const char *GDALGetProjectionRef( GDALDatasetH hDS )
+const char * CPL_STDCALL GDALGetProjectionRef( GDALDatasetH hDS )
 
 {
     return( ((GDALDataset *) hDS)->GetProjectionRef() );
@@ -685,8 +663,9 @@ const char *GDALGetProjectionRef( GDALDatasetH hDS )
 CPLErr GDALDataset::SetProjection( const char * )
 
 {
-    CPLError( CE_Failure, CPLE_NotSupported, 
-              "Dataset does not support the SetProjection() method." );
+    if( !(GetMOFlags() & GMO_IGNORE_UNIMPLEMENTED) )
+        CPLError( CE_Failure, CPLE_NotSupported, 
+                  "Dataset does not support the SetProjection() method." );
     return CE_Failure;
 }
 
@@ -698,7 +677,7 @@ CPLErr GDALDataset::SetProjection( const char * )
  * @see GDALDataset::SetProjection()
  */
 
-CPLErr GDALSetProjection( GDALDatasetH hDS, const char * pszProjection )
+CPLErr CPL_STDCALL GDALSetProjection( GDALDatasetH hDS, const char * pszProjection )
 
 {
     return( ((GDALDataset *) hDS)->SetProjection(pszProjection) );
@@ -764,7 +743,7 @@ CPLErr GDALDataset::GetGeoTransform( double * padfTransform )
  * @see GDALDataset::GetGeoTransform()
  */
 
-CPLErr GDALGetGeoTransform( GDALDatasetH hDS, double * padfTransform )
+CPLErr CPL_STDCALL GDALGetGeoTransform( GDALDatasetH hDS, double * padfTransform )
 
 {
     return( ((GDALDataset *) hDS)->GetGeoTransform(padfTransform) );
@@ -792,8 +771,9 @@ CPLErr GDALGetGeoTransform( GDALDatasetH hDS, double * padfTransform )
 CPLErr GDALDataset::SetGeoTransform( double * )
 
 {
-    CPLError( CE_Failure, CPLE_NotSupported,
-              "SetGeoTransform() not supported for this dataset." );
+    if( !(GetMOFlags() & GMO_IGNORE_UNIMPLEMENTED) )
+        CPLError( CE_Failure, CPLE_NotSupported,
+                  "SetGeoTransform() not supported for this dataset." );
     
     return( CE_Failure );
 }
@@ -806,7 +786,8 @@ CPLErr GDALDataset::SetGeoTransform( double * )
  * @see GDALDataset::SetGeoTransform()
  */
 
-CPLErr GDALSetGeoTransform( GDALDatasetH hDS, double * padfTransform )
+CPLErr CPL_STDCALL 
+GDALSetGeoTransform( GDALDatasetH hDS, double * padfTransform )
 
 {
     return( ((GDALDataset *) hDS)->SetGeoTransform(padfTransform) );
@@ -841,7 +822,8 @@ void *GDALDataset::GetInternalHandle( const char * )
  * @see GDALDataset::GetInternalHandle()
  */
 
-void *GDALGetInternalHandle( GDALDatasetH hDS, const char * pszRequest )
+void * CPL_STDCALL 
+GDALGetInternalHandle( GDALDatasetH hDS, const char * pszRequest )
 
 {
     return( ((GDALDataset *) hDS)->GetInternalHandle(pszRequest) );
@@ -874,7 +856,7 @@ GDALDriver * GDALDataset::GetDriver()
  * @see GDALDataset::GetDriver()
  */
 
-GDALDriverH GDALGetDatasetDriver( GDALDatasetH hDataset )
+GDALDriverH CPL_STDCALL GDALGetDatasetDriver( GDALDatasetH hDataset )
 
 {
     return (GDALDriverH) ((GDALDataset *) hDataset)->GetDriver();
@@ -908,7 +890,7 @@ int GDALDataset::Reference()
  * @see GDALDataset::Reference()
  */
 
-int GDALReferenceDataset( GDALDatasetH hDataset )
+int CPL_STDCALL GDALReferenceDataset( GDALDatasetH hDataset )
 
 {
     return ((GDALDataset *) hDataset)->Reference();
@@ -943,7 +925,7 @@ int GDALDataset::Dereference()
  * @see GDALDataset::Dereference()
  */
 
-int GDALDereferenceDataset( GDALDatasetH hDataset )
+int CPL_STDCALL GDALDereferenceDataset( GDALDatasetH hDataset )
 
 {
     return ((GDALDataset *) hDataset)->Dereference();
@@ -1007,7 +989,7 @@ int GDALDataset::GetGCPCount()
  * @see GDALDataset::GetGCPCount()
  */
 
-int GDALGetGCPCount( GDALDatasetH hDS )
+int CPL_STDCALL GDALGetGCPCount( GDALDatasetH hDS )
 
 {
     return ((GDALDataset *) hDS)->GetGCPCount();
@@ -1041,7 +1023,7 @@ const char *GDALDataset::GetGCPProjection()
  * @see GDALDataset::GetGCPProjection()
  */
 
-const char *GDALGetGCPProjection( GDALDatasetH hDS )
+const char * CPL_STDCALL GDALGetGCPProjection( GDALDatasetH hDS )
 
 {
     return ((GDALDataset *) hDS)->GetGCPProjection();
@@ -1074,7 +1056,7 @@ const GDAL_GCP *GDALDataset::GetGCPs()
  * @see GDALDataset::GetGCPs()
  */
 
-const GDAL_GCP *GDALGetGCPs( GDALDatasetH hDS )
+const GDAL_GCP * CPL_STDCALL GDALGetGCPs( GDALDatasetH hDS )
 
 {
     return ((GDALDataset *) hDS)->GetGCPs();
@@ -1119,8 +1101,9 @@ CPLErr GDALDataset::SetGCPs( int nGCPCount,
     (void) pasGCPList;
     (void) pszGCPProjection;
 
-    CPLError( CE_Failure, CPLE_NotSupported, 
-              "Dataset does not support the SetGCPs() method." );
+    if( !(GetMOFlags() & GMO_IGNORE_UNIMPLEMENTED) )
+        CPLError( CE_Failure, CPLE_NotSupported, 
+                  "Dataset does not support the SetGCPs() method." );
 
     return CE_Failure;
 }
@@ -1133,7 +1116,7 @@ CPLErr GDALDataset::SetGCPs( int nGCPCount,
  * @see GDALDataset::SetGCPs()
  */
 
-CPLErr GDALSetGCPs( GDALDatasetH hDS, int nGCPCount, 
+CPLErr CPL_STDCALL GDALSetGCPs( GDALDatasetH hDS, int nGCPCount, 
                     const GDAL_GCP *pasGCPList, 
                     const char *pszGCPProjection )
 
@@ -1150,7 +1133,8 @@ CPLErr GDALSetGCPs( GDALDatasetH hDS, int nGCPCount,
  * Build raster overview(s)
  *
  * If the operation is unsupported for the indicated dataset, then 
- * CE_Failure is returned, and CPLGetLastError() will return CPLE_NonSupported.
+ * CE_Failure is returned, and CPLGetLastErrorNo() will return 
+ * CPLE_NotSupported.
  *
  * This method is the same as the C function GDALBuildOverviews().
  *
@@ -1216,7 +1200,7 @@ CPLErr GDALDataset::BuildOverviews( const char *pszResampling,
  * @see GDALDataset::BuildOverviews()
  */
 
-CPLErr GDALBuildOverviews( GDALDatasetH hDataset,
+CPLErr CPL_STDCALL GDALBuildOverviews( GDALDatasetH hDataset,
                            const char *pszResampling, 
                            int nOverviews, int *panOverviewList, 
                            int nListBands, int *panBandList,
@@ -1502,13 +1486,14 @@ CPLErr GDALDataset::RasterIO( GDALRWFlag eRWFlag,
  * @see GDALDataset::RasterIO()
  */
 
-CPLErr GDALDatasetRasterIO( GDALDatasetH hDS, GDALRWFlag eRWFlag,
-                            int nXOff, int nYOff, int nXSize, int nYSize,
-                            void * pData, int nBufXSize, int nBufYSize,
-                            GDALDataType eBufType,
-                            int nBandCount, int *panBandMap,
-                            int nPixelSpace, int nLineSpace, int nBandSpace )
-
+CPLErr CPL_STDCALL 							
+GDALDatasetRasterIO( GDALDatasetH hDS, GDALRWFlag eRWFlag,
+                     int nXOff, int nYOff, int nXSize, int nYSize,
+                     void * pData, int nBufXSize, int nBufYSize,
+                     GDALDataType eBufType,
+                     int nBandCount, int *panBandMap,
+                     int nPixelSpace, int nLineSpace, int nBandSpace )
+    
 {
     GDALDataset    *poDS = (GDALDataset *) hDS;
     
@@ -1527,6 +1512,9 @@ CPLErr GDALDatasetRasterIO( GDALDatasetH hDS, GDALRWFlag eRWFlag,
  *
  * This method is the same as the C function GDALGetOpenDatasets().
  *
+ * NOTE: This method is not thread safe.  The returned list may changed
+ * at any time.
+ *
  * @param pnCount integer into which to place the count of dataset pointers
  * being returned.
  *
@@ -1537,7 +1525,7 @@ GDALDataset **GDALDataset::GetOpenDatasets( int *pnCount )
 
 {
     *pnCount = nGDALDatasetCount;
-    return papoGDALDatasetList;
+    return (GDALDataset **) papoGDALDatasetList;
 }
 
 /************************************************************************/
@@ -1548,7 +1536,7 @@ GDALDataset **GDALDataset::GetOpenDatasets( int *pnCount )
  * @see GDALDataset::GetOpenDatasets()
  */
 
-void GDALGetOpenDatasets( GDALDatasetH ***hDS, int *pnCount )
+void CPL_STDCALL GDALGetOpenDatasets( GDALDatasetH ***hDS, int *pnCount )
 
 {
     *hDS = (GDALDatasetH **) GDALDataset::GetOpenDatasets( pnCount);
@@ -1562,7 +1550,7 @@ void GDALGetOpenDatasets( GDALDatasetH ***hDS, int *pnCount )
  * @see GDALDataset::GetAccess()
  */
 
-int GDALGetAccess( GDALDatasetH hDS )
+int CPL_STDCALL GDALGetAccess( GDALDatasetH hDS )
 {
     return ((GDALDataset *) hDS)->GetAccess();
 }
@@ -1648,13 +1636,12 @@ CPLErr GDALDataset::AdviseRead( int nXOff, int nYOff, int nXSize, int nYSize,
 /*                       GDALDatasetAdviseRead()                        */
 /************************************************************************/
 
-CPLErr GDALDatasetAdviseRead( GDALDatasetH hDS, 
-                              int nXOff, int nYOff, int nXSize, int nYSize,
-                              int nBufXSize, int nBufYSize, 
-                              GDALDataType eDT, 
-                              int nBandCount, int *panBandMap,
-                              char **papszOptions )
-
+CPLErr CPL_STDCALL 
+GDALDatasetAdviseRead( GDALDatasetH hDS, 
+                       int nXOff, int nYOff, int nXSize, int nYSize,
+                       int nBufXSize, int nBufYSize, GDALDataType eDT, 
+                       int nBandCount, int *panBandMap,char **papszOptions )
+    
 {
     return ((GDALDataset *) hDS)->AdviseRead( nXOff, nYOff, nXSize, nYSize, 
                                               nBufXSize, nBufYSize, eDT, 
@@ -1662,3 +1649,232 @@ CPLErr GDALDatasetAdviseRead( GDALDatasetH hDS,
                                               papszOptions );
 }
 
+/************************************************************************/
+/*                              GDALOpen()                              */
+/************************************************************************/
+
+/**
+ * Open a raster file as a GDALDataset.
+ *
+ * This function will try to open the passed file, or virtual dataset
+ * name by invoking the Open method of each registered GDALDriver in turn. 
+ * The first successful open will result in a returned dataset.  If all
+ * drivers fail then NULL is returned.
+ *
+ * \sa GDALOpenShared()
+ *
+ * @param pszFilename the name of the file to access.  In the case of
+ * exotic drivers this may not refer to a physical file, but instead contain
+ * information for the driver on how to access a dataset.
+ *
+ * @param eAccess the desired access, either GA_Update or GA_ReadOnly.  Many
+ * drivers support only read only access.
+ *
+ * @return A GDALDatasetH handle or NULL on failure.  For C++ applications
+ * this handle can be cast to a GDALDataset *. 
+ */
+
+GDALDatasetH CPL_STDCALL 
+GDALOpen( const char * pszFilename, GDALAccess eAccess )
+
+{
+    int         iDriver;
+    GDALDriverManager *poDM = GetGDALDriverManager();
+    GDALOpenInfo oOpenInfo( pszFilename, eAccess );
+
+    CPLErrorReset();
+    
+    for( iDriver = 0; iDriver < poDM->GetDriverCount(); iDriver++ )
+    {
+        GDALDriver      *poDriver = poDM->GetDriver( iDriver );
+        GDALDataset     *poDS;
+
+        poDS = poDriver->pfnOpen( &oOpenInfo );
+        if( poDS != NULL )
+        {
+            poDS->SetDescription( pszFilename );
+
+            if( poDS->poDriver == NULL )
+                poDS->poDriver = poDriver;
+
+            
+            CPLDebug( "GDAL", "GDALOpen(%s) succeeds as %s.\n",
+                      pszFilename, poDriver->GetDescription() );
+
+            return (GDALDatasetH) poDS;
+        }
+
+        if( CPLGetLastErrorNo() != 0 )
+            return NULL;
+    }
+
+    if( oOpenInfo.bStatOK )
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "`%s' not recognised as a supported file format.\n",
+                  pszFilename );
+    else
+        CPLError( CE_Failure, CPLE_OpenFailed,
+                  "`%s' does not exist in the file system,\n"
+                  "and is not recognised as a supported dataset name.\n",
+                  pszFilename );
+              
+    return NULL;
+}
+
+/************************************************************************/
+/*                           GDALOpenShared()                           */
+/************************************************************************/
+
+/**
+ * Open a raster file as a GDALDataset.
+ *
+ * This function works the same as GDALOpen(), but allows the sharing of
+ * GDALDataset handles for a dataset with other callers to GDALOpenShared().
+ * 
+ * In particular, GDALOpenShared() will first consult it's list of currently
+ * open and shared GDALDataset's, and if the GetDescription() name for one
+ * exactly matches the pszFilename passed to GDALOpenShared() it will be
+ * referenced and returned.
+ *
+ * \sa GDALOpen()
+ *
+ * @param pszFilename the name of the file to access.  In the case of
+ * exotic drivers this may not refer to a physical file, but instead contain
+ * information for the driver on how to access a dataset.
+ *
+ * @param eAccess the desired access, either GA_Update or GA_ReadOnly.  Many
+ * drivers support only read only access.
+ *
+ * @return A GDALDatasetH handle or NULL on failure.  For C++ applications
+ * this handle can be cast to a GDALDataset *. 
+ */
+ 
+GDALDatasetH CPL_STDCALL 
+GDALOpenShared( const char *pszFilename, GDALAccess eAccess )
+
+{
+/* -------------------------------------------------------------------- */
+/*      First scan the existing list to see if it could already         */
+/*      contain the requested dataset.                                  */
+/* -------------------------------------------------------------------- */
+    {
+        CPLMutexHolderD( &hDLMutex );
+        int         i;
+    
+        for( i = 0; i < nGDALDatasetCount; i++ )
+        {
+            if( strcmp(pszFilename,
+                       papoGDALDatasetList[i]->GetDescription()) == 0 
+                && (eAccess == GA_ReadOnly 
+                    || papoGDALDatasetList[i]->GetAccess() == eAccess ) )
+                
+            {
+                papoGDALDatasetList[i]->Reference();
+                return papoGDALDatasetList[i];
+            }
+        }
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Try opening the the requested dataset.                          */
+/* -------------------------------------------------------------------- */
+    GDALDataset *poDataset;
+
+    poDataset = (GDALDataset *) GDALOpen( pszFilename, eAccess );
+    if( poDataset != NULL )
+        poDataset->MarkAsShared();
+    
+    return (GDALDatasetH) poDataset;
+}
+
+/************************************************************************/
+/*                             GDALClose()                              */
+/************************************************************************/
+
+/**
+ * Close GDAL dataset. 
+ *
+ * For non-shared datasets (opened with GDALOpen()) the dataset is closed
+ * using the C++ "delete" operator, recovering all dataset related resources.  
+ * For shared datasets (opened with GDALOpenShared()) the dataset is 
+ * dereferenced, and closed only if the referenced count has dropped below 1.
+ *
+ * @param hDS The dataset to close.  May be cast from a "GDALDataset *". 
+ */
+
+void CPL_STDCALL GDALClose( GDALDatasetH hDS )
+
+{
+    GDALDataset *poDS = (GDALDataset *) hDS;
+    int         i;
+    CPLMutexHolderD( &hDLMutex );
+
+/* -------------------------------------------------------------------- */
+/*      If this file is in the shared dataset list then dereference     */
+/*      it, and only delete/remote it if the reference count has        */
+/*      dropped to zero.                                                */
+/* -------------------------------------------------------------------- */
+    for( i = 0; i < nGDALDatasetCount; i++ )
+    {
+        if( papoGDALDatasetList[i] == poDS )
+        {
+            if( poDS->Dereference() > 0 )
+                return;
+
+            delete poDS;
+            return;
+        }
+    }
+
+/* -------------------------------------------------------------------- */
+/*      This is not shared dataset, so directly delete it.              */
+/* -------------------------------------------------------------------- */
+    delete poDS;
+}
+
+/************************************************************************/
+/*                        GDALDumpOpenDataset()                         */
+/************************************************************************/
+
+/**
+ * List open datasets.
+ *
+ * Dumps a list of all open datasets (shared or not) to the indicated 
+ * text file (may be stdout or stderr).   This function is primariliy intended
+ * to assist in debugging "dataset leaks" and reference counting issues. 
+ * The information reported includes the dataset name, referenced count, 
+ * shared status, driver name, size, and band count. 
+ */
+
+int CPL_STDCALL GDALDumpOpenDatasets( FILE *fp )
+   
+{
+    CPLMutexHolderD( &hDLMutex );
+    int         i;
+
+    if( nGDALDatasetCount > 0 )
+        VSIFPrintf( fp, "Open GDAL Datasets:\n" );
+    
+    for( i = 0; i < nGDALDatasetCount; i++ )
+    {
+        const char *pszDriverName;
+        GDALDataset *poDS = (GDALDataset *) papoGDALDatasetList[i];
+        
+        if( poDS->GetDriver() == NULL )
+            pszDriverName = "DriverIsNULL";
+        else
+            pszDriverName = poDS->GetDriver()->GetDescription();
+
+        poDS->Reference();
+        VSIFPrintf( fp, "  %d %c %-6s %dx%dx%d %s\n", 
+                    poDS->Dereference(), 
+                    poDS->GetShared() ? 'S' : 'N',
+                    pszDriverName, 
+                    poDS->GetRasterXSize(),
+                    poDS->GetRasterYSize(),
+                    poDS->GetRasterCount(),
+                    poDS->GetDescription() );
+    }
+    
+    return nGDALDatasetCount;
+}
