@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaldefaultoverviews.cpp,v 1.10 2004/07/26 22:32:30 warmerda Exp $
+ * $Id: gdaldefaultoverviews.cpp,v 1.14 2005/09/26 15:52:03 fwarmerdam Exp $
  *
  * Project:  GDAL Core
  * Purpose:  Helper code to implement overview support in different drivers.
@@ -28,6 +28,18 @@
  ******************************************************************************
  *
  * $Log: gdaldefaultoverviews.cpp,v $
+ * Revision 1.14  2005/09/26 15:52:03  fwarmerdam
+ * centralized .aux opening logic
+ *
+ * Revision 1.13  2005/09/17 03:46:18  fwarmerdam
+ * added USE_RRD support to create overviews
+ *
+ * Revision 1.12  2005/09/16 20:32:44  fwarmerdam
+ * added preliminary .aux support (read only)
+ *
+ * Revision 1.11  2005/05/10 04:48:42  fwarmerdam
+ * GDALOvLevelAdjust now public
+ *
  * Revision 1.10  2004/07/26 22:32:30  warmerda
  * Support .OVR files as well as .ovr
  *
@@ -62,8 +74,9 @@
  */
 
 #include "gdal_priv.h"
+#include "cpl_string.h"
 
-CPL_CVSID("$Id: gdaldefaultoverviews.cpp,v 1.10 2004/07/26 22:32:30 warmerda Exp $");
+CPL_CVSID("$Id: gdaldefaultoverviews.cpp,v 1.14 2005/09/26 15:52:03 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                        GDALDefaultOverviews()                        */
@@ -75,6 +88,7 @@ GDALDefaultOverviews::GDALDefaultOverviews()
     poDS = NULL;
     poODS = NULL;
     pszOvrFilename = NULL;
+    bOvrIsAux = FALSE;
 }
 
 /************************************************************************/
@@ -101,7 +115,7 @@ void GDALDefaultOverviews::Initialize( GDALDataset *poDSIn,
                                        int bNameIsOVR )
 
 {
-    VSIStatBuf sStatBuf;
+    VSIStatBufL sStatBuf;
 
 /* -------------------------------------------------------------------- */
 /*      If we were already initialized, destroy the old overview        */
@@ -129,20 +143,39 @@ void GDALDefaultOverviews::Initialize( GDALDataset *poDSIn,
     else
         sprintf( pszOvrFilename, "%s.ovr", pszBasename );
 
-    bExists = VSIStat( pszOvrFilename, &sStatBuf ) == 0;
+    bExists = VSIStatL( pszOvrFilename, &sStatBuf ) == 0;
 
 #if !defined(WIN32)
     if( !bNameIsOVR && !bExists )
     {
         sprintf( pszOvrFilename, "%s.OVR", pszBasename );
-        bExists = VSIStat( pszOvrFilename, &sStatBuf ) == 0;
+        bExists = VSIStatL( pszOvrFilename, &sStatBuf ) == 0;
         if( !bExists )
             sprintf( pszOvrFilename, "%s.ovr", pszBasename );
     }
 #endif
 
     if( bExists )
+    {
         poODS = (GDALDataset *) GDALOpen( pszOvrFilename, poDS->GetAccess() );
+    }
+
+/* -------------------------------------------------------------------- */
+/*      We didn't find that, so try and find a corresponding aux        */
+/*      file.  Check that we are the dependent file of the aux          */
+/*      file.                                                           */
+/* -------------------------------------------------------------------- */
+    if( !poODS )
+    {
+        poODS = GDALFindAssociatedAuxFile( pszBasename, poDS->GetAccess() );
+
+        if( poODS )
+        {
+            bOvrIsAux = TRUE;
+            CPLFree( pszOvrFilename );
+            pszOvrFilename = CPLStrdup(poODS->GetDescription());
+        }
+    }
 }
 
 /************************************************************************/
@@ -161,7 +194,12 @@ int GDALDefaultOverviews::GetOverviewCount( int nBand )
     if( poBand == NULL )
         return 0;
     else
-        return poBand->GetOverviewCount() + 1;
+    {
+        if( bOvrIsAux )
+            return poBand->GetOverviewCount();
+        else
+            return poBand->GetOverviewCount() + 1;
+    }
 }
 
 /************************************************************************/
@@ -181,12 +219,18 @@ GDALDefaultOverviews::GetOverview( int nBand, int iOverview )
     if( poBand == NULL )
         return NULL;
 
-    if( iOverview == 0 )
-        return poBand;
-    else if( iOverview-1 >= poBand->GetOverviewCount() )
-        return NULL;
-    else
-        return poBand->GetOverview( iOverview-1 );
+    if( bOvrIsAux )
+        return poBand->GetOverview( iOverview );
+
+    else // TIFF case, base is overview 0.
+    {
+        if( iOverview == 0 )
+            return poBand;
+        else if( iOverview-1 >= poBand->GetOverviewCount() )
+            return NULL;
+        else
+            return poBand->GetOverview( iOverview-1 );
+    }
 }
 
 /************************************************************************/
@@ -204,7 +248,7 @@ GDALDefaultOverviews::GetOverview( int nBand, int iOverview )
 /*      the image size.                                                 */
 /************************************************************************/
 
-static int GDALOvLevelAdjust( int nOvLevel, int nXSize )
+int GDALOvLevelAdjust( int nOvLevel, int nXSize )
 
 {
     int nOXSize = (nXSize + nOvLevel - 1) / nOvLevel;
@@ -229,6 +273,21 @@ GDALDefaultOverviews::BuildOverviews(
     CPLErr       eErr;
     int          i;
 
+/* -------------------------------------------------------------------- */
+/*      If we don't already have an overview file, we need to decide    */
+/*      what format to use.                                             */
+/* -------------------------------------------------------------------- */
+    if( poODS == NULL )
+    {
+        bOvrIsAux = CSLTestBoolean(CPLGetConfigOption( "USE_RRD", "NO" ));
+        if( bOvrIsAux )
+        {
+            CPLFree( pszOvrFilename );
+            pszOvrFilename = 
+                CPLStrdup(CPLResetExtension(poDS->GetDescription(),"aux"));
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Our TIFF overview support currently only works safely if all    */
 /*      bands are handled at the same time.                             */
@@ -253,7 +312,11 @@ GDALDefaultOverviews::BuildOverviews(
     {
         CPLFree( pszOvrFilename );
         pszOvrFilename = (char *) CPLMalloc(strlen(pszBasename)+5);
-        sprintf( pszOvrFilename, "%s.ovr", pszBasename );
+        
+        if( bOvrIsAux )
+            sprintf( pszOvrFilename, "%s.aux", pszBasename );
+        else
+            sprintf( pszOvrFilename, "%s.ovr", pszBasename );
     }
 
 /* -------------------------------------------------------------------- */
@@ -288,15 +351,6 @@ GDALDefaultOverviews::BuildOverviews(
             panNewOverviewList[nNewOverviews++] = panOverviewList[i];
     }
 
-/* -------------------------------------------------------------------- */
-/*      If we have an existing overview file open, close it now.        */
-/* -------------------------------------------------------------------- */
-    if( poODS != NULL )
-    {
-        delete poODS;
-        poODS = NULL;
-    }
-
 /* -------------------------------------------------------------------- */
 /*      Build band list.                                                */
 /* -------------------------------------------------------------------- */
@@ -305,25 +359,57 @@ GDALDefaultOverviews::BuildOverviews(
         pahBands[i] = poDS->GetRasterBand( panBandList[i] );
 
 /* -------------------------------------------------------------------- */
-/*      Build new overviews.                                            */
+/*      Build new overviews - Imagine.  Keep existing file open if      */
+/*      we have it.  But mark all overviews as in need of               */
+/*      regeneration, since HFAAuxBuildOverviews() doesn't actually     */
+/*      produce the imagery.                                            */
 /* -------------------------------------------------------------------- */
+    if( bOvrIsAux )
+    {
+        eErr = HFAAuxBuildOverviews( pszOvrFilename, poDS, &poODS,
+                                     nBands, panBandList,
+                                     nNewOverviews, panNewOverviewList, 
+                                     pszResampling, 
+                                     pfnProgress, pProgressData );
 
-    eErr = GTIFFBuildOverviews( pszOvrFilename, nBands, pahBands, 
-                                nNewOverviews, panNewOverviewList, 
-                                pszResampling, pfnProgress, pProgressData );
+        int j;
+        
+        for( j = 0; j < poBand->GetOverviewCount(); j++ )
+        {
+            if( panOverviewList[j] > 0 )
+                panOverviewList[j] *= -1;
+        }
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Build new overviews - TIFF.  Close TIFF files while we          */
+/*      operate on it.                                                  */
+/* -------------------------------------------------------------------- */
+    else
+    {
+        if( poODS != NULL )
+        {
+            delete poODS;
+            poODS = NULL;
+        }
+
+        eErr = GTIFFBuildOverviews( pszOvrFilename, nBands, pahBands, 
+                                    nNewOverviews, panNewOverviewList, 
+                                    pszResampling, pfnProgress, pProgressData );
+
+        if( eErr == CE_None )
+        {
+            poODS = (GDALDataset *) GDALOpen( pszOvrFilename, GA_Update );
+            if( poODS == NULL )
+                eErr = CE_Failure;
+        }
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Refresh old overviews that were listed.                         */
 /* -------------------------------------------------------------------- */
     GDALRasterBand **papoOverviewBands;
 
-    if( eErr == CE_None )
-    {
-        poODS = (GDALDataset *) GDALOpen( pszOvrFilename, GA_Update );
-        if( poODS == NULL )
-            eErr = CE_Failure;
-    }
-
     papoOverviewBands = (GDALRasterBand **) 
         CPLCalloc(sizeof(void*),nOverviews);
 
@@ -348,7 +434,7 @@ GDALDefaultOverviews::BuildOverviews(
                     || nOvFactor == GDALOvLevelAdjust( -panOverviewList[i], 
                                                        poBand->GetXSize() ) )
                 {
-                    panOverviewList[i] *= -1;
+                    //panOverviewList[i] *= -1;
                     papoOverviewBands[nNewOverviews++] = poOverview;
                 }
             }
@@ -359,7 +445,7 @@ GDALDefaultOverviews::BuildOverviews(
             eErr = GDALRegenerateOverviews( poBand, 
                                             nNewOverviews, papoOverviewBands,
                                             pszResampling, 
-                                            GDALDummyProgress, NULL );
+                                            pfnProgress, pProgressData );
         }
     }
 
