@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaldriver.cpp,v 1.31 2005/01/15 16:10:10 fwarmerdam Exp $
+ * $Id: gdaldriver.cpp,v 1.35 2005/05/24 18:13:15 dron Exp $
  *
  * Project:  GDAL Core
  * Purpose:  Implementation of GDALDriver class (and C wrappers)
@@ -28,6 +28,18 @@
  ******************************************************************************
  *
  * $Log: gdaldriver.cpp,v $
+ * Revision 1.35  2005/05/24 18:13:15  dron
+ * Added GDALGetDriverCreationOptionList() function.
+ *
+ * Revision 1.34  2005/05/17 15:13:06  fwarmerdam
+ * Added nodata support for bands
+ *
+ * Revision 1.33  2005/04/04 15:24:48  fwarmerdam
+ * Most C entry points now CPL_STDCALL
+ *
+ * Revision 1.32  2005/03/16 11:01:10  lichun
+ * in CreateCopy() the category names are copied as well
+ *
  * Revision 1.31  2005/01/15 16:10:10  fwarmerdam
  * added offset, scale and colorinterp to default createcopy
  *
@@ -73,7 +85,7 @@
 
 #include "gdal_priv.h"
 
-CPL_CVSID("$Id: gdaldriver.cpp,v 1.31 2005/01/15 16:10:10 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdaldriver.cpp,v 1.35 2005/05/24 18:13:15 dron Exp $");
 
 /************************************************************************/
 /*                             GDALDriver()                             */
@@ -172,11 +184,10 @@ GDALDataset * GDALDriver::Create( const char * pszFilename,
  * @see GDALDriver::Create()
  */
 
-GDALDatasetH CPL_DLL GDALCreate( GDALDriverH hDriver,
-                                 const char * pszFilename,
-                                 int nXSize, int nYSize, int nBands,
-                                 GDALDataType eBandType,
-                                 char ** papszOptions )
+GDALDatasetH CPL_DLL CPL_STDCALL 
+GDALCreate( GDALDriverH hDriver, const char * pszFilename,
+            int nXSize, int nYSize, int nBands, GDALDataType eBandType,
+            char ** papszOptions )
 
 {
     return( ((GDALDriver *) hDriver)->Create( pszFilename,
@@ -316,6 +327,7 @@ GDALDataset *GDALDriver::CreateCopy( const char * pszFilename,
 /*      Do we need to copy a colortable or other metadata?              */
 /* -------------------------------------------------------------------- */
         GDALColorTable *poCT;
+        char** catNames;
         int bSuccess;
         double dfValue;
 
@@ -339,10 +351,18 @@ GDALDataset *GDALDriver::CreateCopy( const char * pszFilename,
         if( bSuccess )
             poDstBand->SetScale( dfValue );
 
+        dfValue = poSrcBand->GetNoDataValue( &bSuccess );
+        if( bSuccess )
+            poDstBand->SetNoDataValue( dfValue );
+
         if( poSrcBand->GetColorInterpretation() != GCI_Undefined )
             poDstBand->SetColorInterpretation( 
                 poSrcBand->GetColorInterpretation() );
 
+        catNames = poSrcBand->GetCategoryNames();
+        if (0 != catNames)
+            poDstBand->SetCategoryNames(catNames);
+
         if( !bStrict )
             CPLPopErrorHandler();
 
@@ -395,7 +415,7 @@ GDALDataset *GDALDriver::CreateCopy( const char * pszFilename,
  * @see GDALDriver::CreateCopy()
  */
 
-GDALDatasetH GDALCreateCopy( GDALDriverH hDriver, 
+GDALDatasetH CPL_STDCALL GDALCreateCopy( GDALDriverH hDriver, 
                              const char * pszFilename, 
                              GDALDatasetH hSrcDS, 
                              int bStrict, char ** papszOptions,
@@ -470,7 +490,7 @@ CPLErr GDALDriver::Delete( const char * pszFilename )
  * @see GDALDriver::Delete()
  */
 
-CPLErr GDALDeleteDataset( GDALDriverH hDriver, const char * pszFilename )
+CPLErr CPL_STDCALL GDALDeleteDataset( GDALDriverH hDriver, const char * pszFilename )
 
 {
     return ((GDALDriver *) hDriver)->Delete( pszFilename );
@@ -480,7 +500,7 @@ CPLErr GDALDeleteDataset( GDALDriverH hDriver, const char * pszFilename )
 /*                       GDALGetDriverShortName()                       */
 /************************************************************************/
 
-const char * GDALGetDriverShortName( GDALDriverH hDriver )
+const char * CPL_STDCALL GDALGetDriverShortName( GDALDriverH hDriver )
 
 {
     if( hDriver == NULL )
@@ -493,7 +513,7 @@ const char * GDALGetDriverShortName( GDALDriverH hDriver )
 /*                       GDALGetDriverLongName()                        */
 /************************************************************************/
 
-const char * GDALGetDriverLongName( GDALDriverH hDriver )
+const char * CPL_STDCALL GDALGetDriverLongName( GDALDriverH hDriver )
 
 {
     if( hDriver == NULL )
@@ -512,7 +532,7 @@ const char * GDALGetDriverLongName( GDALDriverH hDriver )
 /*                       GDALGetDriverHelpTopic()                       */
 /************************************************************************/
 
-const char * GDALGetDriverHelpTopic( GDALDriverH hDriver )
+const char * CPL_STDCALL GDALGetDriverHelpTopic( GDALDriverH hDriver )
 
 {
     if( hDriver == NULL )
@@ -521,3 +541,21 @@ const char * GDALGetDriverHelpTopic( GDALDriverH hDriver )
     return ((GDALDriver *) hDriver)->GetMetadataItem( GDAL_DMD_HELPTOPIC );
 }
 
+/************************************************************************/
+/*                   GDALGetDriverCreationOptionList()                  */
+/************************************************************************/
+
+const char * CPL_STDCALL GDALGetDriverCreationOptionList( GDALDriverH hDriver )
+
+{
+    if( hDriver == NULL )
+        return NULL;
+
+    const char *pszOptionList = 
+        ((GDALDriver *) hDriver)->GetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST );
+
+    if( pszOptionList == NULL )
+        return "";
+    else
+        return pszOptionList;
+}
