@@ -25,6 +25,33 @@
  * The GDALDriverManager class from gdal_priv.h.
  * 
  * $Log: gdaldrivermanager.cpp,v $
+ * Revision 1.31  2005/09/15 18:39:43  fwarmerdam
+ * cleanup TLS and VSI with driver manager
+ *
+ * Revision 1.30  2005/07/11 17:29:48  fwarmerdam
+ * More use of mutex in some "read" cases.
+ *
+ * Revision 1.29  2005/07/11 17:25:49  fwarmerdam
+ * Fixed minior race condition in GDALDriverManager creation.
+ *
+ * Revision 1.28  2005/07/11 17:17:21  fwarmerdam
+ * Added note on thread un-safety of GDALDestroyDriverManager.
+ *
+ * Revision 1.27  2005/07/11 17:15:32  fwarmerdam
+ * Grab mutex to create drive manager
+ *
+ * Revision 1.26  2005/06/10 15:37:35  fwarmerdam
+ * avoid use of MAX_PATH
+ *
+ * Revision 1.25  2005/06/10 14:59:12  fwarmerdam
+ * Added option to search under executable directory for plugins on win32.
+ *
+ * Revision 1.24  2005/05/23 06:46:24  fwarmerdam
+ * added mutex to protect driver manager
+ *
+ * Revision 1.23  2005/04/04 15:24:48  fwarmerdam
+ * Most C entry points now CPL_STDCALL
+ *
  * Revision 1.22  2005/01/19 20:27:17  fwarmerdam
  * Search in lib/gdalplugins instead of directly in lib directory.
  * Only try files endning in .dll, .so or .dylib.  see bug 746
@@ -96,11 +123,13 @@
 
 #include "gdal_priv.h"
 #include "cpl_string.h"
+#include "cpl_multiproc.h"
 #include "ogr_srs_api.h"
+#include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gdaldrivermanager.cpp,v 1.22 2005/01/19 20:27:17 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdaldrivermanager.cpp,v 1.31 2005/09/15 18:39:43 fwarmerdam Exp $");
 
-static char *pszUpdatableINST_DATA = 
+static const char *pszUpdatableINST_DATA = 
 "__INST_DATA_TARGET:                                                                                                                                      ";
 
 /************************************************************************/
@@ -109,7 +138,8 @@ static char *pszUpdatableINST_DATA =
 /* ==================================================================== */
 /************************************************************************/
 
-static GDALDriverManager        *poDM = NULL;
+static volatile GDALDriverManager        *poDM = NULL;
+static void *hDMMutex = NULL;
 
 /************************************************************************/
 /*                        GetGDALDriverManager()                        */
@@ -128,13 +158,18 @@ static GDALDriverManager        *poDM = NULL;
  * to fail.
  */
 
-GDALDriverManager *GetGDALDriverManager()
+GDALDriverManager * GetGDALDriverManager()
 
 {
     if( poDM == NULL )
-        new GDALDriverManager();
+    {
+        CPLMutexHolderD( &hDMMutex );
 
-    return( poDM );
+        if( poDM == NULL )
+            poDM = new GDALDriverManager();
+    }
+
+    return( (GDALDriverManager *) poDM );
 }
 
 /************************************************************************/
@@ -149,7 +184,6 @@ GDALDriverManager::GDALDriverManager()
     pszHome = CPLStrdup("");
 
     CPLAssert( poDM == NULL );
-    poDM = this;
 
 /* -------------------------------------------------------------------- */
 /*      We want to push a location to search for data files             */
@@ -221,6 +255,17 @@ GDALDriverManager::~GDALDriverManager()
 /* -------------------------------------------------------------------- */
     OSRCleanup();
 
+/* -------------------------------------------------------------------- */
+/*      Cleanup VSIFileManager.                                         */
+/* -------------------------------------------------------------------- */
+    VSICleanupFileManager();
+
+/* -------------------------------------------------------------------- */
+/*      Cleanup thread local storage ... I hope the program is all      */
+/*      done with GDAL/OGR!                                             */
+/* -------------------------------------------------------------------- */
+    CPLCleanupTLS();
+
 /* -------------------------------------------------------------------- */
 /*      Ensure the global driver manager pointer is NULLed out.         */
 /* -------------------------------------------------------------------- */
@@ -254,7 +299,7 @@ int GDALDriverManager::GetDriverCount()
  * @see GDALDriverManager::GetDriverCount()
  */
 
-int GDALGetDriverCount()
+int CPL_STDCALL GDALGetDriverCount()
 
 {
     return GetGDALDriverManager()->GetDriverCount();
@@ -277,6 +322,8 @@ int GDALGetDriverCount()
 GDALDriver * GDALDriverManager::GetDriver( int iDriver )
 
 {
+    CPLMutexHolderD( &hDMMutex );
+
     if( iDriver < 0 || iDriver >= nDrivers )
         return NULL;
     else
@@ -291,7 +338,7 @@ GDALDriver * GDALDriverManager::GetDriver( int iDriver )
  * @see GDALDriverManager::GetDriver()
  */
 
-GDALDriverH GDALGetDriver( int iDriver )
+GDALDriverH CPL_STDCALL GDALGetDriver( int iDriver )
 
 {
     return (GDALDriverH) GetGDALDriverManager()->GetDriver(iDriver);
@@ -323,6 +370,8 @@ GDALDriverH GDALGetDriver( int iDriver )
 int GDALDriverManager::RegisterDriver( GDALDriver * poDriver )
 
 {
+    CPLMutexHolderD( &hDMMutex );
+
 /* -------------------------------------------------------------------- */
 /*      If it is already registered, just return the existing           */
 /*      index.                                                          */
@@ -334,7 +383,9 @@ int GDALDriverManager::RegisterDriver( GDALDriver * poDriver )
         for( i = 0; i < nDrivers; i++ )
         {
             if( papoDrivers[i] == poDriver )
+            {
                 return i;
+            }
         }
 
         CPLAssert( FALSE );
@@ -355,7 +406,9 @@ int GDALDriverManager::RegisterDriver( GDALDriver * poDriver )
     if( poDriver->pfnCreateCopy != NULL )
         poDriver->SetMetadataItem( GDAL_DCAP_CREATECOPY, "YES" );
 
-    return( nDrivers - 1 );
+    int iResult = nDrivers - 1;
+
+    return iResult;
 }
 
 /************************************************************************/
@@ -366,7 +419,7 @@ int GDALDriverManager::RegisterDriver( GDALDriver * poDriver )
  * @see GDALDriverManager::GetRegisterDriver()
  */
 
-int GDALRegisterDriver( GDALDriverH hDriver )
+int CPL_STDCALL GDALRegisterDriver( GDALDriverH hDriver )
 
 {
     return GetGDALDriverManager()->RegisterDriver( (GDALDriver *) hDriver );
@@ -391,6 +444,7 @@ void GDALDriverManager::DeregisterDriver( GDALDriver * poDriver )
 
 {
     int         i;
+    CPLMutexHolderD( &hDMMutex );
 
     for( i = 0; i < nDrivers; i++ )
     {
@@ -417,7 +471,7 @@ void GDALDriverManager::DeregisterDriver( GDALDriver * poDriver )
  * @see GDALDriverManager::GetDeregisterDriver()
  */
 
-void GDALDeregisterDriver( GDALDriverH hDriver )
+void CPL_STDCALL GDALDeregisterDriver( GDALDriverH hDriver )
 
 {
     GetGDALDriverManager()->DeregisterDriver( (GDALDriver *) hDriver );
@@ -443,6 +497,8 @@ GDALDriver * GDALDriverManager::GetDriverByName( const char * pszName )
 {
     int         i;
 
+    CPLMutexHolderD( &hDMMutex );
+
     for( i = 0; i < nDrivers; i++ )
     {
         if( EQUAL(papoDrivers[i]->GetDescription(), pszName) )
@@ -460,7 +516,7 @@ GDALDriver * GDALDriverManager::GetDriverByName( const char * pszName )
  * @see GDALDriverManager::GetDriverByName()
  */
 
-GDALDriverH GDALGetDriverByName( const char * pszName )
+GDALDriverH CPL_STDCALL GDALGetDriverByName( const char * pszName )
 
 {
     return( GetGDALDriverManager()->GetDriverByName( pszName ) );
@@ -483,6 +539,8 @@ const char *GDALDriverManager::GetHome()
 void GDALDriverManager::SetHome( const char * pszNewHome )
 
 {
+    CPLMutexHolderD( &hDMMutex );
+
     CPLFree( pszHome );
     pszHome = CPLStrdup(pszNewHome);
 }
@@ -525,7 +583,6 @@ void GDALDriverManager::AutoSkipDrivers()
             DeregisterDriver( poDriver );
             delete poDriver;
         }
-            
     }
 
     CSLDestroy( papszList );
@@ -578,8 +635,20 @@ void GDALDriverManager::AutoLoadDrivers()
         papszSearchPath = CSLAddString( papszSearchPath, 
                                         GDAL_PREFIX "/lib/gdalplugins" );
 #else
-        papszSearchPath = CSLAddString( papszSearchPath, 
-                                        "/usr/local/lib/gdalplugins" );
+        char szExecPath[1024];
+
+        if( CPLGetExecPath( szExecPath, sizeof(szExecPath) ) )
+        {
+            char szPluginDir[sizeof(szExecPath)+50];
+            strcpy( szPluginDir, CPLGetDirname( szExecPath ) );
+            strcat( szPluginDir, "\\gdalplugins\\" );
+            papszSearchPath = CSLAddString( papszSearchPath, szPluginDir );
+        }
+        else
+        {
+            papszSearchPath = CSLAddString( papszSearchPath, 
+                                            "/usr/local/lib/gdalplugins" );
+        }
 #endif
 
         if( strlen(GetHome()) > 0 )
@@ -652,11 +721,17 @@ void GDALDriverManager::AutoLoadDrivers()
  * Destroy the driver manager.
  *
  * Incidently unloads all managed drivers.
+ *
+ * NOTE: This function is not thread safe.  It should not be called while
+ * other threads are actively using GDAL. 
  */
 
-void GDALDestroyDriverManager( void )
+void CPL_STDCALL GDALDestroyDriverManager( void )
 
 {
+    // THREADSAFETY: We would like to lock the mutex here, but it 
+    // needs to be reacquired within the destructor during driver
+    // deregistration.
     if( poDM != NULL )
         delete poDM;
 }
