@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalrasterblock.cpp,v 1.16 2004/04/07 13:15:57 warmerda Exp $
+ * $Id: gdalrasterblock.cpp,v 1.20 2005/07/11 19:06:36 fwarmerdam Exp $
  *
  * Project:  GDAL Core
  * Purpose:  Implementation of GDALRasterBlock class and related global 
@@ -29,6 +29,18 @@
  ******************************************************************************
  *
  * $Log: gdalrasterblock.cpp,v $
+ * Revision 1.20  2005/07/11 19:06:36  fwarmerdam
+ * Removed tile ticker, and GDALRasterBlock age.
+ *
+ * Revision 1.19  2005/05/23 14:28:21  fwarmerdam
+ * fixed serious flaw in Detach() method
+ *
+ * Revision 1.18  2005/05/23 06:43:37  fwarmerdam
+ * adding mutex for block list
+ *
+ * Revision 1.17  2005/04/04 15:24:48  fwarmerdam
+ * Most C entry points now CPL_STDCALL
+ *
  * Revision 1.16  2004/04/07 13:15:57  warmerda
  * Ensure that GDALSetCacheMax() keeps flushing till the new limit is
  * honoured. http://bugzilla.remotesensing.org/show_bug.cgi?id=542
@@ -82,16 +94,18 @@
  */
 
 #include "gdal_priv.h"
+#include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gdalrasterblock.cpp,v 1.16 2004/04/07 13:15:57 warmerda Exp $");
+CPL_CVSID("$Id: gdalrasterblock.cpp,v 1.20 2005/07/11 19:06:36 fwarmerdam Exp $");
 
-static int nTileAgeTicker = 0; 
 static int bCacheMaxInitialized = FALSE;
 static int nCacheMax = 10 * 1024*1024;
-static int nCacheUsed = 0;
+static volatile int nCacheUsed = 0;
 
-static GDALRasterBlock   *poOldest = NULL;    /* tail */
-static GDALRasterBlock   *poNewest = NULL;    /* head */
+static volatile GDALRasterBlock *poOldest = NULL;    /* tail */
+static volatile GDALRasterBlock *poNewest = NULL;    /* head */
+
+static void *hRBMutex = NULL;
 
 
 /************************************************************************/
@@ -107,7 +121,7 @@ static GDALRasterBlock   *poNewest = NULL;    /* head */
  * @param nNewSize the maximum number of bytes for caching.  Maximum is 2GB.
  */
 
-void GDALSetCacheMax( int nNewSize )
+void CPL_STDCALL GDALSetCacheMax( int nNewSize )
 
 {
     nCacheMax = nNewSize;
@@ -140,7 +154,7 @@ void GDALSetCacheMax( int nNewSize )
  * @return maximum in bytes. 
  */
 
-int GDALGetCacheMax()
+int CPL_STDCALL GDALGetCacheMax()
 {
     if( !bCacheMaxInitialized )
     {
@@ -167,7 +181,7 @@ int GDALGetCacheMax()
  * GDALRasterBlock memory caching.
  */
 
-int GDALGetCacheUsed()
+int CPL_STDCALL GDALGetCacheUsed()
 {
     return nCacheUsed;
 }
@@ -178,7 +192,7 @@ int GDALGetCacheUsed()
 /*      The workhorse of cache management!                              */
 /************************************************************************/
 
-int GDALFlushCacheBlock()
+int CPL_STDCALL GDALFlushCacheBlock()
 
 {
     return GDALRasterBlock::FlushCacheBlock();
@@ -186,24 +200,40 @@ int GDALFlushCacheBlock()
 
 /************************************************************************/
 /*                          FlushCacheBlock()                           */
+/*                                                                      */
+/*      Note, if we have alot of blocks locked for a long time, this    */
+/*      method is going to get slow because it will have to traverse    */
+/*      the linked list a long ways looking for a flushing              */
+/*      candidate.   It might help to re-touch locked blocks to push    */
+/*      them to the top of the list.                                    */
 /************************************************************************/
 
 int GDALRasterBlock::FlushCacheBlock()
 
 {
-    GDALRasterBlock *poTarget = poOldest;
+    int nXOff, nYOff;
+    GDALRasterBand *poBand;
 
-    while( poTarget != NULL && poTarget->GetLockCount() > 0 ) 
-        poTarget = poTarget->poPrevious;
-
-    if( poTarget != NULL )
     {
-        poTarget->GetBand()->FlushBlock( poTarget->GetXOff(), 
-                                         poTarget->GetYOff() );
-        return TRUE;
+        CPLMutexHolderD( &hRBMutex );
+        GDALRasterBlock *poTarget = (GDALRasterBlock *) poOldest;
+
+        while( poTarget != NULL && poTarget->GetLockCount() > 0 ) 
+            poTarget = poTarget->poPrevious;
+        
+        if( poTarget == NULL )
+            return FALSE;
+
+        poTarget->Detach();
+
+        nXOff = poTarget->GetXOff();
+        nYOff = poTarget->GetYOff();
+        poBand = poTarget->GetBand();
     }
-    else
-        return FALSE;
+
+    poBand->FlushBlock( nXOff, nYOff );
+
+    return TRUE;
 }
 
 /************************************************************************/
@@ -235,6 +265,8 @@ GDALRasterBlock::GDALRasterBlock( GDALRasterBand *poBandIn,
 GDALRasterBlock::~GDALRasterBlock()
 
 {
+    Detach();
+
     if( pData != NULL )
     {
         int nSizeInBytes;
@@ -245,6 +277,24 @@ GDALRasterBlock::~GDALRasterBlock()
         nCacheUsed -= nSizeInBytes;
     }
 
+    CPLAssert( nLockCount == 0 );
+
+#ifdef ENABLE_DEBUG
+    Verify();
+#endif
+}
+
+/************************************************************************/
+/*                               Detach()                               */
+/*                                                                      */
+/*      Remove from block lists.                                        */
+/************************************************************************/
+
+void GDALRasterBlock::Detach()
+
+{
+    CPLMutexHolderD( &hRBMutex );
+
     if( poOldest == this )
         poOldest = poPrevious;
 
@@ -259,13 +309,8 @@ GDALRasterBlock::~GDALRasterBlock()
     if( poNext != NULL )
         poNext->poPrevious = poPrevious;
 
-    CPLAssert( nLockCount == 0 );
-
-#ifdef ENABLE_DEBUG
-    Verify();
-#endif
-
-    nAge = -1;
+    poPrevious = NULL;
+    poNext = NULL;
 }
 
 /************************************************************************/
@@ -275,6 +320,8 @@ GDALRasterBlock::~GDALRasterBlock()
 void GDALRasterBlock::Verify()
 
 {
+    CPLMutexHolderD( &hRBMutex );
+
     CPLAssert( (poNewest == NULL && poOldest == NULL)
                || (poNewest != NULL && poOldest != NULL) );
 
@@ -283,8 +330,7 @@ void GDALRasterBlock::Verify()
         CPLAssert( poNewest->poPrevious == NULL );
         CPLAssert( poOldest->poNext == NULL );
         
-
-        for( GDALRasterBlock *poBlock = poNewest; 
+        for( GDALRasterBlock *poBlock = (GDALRasterBlock *) poNewest; 
              poBlock != NULL;
              poBlock = poBlock->poNext )
         {
@@ -326,7 +372,7 @@ CPLErr GDALRasterBlock::Write()
 void GDALRasterBlock::Touch()
 
 {
-    nAge = nTileAgeTicker++;
+    CPLMutexHolderD( &hRBMutex );
 
     if( poNewest == this )
         return;
@@ -341,7 +387,7 @@ void GDALRasterBlock::Touch()
         poNext->poPrevious = poPrevious;
 
     poPrevious = NULL;
-    poNext = poNewest;
+    poNext = (GDALRasterBlock *) poNewest;
 
     if( poNewest != NULL )
     {
@@ -367,6 +413,7 @@ void GDALRasterBlock::Touch()
 CPLErr GDALRasterBlock::Internalize()
 
 {
+    CPLMutexHolderD( &hRBMutex );
     void        *pNewData;
     int         nSizeInBytes;
     int         nCurCacheMax = GDALGetCacheMax();
@@ -398,12 +445,11 @@ CPLErr GDALRasterBlock::Internalize()
             break;
     }
 
-    DropLock();
-
 /* -------------------------------------------------------------------- */
 /*      Add this block to the list.                                     */
 /* -------------------------------------------------------------------- */
     Touch();
+    DropLock();
 
     return( CE_None );
 }
@@ -429,3 +475,34 @@ void GDALRasterBlock::MarkClean()
     bDirty = FALSE;
 }
 
+/************************************************************************/
+/*                           SafeLockBlock()                            */
+/************************************************************************/
+
+/**
+ * Safely lock block.
+ *
+ * This method locks a GDALRasterBlock (and touches it) in a thread-safe
+ * manner.  The global block cache mutex is held while locking the block,
+ * in order to avoid race conditions with other threads that might be
+ * trying to expire the block at the same time.  The block pointer may be
+ * safely NULL, in which case this method does nothing. 
+ *
+ * @param ppBlock Pointer to the block pointer to try and lock/touch.
+ */
+ 
+int GDALRasterBlock::SafeLockBlock( GDALRasterBlock ** ppBlock )
+
+{
+    CPLMutexHolderD( &hRBMutex );
+
+    if( *ppBlock != NULL )
+    {
+        (*ppBlock)->AddLock();
+        (*ppBlock)->Touch();
+        
+        return TRUE;
+    }
+    else
+        return FALSE;
+}
