@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: overview.cpp,v 1.11 2003/05/21 04:24:38 warmerda Exp $
+ * $Id: overview.cpp,v 1.12 2005/04/04 15:24:48 fwarmerdam Exp $
  *
  * Project:  GDAL Core
  * Purpose:  Helper code to implement overview support in different drivers.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: overview.cpp,v $
+ * Revision 1.12  2005/04/04 15:24:48  fwarmerdam
+ * Most C entry points now CPL_STDCALL
+ *
  * Revision 1.11  2003/05/21 04:24:38  warmerda
  * avoid warnings with cast
  *
@@ -65,7 +68,7 @@
 
 #include "gdal_priv.h"
 
-CPL_CVSID("$Id: overview.cpp,v 1.11 2003/05/21 04:24:38 warmerda Exp $");
+CPL_CVSID("$Id: overview.cpp,v 1.12 2005/04/04 15:24:48 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                       GDALDownsampleChunk32R()                       */
@@ -531,7 +534,7 @@ GDALRegenerateOverviews( GDALRasterBand *poSrcBand,
 /*                        GDALComputeBandStats()                        */
 /************************************************************************/
 
-CPLErr
+CPLErr CPL_STDCALL 
 GDALComputeBandStats( GDALRasterBandH hSrcBand,
                       int nSampleStep,
                       double *pdfMean, double *pdfStdDev, 
