@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr2ogr.cpp,v 1.25 2004/10/30 15:50:41 fwarmerdam Exp $
+ * $Id: ogr2ogr.cpp,v 1.28 2005/04/14 14:20:24 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Simple client for translating between formats.
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: ogr2ogr.cpp,v $
+ * Revision 1.28  2005/04/14 14:20:24  fwarmerdam
+ * More stuff to avoid feature leaks.
+ *
+ * Revision 1.27  2005/04/14 14:16:37  fwarmerdam
+ * Fix from Julien for destroying features in error case.
+ *
+ * Revision 1.26  2005/03/15 21:18:47  fwarmerdam
+ * Added -sql to usage message.
+ *
  * Revision 1.25  2004/10/30 15:50:41  fwarmerdam
  * added -skipfailures doc
  *
@@ -109,7 +118,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogr2ogr.cpp,v 1.25 2004/10/30 15:50:41 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr2ogr.cpp,v 1.28 2005/04/14 14:20:24 fwarmerdam Exp $");
 
 static void Usage();
 
@@ -527,7 +536,8 @@ static void Usage()
     OGRSFDriverRegistrar        *poR = OGRSFDriverRegistrar::GetRegistrar();
 
     printf( "Usage: ogr2ogr [-skipfailures] [-append] [-update] [-f format_name]\n"
-            "               [-select field_list] [-where restricted_where]\n"
+            "               [-select field_list] [-where restricted_where] \n"
+            "               [-sql <sql statement>] \n" 
             "               [-spat xmin ymin xmax ymax] [-preserve_fid] [-fid FID]\n"
             "               [-a_srs srs_def] [-t_srs srs_def] [-s_srs srs_def]\n"
             "               [[-dsco NAME=VALUE] ...] dst_datasource_name\n"
@@ -549,6 +559,7 @@ static void Usage()
             " -select field_list: Comma-delimited list of fields from input layer to\n"
             "                     copy to the new layer (defaults to all)\n" 
             " -where restricted_where: Attribute query (like SQL WHERE)\n" 
+            " -sql statement: Execute given SQL statement and save result.\n"
             " -skipfailures: skip features or layers that fail to convert\n"
             " -spat xmin ymin xmax ymax: spatial query extents\n"
             " -dsco NAME=VALUE: Dataset creation option (format specific)\n"
@@ -764,11 +775,12 @@ static int TranslateLayer( OGRDataSource *poSrcDS,
             if( nGroupTransactions )
                 poDstLayer->CommitTransaction();
             
-            delete poFeature;
-            
             CPLError( CE_Failure, CPLE_AppDefined,
                       "Unable to translate feature %d from layer %s.\n",
                       poFeature->GetFID(), poFDefn->GetName() );
+            
+            OGRFeature::DestroyFeature( poFeature );
+            OGRFeature::DestroyFeature( poDstFeature );
             return FALSE;
         }
 
@@ -786,7 +798,11 @@ static int TranslateLayer( OGRDataSource *poSrcDS,
                 printf( "Failed to transform feature %d.\n", 
                         (int) poFeature->GetFID() );
                 if( !bSkipFailures )
+                {
+                    OGRFeature::DestroyFeature( poFeature );
+                    OGRFeature::DestroyFeature( poDstFeature );
                     return FALSE;
+                }
             }
         }
 
