@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_api.h,v 1.25 2005/02/22 12:37:26 fwarmerdam Exp $
+ * $Id: ogr_api.h,v 1.28 2005/09/21 00:50:08 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  C API for OGR Geometry, Feature, Layers, DataSource and drivers.
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: ogr_api.h,v $
+ * Revision 1.28  2005/09/21 00:50:08  fwarmerdam
+ * Added Release
+ *
+ * Revision 1.27  2005/08/05 20:28:55  fwarmerdam
+ * pass nbytes in OGR_G_CreateFromWkb()
+ *
+ * Revision 1.26  2005/07/20 01:43:51  fwarmerdam
+ * upgraded OGR geometry dimension handling
+ *
  * Revision 1.25  2005/02/22 12:37:26  fwarmerdam
  * rename Equal/Intersect to Equals/Intersects
  *
@@ -139,7 +148,7 @@ struct _CPLXMLNode;
 /* From base OGRGeometry class */
 
 OGRErr CPL_DLL OGR_G_CreateFromWkb( unsigned char *, OGRSpatialReferenceH, 
-                                    OGRGeometryH * );
+                                    OGRGeometryH *, int );
 OGRErr CPL_DLL OGR_G_CreateFromWkt( char **, OGRSpatialReferenceH, 
                                     OGRGeometryH * );
 void   CPL_DLL OGR_G_DestroyGeometry( OGRGeometryH );
@@ -147,6 +156,7 @@ OGRGeometryH CPL_DLL OGR_G_CreateGeometry( OGRwkbGeometryType );
 
 int    CPL_DLL OGR_G_GetDimension( OGRGeometryH );
 int    CPL_DLL OGR_G_GetCoordinateDimension( OGRGeometryH );
+void   CPL_DLL OGR_G_SetCoordinateDimension( OGRGeometryH, int );
 OGRGeometryH CPL_DLL OGR_G_Clone( OGRGeometryH );
 void   CPL_DLL OGR_G_GetEnvelope( OGRGeometryH, OGREnvelope * );
 OGRErr CPL_DLL OGR_G_ImportFromWkb( OGRGeometryH, unsigned char *, int );
@@ -211,7 +221,10 @@ void   CPL_DLL OGR_G_GetPoint( OGRGeometryH, int iPoint,
                                double *, double *, double * );
 void   CPL_DLL OGR_G_SetPoint( OGRGeometryH, int iPoint, 
                                double, double, double );
+void   CPL_DLL OGR_G_SetPoint_2D( OGRGeometryH, int iPoint, 
+                                  double, double );
 void   CPL_DLL OGR_G_AddPoint( OGRGeometryH, double, double, double );
+void   CPL_DLL OGR_G_AddPoint_2D( OGRGeometryH, double, double );
 
 /* Methods for getting/setting rings and members collections */
 
@@ -264,6 +277,7 @@ const char CPL_DLL *OGR_GetFieldTypeName( OGRFieldType );
 
 OGRFeatureDefnH CPL_DLL OGR_FD_Create( const char * );
 void   CPL_DLL OGR_FD_Destroy( OGRFeatureDefnH );
+void   CPL_DLL OGR_FD_Release( OGRFeatureDefnH );
 const char CPL_DLL *OGR_FD_GetName( OGRFeatureDefnH );
 int    CPL_DLL OGR_FD_GetFieldCount( OGRFeatureDefnH );
 OGRFieldDefnH CPL_DLL OGR_FD_GetFieldDefn( OGRFeatureDefnH, int );
