@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_core.h,v 1.25 2005/02/02 19:59:47 fwarmerdam Exp $
+ * $Id: ogr_core.h,v 1.28 2005/08/30 23:52:35 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Define some core portability services for cross-platform OGR code.
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: ogr_core.h,v $
+ * Revision 1.28  2005/08/30 23:52:35  fwarmerdam
+ * implement preliminary OFTBinary support
+ *
+ * Revision 1.27  2005/07/29 15:54:07  fwarmerdam
+ * ogrmakewktcoordinate defined in ogr_p.h
+ *
+ * Revision 1.26  2005/07/29 04:12:47  fwarmerdam
+ * expose OGRMakeWktCoordinates
+ *
  * Revision 1.25  2005/02/02 19:59:47  fwarmerdam
  * added SetNextByIndex support
  *
@@ -304,6 +313,11 @@ typedef union {
     } WideStringList;
     */
 
+    struct {
+        int     nCount;
+        GByte   *paData;
+    } Binary;
+    
     struct {
         int     nMarker1;
         int     nMarker2;
