@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_feature.h,v 1.29 2004/02/23 21:47:23 warmerda Exp $
+ * $Id: ogr_feature.h,v 1.31 2005/09/21 00:50:08 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Class for representing a whole feature, and layer schemas.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogr_feature.h,v $
+ * Revision 1.31  2005/09/21 00:50:08  fwarmerdam
+ * Added Release
+ *
+ * Revision 1.30  2005/08/30 23:52:35  fwarmerdam
+ * implement preliminary OFTBinary support
+ *
  * Revision 1.29  2004/02/23 21:47:23  warmerda
  * Added GetUsedFields() and GetSWGExpr() methods on OGRFeatureQuery class
  *
@@ -234,6 +240,7 @@ class CPL_DLL OGRFeatureDefn
     int         Reference() { return ++nRefCount; }
     int         Dereference() { return --nRefCount; }
     int         GetReferenceCount() { return nRefCount; }
+    void        Release();
 
     static OGRFeatureDefn  *CreateFeatureDefn( const char *pszName = NULL );
     static void         DestroyFeatureDefn( OGRFeatureDefn * );
@@ -297,6 +304,7 @@ class CPL_DLL OGRFeature
     const int          *GetFieldAsIntegerList( int i, int *pnCount );
     const double       *GetFieldAsDoubleList( int i, int *pnCount );
     char              **GetFieldAsStringList( int i );
+    GByte               *GetFieldAsBinary( int i, int *pnCount );
 
     int                 GetFieldAsInteger( const char *pszFName )
                       { return GetFieldAsInteger( GetFieldIndex(pszFName) ); }
@@ -322,6 +330,7 @@ class CPL_DLL OGRFeature
     void                SetField( int i, int nCount, double * padfValues );
     void                SetField( int i, char ** papszValues );
     void                SetField( int i, OGRField * puValue );
+    void                SetField( int i, int nCount, GByte * pabyBinary );
 
     void                SetField( const char *pszFName, int nValue )
                            { SetField( GetFieldIndex(pszFName), nValue ); }
