@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_fromepsg.cpp,v 1.38 2004/09/10 21:02:41 fwarmerdam Exp $
+ * $Id: ogr_fromepsg.cpp,v 1.39 2005/04/06 00:02:05 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Generate an OGRSpatialReference object based on an EPSG
@@ -29,6 +29,9 @@
  ******************************************************************************
  *
  * $Log: ogr_fromepsg.cpp,v $
+ * Revision 1.39  2005/04/06 00:02:05  fwarmerdam
+ * various osr and oct functions now stdcall
+ *
  * Revision 1.38  2004/09/10 21:02:41  fwarmerdam
  * added note about 9814
  *
@@ -148,7 +151,7 @@
 #include "ogr_spatialref.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogr_fromepsg.cpp,v 1.38 2004/09/10 21:02:41 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr_fromepsg.cpp,v 1.39 2005/04/06 00:02:05 fwarmerdam Exp $");
 
 #ifndef PI
 #  define PI 3.14159265358979323846
@@ -1524,7 +1527,7 @@ OGRErr OGRSpatialReference::importFromEPSG( int nCode )
 /*                         OSRImportFromEPSG()                          */
 /************************************************************************/
 
-OGRErr OSRImportFromEPSG( OGRSpatialReferenceH hSRS, int nCode )
+OGRErr CPL_STDCALL OSRImportFromEPSG( OGRSpatialReferenceH hSRS, int nCode )
 
 {
     return ((OGRSpatialReference *) hSRS)->importFromEPSG( nCode );
