@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_geometry.h,v 1.52 2005/02/22 12:48:09 fwarmerdam Exp $
+ * $Id: ogr_geometry.h,v 1.56 2005/09/17 12:11:30 osemykin Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Classes for manipulating simple features that is not specific
@@ -29,6 +29,18 @@
  ******************************************************************************
  *
  * $Log: ogr_geometry.h,v $
+ * Revision 1.56  2005/09/17 12:11:30  osemykin
+ * added OGRLineString::getPoints
+ *
+ * Revision 1.55  2005/08/04 17:18:59  fwarmerdam
+ * now have separate 2D and 3D OGRPoint constructors
+ *
+ * Revision 1.54  2005/07/20 01:43:51  fwarmerdam
+ * upgraded OGR geometry dimension handling
+ *
+ * Revision 1.53  2005/03/25 06:31:12  fwarmerdam
+ * added addSubLineString
+ *
  * Revision 1.52  2005/02/22 12:48:09  fwarmerdam
  * added OGRGeometryFactory::haveGEOS()
  *
@@ -239,6 +251,9 @@ class CPL_DLL OGRGeometry
 {
   private:
     OGRSpatialReference * poSRS;                // may be NULL
+
+  protected:
+    int                   nCoordDimension;
     
   public:
                 OGRGeometry();
@@ -246,7 +261,7 @@ class CPL_DLL OGRGeometry
                         
     // standard IGeometry
     virtual int getDimension() const = 0;
-    virtual int getCoordinateDimension() const = 0;
+    virtual int getCoordinateDimension() const;
     virtual OGRBoolean  IsEmpty() const { return 0; } 
     virtual OGRBoolean  IsSimple() const { return 1; }
     virtual void        empty() = 0;
@@ -269,6 +284,8 @@ class CPL_DLL OGRGeometry
     virtual geos::Geometry *exportToGEOS() const;
     virtual void closeRings();
 
+    virtual void setCoordinateDimension( int nDimension ); 
+
     void    assignSpatialReference( OGRSpatialReference * poSR );
     OGRSpatialReference *getSpatialReference( void ) const { return poSRS; }
 
@@ -321,7 +338,8 @@ class CPL_DLL OGRPoint : public OGRGeometry
 
   public:
                 OGRPoint();
-                OGRPoint( double x, double y, double z = 0.0 );
+                OGRPoint( double x, double y );
+                OGRPoint( double x, double y, double z );
     virtual     ~OGRPoint();
 
     // IWks Interface
@@ -333,7 +351,6 @@ class CPL_DLL OGRPoint : public OGRGeometry
     
     // IGeometry
     virtual int getDimension() const;
-    virtual int getCoordinateDimension() const;
     virtual OGRGeometry *clone() const;
     virtual void empty();
     virtual void getEnvelope( OGREnvelope * psEnvelope ) const;
@@ -346,7 +363,7 @@ class CPL_DLL OGRPoint : public OGRGeometry
     // Non standard
     void        setX( double xIn ) { x = xIn; }
     void        setY( double yIn ) { y = yIn; }
-    void        setZ( double zIn ) { z = zIn; }
+    void        setZ( double zIn ) { z = zIn; nCoordDimension=3; }
 
     // ISpatialRelation
     virtual OGRBoolean  Equals( OGRGeometry * ) const;
@@ -412,7 +429,6 @@ class CPL_DLL OGRLineString : public OGRCurve
 
     // IGeometry interface
     virtual int getDimension() const;
-    virtual int getCoordinateDimension() const;
     virtual OGRGeometry *clone() const;
     virtual void empty();
     virtual void getEnvelope( OGREnvelope * psEnvelope ) const;
@@ -434,14 +450,22 @@ class CPL_DLL OGRLineString : public OGRCurve
     virtual OGRBoolean  Equals( OGRGeometry * ) const;
     
     // non standard.
+    virtual void setCoordinateDimension( int nDimension ); 
     void        setNumPoints( int );
     void        setPoint( int, OGRPoint * );
-    void        setPoint( int, double, double, double = 0.0 );
+    void        setPoint( int, double, double );
+    void        setPoint( int, double, double, double );
     void        setPoints( int, OGRRawPoint *, double * = NULL );
     void        setPoints( int, double * padfX, double * padfY,
                            double *padfZ = NULL );
     void        addPoint( OGRPoint * );
-    void        addPoint( double, double, double = 0.0 );
+    void        addPoint( double, double );
+    void        addPoint( double, double, double );
+
+    void        getPoints( OGRRawPoint *, double * = NULL );
+
+    void        addSubLineString( const OGRLineString *, 
+                                  int nStartVertex = 0, int nEndVertex = -1 );
 
     // non-standard from OGRGeometry
     virtual OGRwkbGeometryType getGeometryType() const;
@@ -557,13 +581,14 @@ class CPL_DLL OGRPolygon : public OGRSurface
 
     // IGeometry
     virtual int getDimension() const;
-    virtual int getCoordinateDimension() const;
     virtual void getEnvelope( OGREnvelope * psEnvelope ) const;
 
     // ISpatialRelation
     virtual OGRBoolean  Equals( OGRGeometry * ) const;
     
     // Non standard
+    virtual void setCoordinateDimension( int nDimension ); 
+
     void        addRing( OGRLinearRing * );
     void        addRingDirectly( OGRLinearRing * );
 
@@ -615,7 +640,6 @@ class CPL_DLL OGRGeometryCollection : public OGRGeometry
 
     // IGeometry methods
     virtual int getDimension() const;
-    virtual int getCoordinateDimension() const;
     virtual void getEnvelope( OGREnvelope * psEnvelope ) const;
 
     // IGeometryCollection
@@ -627,6 +651,7 @@ class CPL_DLL OGRGeometryCollection : public OGRGeometry
     virtual OGRBoolean  Equals( OGRGeometry * ) const;
     
     // Non standard
+    virtual void setCoordinateDimension( int nDimension ); 
     virtual OGRErr addGeometry( const OGRGeometry * );
     virtual OGRErr addGeometryDirectly( OGRGeometry * );
     virtual OGRErr removeGeometry( int iIndex, int bDelete = TRUE );
