@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_esri.cpp,v 1.38 2005/02/18 21:52:44 fwarmerdam Exp $
+ * $Id: ogr_srs_esri.cpp,v 1.40 2005/08/03 16:11:18 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  OGRSpatialReference translation to/from ESRI .prj definitions.
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: ogr_srs_esri.cpp,v $
+ * Revision 1.40  2005/08/03 16:11:18  fwarmerdam
+ * Added additional morphToESRI() rules to:
+ *  o Change NAD27/NAD83/WGS84 GEOGCSes to GCS_ names used by ESRI.
+ *  o Change the word Unnamed to Unknown in names.
+ *  o Set exact ESRI names for UTM PROJCSes (for WGS84, NAD27 and NAD83).
+ *
+ * Revision 1.39  2005/05/04 14:29:20  fwarmerdam
+ * convert Standard_Parallel_1 in ESRI Mercator to Latitude_Of_Origin.
+ *
  * Revision 1.38  2005/02/18 21:52:44  fwarmerdam
  * Fixed support for .prj files with blank lines between the parameters.
  *
@@ -155,7 +164,7 @@
 #include "ogr_p.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogr_srs_esri.cpp,v 1.38 2005/02/18 21:52:44 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr_srs_esri.cpp,v 1.40 2005/08/03 16:11:18 fwarmerdam Exp $");
 
 static char *apszProjMapping[] = {
     "Albers", SRS_PT_ALBERS_CONIC_EQUAL_AREA,
@@ -179,6 +188,10 @@ static char *apszAlbersMapping[] = {
     "Central_Parallel", SRS_PP_LATITUDE_OF_CENTER,
     NULL, NULL };
 
+static char *apszMercatorMapping[] = {
+    SRS_PP_STANDARD_PARALLEL_1, SRS_PP_LATITUDE_OF_ORIGIN,
+    NULL, NULL };
+
 static char **papszDatumMapping = NULL;
  
 static char *apszDefaultDatumMapping[] = {
@@ -979,6 +992,26 @@ OGRErr OGRSpatialReference::morphToESRI()
         /* ideally we should strip out of the rectified_grid_angle */
     }
 
+/* -------------------------------------------------------------------- */
+/*      Force Unnamed to Unknown for most common locations.             */
+/* -------------------------------------------------------------------- */
+    static char *apszUnknownMapping[] = { 
+        "Unknown", "Unnamed",
+        NULL, NULL 
+    };
+
+    GetRoot()->applyRemapper( "PROJCS", 
+                              apszUnknownMapping+1, apszUnknownMapping+0, 2 );
+    GetRoot()->applyRemapper( "GEOGCS", 
+                              apszUnknownMapping+1, apszUnknownMapping+0, 2 );
+    GetRoot()->applyRemapper( "DATUM", 
+                              apszUnknownMapping+1, apszUnknownMapping+0, 2 );
+    GetRoot()->applyRemapper( "SPHEROID", 
+                              apszUnknownMapping+1, apszUnknownMapping+0, 2 );
+    GetRoot()->applyRemapper( "PRIMEM", 
+                              apszUnknownMapping+1, apszUnknownMapping+0, 2 );
+    
+
 /* -------------------------------------------------------------------- */
 /*      Translate PROJECTION keywords that are misnamed.                */
 /* -------------------------------------------------------------------- */
@@ -993,6 +1026,65 @@ OGRErr OGRSpatialReference::morphToESRI()
     GetRoot()->applyRemapper( "DATUM", 
                               papszDatumMapping+2, papszDatumMapping+1, 3 );
 
+/* -------------------------------------------------------------------- */
+/*      Very specific handling for some well known geographic           */
+/*      coordinate systems.                                             */
+/* -------------------------------------------------------------------- */
+    OGR_SRSNode *poGeogCS = GetAttrNode( "GEOGCS" );
+    if( poGeogCS != NULL )
+    {
+        const char *pszGeogCSName = poGeogCS->GetChild(0)->GetValue();
+        const char *pszAuthName = GetAuthorityName("GEOGCS");
+        const char *pszUTMPrefix = NULL;
+        int nGCSCode = -1;
+        
+        if( pszAuthName != NULL && EQUAL(pszAuthName,"EPSG") )
+            nGCSCode = atoi(GetAuthorityCode("GEOGCS"));
+
+        if( nGCSCode == 4326 
+            || EQUAL(pszGeogCSName,"WGS84") 
+            || EQUAL(pszGeogCSName,"WGS 84") )
+        {
+            poGeogCS->GetChild(0)->SetValue( "GCS_WGS_1984" );
+            pszUTMPrefix = "WGS_1984";
+        }
+        else if( nGCSCode == 4267
+                 || EQUAL(pszGeogCSName,"NAD27") 
+                 || EQUAL(pszGeogCSName,"NAD 27") )
+        {
+            poGeogCS->GetChild(0)->SetValue( "GCS_North_American_1927" );
+            pszUTMPrefix = "NAD_1927";
+        }
+        else if( nGCSCode == 4269
+                 || EQUAL(pszGeogCSName,"NAD83") 
+                 || EQUAL(pszGeogCSName,"NAD 83") )
+        {
+            poGeogCS->GetChild(0)->SetValue( "GCS_North_American_1983" );
+            pszUTMPrefix = "NAD_1983";
+        }
+
+/* -------------------------------------------------------------------- */
+/*      Prepare very specific PROJCS names for UTM coordinate           */
+/*      systems.                                                        */
+/* -------------------------------------------------------------------- */
+        int bNorth, nZone;
+
+        nZone = GetUTMZone( &bNorth );
+        if( nZone > 0 && pszUTMPrefix != NULL )
+        {
+            char szUTMName[128];
+
+            if( bNorth )
+                sprintf( szUTMName, "%s_UTM_Zone_%dN", pszUTMPrefix, nZone );
+            else
+                sprintf( szUTMName, "%s_UTM_Zone_%dS", pszUTMPrefix, nZone );
+            
+            OGR_SRSNode *poProjCS = GetAttrNode( "PROJCS" );
+            if( poProjCS != NULL )
+                poProjCS->GetChild(0)->SetValue( szUTMName );
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Translate UNIT keywords that are misnamed, or even the wrong    */
 /*      case.                                                           */
@@ -1026,7 +1118,7 @@ OGRErr OGRSpatialReference::morphToESRI()
     }
 
 /* -------------------------------------------------------------------- */
-/*      Remap parameters used for Albers.                               */
+/*      Remap parameters used for Albers and Mercator.                  */
 /* -------------------------------------------------------------------- */
     pszProjection = GetAttrValue("PROJECTION");
     
@@ -1034,6 +1126,10 @@ OGRErr OGRSpatialReference::morphToESRI()
         GetRoot()->applyRemapper( 
             "PARAMETER", apszAlbersMapping + 1, apszAlbersMapping + 0, 2 );
 
+    if( pszProjection != NULL && EQUAL(pszProjection,"Mercator") )
+        GetRoot()->applyRemapper( 
+            "PARAMETER", apszAlbersMapping + 1, apszAlbersMapping + 0, 2 );
+
 /* -------------------------------------------------------------------- */
 /*      Convert SPHEROID name to use underscores instead of spaces.     */
 /* -------------------------------------------------------------------- */
@@ -1176,12 +1272,16 @@ OGRErr OGRSpatialReference::morphFromESRI()
     }
 
 /* -------------------------------------------------------------------- */
-/*      Remap albers parameters.                                        */
+/*      Remap albers and Mercator parameters.                           */
 /* -------------------------------------------------------------------- */
     if( pszProjection != NULL && EQUAL(pszProjection,"Albers") )
         GetRoot()->applyRemapper( 
             "PARAMETER", apszAlbersMapping + 0, apszAlbersMapping + 1, 2 );
 
+    if( pszProjection != NULL && EQUAL(pszProjection,"Mercator") )
+        GetRoot()->applyRemapper( 
+            "PARAMETER", apszMercatorMapping + 0, apszMercatorMapping + 1, 2 );
+
 /* -------------------------------------------------------------------- */
 /*      Translate PROJECTION keywords that are misnamed.                */
 /* -------------------------------------------------------------------- */
