@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_proj4.cpp,v 1.53 2005/02/28 15:01:56 fwarmerdam Exp $
+ * $Id: ogr_srs_proj4.cpp,v 1.56 2005/09/29 20:22:26 dmorissette Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  OGRSpatialReference interface to PROJ.4.
@@ -28,6 +28,16 @@
  ******************************************************************************
  *
  * $Log: ogr_srs_proj4.cpp,v $
+ * Revision 1.56  2005/09/29 20:22:26  dmorissette
+ * Improved support for MapInfo modified TM projections in exportToProj4()
+ * (Anthony D - MITAB bug 1155)
+ *
+ * Revision 1.55  2005/09/05 20:42:52  fwarmerdam
+ * ensure that non-geographic, non-projected srses return empty string
+ *
+ * Revision 1.54  2005/04/06 00:02:05  fwarmerdam
+ * various osr and oct functions now stdcall
+ *
  * Revision 1.53  2005/02/28 15:01:56  fwarmerdam
  * Applied patch to add +towgs84 parameter if one is well known for a given
  * EPSG GEOGCS, and no datum shifting info is available in the source defn.
@@ -196,7 +206,7 @@
 
 extern int EPSGGetWGS84Transform( int nGeogCS, double *padfTransform );
 
-CPL_CVSID("$Id: ogr_srs_proj4.cpp,v 1.53 2005/02/28 15:01:56 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr_srs_proj4.cpp,v 1.56 2005/09/29 20:22:26 dmorissette Exp $");
 
 /* -------------------------------------------------------------------- */
 /*      The following list comes from osrs/proj/src/pj_ellps.c          */
@@ -1018,7 +1028,8 @@ OGRErr OGRSpatialReference::importFromProj4( const char * pszProj4 )
 /*                          OSRExportToProj4()                          */
 /************************************************************************/
 
-OGRErr OSRExportToProj4( OGRSpatialReferenceH hSRS, char ** ppszReturn )
+OGRErr CPL_STDCALL OSRExportToProj4( OGRSpatialReferenceH hSRS, 
+                                     char ** ppszReturn )
 
 {
     *ppszReturn = NULL;
@@ -1072,16 +1083,16 @@ OGRErr OGRSpatialReference::exportToProj4( char ** ppszProj4 ) const
 /*      Handle the projection definition.                               */
 /* ==================================================================== */
 
-    if( IsLocal() )
-    {
-        /* LOCAL_CS (NonEarth): return an empty PROJ4 string */
-        *ppszProj4 = CPLStrdup("");
-        return OGRERR_NONE;
-    }
-    else if( pszProjection == NULL )
+    if( pszProjection == NULL && IsGeographic() )
     {
         sprintf( szProj4+strlen(szProj4), "+proj=longlat " );
     }
+    else if( pszProjection == NULL && !IsGeographic() )
+    {
+        // LOCAL_CS, or incompletely initialized coordinate systems.
+        *ppszProj4 = CPLStrdup("");
+        return OGRERR_NONE;
+    }
     else if( EQUAL(pszProjection,SRS_PT_CYLINDRICAL_EQUAL_AREA) )
     {
         sprintf( szProj4+strlen(szProj4),
@@ -1126,7 +1137,12 @@ OGRErr OGRSpatialReference::exportToProj4( char ** ppszProj4 ) const
                  GetNormProjParm(SRS_PP_FALSE_NORTHING,0.0) );
     }
 
-    else if( EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR) )
+    else if( EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR) ||
+             EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR_MI_21) ||
+             EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR_MI_22) ||
+             EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR_MI_23) ||
+             EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR_MI_24) ||
+             EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR_MI_25) )
     {
         int bNorth;
         int nZone = GetUTMZone( &bNorth );
