@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_usgs.cpp,v 1.5 2005/01/27 16:57:24 fwarmerdam Exp $
+ * $Id: ogr_srs_usgs.cpp,v 1.7 2005/09/29 11:50:37 dron Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  OGRSpatialReference translation to/from USGS georeferencing
@@ -29,6 +29,12 @@
  ******************************************************************************
  *
  * $Log: ogr_srs_usgs.cpp,v $
+ * Revision 1.7  2005/09/29 11:50:37  dron
+ * Calculate ellipsoid array size dynamically.
+ *
+ * Revision 1.6  2005/04/19 12:13:36  dron
+ * Do not fetch zone code from the projection parameters if the one supplied.
+ *
  * Revision 1.5  2005/01/27 16:57:24  fwarmerdam
  * provide fallback if EPSG lookup fails
  *
@@ -50,7 +56,7 @@
 #include "cpl_conv.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogr_srs_usgs.cpp,v 1.5 2005/01/27 16:57:24 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr_srs_usgs.cpp,v 1.7 2005/09/29 11:50:37 dron Exp $");
 
 /************************************************************************/
 /*  GCTP projection codes.                                              */
@@ -134,7 +140,7 @@ static long aoEllips[] =
     0// FIXME: WGS 60 --- skipped
 };
 
-#define NUMBER_OF_ELLIPSOIDS    31
+#define NUMBER_OF_ELLIPSOIDS    (sizeof(aoEllips)/sizeof(aoEllips[0]))
 
 /************************************************************************/
 /*                        USGSGetUOMLengthInfo()                        */
@@ -291,7 +297,7 @@ USGSGetEllipsoidInfo( int nCode, char ** ppszName,
 }
 
 /************************************************************************/
-/*                         OSRImportFromUSGSI()                         */
+/*                         OSRImportFromUSGS()                          */
 /************************************************************************/
 
 OGRErr OSRImportFromUSGS( OGRSpatialReferenceH hSRS, long iProjsys,
@@ -414,7 +420,7 @@ OGRErr OSRImportFromUSGS( OGRSpatialReferenceH hSRS, long iProjsys,
  *   where
  *
  *    Lon/Z     Longitude of any point in the UTM zone or zero.  If zero,
- *              a zone code must be specified. <br>
+ *              a zone code must be specified.
  *    Lat/Z     Latitude of any point in the UTM zone or zero.  If zero, a
  *              zone code must be specified.
  *    SMajor    Semi-major axis of ellipsoid.  If zero, Clarke 1866 in meters
@@ -551,14 +557,18 @@ OGRErr OGRSpatialReference::importFromUSGS( long iProjSys, long iZone,
         case UTM:
             {
                 int bNorth = TRUE;
-                if ( padfPrjParams[2] != 0.0 )
-                    iZone = (long) padfPrjParams[2];
-                else if ( padfPrjParams[0] != 0.0 && padfPrjParams[1] != 0.0 )
+
+                if ( !iZone )
                 {
-                    iZone = (long)(((CPLPackedDMSToDec(padfPrjParams[0])
-                                     + 180.0) / 6.0) + 1.0);
-                    if ( CPLPackedDMSToDec(padfPrjParams[0]) < 0 )
-                        bNorth = FALSE;
+                    if ( padfPrjParams[2] != 0.0 )
+                        iZone = (long) padfPrjParams[2];
+                    else if (padfPrjParams[0] != 0.0 && padfPrjParams[1] != 0.0)
+                    {
+                        iZone = (long)(((CPLPackedDMSToDec(padfPrjParams[0])
+                                         + 180.0) / 6.0) + 1.0);
+                        if ( CPLPackedDMSToDec(padfPrjParams[0]) < 0 )
+                            bNorth = FALSE;
+                    }
                 }
 
                 if ( iZone < 0 )
