@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_validate.cpp,v 1.6 2004/11/11 18:28:45 fwarmerdam Exp $
+ * $Id: ogr_srs_validate.cpp,v 1.7 2005/10/03 17:34:19 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implementation of the OGRSpatialReference::Validate() method and
@@ -29,6 +29,9 @@
  ******************************************************************************
  *
  * $Log: ogr_srs_validate.cpp,v $
+ * Revision 1.7  2005/10/03 17:34:19  fwarmerdam
+ * Added a bunch of missing definitions as per 937.
+ *
  * Revision 1.6  2004/11/11 18:28:45  fwarmerdam
  * added Bonne projection support
  *
@@ -52,7 +55,7 @@
 #include "ogr_spatialref.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogr_srs_validate.cpp,v 1.6 2004/11/11 18:28:45 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr_srs_validate.cpp,v 1.7 2005/10/03 17:34:19 fwarmerdam Exp $");
 
 /* why would fipszone and zone be paramers when they relate to a composite
    projection which renders done into a non-zoned projection? */
@@ -81,6 +84,9 @@ static char *papszParameters[] =
     SRS_PP_PERSPECTIVE_POINT_HEIGHT,
     SRS_PP_FIPSZONE,
     SRS_PP_ZONE,
+    SRS_PP_RECTIFIED_GRID_ANGLE,
+    SRS_PP_SATELLITE_HEIGHT,
+    SRS_PP_PSEUDO_STD_PARALLEL_1,
     NULL
 };
 
@@ -121,6 +127,7 @@ static char *papszProjectionSupported[] =
     SRS_PT_ORTHOGRAPHIC,
     SRS_PT_POLYCONIC,
     SRS_PT_VANDERGRINTEN,
+    SRS_PT_GEOSTATIONARY_SATELLITE,
     NULL
 };
 
@@ -375,6 +382,23 @@ static char *papszProjWithParms[] = {
     SRS_PP_FALSE_NORTHING,
     NULL,
 
+    SRS_PT_GEOSTATIONARY_SATELLITE,
+    SRS_PP_CENTRAL_MERIDIAN,
+    SRS_PP_SATELLITE_HEIGHT,
+    SRS_PP_FALSE_EASTING,
+    SRS_PP_FALSE_NORTHING,
+    NULL,
+
+    SRS_PT_KROVAK,
+    SRS_PP_LATITUDE_OF_CENTER,
+    SRS_PP_LONGITUDE_OF_CENTER,
+    SRS_PP_AZIMUTH,
+    SRS_PP_PSEUDO_STD_PARALLEL_1,
+    SRS_PP_SCALE_FACTOR,
+    SRS_PP_FALSE_EASTING,
+    SRS_PP_FALSE_NORTHING,
+    NULL,
+
     NULL
 };
 
