@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrct.cpp,v 1.26 2004/12/16 16:58:29 fwarmerdam Exp $
+ * $Id: ogrct.cpp,v 1.30 2005/09/05 20:45:16 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRSCoordinateTransformation class.
@@ -28,6 +28,18 @@
  ******************************************************************************
  *
  * $Log: ogrct.cpp,v $
+ * Revision 1.30  2005/09/05 20:45:16  fwarmerdam
+ * fail in intialize() if exportToProj4 result is empty
+ *
+ * Revision 1.29  2005/07/13 18:06:03  mbrudka
+ * Just for a case moved proj4 lock above static bTriedToLoad
+ *
+ * Revision 1.28  2005/06/08 19:38:14  fwarmerdam
+ * protect PROJ.4 use with a mutex
+ *
+ * Revision 1.27  2005/04/06 00:02:05  fwarmerdam
+ * various osr and oct functions now stdcall
+ *
  * Revision 1.26  2004/12/16 16:58:29  fwarmerdam
  * Use libproj.dylib on Apple
  *
@@ -112,12 +124,13 @@
 #include "cpl_error.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
+#include "cpl_multiproc.h"
 
 #ifdef PROJ_STATIC
 #include "proj_api.h"
 #endif
 
-CPL_CVSID("$Id: ogrct.cpp,v 1.26 2004/12/16 16:58:29 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrct.cpp,v 1.30 2005/09/05 20:45:16 fwarmerdam Exp $");
 
 /* ==================================================================== */
 /*      PROJ.4 interface stuff.                                         */
@@ -132,6 +145,8 @@ typedef struct { double u, v; } projUV;
 
 #endif
 
+static void *hPROJMutex = NULL;
+
 static projPJ       (*pfn_pj_init_plus)(const char *) = NULL;
 static projPJ       (*pfn_pj_init)(int, char**) = NULL;
 static projUV       (*pfn_pj_fwd)(projUV, projPJ) = NULL;
@@ -197,6 +212,7 @@ public:
 static int LoadProjLibrary()
 
 {
+    CPLMutexHolderD( &hPROJMutex );
     static int  bTriedToLoad = FALSE;
     const char *pszLibName = LIBNAME;
     
@@ -283,6 +299,7 @@ char *OCTProj4Normalize( const char *pszProj4Src )
 {
     char        *pszNewProj4Def, *pszCopy;
     projPJ      psPJSource = NULL;
+    CPLMutexHolderD( &hPROJMutex );
 
     if( !LoadProjLibrary() || pfn_pj_dalloc == NULL || pfn_pj_get_def == NULL )
         return CPLStrdup( pszProj4Src );
@@ -309,7 +326,8 @@ char *OCTProj4Normalize( const char *pszProj4Src )
 /*                 OCTDestroyCoordinateTransformation()                 */
 /************************************************************************/
 
-void OCTDestroyCoordinateTransformation( OGRCoordinateTransformationH hCT )
+void CPL_STDCALL
+OCTDestroyCoordinateTransformation( OGRCoordinateTransformationH hCT )
 
 {
     delete (OGRCoordinateTransformation *) hCT;
@@ -332,7 +350,7 @@ void OCTDestroyCoordinateTransformation( OGRCoordinateTransformationH hCT )
  * @return NULL on failure or a ready to use transformation object.
  */
 
-OGRCoordinateTransformation*
+OGRCoordinateTransformation*  
 OGRCreateCoordinateTransformation( OGRSpatialReference *poSource, 
                                    OGRSpatialReference *poTarget )
 
@@ -365,7 +383,8 @@ OGRCreateCoordinateTransformation( OGRSpatialReference *poSource,
 /*                   OCTNewCoordinateTransformation()                   */
 /************************************************************************/
 
-OGRCoordinateTransformationH OCTNewCoordinateTransformation(
+OGRCoordinateTransformationH CPL_STDCALL 
+OCTNewCoordinateTransformation(
     OGRSpatialReferenceH hSourceSRS, OGRSpatialReferenceH hTargetSRS )
 
 {
@@ -409,6 +428,8 @@ OGRProj4CT::~OGRProj4CT()
             delete poSRSTarget;
     }
 
+    CPLMutexHolderD( &hPROJMutex );
+
     if( psPJSource != NULL )
         pfn_pj_free( psPJSource );
 
@@ -424,6 +445,11 @@ int OGRProj4CT::Initialize( OGRSpatialReference * poSourceIn,
                             OGRSpatialReference * poTargetIn )
 
 {
+    CPLMutexHolderD( &hPROJMutex );
+
+    if( poSourceIn == NULL || poTargetIn == NULL )
+        return FALSE;
+
     poSRSSource = poSourceIn->Clone();
     poSRSTarget = poTargetIn->Clone();
 
@@ -475,6 +501,14 @@ int OGRProj4CT::Initialize( OGRSpatialReference * poSourceIn,
     if( poSRSSource->exportToProj4( &pszProj4Defn ) != OGRERR_NONE )
         return FALSE;
 
+    if( strlen(pszProj4Defn) == 0 )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "No PROJ.4 translation for source SRS, coordinate\n"
+                  "transformation initialization has failed." );
+        return FALSE;
+    }
+
     psPJSource = pfn_pj_init_plus( pszProj4Defn );
     
     if( psPJSource == NULL )
@@ -507,6 +541,14 @@ int OGRProj4CT::Initialize( OGRSpatialReference * poSourceIn,
     if( poSRSTarget->exportToProj4( &pszProj4Defn ) != OGRERR_NONE )
         return FALSE;
 
+    if( strlen(pszProj4Defn) == 0 )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "No PROJ.4 translation for destination SRS, coordinate\n"
+                  "transformation initialization has failed." );
+        return FALSE;
+    }
+
     psPJTarget = pfn_pj_init_plus( pszProj4Defn );
     
     if( psPJTarget == NULL )
@@ -574,8 +616,8 @@ int OGRProj4CT::Transform( int nCount, double *x, double *y, double *z )
 /*                            OCTTransform()                            */
 /************************************************************************/
 
-int OCTTransform( OGRCoordinateTransformationH hTransform,
-                  int nCount, double *x, double *y, double *z )
+int CPL_STDCALL OCTTransform( OGRCoordinateTransformationH hTransform,
+                              int nCount, double *x, double *y, double *z )
 
 {
     return ((OGRCoordinateTransformation*) hTransform)->
@@ -607,6 +649,7 @@ int OGRProj4CT::TransformEx( int nCount, double *x, double *y, double *z,
 /* -------------------------------------------------------------------- */
 /*      Do the transformation using PROJ.4.                             */
 /* -------------------------------------------------------------------- */
+    CPLMutexHolderD( &hPROJMutex );
     err = pfn_pj_transform( psPJSource, psPJTarget, nCount, 1, x, y, z );
 
 /* -------------------------------------------------------------------- */
@@ -679,9 +722,9 @@ int OGRProj4CT::TransformEx( int nCount, double *x, double *y, double *z,
 /*                           OCTTransformEx()                           */
 /************************************************************************/
 
-int OCTTransformEx( OGRCoordinateTransformationH hTransform,
-                    int nCount, double *x, double *y, double *z,
-                    int *pabSuccess )
+int CPL_STDCALL OCTTransformEx( OGRCoordinateTransformationH hTransform,
+                                int nCount, double *x, double *y, double *z,
+                                int *pabSuccess )
 
 {
     return ((OGRCoordinateTransformation*) hTransform)->
