@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrfeaturedefn.cpp,v 1.12 2003/09/18 21:31:34 warmerda Exp $
+ * $Id: ogrfeaturedefn.cpp,v 1.13 2005/09/21 00:51:59 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRFeatureDefn class implementation.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrfeaturedefn.cpp,v $
+ * Revision 1.13  2005/09/21 00:51:59  fwarmerdam
+ * Added Release
+ *
  * Revision 1.12  2003/09/18 21:31:34  warmerda
  * Fixed docs for GetGeomType().
  *
@@ -69,7 +72,7 @@
 #include "ogr_api.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrfeaturedefn.cpp,v 1.12 2003/09/18 21:31:34 warmerda Exp $");
+CPL_CVSID("$Id: ogrfeaturedefn.cpp,v 1.13 2005/09/21 00:51:59 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           OGRFeatureDefn()                           */
@@ -105,10 +108,9 @@ OGRFeatureDefn::OGRFeatureDefn( const char * pszName )
  * Create a new feature definition object to held the field definitions.
  *
  * The OGRFeatureDefn maintains a reference count, but this starts at
- * zero.  It is mainly intended to represent a count of OGRFeature's
- * based on this definition.
+ * zero, and should normally be incremented by the owner.
  *
- * This function is the same as the CPP method 
+ * This function is the same as the C++ method 
  * OGRFeatureDefn::OGRFeatureDefn().
  *
  * @param pszName the name to be assigned to this layer/class.  It does not
@@ -154,7 +156,7 @@ OGRFeatureDefn::~OGRFeatureDefn()
  * Destroy a feature definition object and release all memory 
  * associated with it. 
  *
- * This function is the same as the CPP method 
+ * This function is the same as the C++ method 
  * OGRFeatureDefn::~OGRFeatureDefn().
  *
  * @param hDefn handle to the feature definition to be destroyed.
@@ -166,6 +168,41 @@ void OGR_FD_Destroy( OGRFeatureDefnH hDefn )
     delete (OGRFeatureDefn *) hDefn;
 }
 
+/************************************************************************/
+/*                              Release()                               */
+/************************************************************************/
+
+/**
+ * \fn void OGRFeatureDefn::Release();
+ *
+ * Drop a reference to this object, and destroy if no longer referenced.
+ */
+
+void OGRFeatureDefn::Release()
+
+{
+    if( this && Dereference() == 0 )
+        delete this;
+}
+
+/************************************************************************/
+/*                           OGR_FD_Release()                           */
+/************************************************************************/
+
+/**
+ * Drop a reference, and destroy if unreferenced.
+ *
+ * This function is the same as the C++ method OGRFeatureDefn::Release().
+ *
+ * @param hDefn handle to the feature definition to be released.
+ */
+
+void OGR_FD_Release( OGRFeatureDefnH hDefn )
+
+{
+    ((OGRFeatureDefn *) hDefn)->Release();
+}
+
 /************************************************************************/
 /*                               Clone()                                */
 /************************************************************************/
@@ -216,7 +253,7 @@ OGRFeatureDefn *OGRFeatureDefn::Clone()
 /**
  * Get name of the OGRFeatureDefn passed as an argument.
  *
- * This function is the same as the CPP method OGRFeatureDefn::GetName().
+ * This function is the same as the C++ method OGRFeatureDefn::GetName().
  *
  * @param hDefn handle to the feature definition to get the name from.
  * @return the name.  This name is internal and should not be modified, or
@@ -249,7 +286,7 @@ const char *OGR_FD_GetName( OGRFeatureDefnH hDefn )
 /**
  * Fetch number of fields on the passed feature definition.
  *
- * This function is the same as the CPP OGRFeatureDefn::GetFieldCount().
+ * This function is the same as the C++ OGRFeatureDefn::GetFieldCount().
  *
  * @param hDefn handle to the feature definition to get the fields count from.
  * @return count of fields.
@@ -294,7 +331,7 @@ OGRFieldDefn *OGRFeatureDefn::GetFieldDefn( int iField )
 /**
  * Fetch field definition of the passed feature definition.
  *
- * This function is the same as the CPP method 
+ * This function is the same as the C++ method 
  * OGRFeatureDefn::GetFieldDefn().
  *
  * @param hDefn handle to the feature definition to get the field definition
@@ -348,7 +385,7 @@ void OGRFeatureDefn::AddFieldDefn( OGRFieldDefn * poNewDefn )
  * objects in existance based on this OGRFeatureDefn.  The OGRFieldDefn
  * passed in is copied, and remains the responsibility of the caller.
  *
- * This function is the same as the CPP method OGRFeatureDefn::AddFieldDefn.
+ * This function is the same as the C++ method OGRFeatureDefn::AddFieldDefn.
  *
  * @param hDefn handle to the feature definition to add the field definition
  * to.
@@ -388,7 +425,7 @@ void OGR_FD_AddFieldDefn( OGRFeatureDefnH hDefn, OGRFieldDefnH hNewField )
 /**
  * Fetch the geometry base type of the passed feature definition.
  *
- * This function is the same as the CPP method OGRFeatureDefn::GetGeomType().
+ * This function is the same as the C++ method OGRFeatureDefn::GetGeomType().
  *
  * @param hDefn handle to the feature definition to get the geometry type from.
  * @return the base type for all geometry related to this definition.
@@ -436,7 +473,7 @@ void OGRFeatureDefn::SetGeomType( OGRwkbGeometryType eNewType )
  * any geometry type.  The geometry type should generally not be changed
  * after any OGRFeatures have been created against this definition. 
  *
- * This function is the same as the CPP method OGRFeatureDefn::SetGeomType().
+ * This function is the same as the C++ method OGRFeatureDefn::SetGeomType().
  *
  * @param hDefn handle to the layer or feature definition to set the geometry
  * type to.
@@ -476,7 +513,7 @@ void OGR_FD_SetGeomType( OGRFeatureDefnH hDefn, OGRwkbGeometryType eType )
  * The reference count is used keep track of the number of OGRFeature
  * objects referencing this definition. 
  *
- * This function is the same as the CPP method OGRFeatureDefn::Reference().
+ * This function is the same as the C++ method OGRFeatureDefn::Reference().
  *
  * @param hDefn handle to the feature definition on witch OGRFeature are
  * based on.
@@ -510,7 +547,7 @@ int OGR_FD_Reference( OGRFeatureDefnH hDefn )
 /**
  * Decrements the reference count by one.
  *
- * This function is the same as the CPP method OGRFeatureDefn::Dereference().
+ * This function is the same as the C++ method OGRFeatureDefn::Dereference().
  *
  * @param hDefn handle to the feature definition on witch OGRFeature are
  * based on. 
@@ -544,7 +581,7 @@ int OGR_FD_Dereference( OGRFeatureDefnH hDefn )
 /**
  * Fetch current reference count.
  *
- * This function is the same as the CPP method 
+ * This function is the same as the C++ method 
  * OGRFeatureDefn::GetReferenceCount().
  *
  * @param hDefn hanlde to the feature definition on witch OGRFeature are
@@ -597,7 +634,7 @@ int OGRFeatureDefn::GetFieldIndex( const char * pszFieldName )
  * The field index of the first field matching the passed field name (case
  * insensitively) is returned.
  *
- * This function is the same as the CPP method OGRFeatureDefn::GetFieldIndex.
+ * This function is the same as the C++ method OGRFeatureDefn::GetFieldIndex.
  *
  * @param hDefn handle to the feature definition to get field index from. 
  * @param pszFieldName the field name to search for.
