@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeometry.cpp,v 1.28 2005/02/22 12:39:32 fwarmerdam Exp $
+ * $Id: ogrgeometry.cpp,v 1.32 2005/09/21 00:50:53 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements a few base methods on OGRGeometry.
@@ -28,6 +28,18 @@
  ******************************************************************************
  *
  * $Log: ogrgeometry.cpp,v $
+ * Revision 1.32  2005/09/21 00:50:53  fwarmerdam
+ * Release SRS on geometry destruction
+ *
+ * Revision 1.31  2005/07/20 01:43:51  fwarmerdam
+ * upgraded OGR geometry dimension handling
+ *
+ * Revision 1.30  2005/04/18 15:42:17  fwarmerdam
+ * fix geos exception catching
+ *
+ * Revision 1.29  2005/04/06 20:43:00  fwarmerdam
+ * fixed a variety of method signatures for documentation
+ *
  * Revision 1.28  2005/02/22 12:39:32  fwarmerdam
  * Rename Equal to Equals, and Intersect to Intersects.
  * Intersects() now always uses GEOS if available.  Also, Intersects()
@@ -130,7 +142,7 @@
 #  include "geos/geosAlgorithm.h"
 #endif
 
-CPL_CVSID("$Id: ogrgeometry.cpp,v 1.28 2005/02/22 12:39:32 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrgeometry.cpp,v 1.32 2005/09/21 00:50:53 fwarmerdam Exp $");
 
 int OGRGeometry::bGenerate_DB2_V72_BYTE_ORDER = FALSE;
 
@@ -142,6 +154,7 @@ OGRGeometry::OGRGeometry()
 
 {
     poSRS = NULL;
+    nCoordDimension = 2;
 }
 
 /************************************************************************/
@@ -152,9 +165,7 @@ OGRGeometry::~OGRGeometry()
 
 {
     if( poSRS != NULL )
-    {
-        poSRS->Dereference();
-    }
+        poSRS->Release();
 }
 
 
@@ -232,7 +243,7 @@ void OGRGeometry::assignSpatialReference( OGRSpatialReference * poSR )
 
 {
     if( poSRS != NULL )
-        poSRS->Dereference();
+        poSRS->Release();
 
     poSRS = poSR;
     if( poSRS != NULL )
@@ -526,7 +537,7 @@ OGRErr OGR_G_Transform( OGRGeometryH hGeom,
 }
 
 /**
- * \fn int OGRGeometry::getDimension();
+ * \fn int OGRGeometry::getDimension() const;
  *
  * Get the dimension of this object.
  *
@@ -564,9 +575,10 @@ int OGR_G_GetDimension( OGRGeometryH hGeom )
     return ((OGRGeometry *) hGeom)->getDimension();
 }
 
+/************************************************************************/
+/*                       getCoordinateDimension()                       */
+/************************************************************************/
 /**
- * \fn int OGRGeometry::getCoordinateDimension();
- *
  * Get the dimension of the coordinates in this object.
  *
  * This method corresponds to the SFCOM IGeometry::GetDimension() method.
@@ -577,6 +589,12 @@ int OGR_G_GetDimension( OGRGeometryH hGeom )
  * specified within a two dimensional space.
  */
 
+int OGRGeometry::getCoordinateDimension() const
+
+{
+    return nCoordDimension;
+}
+
 /************************************************************************/
 /*                    OGR_G_GetCoordinateDimension()                    */
 /************************************************************************/
@@ -601,9 +619,40 @@ int OGR_G_GetCoordinateDimension( OGRGeometryH hGeom )
     return ((OGRGeometry *) hGeom)->getCoordinateDimension();
 }
 
+/************************************************************************/
+/*                       setCoordinateDimension()                       */
+/************************************************************************/
 
 /**
- * \fn OGRBoolean OGRGeometry::IsEmpty();
+ * Set the coordinate dimension. 
+ *
+ * This method sets the explicit coordinate dimension.  Setting the coordinate
+ * dimension of a geometry to 2 should zero out any existing Z values.  Setting
+ * the dimension of a geometry collection will not necessarily affect the
+ * children geometries. 
+ *
+ * @param nNewDimension New coordinate dimension value, either 2 or 3.
+ */
+
+void OGRGeometry::setCoordinateDimension( int nNewDimension )
+
+{
+    nCoordDimension = nNewDimension;
+}
+
+/************************************************************************/
+/*                    OGR_G_SetCoordinateDimension()                    */
+/************************************************************************/
+
+void OGR_G_SetCoordinateDimension( OGRGeometryH hGeom, int nNewDimension)
+
+{
+    ((OGRGeometry *) hGeom)->setCoordinateDimension( nNewDimension );
+}
+
+
+/**
+ * \fn OGRBoolean OGRGeometry::IsEmpty() const;
  *
  * Returns TRUE (non-zero) if the object has no points.  Normally this
  * returns FALSE except between when an object is instantiated and points
@@ -617,7 +666,7 @@ int OGR_G_GetCoordinateDimension( OGRGeometryH hGeom )
  */
 
 /**
- * \fn OGRBoolean OGRGeometry::IsSimple();
+ * \fn OGRBoolean OGRGeometry::IsSimple() const;
  *
  * Returns TRUE if the geometry is simple.
  * 
@@ -679,7 +728,7 @@ int OGR_G_Equal( OGRGeometryH hGeom, OGRGeometryH hOther )
 
 
 /**
- * \fn int OGRGeometry::WkbSize();
+ * \fn int OGRGeometry::WkbSize() const;
  *
  * Returns size of related binary representation.
  *
@@ -719,7 +768,7 @@ int OGR_G_WkbSize( OGRGeometryH hGeom )
 }
 
 /**
- * \fn void OGRGeometry::getEnvelope(OGREnvelope *psEnvelope);
+ * \fn void OGRGeometry::getEnvelope(OGREnvelope *psEnvelope) const;
  *
  * Computes and returns the bounding envelope for this geometry in the
  * passed psEnvelope structure.
@@ -801,7 +850,7 @@ OGRErr OGR_G_ImportFromWkb( OGRGeometryH hGeom,
 
 /**
  * \fn OGRErr OGRGeometry::exportToWkb( OGRwkbByteOrder eByteOrder,
-                                        unsigned char * pabyData );
+                                        unsigned char * pabyData ) const;
  *
  * Convert a geometry into well known binary format.
  *
@@ -897,7 +946,7 @@ OGRErr OGR_G_ImportFromWkt( OGRGeometryH hGeom, char ** ppszSrcText )
 }
 
 /**
- * \fn OGRErr OGRGeometry::exportToWkt( char ** ppszDstText );
+ * \fn OGRErr OGRGeometry::exportToWkt( char ** ppszDstText ) const;
  *
  * Convert a geometry into well known text format.
  *
@@ -915,8 +964,6 @@ OGRErr OGR_G_ImportFromWkt( OGRGeometryH hGeom, char ** ppszSrcText )
 /*                         OGR_G_ExportToWkt()                          */
 /************************************************************************/
 /**
- * \fn OGRErr ;
- *
  * Convert a geometry into well known text format.
  *
  * This function relates to the SFCOM IWks::ExportToWKT() method.
@@ -937,7 +984,7 @@ OGRErr OGR_G_ExportToWkt( OGRGeometryH hGeom, char **ppszSrcText )
 }
 
 /**
- * \fn OGRwkbGeometryType OGRGeometry::getGeometryType();
+ * \fn OGRwkbGeometryType OGRGeometry::getGeometryType() const;
  *
  * Fetch geometry type.
  *
@@ -973,7 +1020,7 @@ OGRwkbGeometryType OGR_G_GetGeometryType( OGRGeometryH hGeom )
 }
 
 /**
- * \fn const char * OGRGeometry::getGeometryName();
+ * \fn const char * OGRGeometry::getGeometryName() const;
  *
  * Fetch WKT name for geometry type.
  *
@@ -1007,7 +1054,7 @@ const char *OGR_G_GetGeometryName( OGRGeometryH hGeom )
 }
 
 /**
- * \fn OGRGeometry *OGRGeometry::clone();
+ * \fn OGRGeometry *OGRGeometry::clone() const;
  *
  * Make a copy of this object.
  *
@@ -1111,8 +1158,6 @@ void OGR_G_Empty( OGRGeometryH hGeom )
 /************************************************************************/
 
 /**
- * \fn const char *OGRGeometryTypeToName(OGRwkbGeometryType)
- *
  * Fetch a human readable name corresponding to an OGRwkBGeometryType value.
  * The returned value should not be modified, or freed by the application.
  *
@@ -1300,12 +1345,13 @@ geos::Geometry *OGRGeometry::exportToGEOS() const
         geosGeometry = geosWktReader.read( oWKT );
         return geosGeometry;
     }
-    catch( geos::GEOSException &e )
+    catch( geos::GEOSException *e )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "GEOSException: %s", 
-                  e.toString().c_str() );
+                  e->toString().c_str() );
 
+        delete e;
         return NULL;
     }
 
@@ -2317,3 +2363,4 @@ void OGR_G_CloseRings( OGRGeometryH hGeom )
 {
     ((OGRGeometry *) hGeom)->closeRings();
 }
+
