@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeometrycollection.cpp,v 1.29 2005/02/22 12:38:01 fwarmerdam Exp $
+ * $Id: ogrgeometrycollection.cpp,v 1.31 2005/07/20 01:43:51 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRGeometryCollection class.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogrgeometrycollection.cpp,v $
+ * Revision 1.31  2005/07/20 01:43:51  fwarmerdam
+ * upgraded OGR geometry dimension handling
+ *
+ * Revision 1.30  2005/07/12 17:34:00  fwarmerdam
+ * updated to produce proper empty syntax and consume either
+ *
  * Revision 1.29  2005/02/22 12:38:01  fwarmerdam
  * rename Equal/Intersect to Equals/Intersects
  *
@@ -120,7 +126,7 @@
 #include "ogr_geometry.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrgeometrycollection.cpp,v 1.29 2005/02/22 12:38:01 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrgeometrycollection.cpp,v 1.31 2005/07/20 01:43:51 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                       OGRGeometryCollection()                        */
@@ -213,28 +219,6 @@ int OGRGeometryCollection::getDimension() const
     return 2; // This isn't strictly correct.  It should be based on members.
 }
 
-/************************************************************************/
-/*                       getCoordinateDimension()                       */
-/*                                                                      */
-/*      Returning 2 is a dubious solution.  This should at least be     */
-/*      overridden by some of the subclasses.                           */
-/************************************************************************/
-
-int OGRGeometryCollection::getCoordinateDimension() const
-
-{
-    if( nCoordinateDimension == 0 )
-    {
-        ((OGRGeometryCollection *)this)->nCoordinateDimension = 2;
-
-        for( int i = 0; i < nGeomCount; i++ )
-            if( papoGeoms[i]->getCoordinateDimension() == 3 )
-                ((OGRGeometryCollection *)this)->nCoordinateDimension = 3;
-    }
-
-    return nCoordinateDimension;
-}
-
 /************************************************************************/
 /*                            flattenTo2D()                             */
 /************************************************************************/
@@ -662,6 +646,13 @@ OGRErr OGRGeometryCollection::importFromWkt( char ** ppszInput )
 /*      list of objects.                                                */
 /* -------------------------------------------------------------------- */
     pszInput = OGRWktReadToken( pszInput, szToken );
+
+    if( EQUAL(szToken,"EMPTY") )
+    {
+        *ppszInput = (char *) pszInput;
+        return OGRERR_NONE;
+    }
+
     if( szToken[0] != '(' )
         return OGRERR_CORRUPT_DATA;
 
@@ -733,7 +724,7 @@ OGRErr OGRGeometryCollection::exportToWkt( char ** ppszDstText ) const
 
     if( getNumGeometries() == 0 )
     {
-        *ppszDstText = CPLStrdup("GEOMETRYCOLLECTION(EMPTY)");
+        *ppszDstText = CPLStrdup("GEOMETRYCOLLECTION EMPTY");
         return OGRERR_NONE;
     }
 
@@ -889,3 +880,19 @@ void OGRGeometryCollection::closeRings()
             ((OGRPolygon *) papoGeoms[iGeom])->closeRings();
     }
 }
+
+/************************************************************************/
+/*                       setCoordinateDimension()                       */
+/************************************************************************/
+
+void OGRGeometryCollection::setCoordinateDimension( int nNewDimension )
+
+{
+    for( int iGeom = 0; iGeom < nGeomCount; iGeom++ )
+    {
+        papoGeoms[iGeom]->setCoordinateDimension( nNewDimension );
+    }
+
+    OGRGeometry::setCoordinateDimension( nNewDimension );
+}
+
