@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgeometryfactory.cpp,v 1.24 2005/02/22 17:17:30 hobu Exp $
+ * $Id: ogrgeometryfactory.cpp,v 1.26 2005/08/05 20:28:55 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Factory for converting geometry to and from well known binary
@@ -29,6 +29,12 @@
  ******************************************************************************
  *
  * $Log: ogrgeometryfactory.cpp,v $
+ * Revision 1.26  2005/08/05 20:28:55  fwarmerdam
+ * pass nbytes in OGR_G_CreateFromWkb()
+ *
+ * Revision 1.25  2005/04/18 15:42:17  fwarmerdam
+ * fix geos exception catching
+ *
  * Revision 1.24  2005/02/22 17:17:30  hobu
  * typo in the haveGEOS() method
  *
@@ -111,7 +117,7 @@
 #include <assert.h>
 #include "ogr_geos.h"
 
-CPL_CVSID("$Id: ogrgeometryfactory.cpp,v 1.24 2005/02/22 17:17:30 hobu Exp $");
+CPL_CVSID("$Id: ogrgeometryfactory.cpp,v 1.26 2005/08/05 20:28:55 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           createFromWkb()                            */
@@ -249,7 +255,9 @@ OGRErr OGRGeometryFactory::createFromWkb(unsigned char *pabyData,
  *             created geometry object.  This may be NULL.
  * @param phGeometry the newly created geometry object will 
  * be assigned to the indicated handle on return.  This will be NULL in case
- *                  of failure.
+ * of failure.
+ * @param nBytes the number of bytes of data available in pabyData, or -1
+ * if it is not known, but assumed to be sufficient.
  *
  * @return OGRERR_NONE if all goes well, otherwise any of
  * OGRERR_NOT_ENOUGH_DATA, OGRERR_UNSUPPORTED_GEOMETRY_TYPE, or
@@ -258,12 +266,14 @@ OGRErr OGRGeometryFactory::createFromWkb(unsigned char *pabyData,
 
 OGRErr CPL_DLL OGR_G_CreateFromWkb( unsigned char *pabyData, 
                                     OGRSpatialReferenceH hSRS,
-                                    OGRGeometryH *phGeometry )
+                                    OGRGeometryH *phGeometry, 
+                                    int nBytes )
 
 {
     return OGRGeometryFactory::createFromWkb( pabyData, 
                                               (OGRSpatialReference *) hSRS,
-                                              (OGRGeometry **) phGeometry );
+                                              (OGRGeometry **) phGeometry,
+                                              nBytes );
 }
 
 /************************************************************************/
@@ -821,12 +831,13 @@ OGRGeometryFactory::createFromGEOS( const geos::Geometry *geosGeom )
     {
         oWKT = oWKTWriter.write( geosGeom );
     }
-    catch( geos::GEOSException &e )
+    catch( geos::GEOSException *e )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "GEOSException: %s", 
-                  e.toString().c_str() );
+                  e->toString().c_str() );
 
+        delete e;
         return NULL;
     }
 
