@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrlinestring.cpp,v 1.45 2005/02/22 12:38:01 fwarmerdam Exp $
+ * $Id: ogrlinestring.cpp,v 1.51 2005/09/21 07:45:53 osemykin Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRLineString geometry class.
@@ -28,6 +28,24 @@
  ******************************************************************************
  *
  * $Log: ogrlinestring.cpp,v $
+ * Revision 1.51  2005/09/21 07:45:53  osemykin
+ * Fixed getPoints() for support Z coordinate NULL buffer
+ *
+ * Revision 1.50  2005/09/17 12:11:30  osemykin
+ * added OGRLineString::getPoints
+ *
+ * Revision 1.49  2005/07/22 19:28:56  fwarmerdam
+ * reserve extra space for WKT encoding of weird numbers
+ *
+ * Revision 1.48  2005/07/20 01:43:51  fwarmerdam
+ * upgraded OGR geometry dimension handling
+ *
+ * Revision 1.47  2005/07/12 17:34:00  fwarmerdam
+ * updated to produce proper empty syntax and consume either
+ *
+ * Revision 1.46  2005/03/25 06:31:12  fwarmerdam
+ * added addSubLineString
+ *
  * Revision 1.45  2005/02/22 12:38:01  fwarmerdam
  * rename Equal/Intersect to Equals/Intersects
  *
@@ -172,7 +190,7 @@
 #include "ogr_p.h"
 #include <assert.h>
 
-CPL_CVSID("$Id: ogrlinestring.cpp,v 1.45 2005/02/22 12:38:01 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrlinestring.cpp,v 1.51 2005/09/21 07:45:53 osemykin Exp $");
 
 /************************************************************************/
 /*                           OGRLineString()                            */
@@ -249,9 +267,8 @@ OGRGeometry *OGRLineString::clone() const
 
     poNewLineString->assignSpatialReference( getSpatialReference() );
     poNewLineString->setPoints( nPointCount, paoPoints, padfZ );
-
-    // notdef: not 3D
-
+    poNewLineString->setCoordinateDimension( getCoordinateDimension() );
+    
     return poNewLineString;
 }
 
@@ -265,7 +282,6 @@ void OGRLineString::empty()
     setNumPoints( 0 );
 }
 
-
 /************************************************************************/
 /*                            getDimension()                            */
 /************************************************************************/
@@ -277,16 +293,17 @@ int OGRLineString::getDimension() const
 }
 
 /************************************************************************/
-/*                       getCoordinateDimension()                       */
+/*                       setCoordinateDimension()                       */
 /************************************************************************/
 
-int OGRLineString::getCoordinateDimension() const
+void OGRLineString::setCoordinateDimension( int nNewDimension )
 
 {
-    if( padfZ != NULL )
-        return 3;
-    else
-        return 2;
+    nCoordDimension = nNewDimension;
+    if( nNewDimension == 2 )
+        Make2D();
+    else if( nNewDimension == 3 )
+        Make3D();
 }
 
 /************************************************************************/
@@ -314,6 +331,7 @@ void OGRLineString::Make2D()
         OGRFree( padfZ );
         padfZ = NULL;
     }
+    nCoordDimension = 2;
 }
 
 /************************************************************************/
@@ -330,6 +348,7 @@ void OGRLineString::Make3D()
         else
             padfZ = (double *) OGRCalloc(sizeof(double),nPointCount);
     }
+    nCoordDimension = 3;
 }
 
 /************************************************************************/
@@ -355,7 +374,7 @@ void    OGRLineString::getPoint( int i, OGRPoint * poPoint ) const
     poPoint->setX( paoPoints[i].x );
     poPoint->setY( paoPoints[i].y );
 
-    if( getCoordinateDimension() == 3 )
+    if( getCoordinateDimension() == 3 && padfZ != NULL )
         poPoint->setZ( padfZ[i] );
 }
 
@@ -366,7 +385,7 @@ void    OGRLineString::getPoint( int i, OGRPoint * poPoint ) const
 double OGRLineString::getZ( int i ) const
 
 {
-    if( padfZ != NULL && i >= 0 && i < nPointCount )
+    if( padfZ != NULL && i >= 0 && i < nPointCount && nCoordDimension >= 3 )
         return( padfZ[i] );
     else
         return 0.0;
@@ -470,6 +489,9 @@ void OGRLineString::setPoint( int iPoint, OGRPoint * poPoint )
 void OGRLineString::setPoint( int iPoint, double xIn, double yIn, double zIn )
 
 {
+    if( getCoordinateDimension() == 2 )
+        Make3D();
+
     if( iPoint >= nPointCount )
         setNumPoints( iPoint+1 );
 
@@ -487,6 +509,16 @@ void OGRLineString::setPoint( int iPoint, double xIn, double yIn, double zIn )
     }
 }
 
+void OGRLineString::setPoint( int iPoint, double xIn, double yIn )
+
+{
+    if( iPoint >= nPointCount )
+        setNumPoints( iPoint+1 );
+
+    paoPoints[iPoint].x = xIn;
+    paoPoints[iPoint].y = yIn;
+}
+
 /************************************************************************/
 /*                              addPoint()                              */
 /************************************************************************/
@@ -531,6 +563,12 @@ void OGRLineString::addPoint( double x, double y, double z )
     setPoint( nPointCount, x, y, z );
 }
 
+void OGRLineString::addPoint( double x, double y )
+
+{
+    setPoint( nPointCount, x, y );
+}
+
 /************************************************************************/
 /*                             setPoints()                              */
 /************************************************************************/
@@ -559,26 +597,11 @@ void OGRLineString::setPoints( int nPointsIn, OGRRawPoint * paoPointsIn,
 /* -------------------------------------------------------------------- */
 /*      Check 2D/3D.                                                    */
 /* -------------------------------------------------------------------- */
-    if( padfZ != NULL )
+    if( padfZ == NULL && getCoordinateDimension() > 2 )
     {
-        int     i, bIs3D = FALSE;
-
-        for( i = 0; i < nPointsIn && !bIs3D; i++ )
-        {
-            if( padfZ[i] != 0.0 )
-                bIs3D = TRUE;
-        }
-
-        if( !bIs3D )
-            padfZ = NULL;
+        Make2D();
     }
-
-    if( padfZ == NULL )
-    {
-        if( this->padfZ != NULL )
-            Make2D();
-    }
-    else
+    else if( padfZ )
     {
         Make3D();
         memcpy( this->padfZ, padfZ, sizeof(double) * nPointsIn );
@@ -613,20 +636,6 @@ void OGRLineString::setPoints( int nPointsIn, double * padfX, double * padfY,
 /* -------------------------------------------------------------------- */
 /*      Check 2D/3D.                                                    */
 /* -------------------------------------------------------------------- */
-    if( padfZ != NULL )
-    {
-        int     bIs3D = FALSE;
-
-        for( i = 0; i < nPointsIn && !bIs3D; i++ )
-        {
-            if( padfZ[i] != 0.0 )
-                bIs3D = TRUE;
-        }
-
-        if( !bIs3D )
-            padfZ = NULL;
-    }
-
     if( padfZ == NULL )
         Make2D();
     else
@@ -647,6 +656,130 @@ void OGRLineString::setPoints( int nPointsIn, double * padfX, double * padfY,
         memcpy( this->padfZ, padfZ, sizeof(double) * nPointsIn );
 }
 
+/************************************************************************/
+/*                          getPoints()                                 */
+/************************************************************************/
+
+/**
+ * Returns all points of line string.
+ *
+ * This method copies all points into user list. This list must be at
+ * least sizeof(OGRRawPoint) * OGRGeometry::getNumPoints() byte in size.
+ * It also copies all Z coordinates.
+ *
+ * There is no SFCOM analog to this method.
+ *
+ * @param paoPointsOut a buffer into which the points is written.
+ * @param padfZ the Z values that go with the points (optional, may be NULL).
+ */
+
+void OGRLineString::getPoints( OGRRawPoint * paoPointsOut, double * padfZ )
+{
+    if ( ! paoPointsOut )
+        return;
+        
+    memcpy( paoPointsOut, paoPoints, sizeof(OGRRawPoint) * nPointCount );
+
+/* -------------------------------------------------------------------- */
+/*      Check 2D/3D.                                                    */
+/* -------------------------------------------------------------------- */
+    if( padfZ )
+    {
+        if ( this->padfZ )
+            memcpy( padfZ, this->padfZ, sizeof(double) * nPointCount );
+        else
+            memset( padfZ, 0, sizeof(double) * nPointCount );
+    }
+}
+
+
+/************************************************************************/
+/*                          addSubLineString()                          */
+/************************************************************************/
+
+/**
+ * Add a segment of another linestring to this one.
+ *
+ * Adds the request range of vertices to the end of this line string
+ * in an efficient manner.  If the nStartVertex is larger than the
+ * nEndVertex then the vertices will be reversed as they are copied. 
+ *
+ * @param poOtherLine the other OGRLineString. 
+ * @param nStartVertex the first vertex to copy, defaults to 0 to start
+ * with the first vertex in the other linestring. 
+ * @param nEndVertex the last vertex to copy, defaults to -1 indicating 
+ * the last vertex of the other line string. 
+ */
+
+void OGRLineString::addSubLineString( const OGRLineString *poOtherLine, 
+                                      int nStartVertex, int nEndVertex )
+
+{
+/* -------------------------------------------------------------------- */
+/*      Do a bit of argument defaulting and validation.                 */
+/* -------------------------------------------------------------------- */
+    if( nEndVertex == -1 )
+        nEndVertex = poOtherLine->getNumPoints() - 1;
+
+    if( nStartVertex < 0 || nEndVertex < 0 
+        || nStartVertex >= poOtherLine->getNumPoints() 
+        || nEndVertex >= poOtherLine->getNumPoints() )
+    {
+        CPLAssert( FALSE );
+        return;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Grow this linestring to hold the additional points.             */
+/* -------------------------------------------------------------------- */
+    int nOldPoints = nPointCount;
+    int nPointsToAdd = ABS(nEndVertex-nStartVertex) + 1;
+
+    setNumPoints( nPointsToAdd + nOldPoints );
+
+/* -------------------------------------------------------------------- */
+/*      Copy the x/y points - forward copies use memcpy.                */
+/* -------------------------------------------------------------------- */
+    if( nEndVertex >= nStartVertex )
+    {
+        memcpy( paoPoints + nOldPoints, 
+                poOtherLine->paoPoints + nStartVertex, 
+                sizeof(OGRRawPoint) * nPointsToAdd );
+        if( poOtherLine->padfZ != NULL )
+        {
+            Make3D();
+            memcpy( padfZ + nOldPoints, poOtherLine->padfZ + nStartVertex,
+                    sizeof(double) * nPointsToAdd );
+        }
+    }
+    
+/* -------------------------------------------------------------------- */
+/*      Copy the x/y points - reverse copies done double by double.     */
+/* -------------------------------------------------------------------- */
+    else
+    {
+        int i;
+
+        for( i = 0; i < nPointsToAdd; i++ )
+        {
+            paoPoints[i+nOldPoints].x = 
+                poOtherLine->paoPoints[nStartVertex-i].x;
+            paoPoints[i+nOldPoints].y = 
+                poOtherLine->paoPoints[nStartVertex-i].y;
+        }
+
+        if( poOtherLine->padfZ != NULL )
+        {
+            Make3D();
+
+            for( i = 0; i < nPointsToAdd; i++ )
+            {
+                padfZ[i+nOldPoints] = poOtherLine->padfZ[nStartVertex-i];
+            }
+        }
+    }
+}
+
 /************************************************************************/
 /*                           importFromWkb()                            */
 /*                                                                      */
@@ -848,11 +981,17 @@ OGRErr OGRLineString::importFromWkt( char ** ppszInput )
         return OGRERR_CORRUPT_DATA;
 
 /* -------------------------------------------------------------------- */
-/*      Check for EMPTY ... but treat like a point at 0,0.              */
+/*      Check for EMPTY or (EMPTY).                                     */
 /* -------------------------------------------------------------------- */
     const char *pszPreScan;
 
     pszPreScan = OGRWktReadToken( pszInput, szToken );
+    if( EQUAL(szToken,"EMPTY") )
+    {
+        *ppszInput = (char *) pszPreScan;
+        return OGRERR_NONE;
+    }
+
     if( !EQUAL(szToken,"(") )
         return OGRERR_CORRUPT_DATA;
     
@@ -872,7 +1011,7 @@ OGRErr OGRLineString::importFromWkt( char ** ppszInput )
 /* -------------------------------------------------------------------- */
 /*      Read the point list.                                            */
 /* -------------------------------------------------------------------- */
-    int                 nMaxPoint = 0;
+    int      nMaxPoint = 0;
 
     nPointCount = 0;
 
@@ -882,6 +1021,11 @@ OGRErr OGRLineString::importFromWkt( char ** ppszInput )
         return OGRERR_CORRUPT_DATA;
 
     *ppszInput = (char *) pszInput;
+
+    if( padfZ == NULL )
+        nCoordDimension = 2;
+    else
+        nCoordDimension = 3;
     
     return OGRERR_NONE;
 }
@@ -896,7 +1040,7 @@ OGRErr OGRLineString::importFromWkt( char ** ppszInput )
 OGRErr OGRLineString::exportToWkt( char ** ppszDstText ) const
 
 {
-    int         nMaxString = nPointCount * 20 * 3 + 20;
+    int         nMaxString = nPointCount * 40 * 3 + 20;
     int         nRetLen = 0;
 
 /* -------------------------------------------------------------------- */
@@ -904,7 +1048,7 @@ OGRErr OGRLineString::exportToWkt( char ** ppszDstText ) const
 /* -------------------------------------------------------------------- */
     if( nPointCount == 0 )
     {
-        *ppszDstText = CPLStrdup("LINESTRING(EMPTY)");
+        *ppszDstText = CPLStrdup("LINESTRING EMPTY");
         return OGRERR_NONE;
     }
 
@@ -940,12 +1084,14 @@ OGRErr OGRLineString::exportToWkt( char ** ppszDstText ) const
             OGRMakeWktCoordinate( *ppszDstText + nRetLen,
                                   paoPoints[i].x,
                                   paoPoints[i].y,
-                                  padfZ[i] );
+                                  padfZ[i],
+                                  nCoordDimension );
         else
             OGRMakeWktCoordinate( *ppszDstText + nRetLen,
                                   paoPoints[i].x,
                                   paoPoints[i].y,
-                                  0.0 );
+                                  0.0,
+                                  nCoordDimension );
 
         nRetLen += strlen(*ppszDstText + nRetLen);
     }
