@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmultilinestring.cpp,v 1.13 2004/02/21 15:36:14 warmerda Exp $
+ * $Id: ogrmultilinestring.cpp,v 1.14 2005/07/12 17:34:00 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRMultiLineString class.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrmultilinestring.cpp,v $
+ * Revision 1.14  2005/07/12 17:34:00  fwarmerdam
+ * updated to produce proper empty syntax and consume either
+ *
  * Revision 1.13  2004/02/21 15:36:14  warmerda
  * const correctness updates for geometry: bug 289
  *
@@ -74,7 +77,7 @@
 #include "ogr_geometry.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrmultilinestring.cpp,v 1.13 2004/02/21 15:36:14 warmerda Exp $");
+CPL_CVSID("$Id: ogrmultilinestring.cpp,v 1.14 2005/07/12 17:34:00 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                        OGRMultiLineString()                          */
@@ -181,6 +184,13 @@ OGRErr OGRMultiLineString::importFromWkt( char ** ppszInput )
 /*      list of linestrings.                                            */
 /* -------------------------------------------------------------------- */
     pszInput = OGRWktReadToken( pszInput, szToken );
+
+    if( EQUAL(szToken,"EMPTY") )
+    {
+        *ppszInput = (char *) pszInput;
+        return OGRERR_NONE;
+    }
+
     if( szToken[0] != '(' )
         return OGRERR_CORRUPT_DATA;
 
@@ -276,7 +286,7 @@ OGRErr OGRMultiLineString::exportToWkt( char ** ppszDstText ) const
 
     if( getNumGeometries() == 0 )
     {
-        *ppszDstText = CPLStrdup("MULTILINESTRING(EMPTY)");
+        *ppszDstText = CPLStrdup("MULTILINESTRING EMPTY");
         return OGRERR_NONE;
     }
 
