@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmultipoint.cpp,v 1.18 2004/02/21 15:36:14 warmerda Exp $
+ * $Id: ogrmultipoint.cpp,v 1.20 2005/07/20 01:43:51 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRMultiPoint class.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogrmultipoint.cpp,v $
+ * Revision 1.20  2005/07/20 01:43:51  fwarmerdam
+ * upgraded OGR geometry dimension handling
+ *
+ * Revision 1.19  2005/07/12 17:34:00  fwarmerdam
+ * updated to produce proper empty syntax and consume either
+ *
  * Revision 1.18  2004/02/21 15:36:14  warmerda
  * const correctness updates for geometry: bug 289
  *
@@ -90,7 +96,7 @@
 #include "ogr_p.h"
 #include <assert.h>
 
-CPL_CVSID("$Id: ogrmultipoint.cpp,v 1.18 2004/02/21 15:36:14 warmerda Exp $");
+CPL_CVSID("$Id: ogrmultipoint.cpp,v 1.20 2005/07/20 01:43:51 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           OGRMultiPoint()                            */
@@ -176,7 +182,7 @@ OGRErr OGRMultiPoint::exportToWkt( char ** ppszDstText ) const
 
     if( getNumGeometries() == 0 )
     {
-        *ppszDstText = CPLStrdup("MULTIPOINT(EMPTY)");
+        *ppszDstText = CPLStrdup("MULTIPOINT EMPTY");
         return OGRERR_NONE;
     }
 
@@ -201,16 +207,11 @@ OGRErr OGRMultiPoint::exportToWkt( char ** ppszDstText ) const
             *ppszDstText = (char *) CPLRealloc(*ppszDstText,nMaxString);
         }
         
-        if( poPoint->getCoordinateDimension() == 3 )
-            OGRMakeWktCoordinate( *ppszDstText + nRetLen,
-                                  poPoint->getX(), 
-                                  poPoint->getY(),
-                                  poPoint->getZ() );
-        else
-            OGRMakeWktCoordinate( *ppszDstText + nRetLen,
-                                  poPoint->getX(), 
-                                  poPoint->getY(),
-                                  0.0 );
+        OGRMakeWktCoordinate( *ppszDstText + nRetLen,
+                              poPoint->getX(), 
+                              poPoint->getY(),
+                              poPoint->getZ(),
+                              poPoint->getCoordinateDimension() );
     }
 
     strcat( *ppszDstText+nRetLen, ")" );
@@ -255,6 +256,13 @@ OGRErr OGRMultiPoint::importFromWkt( char ** ppszInput )
     while( *pszPreScan == ' ' || *pszPreScan == '\t' )
         pszPreScan++;
 
+    // Handle the proper EMPTY syntax.
+    if( EQUALN(pszPreScan,"EMPTY",5) )
+    {
+        *ppszInput = (char *) pszPreScan+5;
+        return OGRERR_NONE;
+    }
+
     // Skip outer bracket.
     if( *pszPreScan != '(' )
         return OGRERR_CORRUPT_DATA;
