@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpoint.cpp,v 1.29 2005/02/22 12:38:01 fwarmerdam Exp $
+ * $Id: ogrpoint.cpp,v 1.35 2005/09/12 01:43:40 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The Point geometry class.
@@ -28,6 +28,24 @@
  ******************************************************************************
  *
  * $Log: ogrpoint.cpp,v $
+ * Revision 1.35  2005/09/12 01:43:40  fwarmerdam
+ * ensure Z is initialized to zero in 2D constructor
+ *
+ * Revision 1.34  2005/08/04 17:18:59  fwarmerdam
+ * now have separate 2D and 3D OGRPoint constructors
+ *
+ * Revision 1.33  2005/07/22 20:39:36  fwarmerdam
+ * Fixed getGeometryType to look at coordinate dimension, expand wkt buffer.
+ *
+ * Revision 1.32  2005/07/20 01:43:51  fwarmerdam
+ * upgraded OGR geometry dimension handling
+ *
+ * Revision 1.31  2005/07/12 17:34:00  fwarmerdam
+ * updated to produce proper empty syntax and consume either
+ *
+ * Revision 1.30  2005/04/06 20:43:00  fwarmerdam
+ * fixed a variety of method signatures for documentation
+ *
  * Revision 1.29  2005/02/22 12:38:01  fwarmerdam
  * rename Equal/Intersect to Equals/Intersects
  *
@@ -121,7 +139,7 @@
 #include "ogr_p.h"
 #include <assert.h>
 
-CPL_CVSID("$Id: ogrpoint.cpp,v 1.29 2005/02/22 12:38:01 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrpoint.cpp,v 1.35 2005/09/12 01:43:40 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                              OGRPoint()                              */
@@ -151,6 +169,22 @@ OGRPoint::OGRPoint( double xIn, double yIn, double zIn )
     x = xIn;
     y = yIn;
     z = zIn;
+    nCoordDimension = 3;
+}
+
+/************************************************************************/
+/*                              OGRPoint()                              */
+/*                                                                      */
+/*      Initialize point to value.                                      */
+/************************************************************************/
+
+OGRPoint::OGRPoint( double xIn, double yIn )
+
+{
+    x = xIn;
+    y = yIn;
+    z = 0.0;
+    nCoordDimension = 2;
 }
 
 /************************************************************************/
@@ -197,19 +231,6 @@ int OGRPoint::getDimension() const
     return 0;
 }
 
-/************************************************************************/
-/*                       getCoordinateDimension()                       */
-/************************************************************************/
-
-int OGRPoint::getCoordinateDimension() const
-
-{
-    if( z == 0 )
-        return 2;
-    else
-        return 3;
-}
-
 /************************************************************************/
 /*                          getGeometryType()                           */
 /************************************************************************/
@@ -217,7 +238,7 @@ int OGRPoint::getCoordinateDimension() const
 OGRwkbGeometryType OGRPoint::getGeometryType() const
 
 {
-    if( z == 0 )
+    if( nCoordDimension < 3 )
         return wkbPoint;
     else
         return wkbPoint25D;
@@ -241,6 +262,7 @@ void OGRPoint::flattenTo2D()
 
 {
     z = 0;
+    nCoordDimension = 2;
 }
 
 /************************************************************************/
@@ -253,7 +275,7 @@ void OGRPoint::flattenTo2D()
 int OGRPoint::WkbSize() const
 
 {
-    if( z == 0)
+    if( nCoordDimension != 3 )
         return 21;
     else
         return 29;
@@ -320,10 +342,13 @@ OGRErr OGRPoint::importFromWkb( unsigned char * pabyData,
         {
             CPL_SWAPDOUBLE( &z );
         }
-        
+        nCoordDimension = 3;
     }
     else
+    {
         z = 0;
+        nCoordDimension = 2;
+    }
 
     return OGRERR_NONE;
 }
@@ -360,7 +385,7 @@ OGRErr  OGRPoint::exportToWkb( OGRwkbByteOrder eByteOrder,
 /* -------------------------------------------------------------------- */
     memcpy( pabyData+5, &x, 16 );
 
-    if( z != 0 )
+    if( nCoordDimension == 3 )
     {
         memcpy( pabyData + 5 + 16, &z, 8 );
     }
@@ -373,7 +398,7 @@ OGRErr  OGRPoint::exportToWkb( OGRwkbByteOrder eByteOrder,
         CPL_SWAPDOUBLE( pabyData + 5 );
         CPL_SWAPDOUBLE( pabyData + 5 + 8 );
 
-        if( z != 0 )
+        if( nCoordDimension == 3 )
             CPL_SWAPDOUBLE( pabyData + 5 + 16 );
     }
 
@@ -407,6 +432,13 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
     const char *pszPreScan;
 
     pszPreScan = OGRWktReadToken( pszInput, szToken );
+    if( EQUAL(szToken,"EMPTY") )
+    {
+        *ppszInput = (char *) pszInput;
+        empty();
+        return OGRERR_NONE;
+    }
+
     if( !EQUAL(szToken,"(") )
         return OGRERR_CORRUPT_DATA;
 
@@ -444,9 +476,12 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
 
     if( padfZ != NULL )
     {
-        z = padfZ[0];
+        z = padfZ[0];						
+        nCoordDimension = 3;
         CPLFree( padfZ );
     }
+    else
+        nCoordDimension = 2;
 
     *ppszInput = (char *) pszInput;
     
@@ -463,10 +498,10 @@ OGRErr OGRPoint::importFromWkt( char ** ppszInput )
 OGRErr OGRPoint::exportToWkt( char ** ppszDstText ) const
 
 {
-    char        szTextEquiv[100];
+    char        szTextEquiv[140];
     char        szCoordinate[80];
 
-    OGRMakeWktCoordinate(szCoordinate, x, y, z);
+    OGRMakeWktCoordinate(szCoordinate, x, y, z, nCoordDimension );
     sprintf( szTextEquiv, "POINT (%s)", szCoordinate );
     *ppszDstText = CPLStrdup( szTextEquiv );
     
@@ -487,7 +522,7 @@ void OGRPoint::getEnvelope( OGREnvelope * psEnvelope ) const
 
 
 /**
- * \fn double OGRPoint::getX();
+ * \fn double OGRPoint::getX() const;
  *
  * Fetch X coordinate.
  *
@@ -497,7 +532,7 @@ void OGRPoint::getEnvelope( OGREnvelope * psEnvelope ) const
  */
 
 /**
- * \fn double OGRPoint::getY();
+ * \fn double OGRPoint::getY() const;
  *
  * Fetch Y coordinate.
  *
@@ -507,7 +542,7 @@ void OGRPoint::getEnvelope( OGREnvelope * psEnvelope ) const
  */
 
 /**
- * \fn double OGRPoint::getZ();
+ * \fn double OGRPoint::getZ() const;
  *
  * Fetch Z coordinate.
  *
