@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpolygon.cpp,v 1.30 2005/02/22 12:38:01 fwarmerdam Exp $
+ * $Id: ogrpolygon.cpp,v 1.35 2005/07/22 19:30:25 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRPolygon geometry class.
@@ -28,6 +28,21 @@
  ******************************************************************************
  *
  * $Log: ogrpolygon.cpp,v $
+ * Revision 1.35  2005/07/22 19:30:25  fwarmerdam
+ * Document get_Area().
+ *
+ * Revision 1.34  2005/07/20 02:34:09  fwarmerdam
+ * maintain dimension better
+ *
+ * Revision 1.33  2005/07/20 01:43:51  fwarmerdam
+ * upgraded OGR geometry dimension handling
+ *
+ * Revision 1.32  2005/07/12 17:34:00  fwarmerdam
+ * updated to produce proper empty syntax and consume either
+ *
+ * Revision 1.31  2005/04/06 20:43:00  fwarmerdam
+ * fixed a variety of method signatures for documentation
+ *
  * Revision 1.30  2005/02/22 12:38:01  fwarmerdam
  * rename Equal/Intersect to Equals/Intersects
  *
@@ -129,7 +144,7 @@
 #  include "geos/geosAlgorithm.h"
 #endif
 
-CPL_CVSID("$Id: ogrpolygon.cpp,v 1.30 2005/02/22 12:38:01 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrpolygon.cpp,v 1.35 2005/07/22 19:30:25 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                             OGRPolygon()                             */
@@ -219,22 +234,6 @@ int OGRPolygon::getDimension() const
     return 2;
 }
 
-/************************************************************************/
-/*                       getCoordinateDimension()                       */
-/************************************************************************/
-
-int OGRPolygon::getCoordinateDimension() const
-
-{
-    for( int iRing = 0; iRing < nRingCount; iRing++ )
-    {
-        if( papoRings[iRing]->getCoordinateDimension() == 3 )
-            return 3;
-    }
-
-    return 2;
-}
-
 /************************************************************************/
 /*                            flattenTo2D()                             */
 /************************************************************************/
@@ -378,6 +377,9 @@ void OGRPolygon::addRing( OGRLinearRing * poNewRing )
     papoRings[nRingCount] = new OGRLinearRing( poNewRing );
 
     nRingCount++;
+
+    if( poNewRing->getCoordinateDimension() == 3 )
+        nCoordDimension = 3;
 }
 
 /************************************************************************/
@@ -406,6 +408,10 @@ void OGRPolygon::addRingDirectly( OGRLinearRing * poNewRing )
     papoRings[nRingCount] = poNewRing;
 
     nRingCount++;
+
+
+    if( poNewRing->getCoordinateDimension() == 3 )
+        nCoordDimension = 3;
 }
 
 /************************************************************************/
@@ -473,6 +479,11 @@ OGRErr OGRPolygon::importFromWkb( unsigned char * pabyData,
     else
         b3D = pabyData[1] & 0x80 || pabyData[3] & 0x80;
 
+    if( b3D )
+        nCoordDimension = 3;
+    else
+        nCoordDimension = 2;
+
 /* -------------------------------------------------------------------- */
 /*      Do we already have some rings?                                  */
 /* -------------------------------------------------------------------- */
@@ -622,16 +633,20 @@ OGRErr OGRPolygon::importFromWkt( char ** ppszInput )
 
 /* -------------------------------------------------------------------- */
 /*      The next character should be a ( indicating the start of the    */
-/*      list of rings.                                                  */
+/*      list of rings.  We also need to support POLYGON EMPTY and       */
+/*      POLYGON(EMPTY).                                                 */
 /* -------------------------------------------------------------------- */
     pszInput = OGRWktReadToken( pszInput, szToken );
+
+    if( EQUAL(szToken,"EMPTY") )
+    {
+        *ppszInput = (char *) pszInput;
+        return OGRERR_NONE;
+    }
+
     if( szToken[0] != '(' )
         return OGRERR_CORRUPT_DATA;
 
-/* -------------------------------------------------------------------- */
-/*      If the next token is EMPTY, then verify that we have proper     */
-/*      EMPTY format will a trailing closing bracket.                   */
-/* -------------------------------------------------------------------- */
     OGRWktReadToken( pszInput, szToken );
     if( EQUAL(szToken,"EMPTY") )
     {
@@ -654,6 +669,8 @@ OGRErr OGRPolygon::importFromWkt( char ** ppszInput )
     OGRRawPoint *paoPoints = NULL;
     int         nMaxPoints = 0, nMaxRings = 0;
     double      *padfZ = NULL;
+
+    nCoordDimension = 2;
     
     do
     {
@@ -689,6 +706,9 @@ OGRErr OGRPolygon::importFromWkt( char ** ppszInput )
 
         nRingCount++;
 
+        if( padfZ )
+            nCoordDimension = 3;
+
 /* -------------------------------------------------------------------- */
 /*      Read the delimeter following the ring.                          */
 /* -------------------------------------------------------------------- */
@@ -701,7 +721,7 @@ OGRErr OGRPolygon::importFromWkt( char ** ppszInput )
 /* -------------------------------------------------------------------- */
     CPLFree( paoPoints );
     CPLFree( padfZ );
-   
+
     if( szToken[0] != ')' )
         return OGRERR_CORRUPT_DATA;
     
@@ -728,7 +748,7 @@ OGRErr OGRPolygon::exportToWkt( char ** ppszDstText ) const
 /* -------------------------------------------------------------------- */
     if( nRingCount == 0 )
     {
-        *ppszDstText = CPLStrdup("POLYGON(EMPTY)");
+        *ppszDstText = CPLStrdup("POLYGON EMPTY");
         return OGRERR_NONE;
     }
 
@@ -739,6 +759,7 @@ OGRErr OGRPolygon::exportToWkt( char ** ppszDstText ) const
 
     for( iRing = 0; iRing < nRingCount; iRing++ )
     {
+        papoRings[iRing]->setCoordinateDimension( getCoordinateDimension() );
         eErr = papoRings[iRing]->exportToWkt( &(papszRings[iRing]) );
         if( eErr != OGRERR_NONE )
             return eErr;
@@ -1008,3 +1029,17 @@ double OGRPolygon::get_Area() const
 
     return dfArea;
 }
+
+/************************************************************************/
+/*                       setCoordinateDimension()                       */
+/************************************************************************/
+
+void OGRPolygon::setCoordinateDimension( int nNewDimension )
+
+{
+    for( int iRing = 0; iRing < nRingCount; iRing++ )
+        papoRings[iRing]->setCoordinateDimension( nNewDimension );
+
+    OGRGeometry::setCoordinateDimension( nNewDimension );
+}
+
