@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrcsvdatasource.cpp,v 1.6 2004/08/17 21:03:03 warmerda Exp $
+ * $Id: ogrcsvdatasource.cpp,v 1.8 2005/09/21 01:01:01 fwarmerdam Exp $
  *
  * Project:  CSV Translator
  * Purpose:  Implements OGRCSVDataSource class
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogrcsvdatasource.cpp,v $
+ * Revision 1.8  2005/09/21 01:01:01  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.7  2005/06/20 17:54:04  fwarmerdam
+ * added support for external csvt file
+ *
  * Revision 1.6  2004/08/17 21:03:03  warmerda
  * Avoid leak of papoLayers array.
  *
@@ -53,7 +59,7 @@
 #include "cpl_string.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogrcsvdatasource.cpp,v 1.6 2004/08/17 21:03:03 warmerda Exp $");
+CPL_CVSID("$Id: ogrcsvdatasource.cpp,v 1.8 2005/09/21 01:01:01 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                          OGRCSVDataSource()                          */
@@ -152,27 +158,30 @@ int OGRCSVDataSource::Open( const char * pszFilename, int bUpdateIn,
 
     for( i = 0; papszNames != NULL && papszNames[i] != NULL; i++ )
     {
-        const char *pszSubFilename = 
+        CPLString oSubFilename = 
             CPLFormFilename( pszFilename, papszNames[i], NULL );
 
         if( EQUAL(papszNames[i],".") || EQUAL(papszNames[i],"..") )
             continue;
 
-        if( VSIStat( pszSubFilename, &sStatBuf ) != 0 
+        if( VSIStat( oSubFilename, &sStatBuf ) != 0 
             || !VSI_ISREG(sStatBuf.st_mode) 
-            || !EQUAL(pszSubFilename+strlen(pszSubFilename)-4,".csv") )
+            || !EQUAL(CPLGetExtension(oSubFilename),"csv") )
         {
             nNotCSVCount++;
             continue;
         }
 
-        if( !OpenTable( pszSubFilename ) )
+        if( !OpenTable( oSubFilename ) )
         {
+            CSLDestroy( papszNames );
             nNotCSVCount++;
             return FALSE;
         }
     }
 
+    CSLDestroy( papszNames );
+
 /* -------------------------------------------------------------------- */
 /*      We presume that this is indeed intended to be a CSV             */
 /*      datasource if over half the files were .csv files.              */
@@ -227,7 +236,7 @@ int OGRCSVDataSource::OpenTable( const char * pszFilename )
                                              sizeof(void*) * nLayers);
     
     papoLayers[nLayers-1] = 
-        new OGRCSVLayer( CPLGetBasename(pszFilename), fp, FALSE, bUpdate );
+        new OGRCSVLayer( CPLGetBasename(pszFilename), fp, pszFilename, FALSE, bUpdate );
 
     return TRUE;
 }
@@ -299,7 +308,7 @@ OGRCSVDataSource::CreateLayer( const char *pszLayerName,
     papoLayers = (OGRCSVLayer **) CPLRealloc(papoLayers, 
                                              sizeof(void*) * nLayers);
     
-    papoLayers[nLayers-1] = new OGRCSVLayer( pszLayerName, fp, TRUE, TRUE );
+    papoLayers[nLayers-1] = new OGRCSVLayer( pszLayerName, fp, pszFilename, TRUE, TRUE );
 
 /* -------------------------------------------------------------------- */
 /*      Was a partiuclar CRLF order requested?                          */
