@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrcsvlayer.cpp,v 1.4 2005/02/02 20:30:10 fwarmerdam Exp $
+ * $Id: ogrcsvlayer.cpp,v 1.6 2005/09/21 01:01:01 fwarmerdam Exp $
  *
  * Project:  CSV Translator
  * Purpose:  Implements OGRCSVLayer class.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogrcsvlayer.cpp,v $
+ * Revision 1.6  2005/09/21 01:01:01  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.5  2005/06/20 17:54:04  fwarmerdam
+ * added support for external csvt file
+ *
  * Revision 1.4  2005/02/02 20:30:10  fwarmerdam
  * added m_nFeaturesRead support
  *
@@ -47,7 +53,7 @@
 #include "cpl_string.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogrcsvlayer.cpp,v 1.4 2005/02/02 20:30:10 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrcsvlayer.cpp,v 1.6 2005/09/21 01:01:01 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                            OGRCSVLayer()                             */
@@ -57,7 +63,7 @@ CPL_CVSID("$Id: ogrcsvlayer.cpp,v 1.4 2005/02/02 20:30:10 fwarmerdam Exp $");
 /************************************************************************/
 
 OGRCSVLayer::OGRCSVLayer( const char *pszLayerNameIn, 
-                          FILE * fp, int bNew, int bInWriteMode )
+                          FILE * fp, const char *pszFilename, int bNew, int bInWriteMode )
 
 {
     fpCSV = fp;
@@ -71,6 +77,7 @@ OGRCSVLayer::OGRCSVLayer( const char *pszLayerNameIn,
     nNextFID = 1;
 
     poFeatureDefn = new OGRFeatureDefn( pszLayerNameIn );
+    poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( wkbNone );
 
 /* -------------------------------------------------------------------- */
@@ -130,6 +137,25 @@ OGRCSVLayer::OGRCSVLayer( const char *pszLayerNameIn,
     if( !bHasFieldNames )
         VSIRewind( fpCSV );
 
+
+/* -------------------------------------------------------------------- */
+/*      Search a csvt file for types                                */
+/* -------------------------------------------------------------------- */
+    char** papszFieldTypes = NULL;
+    if (!bNew) {
+      char* dname = strdup(CPLGetDirname(pszFilename));
+      char* fname = strdup(CPLGetBasename(pszFilename));
+      FILE* fpCSVT = fopen(CPLFormFilename(dname, fname, ".csvt"), "r");
+      free(dname);
+      free(fname);
+      if (fpCSVT!=NULL) {
+        VSIRewind(fpCSVT);
+        papszFieldTypes = CSVReadParseLine(fpCSVT);
+        fclose(fpCSVT);
+      }
+    }
+    
+
 /* -------------------------------------------------------------------- */
 /*      Build field definitions.                                        */
 /* -------------------------------------------------------------------- */
@@ -155,8 +181,16 @@ OGRCSVLayer::OGRCSVLayer( const char *pszLayerNameIn,
             pszFieldName = szFieldNameBuffer;
             sprintf( szFieldNameBuffer, "field_%d", iField+1 );
         }
-            
-        OGRFieldDefn oField( pszFieldName, OFTString );
+
+        OGRFieldDefn oField(pszFieldName, OFTString);
+        if (papszFieldTypes!=NULL && iField<CSLCount(papszFieldTypes)) {
+          if (!strcmp(papszFieldTypes[iField], "Integer"))
+            oField.SetType(OFTInteger);
+          else if (!strcmp(papszFieldTypes[iField], "Real"))
+            oField.SetType(OFTReal);
+          else if (!strcmp(papszFieldTypes[iField], "String"))
+            oField.SetType(OFTString);
+        }
 
         poFeatureDefn->AddFieldDefn( &oField );
     }
@@ -178,7 +212,7 @@ OGRCSVLayer::~OGRCSVLayer()
                   poFeatureDefn->GetName() );
     }
 
-    delete poFeatureDefn;
+    poFeatureDefn->Release();
     
     VSIFClose( fpCSV );
 }
