@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: dgnread.cpp,v 1.44 2004/01/23 16:15:22 warmerda Exp $
+ * $Id: dgnread.cpp,v 1.46 2004/06/02 15:53:00 warmerda Exp $
  *
  * Project:  Microstation DGN Access Library
  * Purpose:  DGN Access Library element reading code.
@@ -28,6 +28,13 @@
  ******************************************************************************
  *
  * $Log: dgnread.cpp,v $
+ * Revision 1.46  2004/06/02 15:53:00  warmerda
+ * improve divide by zero error checking
+ *
+ * Revision 1.45  2004/04/06 04:24:51  warmerda
+ * Always make any new color table encountered the default color
+ * table when it is read.
+ *
  * Revision 1.44  2004/01/23 16:15:22  warmerda
  * DGN_LEVEL_SYMBOLOGY has no disphdr, DGNParseCore() wont read negative attr
  *
@@ -168,7 +175,7 @@
 
 #include "dgnlibp.h"
 
-CPL_CVSID("$Id: dgnread.cpp,v 1.44 2004/01/23 16:15:22 warmerda Exp $");
+CPL_CVSID("$Id: dgnread.cpp,v 1.46 2004/06/02 15:53:00 warmerda Exp $");
 
 static DGNElemCore *DGNParseTCB( DGNInfo * );
 static DGNElemCore *DGNParseColorTable( DGNInfo * );
@@ -461,7 +468,11 @@ static DGNElemCore *DGNProcessElement( DGNInfo *psDGN, int nType, int nLevel )
               
               psCell->xscale = sqrt(a2 + c2) / 214748;
               psCell->yscale = sqrt(b*b + d*d) / 214748;
-              psCell->rotation = acos(a / sqrt(a2 + c2));
+              if( (a2 + c2) <= 0.0 )
+                  psCell->rotation = 0.0;
+              else
+                  psCell->rotation = acos(a / sqrt(a2 + c2));
+
               if (b <= 0)
                   psCell->rotation = psCell->rotation * 180 / PI;
               else
@@ -1201,11 +1212,14 @@ static DGNElemCore *DGNParseColorTable( DGNInfo * psDGN )
 
     memcpy( psColorTable->color_info[255], psDGN->abyElem+38, 3 );
     memcpy( psColorTable->color_info, psDGN->abyElem+41, 765 ); 
-    if( !psDGN->got_color_table )
-    {
-        memcpy( psDGN->color_table, psColorTable->color_info, 768 );
-        psDGN->got_color_table = 1;
-    }
+
+    // We used to only install a color table as the default color
+    // table if it was the first in the file.  But apparently we should
+    // really be using the last one.  This doesn't necessarily accomplish
+    // that either if the elements are being read out of order but it will
+    // usually do better at least. 
+    memcpy( psDGN->color_table, psColorTable->color_info, 768 );
+    psDGN->got_color_table = 1;
     
     return psElement;
 }
