@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrdgnlayer.cpp,v 1.33 2005/02/22 12:54:50 fwarmerdam Exp $
+ * $Id: ogrdgnlayer.cpp,v 1.35 2005/09/21 01:00:46 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRDGNLayer class.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogrdgnlayer.cpp,v $
+ * Revision 1.35  2005/09/21 01:00:46  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.34  2005/07/20 02:33:03  fwarmerdam
+ * Set feature geometry dimension based on source file.
+ *
  * Revision 1.33  2005/02/22 12:54:50  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -138,7 +144,7 @@
 #include "ogr_featurestyle.h"
 #include "ogr_api.h"
 
-CPL_CVSID("$Id: ogrdgnlayer.cpp,v 1.33 2005/02/22 12:54:50 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrdgnlayer.cpp,v 1.35 2005/09/21 01:00:46 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           OGRDGNLayer()                              */
@@ -177,6 +183,7 @@ OGRDGNLayer::OGRDGNLayer( const char * pszName, DGNHandle hDGN,
 /*      Create the feature definition.                                  */
 /* -------------------------------------------------------------------- */
     poFeatureDefn = new OGRFeatureDefn( pszName );
+    poFeatureDefn->Reference();
     
     OGRFieldDefn        oField( "", OFTInteger );
 
@@ -290,7 +297,8 @@ OGRDGNLayer::~OGRDGNLayer()
     }
 
     delete poEvalFeature;
-    delete poFeatureDefn;
+
+    poFeatureDefn->Release();
 
     CPLFree( pszLinkFormat );
 }
@@ -733,6 +741,13 @@ OGRFeature *OGRDGNLayer::ElementToFeature( DGNElemCore *psElement )
       default:
         break;
     }
+    
+/* -------------------------------------------------------------------- */
+/*      Fixup geometry dimension.                                       */
+/* -------------------------------------------------------------------- */
+    if( poFeature->GetGeometryRef() != NULL )
+        poFeature->GetGeometryRef()->setCoordinateDimension( 
+            DGNGetDimension( hDGN ) );
 
     return poFeature;
 }
