@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrdodsgrid.cpp,v 1.6 2005/02/22 12:57:39 fwarmerdam Exp $
+ * $Id: ogrdodsgrid.cpp,v 1.7 2005/09/21 01:00:28 fwarmerdam Exp $
  *
  * Project:  OGR/DODS Interface
  * Purpose:  Implements OGRDODSGridLayer class, which implements the
@@ -29,6 +29,9 @@
  ******************************************************************************
  *
  * $Log: ogrdodsgrid.cpp,v $
+ * Revision 1.7  2005/09/21 01:00:28  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.6  2005/02/22 12:57:39  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -53,7 +56,7 @@
 #include "ogr_dods.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrdodsgrid.cpp,v 1.6 2005/02/22 12:57:39 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrdodsgrid.cpp,v 1.7 2005/09/21 01:00:28 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                          OGRDODSGridLayer()                          */
@@ -82,6 +85,7 @@ OGRDODSGridLayer::OGRDODSGridLayer( OGRDODSDataSource *poDSIn,
     }
         
     poFeatureDefn = new OGRFeatureDefn( pszLayerName );
+    poFeatureDefn->Reference();
 
 /* -------------------------------------------------------------------- */
 /*      Fetch the target variable.                                      */
