@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrdodslayer.cpp,v 1.7 2005/02/22 12:57:39 fwarmerdam Exp $
+ * $Id: ogrdodslayer.cpp,v 1.8 2005/09/21 01:00:29 fwarmerdam Exp $
  *
  * Project:  OGR/DODS Interface
  * Purpose:  Implements OGRDODSLayer class.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrdodslayer.cpp,v $
+ * Revision 1.8  2005/09/21 01:00:29  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.7  2005/02/22 12:57:39  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -54,7 +57,7 @@
 #include "ogr_dods.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrdodslayer.cpp,v 1.7 2005/02/22 12:57:39 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrdodslayer.cpp,v 1.8 2005/09/21 01:00:29 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                            OGRDODSLayer()                            */
@@ -155,10 +158,10 @@ OGRDODSLayer::~OGRDODSLayer()
     }
 
     if( poSRS != NULL )
-        poSRS->Dereference();
+        poSRS->Release();
 
     if( poFeatureDefn != NULL )
-        delete poFeatureDefn;
+        poFeatureDefn->Release();
 
     CPLFree( pszFIDColumn );
     pszFIDColumn = NULL;
