@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrdodssequencelayer.cpp,v 1.6 2005/02/02 20:54:26 fwarmerdam Exp $
+ * $Id: ogrdodssequencelayer.cpp,v 1.7 2005/09/21 01:00:29 fwarmerdam Exp $
  *
  * Project:  OGR/DODS Interface
  * Purpose:  Implements OGRDODSSequenceLayer class, which implements the
@@ -29,6 +29,9 @@
  ******************************************************************************
  *
  * $Log: ogrdodssequencelayer.cpp,v $
+ * Revision 1.7  2005/09/21 01:00:29  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.6  2005/02/02 20:54:26  fwarmerdam
  * track m_nFeaturesRead
  *
@@ -53,7 +56,7 @@
 #include "ogr_dods.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrdodssequencelayer.cpp,v 1.6 2005/02/02 20:54:26 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrdodssequencelayer.cpp,v 1.7 2005/09/21 01:00:29 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                        OGRDODSSequenceLayer()                        */
@@ -84,6 +87,7 @@ OGRDODSSequenceLayer::OGRDODSSequenceLayer( OGRDODSDataSource *poDSIn,
     }
         
     poFeatureDefn = new OGRFeatureDefn( pszLayerName );
+    poFeatureDefn->Reference();
 
 /* -------------------------------------------------------------------- */
 /*      Fetch the target variable.                                      */
