@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrregisterall.cpp,v 1.22 2005/01/19 20:34:38 fwarmerdam Exp $
+ * $Id: ogrregisterall.cpp,v 1.26 2005/09/05 19:32:52 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Function to register all known OGR drivers.
@@ -28,6 +28,18 @@
  ******************************************************************************
  *
  * $Log: ogrregisterall.cpp,v $
+ * Revision 1.26  2005/09/05 19:32:52  fwarmerdam
+ * Added PGEO
+ *
+ * Revision 1.25  2005/08/05 15:34:49  fwarmerdam
+ * added grass support
+ *
+ * Revision 1.24  2005/07/12 16:06:06  fwarmerdam
+ * Don't register ILI unless enabled.
+ *
+ * Revision 1.23  2005/07/08 22:10:57  pka
+ * Initial import of OGR Interlis driver
+ *
  * Revision 1.22  2005/01/19 20:34:38  fwarmerdam
  * added autoloaddrivers call
  *
@@ -98,7 +110,7 @@
 
 #include "ogrsf_frmts.h"
 
-CPL_CVSID("$Id: ogrregisterall.cpp,v 1.22 2005/01/19 20:34:38 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrregisterall.cpp,v 1.26 2005/09/05 19:32:52 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           OGRRegisterAll()                           */
@@ -122,6 +134,10 @@ void OGRRegisterAll()
     RegisterOGRMEM();
     RegisterOGRCSV();
     RegisterOGRGML();
+#ifdef ILI_ENABLED
+    RegisterOGRILI1();
+    RegisterOGRILI2();
+#endif
 #ifdef SQLITE_ENABLED
     RegisterOGRSQLite();
 #endif
@@ -130,6 +146,7 @@ void OGRRegisterAll()
 #endif
 #ifdef ODBC_ENABLED
     RegisterOGRODBC();
+    RegisterOGRPGeo();
 #endif
 //    RegisterOGRE00();
 #ifdef OGDI_ENABLED
@@ -144,10 +161,11 @@ void OGRRegisterAll()
 #ifdef OCI_ENABLED
     RegisterOGROCI();
 #endif
+#ifdef GRASS_ENABLED
+    RegisterOGRGRASS();
+#endif
 #ifdef FME_ENABLED
     RegisterOGRFME();
 #endif
-
-    
 }
 
