@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsfdriverregistrar.cpp,v 1.17 2005/01/19 20:28:18 fwarmerdam Exp $
+ * $Id: ogrsfdriverregistrar.cpp,v 1.21 2005/08/10 13:49:43 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The OGRSFDriverRegistrar class implementation.
@@ -28,6 +28,18 @@
  ******************************************************************************
  *
  * $Log: ogrsfdriverregistrar.cpp,v $
+ * Revision 1.21  2005/08/10 13:49:43  fwarmerdam
+ * fix logic for dll entry point.
+ *
+ * Revision 1.20  2005/06/10 15:38:10  fwarmerdam
+ * avoid use of MAX_PATH
+ *
+ * Revision 1.19  2005/06/10 14:58:46  fwarmerdam
+ * Added option to search under executable directory for plugins.
+ *
+ * Revision 1.18  2005/06/08 21:11:24  fwarmerdam
+ * bug 865: fixed "ogr_" name test.
+ *
  * Revision 1.17  2005/01/19 20:28:18  fwarmerdam
  * added untested autoloaddrivers.
  *
@@ -85,7 +97,7 @@
 #include "ogr_api.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogrsfdriverregistrar.cpp,v 1.17 2005/01/19 20:28:18 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrsfdriverregistrar.cpp,v 1.21 2005/08/10 13:49:43 fwarmerdam Exp $");
 
 static OGRSFDriverRegistrar *poRegistrar = NULL;
 
@@ -623,8 +635,20 @@ void OGRSFDriverRegistrar::AutoLoadDrivers()
         papszSearchPath = CSLAddString( papszSearchPath, 
                                         GDAL_PREFIX "/lib/gdalplugins" );
 #else
-        papszSearchPath = CSLAddString( papszSearchPath, 
-                                        "/usr/local/lib/gdalplugins" );
+        char szExecPath[1024];
+
+        if( CPLGetExecPath( szExecPath, sizeof(szExecPath) ) )
+        {
+            char szPluginDir[sizeof(szExecPath)+50];
+            strcpy( szPluginDir, CPLGetDirname( szExecPath ) );
+            strcat( szPluginDir, "\\gdalplugins\\" );
+            papszSearchPath = CSLAddString( papszSearchPath, szPluginDir );
+        }
+        else
+        {
+            papszSearchPath = CSLAddString( papszSearchPath, 
+                                            "/usr/local/lib/gdalplugins" );
+        }
 #endif
 
 #ifdef notdef
@@ -650,7 +674,7 @@ void OGRSFDriverRegistrar::AutoLoadDrivers()
             const char *pszExtension = CPLGetExtension( papszFiles[iFile] );
             void   *pRegister;
 
-            if( !EQUALN(papszFiles[iFile],"ogr_",5) )
+            if( !EQUALN(papszFiles[iFile],"ogr_",4) )
                 continue;
 
             if( !EQUAL(pszExtension,"dll") 
@@ -660,7 +684,7 @@ void OGRSFDriverRegistrar::AutoLoadDrivers()
 
             pszFuncName = (char *) CPLCalloc(strlen(papszFiles[iFile])+20,1);
             sprintf( pszFuncName, "RegisterOGR%s", 
-                     CPLGetBasename(papszFiles[iFile]) + 5 );
+                     CPLGetBasename(papszFiles[iFile]) + 4 );
             
             pszFilename = 
                 CPLFormFilename( papszSearchPath[iDir], 
