@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgmldatasource.cpp,v 1.17 2005/01/27 16:34:56 fwarmerdam Exp $
+ * $Id: ogrgmldatasource.cpp,v 1.22 2005/07/05 19:18:58 fwarmerdam Exp $
  *
  * Project:  OGR
  * Purpose:  Implements OGRGMLDataSource class.
@@ -28,6 +28,22 @@
  ******************************************************************************
  *
  * $Log: ogrgmldatasource.cpp,v $
+ * Revision 1.22  2005/07/05 19:18:58  fwarmerdam
+ * Use www.w3.org not www.w3c.org.
+ *
+ * Revision 1.21  2005/06/30 02:16:41  fwarmerdam
+ * more efforts to produce valid GML
+ *
+ * Revision 1.20  2005/06/29 16:08:57  fwarmerdam
+ * Fixed gml:geometryPropertyType to be gml:GeometryPropertyType as per
+ * suggestion from Tom Kralidis.
+ *
+ * Revision 1.19  2005/05/04 19:34:07  fwarmerdam
+ * delete reader in datasource destructor, not layer
+ *
+ * Revision 1.18  2005/04/06 13:38:33  fwarmerdam
+ * Initialize nSchemaStart.
+ *
  * Revision 1.17  2005/01/27 16:34:56  fwarmerdam
  * Fixed gml test.  Don't try to read any old XML file!
  *
@@ -86,7 +102,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrgmldatasource.cpp,v 1.17 2005/01/27 16:34:56 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrgmldatasource.cpp,v 1.22 2005/07/05 19:18:58 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                         OGRGMLDataSource()                         */
@@ -149,6 +165,9 @@ OGRGMLDataSource::~OGRGMLDataSource()
         delete papoLayers[i];
     
     CPLFree( papoLayers );
+
+    if( poReader )
+        delete poReader;
 }
 
 /************************************************************************/
@@ -399,7 +418,7 @@ int OGRGMLDataSource::Create( const char *pszFilename,
     if( pszSchemaURI != NULL )
     {
         VSIFPrintf( fpOutput, 
-              "     xmlns:xsi=\"http://www.w3c.org/2001/XMLSchema-instance\"\n"
+              "     xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
               "     xsi:schemaLocation=\"%s\"\n", 
                     CSLFetchNameValue( papszOptions, "XSISCHEMAURI" ) );
     }
@@ -408,16 +427,14 @@ int OGRGMLDataSource::Create( const char *pszFilename,
         char *pszBasename = CPLStrdup(CPLGetBasename( pszName ));
 
         VSIFPrintf( fpOutput, 
-              "     xmlns:xsi=\"http://www.w3c.org/2001/XMLSchema-instance\"\n"
-              "     xsi:schemaLocation=\". %s\"\n", 
+              "     xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
+              "     xsi:schemaLocation=\"http://ogr.maptools.org/ %s\"\n", 
                     CPLResetExtension( pszBasename, "xsd" ) );
         CPLFree( pszBasename );
     }
 
     VSIFPrintf( fpOutput, "%s", 
                 "     xmlns:ogr=\"http://ogr.maptools.org/\"\n" );
-    VSIFPrintf( fpOutput, "%s", 
-                "     xmlns=\"http://ogr.maptools.org/\"\n" );
     VSIFPrintf( fpOutput, "%s", 
                 "     xmlns:gml=\"http://www.opengis.net/gml\">\n" );
 
@@ -543,7 +560,7 @@ void OGRGMLDataSource::InsertHeader()
 
 {
     FILE        *fpSchema;
-    int         nSchemaStart;
+    int         nSchemaStart = 0;
 
     if( fpOutput == NULL || fpOutput == stdout )
         return;
@@ -651,7 +668,7 @@ void OGRGMLDataSource::InsertHeader()
 /* -------------------------------------------------------------------- */
         VSIFPrintf( 
             fpSchema,
-            "<xs:element name=\"geometryProperty\" type=\"gml:geometryPropertyType\" nillable=\"true\" minOccurs=\"1\" maxOccurs=\"1\"/>\n" );
+            "<xs:element name=\"geometryProperty\" type=\"gml:GeometryPropertyType\" nillable=\"true\" minOccurs=\"1\" maxOccurs=\"1\"/>\n" );
             
 /* -------------------------------------------------------------------- */
 /*      Emit each of the attributes.                                    */
@@ -684,7 +701,7 @@ void OGRGMLDataSource::InsertHeader()
                 int nWidth, nDecimals;
 
                 if( poFieldDefn->GetPrecision() == 0 )
-                    nDecimals = 16;
+                    nDecimals = 0;
                 else
                     nDecimals = poFieldDefn->GetPrecision();
 
