@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrgmllayer.cpp,v 1.16 2005/02/22 12:56:28 fwarmerdam Exp $
+ * $Id: ogrgmllayer.cpp,v 1.19 2005/09/21 00:59:36 fwarmerdam Exp $
  *
  * Project:  OGR
  * Purpose:  Implements OGRGMLLayer class.
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: ogrgmllayer.cpp,v $
+ * Revision 1.19  2005/09/21 00:59:36  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.18  2005/06/30 02:16:41  fwarmerdam
+ * more efforts to produce valid GML
+ *
+ * Revision 1.17  2005/05/04 19:34:07  fwarmerdam
+ * delete reader in datasource destructor, not layer
+ *
  * Revision 1.16  2005/02/22 12:56:28  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -82,7 +91,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrgmllayer.cpp,v 1.16 2005/02/22 12:56:28 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrgmllayer.cpp,v 1.19 2005/09/21 00:59:36 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           OGRGMLLayer()                              */
@@ -105,6 +114,7 @@ OGRGMLLayer::OGRGMLLayer( const char * pszName,
     poDS = poDSIn;
 
     poFeatureDefn = new OGRFeatureDefn( pszName );
+    poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( eReqType );
 
     bWriter = bWriterIn;
@@ -126,13 +136,11 @@ OGRGMLLayer::OGRGMLLayer( const char * pszName,
 OGRGMLLayer::~OGRGMLLayer()
 
 {
-    delete poFeatureDefn;
+    if( poFeatureDefn )
+        poFeatureDefn->Release();
 
     if( poSRS != NULL )
-        delete poSRS;
-
-    if( !bWriter )
-        delete poDS->GetReader();
+        poSRS->Release();
 }
 
 /************************************************************************/
@@ -306,29 +314,10 @@ OGRErr OGRGMLLayer::CreateFeature( OGRFeature *poFeature )
     if( poFeature->GetFID() == OGRNullFID )
         poFeature->SetFID( iNextGMLId++ );
 
-    VSIFPrintf( fp, "    <%s fid=\"%d\">\n", 
+    VSIFPrintf( fp, "    <ogr:%s fid=\"F%d\">\n", 
                 poFeatureDefn->GetName(),
                 poFeature->GetFID() );
 
-    // Write all "set" fields. 
-    for( int iField = 0; iField < poFeatureDefn->GetFieldCount(); iField++ )
-    {
-        
-        OGRFieldDefn *poField = poFeatureDefn->GetFieldDefn( iField );
-
-        if( poFeature->IsFieldSet( iField ) )
-        {
-            char *pszEscaped = 
-                CPLEscapeString( poFeature->GetFieldAsString( iField ), 
-                                 -1, CPLES_XML );
-
-            VSIFPrintf( fp, "      <%s>%s</%s>\n", 
-                        poField->GetNameRef(), pszEscaped, 
-                        poField->GetNameRef() );
-            CPLFree( pszEscaped );
-        }
-    }
-
     // Write out Geometry - for now it isn't indented properly.
     if( poFeature->GetGeometryRef() != NULL )
     {
@@ -344,7 +333,29 @@ OGRErr OGRGMLLayer::CreateFeature( OGRFeature *poFeature )
         poDS->GrowExtents( &sGeomBounds );
     }
 
-    VSIFPrintf( fp, "    </%s>\n", poFeatureDefn->GetName() );
+    // Write all "set" fields. 
+    for( int iField = 0; iField < poFeatureDefn->GetFieldCount(); iField++ )
+    {
+        
+        OGRFieldDefn *poField = poFeatureDefn->GetFieldDefn( iField );
+
+        if( poFeature->IsFieldSet( iField ) )
+        {
+            const char *pszRaw = poFeature->GetFieldAsString( iField );
+
+            while( *pszRaw == ' ' )
+                pszRaw++;
+
+            char *pszEscaped = CPLEscapeString( pszRaw, -1, CPLES_XML );
+
+            VSIFPrintf( fp, "      <ogr:%s>%s</ogr:%s>\n", 
+                        poField->GetNameRef(), pszEscaped, 
+                        poField->GetNameRef() );
+            CPLFree( pszEscaped );
+        }
+    }
+
+    VSIFPrintf( fp, "    </ogr:%s>\n", poFeatureDefn->GetName() );
     VSIFPrintf( fp, "  </gml:featureMember>\n" );
 
     return OGRERR_NONE;
