@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mitab_spatialref.cpp,v 1.41 2004/10/11 20:50:04 dmorissette Exp $
+ * $Id: mitab_spatialref.cpp,v 1.43 2005/05/12 22:07:52 dmorissette Exp $
  *
  * Name:     mitab_spatialref.cpp
  * Project:  MapInfo TAB Read/Write library
@@ -30,6 +30,12 @@
  **********************************************************************
  *
  * $Log: mitab_spatialref.cpp,v $
+ * Revision 1.43  2005/05/12 22:07:52  dmorissette
+ * Improved handling of Danish modified TM proj#21-24 (hss, bugs 976,1010)
+ *
+ * Revision 1.42  2005/03/22 23:24:54  dmorissette
+ * Added support for datum id in .MAP header (bug 910)
+ *
  * Revision 1.41  2004/10/11 20:50:04  dmorissette
  * 7 new datum defns, 1 fixed and list of ellipsoids updated (Bug 608,Uffe K.)
  *
@@ -636,10 +642,6 @@ OGRSpatialReference *TABFile::GetSpatialRef()
          * Transverse Mercator
          *-------------------------------------------------------------*/
       case 8:
-      case 21:
-      case 22:
-      case 23:
-      case 24:
         m_poSpatialRef->SetTM( sTABProj.adProjParams[1],
                                sTABProj.adProjParams[0],
                                sTABProj.adProjParams[2],
@@ -647,6 +649,54 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                sTABProj.adProjParams[4] * dfConv );
         break;
 
+        /*----------------------------------------------------------------
+         * Transverse Mercator,(modified for Danish System 34 Jylland-Fyn)
+         *---------------------------------------------------------------*/
+      case 21:
+         m_poSpatialRef->SetTMVariant( SRS_PT_TRANSVERSE_MERCATOR_MI_21,
+                                       sTABProj.adProjParams[1],
+                                       sTABProj.adProjParams[0],
+                                       sTABProj.adProjParams[2],
+                                       sTABProj.adProjParams[3] * dfConv,
+                                       sTABProj.adProjParams[4] * dfConv );
+         break;
+
+        /*--------------------------------------------------------------
+         * Transverse Mercator,(modified for Danish System 34 Sjaelland)
+         *-------------------------------------------------------------*/
+      case 22:
+         m_poSpatialRef->SetTMVariant( SRS_PT_TRANSVERSE_MERCATOR_MI_22,
+                                       sTABProj.adProjParams[1],
+                                       sTABProj.adProjParams[0],
+                                       sTABProj.adProjParams[2],
+                                       sTABProj.adProjParams[3] * dfConv,
+                                       sTABProj.adProjParams[4] * dfConv );
+         break;
+
+        /*----------------------------------------------------------------
+         * Transverse Mercator,(modified for Danish System 34/45 Bornholm)
+         *---------------------------------------------------------------*/
+      case 23:
+         m_poSpatialRef->SetTMVariant( SRS_PT_TRANSVERSE_MERCATOR_MI_23,
+                                       sTABProj.adProjParams[1],
+                                       sTABProj.adProjParams[0],
+                                       sTABProj.adProjParams[2],
+                                       sTABProj.adProjParams[3] * dfConv,
+                                       sTABProj.adProjParams[4] * dfConv );
+         break;
+
+        /*--------------------------------------------------------------
+         * Transverse Mercator,(modified for Finnish KKJ)
+         *-------------------------------------------------------------*/
+      case 24:
+         m_poSpatialRef->SetTMVariant( SRS_PT_TRANSVERSE_MERCATOR_MI_24,
+                                       sTABProj.adProjParams[1],
+                                       sTABProj.adProjParams[0],
+                                       sTABProj.adProjParams[2],
+                                       sTABProj.adProjParams[3] * dfConv,
+                                       sTABProj.adProjParams[4] * dfConv );
+         break;
+
         /*--------------------------------------------------------------
          * Albers Conic Equal Area
          *-------------------------------------------------------------*/
@@ -836,15 +886,18 @@ OGRSpatialReference *TABFile::GetSpatialRef()
     {
         psDatumInfo = asDatumInfoList + iDatumInfo;
         
-        if( TAB_EQUAL(psDatumInfo->nEllipsoid, sTABProj.nEllipsoidId)
-            && TAB_EQUAL(psDatumInfo->dfShiftX, sTABProj.dDatumShiftX)
-            && TAB_EQUAL(psDatumInfo->dfShiftY, sTABProj.dDatumShiftY)
-            && TAB_EQUAL(psDatumInfo->dfShiftZ, sTABProj.dDatumShiftZ)
-            && TAB_EQUAL(psDatumInfo->dfDatumParm0,sTABProj.adDatumParams[0])
-            && TAB_EQUAL(psDatumInfo->dfDatumParm1,sTABProj.adDatumParams[1])
-            && TAB_EQUAL(psDatumInfo->dfDatumParm2,sTABProj.adDatumParams[2])
-            && TAB_EQUAL(psDatumInfo->dfDatumParm3,sTABProj.adDatumParams[3])
-            && TAB_EQUAL(psDatumInfo->dfDatumParm4,sTABProj.adDatumParams[4]))
+        if( TAB_EQUAL(psDatumInfo->nEllipsoid, sTABProj.nEllipsoidId) &&
+            ((sTABProj.nDatumId > 0 && 
+              sTABProj.nDatumId == psDatumInfo->nMapInfoDatumID) ||
+             (sTABProj.nDatumId <= 0
+              && TAB_EQUAL(psDatumInfo->dfShiftX, sTABProj.dDatumShiftX)
+              && TAB_EQUAL(psDatumInfo->dfShiftY, sTABProj.dDatumShiftY)
+              && TAB_EQUAL(psDatumInfo->dfShiftZ, sTABProj.dDatumShiftZ)
+              && TAB_EQUAL(psDatumInfo->dfDatumParm0,sTABProj.adDatumParams[0])
+              && TAB_EQUAL(psDatumInfo->dfDatumParm1,sTABProj.adDatumParams[1])
+              && TAB_EQUAL(psDatumInfo->dfDatumParm2,sTABProj.adDatumParams[2])
+              && TAB_EQUAL(psDatumInfo->dfDatumParm3,sTABProj.adDatumParams[3])
+              && TAB_EQUAL(psDatumInfo->dfDatumParm4,sTABProj.adDatumParams[4]))))
             break;
 
         psDatumInfo = NULL;
@@ -1006,6 +1059,7 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
     sTABProj.adProjParams[2] = sTABProj.adProjParams[3] = 0.0;
     sTABProj.adProjParams[4] = sTABProj.adProjParams[5] = 0.0;
     
+    sTABProj.nDatumId = 0;
     sTABProj.dDatumShiftX = 0.0;
     sTABProj.dDatumShiftY = 0.0;
     sTABProj.dDatumShiftZ = 0.0;
@@ -1345,6 +1399,7 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
     if( psDatumInfo != NULL )
     {
         sTABProj.nEllipsoidId = psDatumInfo->nEllipsoid;
+        sTABProj.nDatumId = psDatumInfo->nMapInfoDatumID;
         sTABProj.dDatumShiftX = psDatumInfo->dfShiftX;
         sTABProj.dDatumShiftY = psDatumInfo->dfShiftY;
         sTABProj.dDatumShiftZ = psDatumInfo->dfShiftZ;
