@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmysqldatasource.cpp,v 1.3 2004/10/12 16:59:31 fwarmerdam Exp $
+ * $Id: ogrmysqldatasource.cpp,v 1.5 2005/09/21 01:00:01 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMySQLDataSource class.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogrmysqldatasource.cpp,v $
+ * Revision 1.5  2005/09/21 01:00:01  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.4  2005/08/02 13:01:29  fwarmerdam
+ * Removed old postgis create logic.
+ *
  * Revision 1.3  2004/10/12 16:59:31  fwarmerdam
  * rearrange include files for win32
  *
@@ -49,7 +55,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrmysqldatasource.cpp,v 1.3 2004/10/12 16:59:31 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrmysqldatasource.cpp,v 1.5 2005/09/21 01:00:01 fwarmerdam Exp $");
 /************************************************************************/
 /*                         OGRMySQLDataSource()                         */
 /************************************************************************/
@@ -95,8 +101,8 @@ OGRMySQLDataSource::~OGRMySQLDataSource()
 
     for( i = 0; i < nKnownSRID; i++ )
     {
-        if( papoSRS[i] != NULL && papoSRS[i]->Dereference() == 0 )
-            delete papoSRS[i];
+        if( papoSRS[i] != NULL )
+            papoSRS[i]->Release();
     }
     CPLFree( panSRID );
     CPLFree( papoSRS );
@@ -320,312 +326,6 @@ int OGRMySQLDataSource::OpenTable( const char *pszNewName, int bUpdate,
     return TRUE;
 }
 
-#ifdef notdef
-/************************************************************************/
-/*                            DeleteLayer()                             */
-/************************************************************************/
-
-void OGRMySQLDataSource::DeleteLayer( const char *pszLayerName )
-
-{
-    int iLayer;
-
-/* -------------------------------------------------------------------- */
-/*      Try to find layer.                                              */
-/* -------------------------------------------------------------------- */
-    for( iLayer = 0; iLayer < nLayers; iLayer++ )
-    {
-        if( EQUAL(pszLayerName,papoLayers[iLayer]->GetLayerDefn()->GetName()) )
-            break;
-    }
-
-    if( iLayer == nLayers )
-    {
-        CPLError( CE_Failure, CPLE_AppDefined, 
-                  "Attempt to delete layer '%s', but this layer is not known to OGR.", 
-                  pszLayerName );
-        return;
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Blow away our OGR structures related to the layer.  This is     */
-/*      pretty dangerous if anything has a reference to this layer!     */
-/* -------------------------------------------------------------------- */
-    CPLDebug( "OGR_MYSQL", "DeleteLayer(%s)", pszLayerName );
-
-    delete papoLayers[iLayer];
-    memmove( papoLayers + iLayer, papoLayers + iLayer + 1, 
-             sizeof(void *) * (nLayers - iLayer - 1) );
-    nLayers--;
-
-/* -------------------------------------------------------------------- */
-/*      Remove from the database.                                       */
-/* -------------------------------------------------------------------- */
-    PGresult            *hResult;
-    char                szCommand[1024];
-
-    hResult = PQexec(hPGConn, "BEGIN");
-    PQclear( hResult );
-
-    if( bHavePostGIS )
-    {
-        sprintf( szCommand, 
-                 "SELECT DropGeometryColumn('%s','%s','wkb_geometry')",
-                 pszDBName, pszLayerName );
-
-        CPLDebug( "OGR_MYSQL", "PGexec(%s)", szCommand );
-
-        hResult = PQexec( hPGConn, szCommand );
-        PQclear( hResult );
-    }
-
-    sprintf( szCommand, "DROP TABLE %s", pszLayerName );
-    CPLDebug( "OGR_MYSQL", "PGexec(%s)", szCommand );
-    hResult = PQexec( hPGConn, szCommand );
-    PQclear( hResult );
-    
-    hResult = PQexec(hPGConn, "COMMIT");
-    PQclear( hResult );
-
-    hResult = PQexec(hPGConn, "BEGIN");
-    PQclear( hResult );
-
-    sprintf( szCommand, "DROP SEQUENCE %s_ogc_fid_seq", pszLayerName );
-    CPLDebug( "OGR_MYSQL", "PGexec(%s)", szCommand );
-    hResult = PQexec( hPGConn, szCommand );
-    PQclear( hResult );
-    
-    hResult = PQexec(hPGConn, "COMMIT");
-    PQclear( hResult );
-}
-
-/************************************************************************/
-/*                            CreateLayer()                             */
-/************************************************************************/
-
-OGRLayer *
-OGRMySQLDataSource::CreateLayer( const char * pszLayerNameIn,
-                              OGRSpatialReference *poSRS,
-                              OGRwkbGeometryType eType,
-                              char ** papszOptions )
-
-{
-    PGresult            *hResult;
-    char                szCommand[1024];
-    const char          *pszGeomType;
-    char                *pszLayerName;
-
-    if( CSLFetchBoolean(papszOptions,"LAUNDER",TRUE) )
-        pszLayerName = LaunderName( pszLayerNameIn );
-    else
-        pszLayerName = CPLStrdup( pszLayerNameIn );
-
-/* -------------------------------------------------------------------- */
-/*      Do we already have this layer?  If so, should we blow it        */
-/*      away?                                                           */
-/* -------------------------------------------------------------------- */
-    int iLayer;
-
-    for( iLayer = 0; iLayer < nLayers; iLayer++ )
-    {
-        if( EQUAL(pszLayerName,papoLayers[iLayer]->GetLayerDefn()->GetName()) )
-        {
-            if( CSLFetchNameValue( papszOptions, "OVERWRITE" ) != NULL
-                && !EQUAL(CSLFetchNameValue(papszOptions,"OVERWRITE"),"NO") )
-            {
-                DeleteLayer( pszLayerName );
-            }
-            else
-            {
-                CPLFree( pszLayerName );
-                CPLError( CE_Failure, CPLE_AppDefined, 
-                          "Layer %s already exists, CreateLayer failed.\n"
-                          "Use the layer creation option OVERWRITE=YES to "
-                          "replace it.",
-                          pszLayerName );
-                return NULL;
-            }
-        }
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Handle the GEOM_TYPE option.                                    */
-/* -------------------------------------------------------------------- */
-    pszGeomType = CSLFetchNameValue( papszOptions, "GEOM_TYPE" );
-    if( pszGeomType == NULL )
-    {
-        if( bHavePostGIS )
-            pszGeomType = "geometry";
-        else
-            pszGeomType = "bytea";
-    }
-
-    if( bHavePostGIS && !EQUAL(pszGeomType,"geometry") )
-    {
-        CPLError( CE_Failure, CPLE_AppDefined,
-                  "Can't override GEOM_TYPE in PostGIS enabled databases.\n"
-                  "Creation of layer %s with GEOM_TYPE %s has failed.",
-                  pszLayerName, pszGeomType );
-        CPLFree( pszLayerName );
-        return NULL;
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Try to get the SRS Id of this spatial reference system,         */
-/*      adding tot the srs table if needed.                             */
-/* -------------------------------------------------------------------- */
-    int nSRSId = -1;
-
-    if( poSRS != NULL )
-        nSRSId = FetchSRSId( poSRS );
-
-/* -------------------------------------------------------------------- */
-/*      Create a basic table with the FID.  Also include the            */
-/*      geometry if this is not a PostGIS enabled table.                */
-/* -------------------------------------------------------------------- */
-    hResult = PQexec(hPGConn, "BEGIN");
-    PQclear( hResult );
-
-    if( !bHavePostGIS )
-        sprintf( szCommand, 
-                 "CREATE TABLE \"%s\" ( "
-                 "   OGC_FID SERIAL, "
-                 "   WKB_GEOMETRY %s )",
-                 pszLayerName, pszGeomType );
-    else
-        sprintf( szCommand, 
-                 "CREATE TABLE \"%s\" ( OGC_FID SERIAL )", 
-                 pszLayerName );
-
-    CPLDebug( "OGR_MYSQL", "PQexec(%s)", szCommand );
-    hResult = PQexec(hPGConn, szCommand);
-    if( PQresultStatus(hResult) != PGRES_COMMAND_OK )
-    {
-        CPLError( CE_Failure, CPLE_AppDefined, 
-                  "%s\n%s", szCommand, PQerrorMessage(hPGConn) );
-
-        CPLFree( pszLayerName );
-
-        PQclear( hResult );
-        hResult = PQexec( hPGConn, "ROLLBACK" );
-        PQclear( hResult );
-
-        return NULL;
-    }
-    
-    PQclear( hResult );
-
-/* -------------------------------------------------------------------- */
-/*      Eventually we should be adding this table to a table of         */
-/*      "geometric layers", capturing the WKT projection, and           */
-/*      perhaps some other housekeeping.                                */
-/* -------------------------------------------------------------------- */
-    if( bHavePostGIS )
-    {
-        const char *pszGeometryType;
-
-        /* Sometimes there is an old cruft entry in the geometry_columns
-         * table if things were not properly cleaned up before.  We make
-         * an effort to clean out such cruft.
-         */
-        sprintf( szCommand, 
-                 "DELETE FROM geometry_columns WHERE f_table_name = '%s'", 
-                 pszLayerName );
-                 
-        CPLDebug( "OGR_MYSQL", "PQexec(%s)", szCommand );
-        hResult = PQexec(hPGConn, szCommand);
-        PQclear( hResult );
-
-        switch( wkbFlatten(eType) )
-        {
-          case wkbPoint:
-            pszGeometryType = "POINT";
-            break;
-
-          case wkbLineString:
-            pszGeometryType = "LINESTRING";
-            break;
-
-          case wkbPolygon:
-            pszGeometryType = "POLYGON";
-            break;
-
-          case wkbMultiPoint:
-            pszGeometryType = "MULTIPOINT";
-            break;
-
-          case wkbMultiLineString:
-            pszGeometryType = "MULTILINESTRING";
-            break;
-
-          case wkbMultiPolygon:
-            pszGeometryType = "MULTIPOLYGON";
-            break;
-
-          case wkbGeometryCollection:
-            pszGeometryType = "GEOMETRYCOLLECTION";
-            break;
-
-          default:
-            pszGeometryType = "GEOMETRY";
-            break;
-
-        }
-
-        sprintf( szCommand, 
-                 "select AddGeometryColumn('%s','%s','wkb_geometry',%d,'%s',%d)",
-                 pszDBName, pszLayerName, nSRSId, pszGeometryType, 3 );
-
-        CPLDebug( "OGR_MYSQL", "PQexec(%s)", szCommand );
-        hResult = PQexec(hPGConn, szCommand);
-
-        if( !hResult 
-            || PQresultStatus(hResult) != PGRES_TUPLES_OK )
-        {
-            CPLError( CE_Failure, CPLE_AppDefined, 
-                      "AddGeometryColumn failed for layer %s, layer creation has failed.",
-                      pszLayerName );
-            
-            CPLFree( pszLayerName );
-
-            PQclear( hResult );
-
-            hResult = PQexec(hPGConn, "ROLLBACK");
-            PQclear( hResult );
-
-            return NULL;
-        }
-    }
-
-/* -------------------------------------------------------------------- */
-/*      Complete, and commit the transaction.                           */
-/* -------------------------------------------------------------------- */
-    hResult = PQexec(hPGConn, "COMMIT");
-    PQclear( hResult );
-
-/* -------------------------------------------------------------------- */
-/*      Create the layer object.                                        */
-/* -------------------------------------------------------------------- */
-    OGRMySQLTableLayer     *poLayer;
-
-    poLayer = new OGRMySQLTableLayer( this, pszLayerName, TRUE, nSRSId );
-
-    poLayer->SetLaunderFlag( CSLFetchBoolean(papszOptions,"LAUNDER",TRUE) );
-    poLayer->SetPrecisionFlag( CSLFetchBoolean(papszOptions,"PRECISION",TRUE));
-
-/* -------------------------------------------------------------------- */
-/*      Add layer to data source layer list.                            */
-/* -------------------------------------------------------------------- */
-    papoLayers = (OGRMySQLLayer **)
-        CPLRealloc( papoLayers,  sizeof(OGRMySQLLayer *) * (nLayers+1) );
-    
-    papoLayers[nLayers++] = poLayer;
-
-    CPLFree( pszLayerName );
-
-    return poLayer;
-}
-#endif
 /************************************************************************/
 /*                           TestCapability()                           */
 /************************************************************************/
