@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmysqllayer.cpp,v 1.3 2005/02/22 12:54:27 fwarmerdam Exp $
+ * $Id: ogrmysqllayer.cpp,v 1.5 2005/09/21 01:00:01 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMySQLLayer class.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogrmysqllayer.cpp,v $
+ * Revision 1.5  2005/09/21 01:00:01  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.4  2005/08/30 23:53:16  fwarmerdam
+ * implement binary field support
+ *
  * Revision 1.3  2005/02/22 12:54:27  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -43,7 +49,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrmysqllayer.cpp,v 1.3 2005/02/22 12:54:27 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrmysqllayer.cpp,v 1.5 2005/09/21 01:00:01 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           OGRMySQLLayer()                            */
@@ -92,10 +98,10 @@ OGRMySQLLayer::~OGRMySQLLayer()
     CPLFree( pszQueryStatement );
 
     if( poSRS != NULL )
-        poSRS->Dereference();
+        poSRS->Release();
 
     if( poFeatureDefn )
-        delete poFeatureDefn;
+        poFeatureDefn->Release();
 }
 
 /************************************************************************/
@@ -148,7 +154,8 @@ OGRFeature *OGRMySQLLayer::GetNextFeature()
 /*      a feature.                                                      */
 /************************************************************************/
 
-OGRFeature *OGRMySQLLayer::RecordToFeature( char **papszRow )
+OGRFeature *OGRMySQLLayer::RecordToFeature( char **papszRow,
+                                            unsigned long *panLengths )
 
 {
     mysql_field_seek( hResultSet, 0 );
@@ -194,11 +201,20 @@ OGRFeature *OGRMySQLLayer::RecordToFeature( char **papszRow )
 /*      Transfer regular data fields.                                   */
 /* -------------------------------------------------------------------- */
         iOGRField = poFeatureDefn->GetFieldIndex(psMSField->name);
-
         if( iOGRField < 0 )
             continue;
 
-        poFeature->SetField( iOGRField, papszRow[iField] );
+        OGRFieldDefn *psFieldDefn = poFeatureDefn->GetFieldDefn( iOGRField );
+
+        if( psFieldDefn->GetType() == OFTBinary )
+        {
+            poFeature->SetField( iOGRField, panLengths[iField], 
+                                 (GByte *) papszRow[iField] );
+        }
+        else
+        {
+            poFeature->SetField( iOGRField, papszRow[iField] );
+        }
     }
 
     return poFeature;
@@ -238,6 +254,7 @@ OGRFeature *OGRMySQLLayer::GetNextRawFeature()
 /*      Fetch next record.                                              */
 /* -------------------------------------------------------------------- */
     char **papszRow;
+    unsigned long *panLengths;
 
     papszRow = mysql_fetch_row( hResultSet );
     if( papszRow == NULL )
@@ -246,10 +263,12 @@ OGRFeature *OGRMySQLLayer::GetNextRawFeature()
         return NULL;
     }
 
+    panLengths = mysql_fetch_lengths( hResultSet );
+
 /* -------------------------------------------------------------------- */
 /*      Process record.                                                 */
 /* -------------------------------------------------------------------- */
-    OGRFeature *poFeature = RecordToFeature( papszRow );
+    OGRFeature *poFeature = RecordToFeature( papszRow, panLengths );
 
     iNextShapeId++;
 
