@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmysqlresultlayer.cpp,v 1.1 2004/10/08 20:48:12 fwarmerdam Exp $
+ * $Id: ogrmysqlresultlayer.cpp,v 1.2 2005/09/21 01:00:01 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMySQLResultLayer class.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrmysqlresultlayer.cpp,v $
+ * Revision 1.2  2005/09/21 01:00:01  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.1  2004/10/08 20:48:12  fwarmerdam
  * New
  *
@@ -36,7 +39,7 @@
 #include "cpl_conv.h"
 #include "ogr_mysql.h"
 
-CPL_CVSID("$Id: ogrmysqlresultlayer.cpp,v 1.1 2004/10/08 20:48:12 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrmysqlresultlayer.cpp,v 1.2 2005/09/21 01:00:01 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                        OGRMySQLResultLayer()                         */
@@ -87,6 +90,8 @@ OGRFeatureDefn *OGRMySQLResultLayer::ReadResultDefinition()
     OGRFeatureDefn *poDefn = new OGRFeatureDefn( "sql_statement" );
     int            iRawField;
 
+    poDefn->Reference();
+
     mysql_field_seek( hResultSet, 0 );
     for( iRawField = 0; 
          iRawField < (int) mysql_num_fields(hResultSet); 
