@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmysqltablelayer.cpp,v 1.4 2005/02/24 21:54:36 fwarmerdam Exp $
+ * $Id: ogrmysqltablelayer.cpp,v 1.9 2005/09/21 01:00:01 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMySQLTableLayer class.
@@ -28,6 +28,21 @@
  ******************************************************************************
  *
  * $Log: ogrmysqltablelayer.cpp,v $
+ * Revision 1.9  2005/09/21 01:00:01  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.8  2005/08/30 23:53:16  fwarmerdam
+ * implement binary field support
+ *
+ * Revision 1.7  2005/07/25 16:08:50  fwarmerdam
+ * Fixed recognision of some integer types, such as mediumint.
+ *
+ * Revision 1.6  2005/07/25 14:39:55  fwarmerdam
+ * Added FID column related debug statement.
+ *
+ * Revision 1.5  2005/06/27 17:00:50  fwarmerdam
+ * Fixed bug with cleaning up hResultSet in GetFeature(id).
+ *
  * Revision 1.4  2005/02/24 21:54:36  fwarmerdam
  * added support for decimal fields.
  *
@@ -46,7 +61,7 @@
 #include "cpl_string.h"
 #include "ogr_mysql.h"
 
-CPL_CVSID("$Id: ogrmysqltablelayer.cpp,v 1.4 2005/02/24 21:54:36 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrmysqltablelayer.cpp,v 1.9 2005/09/21 01:00:01 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                         OGRMySQLTableLayer()                         */
@@ -123,6 +138,8 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
     OGRFeatureDefn *poDefn = new OGRFeatureDefn( pszTable );
     char           **papszRow;
 
+    poDefn->Reference();
+
     while( (papszRow = mysql_fetch_row( hResult )) != NULL )
     {
         const char      *pszType;
@@ -133,11 +150,15 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
         if( pszType == NULL )
             continue;
 
-        if( EQUAL(pszType,"varchar") 
-            || (strlen(pszType)>3 && EQUAL(pszType+strlen(pszType)-4,"text"))
-            || (strlen(pszType)>3 && EQUAL(pszType+strlen(pszType)-4,"blob"))
-            || EQUAL(pszType+strlen(pszType)-4,"enum") 
-            || EQUAL(pszType+strlen(pszType)-4,"set") )
+        if( EQUAL(pszType,"varbinary")
+            || (strlen(pszType)>3 && EQUAL(pszType+strlen(pszType)-4,"blob")))
+        {
+            oField.SetType( OFTBinary );
+        }
+        else if( EQUAL(pszType,"varchar") 
+                 || (strlen(pszType)>3 && EQUAL(pszType+strlen(pszType)-4,"text"))
+                 || EQUAL(pszType+strlen(pszType)-4,"enum") 
+                 || EQUAL(pszType+strlen(pszType)-4,"set") )
         {
             oField.SetType( OFTString );
         }
@@ -158,11 +179,11 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
         {
             oField.SetType( OFTInteger );
         }
-        else if( EQUAL(pszType,"mediumint") )
+        else if( EQUALN(pszType,"mediumint",9) )
         {
             oField.SetType( OFTInteger );
         }
-        else if( EQUAL(pszType,"bigint") )
+        else if( EQUALN(pszType,"bigint",6) )
         {
             oField.SetType( OFTInteger );
         }
@@ -224,6 +245,14 @@ OGRFeatureDefn *OGRMySQLTableLayer::ReadTableDefinition( const char *pszTable )
 /* -------------------------------------------------------------------- */
     poDefn->SetGeomType( wkbNone );
 
+    if( bHasFid )
+        CPLDebug( "MySQL", "table %s has FID column %s.",
+                  pszTable, pszFIDColumn );
+    else
+        CPLDebug( "MySQL", 
+                  "table %s has no FID column, FIDs will not be reliable!",
+                  pszTable );
+
     return poDefn;
 }
 
@@ -407,7 +436,7 @@ OGRFeature *OGRMySQLTableLayer::GetFeature( long nFeatureId )
 /* -------------------------------------------------------------------- */
 /*      Discard any existing resultset.                                 */
 /* -------------------------------------------------------------------- */
-    poDS->InterruptLongResult();
+    ResetReading();
 
 /* -------------------------------------------------------------------- */
 /*      Prepare query command that will just fetch the one record of    */
@@ -443,17 +472,20 @@ OGRFeature *OGRMySQLTableLayer::GetFeature( long nFeatureId )
 /*      Fetch the result record.                                        */
 /* -------------------------------------------------------------------- */
     char **papszRow;
+    unsigned long *panLengths;
 
     papszRow = mysql_fetch_row( hResultSet );
     if( papszRow == NULL )
         return NULL;
 
+    panLengths = mysql_fetch_lengths( hResultSet );
+
 /* -------------------------------------------------------------------- */
 /*      Transform into a feature.                                       */
 /* -------------------------------------------------------------------- */
     iNextShapeId = nFeatureId;
 
-    OGRFeature *poFeature = RecordToFeature( papszRow );
+    OGRFeature *poFeature = RecordToFeature( papszRow, panLengths );
 
     iNextShapeId = 0;
 
@@ -461,6 +493,7 @@ OGRFeature *OGRMySQLTableLayer::GetFeature( long nFeatureId )
 /*      Cleanup                                                         */
 /* -------------------------------------------------------------------- */
     mysql_free_result( hResultSet );
+    hResultSet = NULL;
 
     return poFeature;
 }
