@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ntf_estlayers.cpp,v 1.19 2003/12/15 16:07:41 warmerda Exp $
+ * $Id: ntf_estlayers.cpp,v 1.20 2005/09/21 00:59:55 fwarmerdam Exp $
  *
  * Project:  NTF Translator
  * Purpose:  NTFFileReader methods related to establishing the schemas
@@ -30,6 +30,9 @@
  ******************************************************************************
  *
  * $Log: ntf_estlayers.cpp,v $
+ * Revision 1.20  2005/09/21 00:59:55  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.19  2003/12/15 16:07:41  warmerda
  * Added CHG_TYPE for landline plus product
  *
@@ -99,7 +102,7 @@
 #include "ntf.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ntf_estlayers.cpp,v 1.19 2003/12/15 16:07:41 warmerda Exp $");
+CPL_CVSID("$Id: ntf_estlayers.cpp,v 1.20 2005/09/21 00:59:55 fwarmerdam Exp $");
 
 #define MAX_LINK        5000
 
@@ -1753,6 +1756,7 @@ void NTFFileReader::EstablishLayer( const char * pszLayerName,
 /* -------------------------------------------------------------------- */
         poDefn = new OGRFeatureDefn( pszLayerName );
         poDefn->SetGeomType( eGeomType );
+        poDefn->Reference();
 
 /* -------------------------------------------------------------------- */
 /*      Fetch definitions of each field in turn.                        */
