@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrntfdatasource.cpp,v 1.15 2002/12/10 04:08:17 warmerda Exp $
+ * $Id: ogrntfdatasource.cpp,v 1.16 2005/09/21 00:59:55 fwarmerdam Exp $
  *
  * Project:  UK NTF Reader
  * Purpose:  Implements OGRNTFDataSource class
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrntfdatasource.cpp,v $
+ * Revision 1.16  2005/09/21 00:59:55  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.15  2002/12/10 04:08:17  warmerda
  * updated British National Grid WKT
  *
@@ -78,7 +81,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrntfdatasource.cpp,v 1.15 2002/12/10 04:08:17 warmerda Exp $");
+CPL_CVSID("$Id: ogrntfdatasource.cpp,v 1.16 2005/09/21 00:59:55 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                          OGRNTFDataSource()                          */
@@ -149,7 +152,8 @@ OGRNTFDataSource::~OGRNTFDataSource()
     CSLDestroy( papszFCNum );
     CSLDestroy( papszFCName );
 
-    delete poSpatialRef;
+    if( poSpatialRef )
+        poSpatialRef->Release();
 }
 
 /************************************************************************/
