@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrntflayer.cpp,v 1.12 2005/02/22 12:54:16 fwarmerdam Exp $
+ * $Id: ogrntflayer.cpp,v 1.13 2005/09/21 00:59:55 fwarmerdam Exp $
  *
  * Project:  UK NTF Reader
  * Purpose:  Implements OGRNTFLayer class.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrntflayer.cpp,v $
+ * Revision 1.13  2005/09/21 00:59:55  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.12  2005/02/22 12:54:16  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -69,7 +72,7 @@
 #include "ntf.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrntflayer.cpp,v 1.12 2005/02/22 12:54:16 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrntflayer.cpp,v 1.13 2005/09/21 00:59:55 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                            OGRNTFLayer()                             */
@@ -106,7 +109,8 @@ OGRNTFLayer::~OGRNTFLayer()
                   poFeatureDefn->GetName() );
     }
 
-    delete poFeatureDefn;
+    if( poFeatureDefn )
+        poFeatureDefn->Release();
 }
 
 /************************************************************************/
