@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrocidatasource.cpp,v 1.26 2005/02/10 15:46:02 fwarmerdam Exp $
+ * $Id: ogrocidatasource.cpp,v 1.29 2005/09/21 00:56:55 fwarmerdam Exp $
  *
  * Project:  Oracle Spatial Driver
  * Purpose:  Implementation of the OGROCIDataSource class.
@@ -28,6 +28,18 @@
  ******************************************************************************
  *
  * $Log: ogrocidatasource.cpp,v $
+ * Revision 1.29  2005/09/21 00:56:55  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.28  2005/08/18 14:45:22  fwarmerdam
+ * Cleanup session on exit.
+ *
+ * Revision 1.27  2005/08/09 20:54:43  hobu
+ * Allow CreateLayer to use the OCI_FID environment
+ * variable if it set.  Also, throw a CPLDebug statement
+ * out there if the layer wasn't found in the layer list
+ * when deleting.
+ *
  * Revision 1.26  2005/02/10 15:46:02  fwarmerdam
  * added GEOMETRY_NAME layer creation option
  *
@@ -116,7 +128,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrocidatasource.cpp,v 1.26 2005/02/10 15:46:02 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrocidatasource.cpp,v 1.29 2005/09/21 00:56:55 fwarmerdam Exp $");
 
 static int anEPSGOracleMapping[] = 
 {
@@ -169,10 +181,13 @@ OGROCIDataSource::~OGROCIDataSource()
 
     for( i = 0; i < nKnownSRID; i++ )
     {
-        delete papoSRS[i];
+        papoSRS[i]->Release();
     }
     CPLFree( papoSRS );
     CPLFree( panSRID );
+
+    if( poSession != NULL )
+        delete poSession;
 }
 
 /************************************************************************/
@@ -434,6 +449,8 @@ void OGROCIDataSource::DeleteLayer( const char *pszLayerName )
     }
 
     if( iLayer == nLayers )
+        CPLDebug( "OCI", "DeleteLayer: %s not found in layer list." \
+                  "  Layer * not* deleted.", pszLayerName );
         return;
 
 /* -------------------------------------------------------------------- */
@@ -534,14 +551,15 @@ OGROCIDataSource::CreateLayer( const char * pszLayerName,
 /*      Create a basic table with the FID.  Also include the            */
 /*      geometry if this is not a PostGIS enabled table.                */
 /* -------------------------------------------------------------------- */
+    const char *pszExpectedFIDName = 
+        CPLGetConfigOption( "OCI_FID", "OGR_FID" );    
    
     OGROCIStatement oStatement( poSession );
-    
     sprintf( szCommand, 
              "CREATE TABLE \"%s\" ( "
-             "OGR_FID INTEGER, "
+             "%s INTEGER, "
              "%s %s )",
-             pszSafeLayerName, pszGeometryName, SDO_GEOMETRY );
+             pszSafeLayerName, pszExpectedFIDName, pszGeometryName, SDO_GEOMETRY );
 
     if( oStatement.Execute( szCommand ) != CE_None )
     {
