@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrocilayer.cpp,v 1.12 2005/02/22 12:57:51 fwarmerdam Exp $
+ * $Id: ogrocilayer.cpp,v 1.13 2005/09/21 00:59:36 fwarmerdam Exp $
  *
  * Project:  Oracle Spatial Driver
  * Purpose:  Implementation of the OGROCILayer class.  This is layer semantics
@@ -30,6 +30,9 @@
  ******************************************************************************
  *
  * $Log: ogrocilayer.cpp,v $
+ * Revision 1.13  2005/09/21 00:59:36  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.12  2005/02/22 12:57:51  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -71,7 +74,7 @@
 #include "ogr_oci.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrocilayer.cpp,v 1.12 2005/02/22 12:57:51 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrocilayer.cpp,v 1.13 2005/09/21 00:59:36 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           OGROCILayer()                               */
@@ -120,8 +123,8 @@ OGROCILayer::~OGROCILayer()
     CPLFree( pszQueryStatement );
     pszQueryStatement = NULL;
 
-    if( poFeatureDefn != NULL && poFeatureDefn->Dereference() <= 0 )
-        delete poFeatureDefn;
+    if( poFeatureDefn != NULL )
+        poFeatureDefn->Release();
 }
 
 /************************************************************************/
