@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrodbcdatasource.cpp,v 1.8 2004/10/17 04:22:19 fwarmerdam Exp $
+ * $Id: ogrodbcdatasource.cpp,v 1.9 2005/09/21 00:56:55 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRODBCDataSource class.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrodbcdatasource.cpp,v $
+ * Revision 1.9  2005/09/21 00:56:55  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.8  2004/10/17 04:22:19  fwarmerdam
  * Slightly improve error message.
  *
@@ -59,7 +62,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrodbcdatasource.cpp,v 1.8 2004/10/17 04:22:19 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrodbcdatasource.cpp,v 1.9 2005/09/21 00:56:55 fwarmerdam Exp $");
 /************************************************************************/
 /*                         OGRODBCDataSource()                          */
 /************************************************************************/
@@ -94,8 +97,8 @@ OGRODBCDataSource::~OGRODBCDataSource()
 
     for( i = 0; i < nKnownSRID; i++ )
     {
-        if( papoSRS[i] != NULL && papoSRS[i]->Dereference() == 0 )
-            delete papoSRS[i];
+        if( papoSRS[i] != NULL )
+            papoSRS[i]->Release();
     }
     CPLFree( panSRID );
     CPLFree( papoSRS );
