@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrodbclayer.cpp,v 1.8 2005/02/22 12:53:56 fwarmerdam Exp $
+ * $Id: ogrodbclayer.cpp,v 1.12 2005/09/21 00:56:55 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRODBCLayer class, code shared between 
@@ -29,6 +29,18 @@
  ******************************************************************************
  *
  * $Log: ogrodbclayer.cpp,v $
+ * Revision 1.12  2005/09/21 00:56:55  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.11  2005/09/09 05:02:12  fwarmerdam
+ * added wkb support
+ *
+ * Revision 1.10  2005/09/05 19:56:42  fwarmerdam
+ * added binary field support
+ *
+ * Revision 1.9  2005/03/21 20:36:31  fwarmerdam
+ * set precision for decimal fields - MS bug 1255
+ *
  * Revision 1.8  2005/02/22 12:53:56  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -59,7 +71,7 @@
 #include "ogr_odbc.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrodbclayer.cpp,v 1.8 2005/02/22 12:53:56 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrodbclayer.cpp,v 1.12 2005/09/21 00:56:55 fwarmerdam Exp $");
 
 
 /************************************************************************/
@@ -71,6 +83,7 @@ OGRODBCLayer::OGRODBCLayer()
 {
     poDS = NULL;
 
+    bGeomColumnWKB = FALSE;
     pszGeomColumn = NULL;
     pszFIDColumn = NULL;
 
@@ -107,12 +120,12 @@ OGRODBCLayer::~OGRODBCLayer()
 
     if( poFeatureDefn != NULL )
     {
-        delete poFeatureDefn;
+        poFeatureDefn->Release();
         poFeatureDefn = NULL;
     }
 
     if( poSRS != NULL )
-        poSRS->Dereference();
+        poSRS->Release();
 }
 
 /************************************************************************/
@@ -129,6 +142,8 @@ CPLErr OGRODBCLayer::BuildFeatureDefn( const char *pszLayerName,
     poFeatureDefn = new OGRFeatureDefn( pszLayerName );
     int    nRawColumns = poStmt->GetColCount();
 
+    poFeatureDefn->Reference();
+
     panFieldOrdinals = (int *) CPLMalloc( sizeof(int) * nRawColumns );
 
     for( int iCol = 0; iCol < nRawColumns; iCol++ )
@@ -147,8 +162,15 @@ CPLErr OGRODBCLayer::BuildFeatureDefn( const char *pszLayerName,
             oField.SetType( OFTInteger );
             break;
 
+          case SQL_BINARY:
+          case SQL_VARBINARY:
+          case SQL_LONGVARBINARY:
+            oField.SetType( OFTBinary );
+            break;
+
           case SQL_DECIMAL:
             oField.SetType( OFTReal );
+            oField.SetPrecision( poStmt->GetColPrecision(iCol) );
             break;
 
           case SQL_FLOAT:
@@ -245,9 +267,16 @@ OGRFeature *OGRODBCLayer::GetNextRawFeature()
 /* -------------------------------------------------------------------- */
     for( iField = 0; iField < poFeatureDefn->GetFieldCount(); iField++ )
     {
-        const char *pszValue = poStmt->GetColData(panFieldOrdinals[iField]-1);
+        int iSrcField = panFieldOrdinals[iField]-1;
+        const char *pszValue = poStmt->GetColData( iSrcField );
 
-        if( pszValue != NULL )
+        if( pszValue == NULL )
+            /* no value */;
+        else if( poFeature->GetFieldDefnRef(iField)->GetType() == OFTBinary )
+            poFeature->SetField( iField, 
+                                 poStmt->GetColDataLength(iSrcField),
+                                 (GByte *) pszValue );
+        else
             poFeature->SetField( iField, pszValue );
     }
 
@@ -260,9 +289,18 @@ OGRFeature *OGRODBCLayer::GetNextRawFeature()
         const char *pszGeomText = poStmt->GetColData( iField );
         OGRGeometry *poGeom = NULL;
 
-        if( pszGeomText != NULL )
+        if( pszGeomText != NULL && !bGeomColumnWKB )
+        {
             OGRGeometryFactory::createFromWkt( (char **) &pszGeomText,
                                                NULL, &poGeom );
+        }
+        else if( pszGeomText != NULL && bGeomColumnWKB )
+        {
+            int nLength = poStmt->GetColDataLength( iField );
+
+            OGRGeometryFactory::createFromWkb( (unsigned char *) pszGeomText,
+                                               NULL, &poGeom, nLength );
+        }
         
         if( poGeom != NULL )
             poFeature->SetGeometryDirectly( poGeom );
