@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrodbctablelayer.cpp,v 1.5 2005/02/22 12:53:56 fwarmerdam Exp $
+ * $Id: ogrodbctablelayer.cpp,v 1.6 2005/09/09 05:02:12 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRODBCTableLayer class, access to an existing table.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrodbctablelayer.cpp,v $
+ * Revision 1.6  2005/09/09 05:02:12  fwarmerdam
+ * added wkb support
+ *
  * Revision 1.5  2005/02/22 12:53:56  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -48,7 +51,7 @@
 #include "cpl_conv.h"
 #include "ogr_odbc.h"
 
-CPL_CVSID("$Id: ogrodbctablelayer.cpp,v 1.5 2005/02/22 12:53:56 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrodbctablelayer.cpp,v 1.6 2005/09/09 05:02:12 fwarmerdam Exp $");
 /************************************************************************/
 /*                          OGRODBCTableLayer()                         */
 /************************************************************************/
@@ -90,12 +93,6 @@ CPLErr OGRODBCTableLayer::Initialize( const char *pszTableName,
 {
     CPLODBCSession *poSession = poDS->GetSession();
 
-    CPLFree( pszGeomColumn );
-    if( pszGeomCol == NULL )
-        pszGeomColumn = NULL;
-    else
-        pszGeomColumn = CPLStrdup( pszGeomCol );
-
     CPLFree( pszFIDColumn );
     pszFIDColumn = NULL;
 
@@ -115,6 +112,15 @@ CPLErr OGRODBCTableLayer::Initialize( const char *pszTableName,
         }
     }
 
+/* -------------------------------------------------------------------- */
+/*      Have we been provided a geometry column?                        */
+/* -------------------------------------------------------------------- */
+    CPLFree( pszGeomColumn );
+    if( pszGeomCol == NULL )
+        pszGeomColumn = NULL;
+    else
+        pszGeomColumn = CPLStrdup( pszGeomCol );
+
 /* -------------------------------------------------------------------- */
 /*      Get the column definitions for this table.                      */
 /* -------------------------------------------------------------------- */
@@ -136,6 +142,30 @@ CPLErr OGRODBCTableLayer::Initialize( const char *pszTableName,
         return CE_Failure;
     }
 
+/* -------------------------------------------------------------------- */
+/*      If we got a geometry column, does it exist?  Is it binary?      */
+/* -------------------------------------------------------------------- */
+    if( pszGeomColumn != NULL )
+    {
+        int iColumn = oGetCol.GetColId( pszGeomColumn );
+        if( iColumn < 0 )
+        {
+            CPLError( CE_Failure, CPLE_AppDefined, 
+                      "Column %s requested for geometry, but it does not exist.", 
+                      pszGeomColumn );
+            CPLFree( pszGeomColumn );
+            pszGeomColumn = NULL;
+        }
+        else
+        {
+            if( oGetCol.GetColType( iColumn ) == SQL_BINARY
+                || oGetCol.GetColType( iColumn ) == SQL_VARBINARY
+                || oGetCol.GetColType( iColumn ) == SQL_LONGVARBINARY )
+                bGeomColumnWKB = TRUE;
+        }
+    }
+
+
     return CE_None;
 }
 
