@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsf_frmts.h,v 1.50 2005/02/22 12:40:37 fwarmerdam Exp $
+ * $Id: ogrsf_frmts.h,v 1.53 2005/09/05 19:31:45 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Classes related to format registration, and file opening.
@@ -28,6 +28,15 @@
  ******************************************************************************
  *
  * $Log: ogrsf_frmts.h,v $
+ * Revision 1.53  2005/09/05 19:31:45  fwarmerdam
+ * Added PGeo driver.
+ *
+ * Revision 1.52  2005/08/05 15:34:34  fwarmerdam
+ * added grass
+ *
+ * Revision 1.51  2005/07/08 22:10:56  pka
+ * Initial import of OGR Interlis driver
+ *
  * Revision 1.50  2005/02/22 12:40:37  fwarmerdam
  * added base OGRLayer spatial filter support
  *
@@ -447,6 +456,10 @@ void CPL_DLL RegisterOGRVRT();
 void CPL_DLL RegisterOGRDODS();
 void CPL_DLL RegisterOGRSQLite();
 void CPL_DLL RegisterOGRCSV();
+void CPL_DLL RegisterOGRILI1();
+void CPL_DLL RegisterOGRILI2();
+void CPL_DLL RegisterOGRGRASS();
+void CPL_DLL RegisterOGRPGeo();
 
 CPL_C_END
 
