@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_pg.h,v 1.16 2005/02/22 12:54:05 fwarmerdam Exp $
+ * $Id: ogr_pg.h,v 1.20 2005/08/06 14:49:27 osemykin Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for OGR/PostgreSQL driver.
@@ -28,6 +28,20 @@
  ******************************************************************************
  *
  * $Log: ogr_pg.h,v $
+ * Revision 1.20  2005/08/06 14:49:27  osemykin
+ * Added BINARY CURSOR support
+ * Use it with 'PGB:dbname=...' instead 'PG:dbname=...'
+ *
+ * Revision 1.19  2005/08/03 21:13:51  osemykin
+ * Changed PostGIS version representation
+ *
+ * Revision 1.18  2005/07/20 01:46:04  fwarmerdam
+ * postgis 8.0 upgrades
+ *
+ * Revision 1.17  2005/05/05 20:47:52  dron
+ * Override GetExtent() method for PostGIS layers with PostGIS standard function
+ * extent() (Oleg Semykin <oleg.semykin@gmail.com>
+ *
  * Revision 1.16  2005/02/22 12:54:05  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -89,7 +103,7 @@
 /************************************************************************/
 
 class OGRPGDataSource;
-    
+
 class OGRPGLayer : public OGRLayer
 {
   protected:
@@ -98,11 +112,13 @@ class OGRPGLayer : public OGRLayer
     // Layer spatial reference system, and srid.
     OGRSpatialReference *poSRS;
     int                 nSRSId;
+    int                 nCoordDimension;
 
     int                 iNextShapeId;
 
     char               *GeometryToBYTEA( OGRGeometry * );
     OGRGeometry        *BYTEAToGeometry( const char * );
+    OGRGeometry        *HEXToGeometry( const char * );
     Oid                 GeometryToOID( OGRGeometry * );
     OGRGeometry        *OIDToGeometry( Oid );
 
@@ -133,7 +149,7 @@ class OGRPGLayer : public OGRLayer
     virtual OGRFeature *GetNextFeature();
 
     virtual OGRFeature *GetFeature( long nFeatureId );
-    
+
     OGRFeatureDefn *    GetLayerDefn() { return poFeatureDefn; }
 
     virtual OGRErr      StartTransaction();
@@ -168,7 +184,7 @@ class OGRPGTableLayer : public OGRPGLayer
 
     int                 bLaunderColumnNames;
     int                 bPreservePrecision;
-    
+
   public:
                         OGRPGTableLayer( OGRPGDataSource *,
                                          const char * pszName,
@@ -186,18 +202,20 @@ class OGRPGTableLayer : public OGRPGLayer
     virtual OGRErr      SetFeature( OGRFeature *poFeature );
     virtual OGRErr      DeleteFeature( long nFID );
     virtual OGRErr      CreateFeature( OGRFeature *poFeature );
-    
+
     virtual OGRErr      CreateField( OGRFieldDefn *poField,
                                      int bApproxOK = TRUE );
-    
+
     virtual OGRSpatialReference *GetSpatialRef();
 
     virtual int         TestCapability( const char * );
 
+	virtual OGRErr		GetExtent( OGREnvelope *psExtent, int bForce );
+
     // follow methods are not base class overrides
-    void                SetLaunderFlag( int bFlag ) 
+    void                SetLaunderFlag( int bFlag )
                                 { bLaunderColumnNames = bFlag; }
-    void                SetPrecisionFlag( int bFlag ) 
+    void                SetPrecisionFlag( int bFlag )
                                 { bPreservePrecision = bFlag; }
 };
 
@@ -235,14 +253,22 @@ class OGRPGResultLayer : public OGRPGLayer
 
 class OGRPGDataSource : public OGRDataSource
 {
+    typedef struct
+    {
+        int nMajor;
+        int nMinor;
+        int nRelease;
+    } PGver;
+
     OGRPGLayer        **papoLayers;
     int                 nLayers;
-    
+
     char               *pszName;
     char               *pszDBName;
 
     int                 bDSUpdate;
     int                 bHavePostGIS;
+
     int                 nSoftTransactionLevel;
 
     PGconn              *hPGConn;
@@ -252,16 +278,21 @@ class OGRPGDataSource : public OGRDataSource
     Oid                 nGeometryOID;
 
     // We maintain a list of known SRID to reduce the number of trips to
-    // the database to get SRSes. 
+    // the database to get SRSes.
     int                 nKnownSRID;
-    int                *panSRID;
+    int                 *panSRID;
     OGRSpatialReference **papoSRS;
+
+  public:
+    PGver               sPostGISVersion;
     
+    int                 bUseBinaryCursor;
+
   public:
                         OGRPGDataSource();
                         ~OGRPGDataSource();
 
-    PGconn             *GetPGConn() { return hPGConn; }
+    PGconn              *GetPGConn() { return hPGConn; }
 
     int                 FetchSRSId( OGRSpatialReference * poSRS );
     OGRSpatialReference *FetchSRS( int nSRSId );
@@ -274,7 +305,7 @@ class OGRPGDataSource : public OGRDataSource
     int                 GetLayerCount() { return nLayers; }
     OGRLayer            *GetLayer( int );
 
-    virtual OGRLayer    *CreateLayer( const char *, 
+    virtual OGRLayer    *CreateLayer( const char *,
                                       OGRSpatialReference * = NULL,
                                       OGRwkbGeometryType = wkbUnknown,
                                       char ** = NULL );
@@ -284,7 +315,7 @@ class OGRPGDataSource : public OGRDataSource
     OGRErr              SoftStartTransaction();
     OGRErr              SoftCommit();
     OGRErr              SoftRollback();
-    
+
     OGRErr              FlushSoftTransaction();
 
     Oid                 GetGeometryOID() { return nGeometryOID; }
@@ -305,13 +336,13 @@ class OGRPGDriver : public OGRSFDriver
 {
   public:
                 ~OGRPGDriver();
-                
+
     const char *GetName();
     OGRDataSource *Open( const char *, int );
 
     virtual OGRDataSource *CreateDataSource( const char *pszName,
                                              char ** = NULL );
-    
+
     int                 TestCapability( const char * );
 };
 
