@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpgdatasource.cpp,v 1.30 2005/03/04 21:04:06 fwarmerdam Exp $
+ * $Id: ogrpgdatasource.cpp,v 1.38 2005/09/23 14:09:27 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGDataSource class.
@@ -28,6 +28,35 @@
  ******************************************************************************
  *
  * $Log: ogrpgdatasource.cpp,v $
+ * Revision 1.38  2005/09/23 14:09:27  fwarmerdam
+ * Fixed last fix to cast rename to text for postgres 7.x support.
+ *
+ * Revision 1.37  2005/09/22 19:29:14  fwarmerdam
+ * Fixed to do exact table name test, instead of a wildcarded test when
+ * scanning for spatial tables.
+ *
+ * Revision 1.36  2005/09/21 00:55:42  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.35  2005/08/06 14:49:27  osemykin
+ * Added BINARY CURSOR support
+ * Use it with 'PGB:dbname=...' instead 'PG:dbname=...'
+ *
+ * Revision 1.34  2005/08/04 19:01:29  fwarmerdam
+ * avoid use of non-standard datatypes like uint
+ *
+ * Revision 1.33  2005/08/04 07:28:28  osemykin
+ * Fixes for support postgis version < 0.9.0
+ * geom as EWKT for PostGIS >= 1.0
+ * geom as Text for PostGIS < 1.0
+ *
+ * Revision 1.32  2005/08/03 21:22:29  osemykin
+ * Changed PostGIS version representation
+ * For PostGIS database layers loads from geometry_columns table
+ *
+ * Revision 1.31  2005/07/20 01:45:56  fwarmerdam
+ * fetch PostGIS version
+ *
  * Revision 1.30  2005/03/04 21:04:06  fwarmerdam
  * Added setting of dimension based on geometry type, or DIM override.
  *
@@ -125,7 +154,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrpgdatasource.cpp,v 1.30 2005/03/04 21:04:06 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrpgdatasource.cpp,v 1.38 2005/09/23 14:09:27 fwarmerdam Exp $");
 
 static void OGRPGNoticeProcessor( void *arg, const char * pszMessage );
 
@@ -142,6 +171,7 @@ OGRPGDataSource::OGRPGDataSource()
     nLayers = 0;
     hPGConn = NULL;
     bHavePostGIS = FALSE;
+    bUseBinaryCursor = FALSE;
     nSoftTransactionLevel = 0;
 
     nKnownSRID = 0;
@@ -165,7 +195,7 @@ OGRPGDataSource::~OGRPGDataSource()
 
     for( i = 0; i < nLayers; i++ )
         delete papoLayers[i];
-    
+
     CPLFree( papoLayers );
 
     if( hPGConn != NULL )
@@ -173,8 +203,8 @@ OGRPGDataSource::~OGRPGDataSource()
 
     for( i = 0; i < nKnownSRID; i++ )
     {
-        if( papoSRS[i] != NULL && papoSRS[i]->Dereference() == 0 )
-            delete papoSRS[i];
+        if( papoSRS[i] != NULL )
+            papoSRS[i]->Release();
     }
     CPLFree( panSRID );
     CPLFree( papoSRS );
@@ -193,23 +223,29 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
 /* -------------------------------------------------------------------- */
 /*      Verify postgresql prefix.                                       */
 /* -------------------------------------------------------------------- */
+    if( EQUALN(pszNewName,"PGB:",4) )
+    {
+        bUseBinaryCursor = TRUE;
+        CPLDebug("OGR_PG","BINARY cursor is used for geometry fetching");
+    }
+    else
     if( !EQUALN(pszNewName,"PG:",3) )
     {
         if( !bTestOpen )
-            CPLError( CE_Failure, CPLE_AppDefined, 
+            CPLError( CE_Failure, CPLE_AppDefined,
                       "%s does not conform to PostgreSQL naming convention,"
                       " PG:*\n" );
         return FALSE;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Try to establish connection.                                    */
 /* -------------------------------------------------------------------- */
-    hPGConn = PQconnectdb( pszNewName + 3 );
+    hPGConn = PQconnectdb( pszNewName + (bUseBinaryCursor ? 4 : 3) );
     if( hPGConn == NULL || PQstatus(hPGConn) == CONNECTION_BAD )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
-                  "PGconnectcb failed.\n%s", 
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "PGconnectcb failed.\n%s",
                   PQerrorMessage(hPGConn) );
         PQfinish(hPGConn);
         hPGConn = NULL;
@@ -217,7 +253,7 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
     }
 
     pszName = CPLStrdup( pszNewName );
-    
+
     bDSUpdate = bUpdate;
 
 /* -------------------------------------------------------------------- */
@@ -237,7 +273,7 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
 
         for( i = 0; pszDBName[i] != '\0'; i++ )
         {
-            if( pszDBName[i] == ' ' )                                   
+            if( pszDBName[i] == ' ' )
             {
                 pszDBName[i] = '\0';
                 break;
@@ -248,7 +284,7 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
         pszDBName = CPLStrdup( getenv("USER") );
     else
         pszDBName = CPLStrdup( "unknown_dbname" );
-        
+
     CPLDebug( "OGR_PG", "DBName=\"%s\"", pszDBName );
 
 /* -------------------------------------------------------------------- */
@@ -257,18 +293,18 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
 /*      so we will get the value of the gist indexes.                   */
 /* -------------------------------------------------------------------- */
     PGresult            *hResult;
-    
+
     hResult = PQexec(hPGConn, "BEGIN");
 
     if( hResult && PQresultStatus(hResult) == PGRES_COMMAND_OK )
     {
         PQclear( hResult );
 
-        hResult = PQexec(hPGConn, 
+        hResult = PQexec(hPGConn,
                          "SELECT oid FROM pg_type WHERE typname = 'geometry'" );
     }
 
-    if( hResult && PQresultStatus(hResult) == PGRES_TUPLES_OK 
+    if( hResult && PQresultStatus(hResult) == PGRES_TUPLES_OK
         && PQntuples(hResult) > 0 )
     {
         bHavePostGIS = TRUE;
@@ -283,6 +319,72 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
     hResult = PQexec(hPGConn, "SET ENABLE_SEQSCAN = OFF");
     PQclear( hResult );
 
+    // find out postgis version.
+    sPostGISVersion.nMajor = -1;
+    sPostGISVersion.nMinor = -1;
+    sPostGISVersion.nRelease = -1;
+
+    if( bHavePostGIS )
+    {
+        hResult = PQexec(hPGConn, "SELECT postgis_version()" );
+        if( hResult && PQresultStatus(hResult) == PGRES_TUPLES_OK
+            && PQntuples(hResult) > 0 )
+        {
+            char * pszVer = PQgetvalue(hResult,0,0);
+            char * ptr = pszVer;
+            char szVer[10];
+            char szNum[25];
+            GUInt32 iLen;
+
+            // get Version string
+            if ( *ptr == ' ' ) *ptr++;
+            while (*ptr && *ptr != ' ') ptr++;
+            iLen = ptr-pszVer;
+            if ( iLen > sizeof(szVer) - 1 ) iLen = sizeof(szVer) - 1;
+            strncpy(szVer,pszVer,iLen);
+            szVer[iLen] = '\0';
+            CPLDebug("OGR_PG","PostSIS version string: '%s' -> '%s'",pszVer,szVer);
+
+            pszVer = ptr = szVer;
+
+            // get Major number
+            while (*ptr && *ptr != '.') ptr++;
+            iLen = ptr-pszVer;
+            if ( iLen > sizeof(szNum) - 1) iLen = sizeof(szNum) - 1;
+            strncpy(szNum,pszVer,iLen);
+            szNum[iLen] = '\0';
+            sPostGISVersion.nMajor = atoi(szNum);
+
+            pszVer = ++ptr;
+
+            // get Minor number
+            while (*ptr && *ptr != '.') ptr++;
+            iLen = ptr-pszVer;
+            if ( iLen > sizeof(szNum) - 1) iLen = sizeof(szNum) - 1;
+            strncpy(szNum,pszVer,iLen);
+            szNum[iLen] = '\0';
+            sPostGISVersion.nMinor = atoi(szNum);
+
+
+            if ( *ptr )
+            {
+                pszVer = ++ptr;
+
+                // get Release number
+                while (*ptr && *ptr != '.') ptr++;
+                iLen = ptr-pszVer;
+                if ( iLen > sizeof(szNum) - 1) iLen = sizeof(szNum) - 1;
+                strncpy(szNum,pszVer,iLen);
+                szNum[iLen] = '\0';
+                sPostGISVersion.nRelease = atoi(szNum);
+            }
+
+            CPLDebug( "OGR_PG", "POSTGIS_VERSION=%s",
+                      PQgetvalue(hResult,0,0));
+        }
+        PQclear(hResult);
+    }
+
     hResult = PQexec(hPGConn, "COMMIT");
     PQclear( hResult );
 
@@ -295,10 +397,17 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
     {
         PQclear( hResult );
 
-        hResult = PQexec(hPGConn, 
-                         "DECLARE mycursor CURSOR for "
-                         "SELECT relname FROM pg_class "
-                         "WHERE (relkind in ('r','v') AND relname !~ '^pg')" );
+        if ( bHavePostGIS )
+            hResult = PQexec(hPGConn,
+                             "DECLARE mycursor CURSOR for "
+                             "SELECT c.relname FROM pg_class c, geometry_columns g "
+                             "WHERE (c.relkind in ('r','v') AND c.relname !~ '^pg' "
+                             "AND c.relname::TEXT = g.f_table_name::TEXT)" );
+        else
+            hResult = PQexec(hPGConn,
+                             "DECLARE mycursor CURSOR for "
+                             "SELECT relname FROM pg_class "
+                             "WHERE (relkind in ('r','v') AND relname !~ '^pg')" );
     }
 
     if( hResult && PQresultStatus(hResult) == PGRES_COMMAND_OK )
@@ -309,7 +418,7 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
 
     if( !hResult || PQresultStatus(hResult) != PGRES_TUPLES_OK )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "%s", PQerrorMessage(hPGConn) );
         return FALSE;
     }
@@ -328,7 +437,7 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
             || EQUAL(pszTable,"geometry_columns") )
             continue;
 
-        papszTableNames = CSLAddString(papszTableNames, 
+        papszTableNames = CSLAddString(papszTableNames,
                                        PQgetvalue(hResult, iRecord, 0));
     }
 
@@ -346,7 +455,7 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
 /* -------------------------------------------------------------------- */
 /*      Get the schema of the available tables.                         */
 /* -------------------------------------------------------------------- */
-    for( iRecord = 0; 
+    for( iRecord = 0;
          papszTableNames != NULL && papszTableNames[iRecord] != NULL;
          iRecord++ )
     {
@@ -354,7 +463,7 @@ int OGRPGDataSource::Open( const char * pszNewName, int bUpdate,
     }
 
     CSLDestroy( papszTableNames );
-    
+
     return nLayers > 0 || bUpdate;
 }
 
@@ -379,7 +488,7 @@ int OGRPGDataSource::OpenTable( const char *pszNewName, int bUpdate,
     papoLayers = (OGRPGLayer **)
         CPLRealloc( papoLayers,  sizeof(OGRPGLayer *) * (nLayers+1) );
     papoLayers[nLayers++] = poLayer;
-    
+
     return TRUE;
 }
 
@@ -403,8 +512,8 @@ void OGRPGDataSource::DeleteLayer( const char *pszLayerName )
 
     if( iLayer == nLayers )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
-                  "Attempt to delete layer '%s', but this layer is not known to OGR.", 
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "Attempt to delete layer '%s', but this layer is not known to OGR.",
                   pszLayerName );
         return;
     }
@@ -416,7 +525,7 @@ void OGRPGDataSource::DeleteLayer( const char *pszLayerName )
     CPLDebug( "OGR_PG", "DeleteLayer(%s)", pszLayerName );
 
     delete papoLayers[iLayer];
-    memmove( papoLayers + iLayer, papoLayers + iLayer + 1, 
+    memmove( papoLayers + iLayer, papoLayers + iLayer + 1,
              sizeof(void *) * (nLayers - iLayer - 1) );
     nLayers--;
 
@@ -431,7 +540,7 @@ void OGRPGDataSource::DeleteLayer( const char *pszLayerName )
 
     if( bHavePostGIS )
     {
-        sprintf( szCommand, 
+        sprintf( szCommand,
                  "SELECT DropGeometryColumn('%s','%s','wkb_geometry')",
                  pszDBName, pszLayerName );
 
@@ -445,7 +554,7 @@ void OGRPGDataSource::DeleteLayer( const char *pszLayerName )
     CPLDebug( "OGR_PG", "PGexec(%s)", szCommand );
     hResult = PQexec( hPGConn, szCommand );
     PQclear( hResult );
-    
+
     hResult = PQexec(hPGConn, "COMMIT");
     PQclear( hResult );
 
@@ -456,7 +565,7 @@ void OGRPGDataSource::DeleteLayer( const char *pszLayerName )
     CPLDebug( "OGR_PG", "PGexec(%s)", szCommand );
     hResult = PQexec( hPGConn, szCommand );
     PQclear( hResult );
-    
+
     hResult = PQexec(hPGConn, "COMMIT");
     PQclear( hResult );
 }
@@ -504,7 +613,7 @@ OGRPGDataSource::CreateLayer( const char * pszLayerNameIn,
             else
             {
                 CPLFree( pszLayerName );
-                CPLError( CE_Failure, CPLE_AppDefined, 
+                CPLError( CE_Failure, CPLE_AppDefined,
                           "Layer %s already exists, CreateLayer failed.\n"
                           "Use the layer creation option OVERWRITE=YES to "
                           "replace it.",
@@ -553,21 +662,21 @@ OGRPGDataSource::CreateLayer( const char * pszLayerNameIn,
     PQclear( hResult );
 
     if( !bHavePostGIS )
-        sprintf( szCommand, 
+        sprintf( szCommand,
                  "CREATE TABLE \"%s\" ( "
                  "   OGC_FID SERIAL, "
                  "   WKB_GEOMETRY %s )",
                  pszLayerName, pszGeomType );
     else
-        sprintf( szCommand, 
-                 "CREATE TABLE \"%s\" ( OGC_FID SERIAL )", 
+        sprintf( szCommand,
+                 "CREATE TABLE \"%s\" ( OGC_FID SERIAL )",
                  pszLayerName );
 
     CPLDebug( "OGR_PG", "PQexec(%s)", szCommand );
     hResult = PQexec(hPGConn, szCommand);
     if( PQresultStatus(hResult) != PGRES_COMMAND_OK )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "%s\n%s", szCommand, PQerrorMessage(hPGConn) );
 
         CPLFree( pszLayerName );
@@ -578,7 +687,7 @@ OGRPGDataSource::CreateLayer( const char * pszLayerNameIn,
 
         return NULL;
     }
-    
+
     PQclear( hResult );
 
 /* -------------------------------------------------------------------- */
@@ -597,10 +706,10 @@ OGRPGDataSource::CreateLayer( const char * pszLayerNameIn,
          * table if things were not properly cleaned up before.  We make
          * an effort to clean out such cruft.
          */
-        sprintf( szCommand, 
-                 "DELETE FROM geometry_columns WHERE f_table_name = '%s'", 
+        sprintf( szCommand,
+                 "DELETE FROM geometry_columns WHERE f_table_name = '%s'",
                  pszLayerName );
-                 
+
         CPLDebug( "OGR_PG", "PQexec(%s)", szCommand );
         hResult = PQexec(hPGConn, szCommand);
         PQclear( hResult );
@@ -641,21 +750,21 @@ OGRPGDataSource::CreateLayer( const char * pszLayerNameIn,
 
         }
 
-        sprintf( szCommand, 
+        sprintf( szCommand,
                  "select AddGeometryColumn('%s','%s','wkb_geometry',%d,'%s',%d)",
-                 pszDBName, pszLayerName, nSRSId, pszGeometryType, 
+                 pszDBName, pszLayerName, nSRSId, pszGeometryType,
                  nDimension );
 
         CPLDebug( "OGR_PG", "PQexec(%s)", szCommand );
         hResult = PQexec(hPGConn, szCommand);
 
-        if( !hResult 
+        if( !hResult
             || PQresultStatus(hResult) != PGRES_TUPLES_OK )
         {
-            CPLError( CE_Failure, CPLE_AppDefined, 
+            CPLError( CE_Failure, CPLE_AppDefined,
                       "AddGeometryColumn failed for layer %s, layer creation has failed.",
                       pszLayerName );
-            
+
             CPLFree( pszLayerName );
 
             PQclear( hResult );
@@ -688,7 +797,7 @@ OGRPGDataSource::CreateLayer( const char * pszLayerNameIn,
 /* -------------------------------------------------------------------- */
     papoLayers = (OGRPGLayer **)
         CPLRealloc( papoLayers,  sizeof(OGRPGLayer *) * (nLayers+1) );
-    
+
     papoLayers[nLayers++] = poLayer;
 
     CPLFree( pszLayerName );
@@ -777,17 +886,17 @@ OGRSpatialReference *OGRPGDataSource::FetchSRS( int nId )
     PGresult        *hResult;
     char            szCommand[1024];
     OGRSpatialReference *poSRS = NULL;
-        
+
     SoftStartTransaction();
 
-    sprintf( szCommand, 
+    sprintf( szCommand,
              "SELECT srtext FROM spatial_ref_sys "
-             "WHERE srid = %d", 
+             "WHERE srid = %d",
              nId );
     hResult = PQexec(hPGConn, szCommand );
 
-    if( hResult 
-        && PQresultStatus(hResult) == PGRES_TUPLES_OK 
+    if( hResult
+        && PQresultStatus(hResult) == PGRES_TUPLES_OK
         && PQntuples(hResult) == 1 )
     {
         char *pszWKT;
@@ -808,7 +917,7 @@ OGRSpatialReference *OGRPGDataSource::FetchSRS( int nId )
 /*      Add to the cache.                                               */
 /* -------------------------------------------------------------------- */
     panSRID = (int *) CPLRealloc(panSRID,sizeof(int) * (nKnownSRID+1) );
-    papoSRS = (OGRSpatialReference **) 
+    papoSRS = (OGRSpatialReference **)
         CPLRealloc(papoSRS, sizeof(void*) * (nKnownSRID + 1) );
     panSRID[nKnownSRID] = nId;
     papoSRS[nKnownSRID] = poSRS;
@@ -839,7 +948,7 @@ int OGRPGDataSource::FetchSRSId( OGRSpatialReference * poSRS )
 /* -------------------------------------------------------------------- */
     if( poSRS->exportToWkt( &pszWKT ) != OGRERR_NONE )
         return -1;
-    
+
     CPLAssert( strlen(pszWKT) < sizeof(szCommand) - 500 );
 
 /* -------------------------------------------------------------------- */
@@ -847,19 +956,19 @@ int OGRPGDataSource::FetchSRSId( OGRSpatialReference * poSRS )
 /* -------------------------------------------------------------------- */
     hResult = PQexec(hPGConn, "BEGIN");
 
-    sprintf( szCommand, 
+    sprintf( szCommand,
              "SELECT srid FROM spatial_ref_sys WHERE srtext = '%s'",
              pszWKT );
     hResult = PQexec(hPGConn, szCommand );
-                     
+
 /* -------------------------------------------------------------------- */
 /*      We got it!  Return it.                                          */
 /* -------------------------------------------------------------------- */
-    if( hResult && PQresultStatus(hResult) == PGRES_TUPLES_OK 
+    if( hResult && PQresultStatus(hResult) == PGRES_TUPLES_OK
         && PQntuples(hResult) > 0 )
     {
         nSRSId = atoi(PQgetvalue( hResult, 0, 0 ));
-        
+
         PQclear( hResult );
 
         hResult = PQexec(hPGConn, "COMMIT");
@@ -867,14 +976,14 @@ int OGRPGDataSource::FetchSRSId( OGRSpatialReference * poSRS )
 
         return nSRSId;
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      If the command actually failed, then the metadata table is      */
 /*      likely missing. Try defining it.                                */
 /* -------------------------------------------------------------------- */
     int         bTableMissing;
 
-    bTableMissing = 
+    bTableMissing =
         hResult == NULL || PQresultStatus(hResult) == PGRES_NONFATAL_ERROR;
 
     hResult = PQexec(hPGConn, "COMMIT");
@@ -901,14 +1010,14 @@ int OGRPGDataSource::FetchSRSId( OGRSpatialReference * poSRS )
     }
     else
         nSRSId = 1;
-    
+
 /* -------------------------------------------------------------------- */
 /*      Try adding the SRS to the SRS table.                            */
 /* -------------------------------------------------------------------- */
-    sprintf( szCommand, 
+    sprintf( szCommand,
              "INSERT INTO spatial_ref_sys (srid,srtext) VALUES (%d,'%s')",
              nSRSId, pszWKT );
-             
+
     hResult = PQexec(hPGConn, szCommand );
     PQclear( hResult );
 
@@ -1011,12 +1120,12 @@ OGRErr OGRPGDataSource::SoftRollback()
 
     PGresult            *hResult;
     PGconn              *hPGConn = GetPGConn();
-    
+
     hResult = PQexec(hPGConn, "ROLLBACK");
-    
+
     if( !hResult || PQresultStatus(hResult) != PGRES_COMMAND_OK )
         return OGRERR_FAILURE;
-    
+
     PQclear( hResult );
 
     return OGRERR_NONE;
@@ -1051,7 +1160,7 @@ OGRLayer * OGRPGDataSource::ExecuteSQL( const char *pszSQLCommand,
 {
     if( poSpatialFilter != NULL )
     {
-        CPLDebug( "OGR_PG", 
+        CPLDebug( "OGR_PG",
           "Spatial filter ignored for now in OGRPGDataSource::ExecuteSQL()" );
     }
 
@@ -1059,8 +1168,8 @@ OGRLayer * OGRPGDataSource::ExecuteSQL( const char *pszSQLCommand,
 /*      Use generic implementation for OGRSQL dialect.                  */
 /* -------------------------------------------------------------------- */
     if( pszDialect != NULL && EQUAL(pszDialect,"OGRSQL") )
-        return OGRDataSource::ExecuteSQL( pszSQLCommand, 
-                                          poSpatialFilter, 
+        return OGRDataSource::ExecuteSQL( pszSQLCommand,
+                                          poSpatialFilter,
                                           pszDialect );
 
 /* -------------------------------------------------------------------- */
@@ -1081,7 +1190,7 @@ OGRLayer * OGRPGDataSource::ExecuteSQL( const char *pszSQLCommand,
 /*      Execute the statement.                                          */
 /* -------------------------------------------------------------------- */
     PGresult            *hResult = NULL;
-    
+
     FlushSoftTransaction();
 
     if( SoftStartTransaction() == OGRERR_NONE  )
@@ -1096,24 +1205,24 @@ OGRLayer * OGRPGDataSource::ExecuteSQL( const char *pszSQLCommand,
 /*      layer for it.                                                   */
 /* -------------------------------------------------------------------- */
 
-    if( hResult && PQresultStatus(hResult) == PGRES_TUPLES_OK 
+    if( hResult && PQresultStatus(hResult) == PGRES_TUPLES_OK
         && PQntuples(hResult) > 0 )
     {
         OGRPGResultLayer *poLayer = NULL;
 
         poLayer = new OGRPGResultLayer( this, pszSQLCommand, hResult );
-        
+
         return poLayer;
     }
 
 /* -------------------------------------------------------------------- */
 /*      Generate an error report if an error occured.                   */
 /* -------------------------------------------------------------------- */
-    if( hResult && 
+    if( hResult &&
         (PQresultStatus(hResult) == PGRES_NONFATAL_ERROR
          || PQresultStatus(hResult) == PGRES_FATAL_ERROR ) )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "%s", PQresultErrorMessage( hResult ) );
     }
 
