@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpglayer.cpp,v 1.17 2005/02/22 12:54:05 fwarmerdam Exp $
+ * $Id: ogrpglayer.cpp,v 1.22 2005/09/30 19:11:16 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGLayer class  which implements shared handling
@@ -30,6 +30,23 @@
  ******************************************************************************
  *
  * $Log: ogrpglayer.cpp,v $
+ * Revision 1.22  2005/09/30 19:11:16  fwarmerdam
+ * Fixed Hextobinary conversion (a and A mistranslated).
+ *
+ * Revision 1.21  2005/09/21 00:55:42  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.20  2005/08/06 14:49:27  osemykin
+ * Added BINARY CURSOR support
+ * Use it with 'PGB:dbname=...' instead 'PG:dbname=...'
+ *
+ * Revision 1.19  2005/07/20 01:45:01  fwarmerdam
+ * added PostGIS 8.0 hex geometry format support
+ *
+ * Revision 1.18  2005/05/05 20:47:52  dron
+ * Override GetExtent() method for PostGIS layers with PostGIS standard function
+ * extent() (Oleg Semykin <oleg.semykin@gmail.com>
+ *
  * Revision 1.17  2005/02/22 12:54:05  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -87,7 +104,7 @@
 #include "ogr_pg.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrpglayer.cpp,v 1.17 2005/02/22 12:54:05 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrpglayer.cpp,v 1.22 2005/09/30 19:11:16 fwarmerdam Exp $");
 
 #define CURSOR_PAGE     1
 
@@ -151,10 +168,10 @@ OGRPGLayer::~OGRPGLayer()
     CPLFree( pszQueryStatement );
 
     if( poSRS != NULL )
-        poSRS->Dereference();
+        poSRS->Release();
 
     if( poFeatureDefn )
-        delete poFeatureDefn;
+        poFeatureDefn->Release();
 }
 
 /************************************************************************/
@@ -252,7 +269,8 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
 /* -------------------------------------------------------------------- */
         if( bHasPostGISGeometry
             && (EQUAL(PQfname(hCursorResult,iField),pszGeomColumn)
-                || EQUAL(PQfname(hCursorResult,iField),"astext") ) )
+                || EQUAL(PQfname(hCursorResult,iField),"asEWKT") 
+                || EQUAL(PQfname(hCursorResult,iField),"asText") ) )
         {
             char        *pszWKT;
             char        *pszPostSRID;
@@ -271,8 +289,15 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
                     pszPostSRID++;
             }
 
-            OGRGeometryFactory::createFromWkt( &pszPostSRID, NULL, 
-                                               &poGeometry );
+            if( EQUALN(pszPostSRID,"00",2) || EQUALN(pszPostSRID,"01",2) )
+            {
+                poGeometry = 
+                    HEXToGeometry( 
+                        PQgetvalue( hCursorResult, iRecord, iField ) );
+            }
+            else
+                OGRGeometryFactory::createFromWkt( &pszPostSRID, NULL, 
+                                                   &poGeometry );
             if( poGeometry != NULL )
                 poFeature->SetGeometryDirectly( poGeometry );
 
@@ -300,6 +325,16 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
             }
             continue;
         }
+        /* Handle binary cursor result */
+        else if ( EQUAL(PQfname(hCursorResult,iField),"AsBinary") )
+        {
+            GByte * pabyWkb = (GByte *)PQgetvalue( hCursorResult,
+                                                   iRecord, iField);
+            OGRGeometry * poGeom = NULL;
+            OGRGeometryFactory::createFromWkb(pabyWkb,NULL,&poGeom);
+            poFeature->SetGeometryDirectly( poGeom );
+            continue;
+        }
 
 /* -------------------------------------------------------------------- */
 /*      Transfer regular data fields.                                   */
@@ -381,8 +416,12 @@ OGRFeature *OGRPGLayer::GetNextRawFeature()
         poDS->FlushSoftTransaction();
         poDS->SoftStartTransaction();
 
-        sprintf( szCommand, "DECLARE %s CURSOR for %s",
-                 pszCursorName, pszQueryStatement );
+        if ( poDS->bUseBinaryCursor )
+            sprintf( szCommand, "DECLARE %s BINARY CURSOR for %s",
+                     pszCursorName, pszQueryStatement );
+        else
+            sprintf( szCommand, "DECLARE %s CURSOR for %s",
+                     pszCursorName, pszQueryStatement );
 
         CPLDebug( "OGR_PG", "PQexec(%s)", szCommand );
 
@@ -471,6 +510,56 @@ OGRFeature *OGRPGLayer::GetFeature( long nFeatureId )
     return NULL;
 }
 
+/************************************************************************/
+/*                           HEXToGeometry()                            */
+/************************************************************************/
+
+OGRGeometry *OGRPGLayer::HEXToGeometry( const char *pszBytea )
+
+{
+    GByte       *pabyWKB;
+    int iSrc=0, iDst=0;
+    OGRGeometry *poGeometry;
+
+    if( pszBytea == NULL )
+        return NULL;
+
+    pabyWKB = (GByte *) CPLMalloc(strlen(pszBytea)+1);
+    while( pszBytea[iSrc] != '\0' )
+    {
+        if( pszBytea[iSrc] >= '0' && pszBytea[iSrc] <= '9' )
+            pabyWKB[iDst] = pszBytea[iSrc] - '0';
+        else if( pszBytea[iSrc] >= 'A' && pszBytea[iSrc] <= 'F' )
+            pabyWKB[iDst] = pszBytea[iSrc] - 'A' + 10;
+        else if( pszBytea[iSrc] >= 'a' && pszBytea[iSrc] <= 'f' )
+            pabyWKB[iDst] = pszBytea[iSrc] - 'a' + 10;
+        else 
+            pabyWKB[iDst] = 0;
+
+        pabyWKB[iDst] *= 16;
+
+        iSrc++;
+
+        if( pszBytea[iSrc] >= '0' && pszBytea[iSrc] <= '9' )
+            pabyWKB[iDst] += pszBytea[iSrc] - '0';
+        else if( pszBytea[iSrc] >= 'A' && pszBytea[iSrc] <= 'F' )
+            pabyWKB[iDst] += pszBytea[iSrc] - 'A' + 10;
+        else if( pszBytea[iSrc] >= 'a' && pszBytea[iSrc] <= 'f' )
+            pabyWKB[iDst] += pszBytea[iSrc] - 'a' + 10;
+        else 
+            pabyWKB[iDst] += 0;
+
+        iSrc++;
+        iDst++;
+    }
+
+    poGeometry = NULL;
+    OGRGeometryFactory::createFromWkb( pabyWKB, NULL, &poGeometry, iDst );
+
+    CPLFree( pabyWKB );
+    return poGeometry;
+}
+
 /************************************************************************/
 /*                          BYTEAToGeometry()                           */
 /************************************************************************/
@@ -642,6 +731,9 @@ int OGRPGLayer::TestCapability( const char * pszCap )
     else if( EQUAL(pszCap,OLCTransactions) )
         return TRUE;
 
+	else if( EQUAL(pszCap,OLCFastGetExtent) )
+		return bHasPostGISGeometry;
+
     else 
         return FALSE;
 }
