@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpgresultlayer.cpp,v 1.4 2004/05/08 02:14:49 warmerda Exp $
+ * $Id: ogrpgresultlayer.cpp,v 1.5 2005/09/21 00:55:42 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGResultLayer class, access the resultset from
@@ -29,6 +29,9 @@
  ******************************************************************************
  *
  * $Log: ogrpgresultlayer.cpp,v $
+ * Revision 1.5  2005/09/21 00:55:42  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.4  2004/05/08 02:14:49  warmerda
  * added GetFeature() on table, generalize FID support a bit
  *
@@ -46,7 +49,7 @@
 #include "cpl_conv.h"
 #include "ogr_pg.h"
 
-CPL_CVSID("$Id: ogrpgresultlayer.cpp,v 1.4 2004/05/08 02:14:49 warmerda Exp $");
+CPL_CVSID("$Id: ogrpgresultlayer.cpp,v 1.5 2005/09/21 00:55:42 fwarmerdam Exp $");
 
 /* These are the OIDs for some builtin types, as returned by PQftype(). */
 /* They were copied from pg_type.h in src/include/catalog/pg_type.h */
@@ -123,6 +126,8 @@ OGRFeatureDefn *OGRPGResultLayer::ReadResultDefinition()
     OGRFeatureDefn *poDefn = new OGRFeatureDefn( "sql_statement" );
     int            iRawField;
 
+    poDefn->Reference();
+
     for( iRawField = 0; iRawField < PQnfields(hInitialResult); iRawField++ )
     {
         OGRFieldDefn    oField( PQfname(hResult,iRawField), OFTString);
