@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpgtablelayer.cpp,v 1.25 2005/02/22 12:54:05 fwarmerdam Exp $
+ * $Id: ogrpgtablelayer.cpp,v 1.37 2005/09/26 04:37:17 cfis Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGTableLayer class, access to an existing table.
@@ -28,6 +28,49 @@
  ******************************************************************************
  *
  * $Log: ogrpgtablelayer.cpp,v $
+ * Revision 1.37  2005/09/26 04:37:17  cfis
+ * If inserting a feature into postgresql failed, the program would crash since the wrong number of parameters were sent to CPLError.  Fixed by passing pszCommand.
+ *
+ * Revision 1.36  2005/09/21 00:55:42  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.35  2005/08/17 16:25:39  fwarmerdam
+ * Patched error output to include command if CreateFeature() fails.
+ *
+ * Revision 1.34  2005/08/06 14:49:27  osemykin
+ * Added BINARY CURSOR support
+ * Use it with 'PGB:dbname=...' instead 'PG:dbname=...'
+ *
+ * Revision 1.33  2005/08/05 13:37:40  fwarmerdam
+ * Bug 902: set pszGeomColumn if WKB_GEOMETRY found.
+ *
+ * Revision 1.32  2005/08/04 07:28:28  osemykin
+ * Fixes for support postgis version < 0.9.0
+ * geom as EWKT for PostGIS >= 1.0
+ * geom as Text for PostGIS < 1.0
+ *
+ * Revision 1.31  2005/08/03 21:27:32  osemykin
+ * GetExtent fixes for old PostGIS version support
+ *
+ * Revision 1.30  2005/08/03 13:22:50  fwarmerdam
+ * Patched GetExtent method to support BOX3D return types as reported
+ * by Markus on gdal-dev.
+ *
+ * Revision 1.29  2005/07/20 01:45:30  fwarmerdam
+ * improved geometry dimension support, PostGIS 8 EWKT upgrades
+ *
+ * Revision 1.28  2005/05/05 20:47:52  dron
+ * Override GetExtent() method for PostGIS layers with PostGIS standard function
+ * extent() (Oleg Semykin <oleg.semykin@gmail.com>
+ *
+ * Revision 1.27  2005/05/04 19:14:27  dron
+ * Determine layer geometry type from Geometry_Columns standard OGC table.
+ * Works only for PostGIS layers. (Oleg Semykin <oleg.semykin@gmail.com>)
+ *
+ * Revision 1.26  2005/04/29 17:08:58  dron
+ * Move nSRSId checking into constructor. Change spatial filter query syntax
+ * to match PostGIS 1.0.0 syntax (Oleg Semykin <oleg.semykin@gmail.com>)
+ *
  * Revision 1.25  2005/02/22 12:54:05  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -111,9 +154,10 @@
  */
 
 #include "cpl_conv.h"
+#include "cpl_string.h"
 #include "ogr_pg.h"
 
-CPL_CVSID("$Id: ogrpgtablelayer.cpp,v 1.25 2005/02/22 12:54:05 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrpgtablelayer.cpp,v 1.37 2005/09/26 04:37:17 cfis Exp $");
 
 #define CURSOR_PAGE     1
 
@@ -121,7 +165,7 @@ CPL_CVSID("$Id: ogrpgtablelayer.cpp,v 1.25 2005/02/22 12:54:05 fwarmerdam Exp $"
 /*                          OGRPGTableLayer()                           */
 /************************************************************************/
 
-OGRPGTableLayer::OGRPGTableLayer( OGRPGDataSource *poDSIn, 
+OGRPGTableLayer::OGRPGTableLayer( OGRPGDataSource *poDSIn,
                                   const char * pszTableName,
                                   int bUpdate, int nSRSIdIn )
 
@@ -131,7 +175,7 @@ OGRPGTableLayer::OGRPGTableLayer( OGRPGDataSource *poDSIn,
     pszQuery = NULL;
     pszWHERE = CPLStrdup( "" );
     pszQueryStatement = NULL;
-    
+
     bUpdateAccess = bUpdate;
 
     iNextShapeId = 0;
@@ -139,13 +183,17 @@ OGRPGTableLayer::OGRPGTableLayer( OGRPGDataSource *poDSIn,
     nSRSId = nSRSIdIn;
 
     poFeatureDefn = ReadTableDefinition( pszTableName );
-    
+
     ResetReading();
 
     bLaunderColumnNames = TRUE;
+
+    // check SRID if it's necessary
+    if( nSRSId == -2 )
+        GetSpatialRef();
 }
 
-/************************************************************************/
+//************************************************************************/
 /*                          ~OGRPGTableLayer()                          */
 /************************************************************************/
 
@@ -169,24 +217,53 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
     PGresult            *hResult;
     char                szCommand[1024];
     PGconn              *hPGConn = poDS->GetPGConn();
-    
+    char                szPrimaryKey[256] = "ogc_fid";
+
+    poDS->FlushSoftTransaction();
+
+/* -------------------------------------------- */
+/*          Detect table primary key            */
+/* -------------------------------------------- */
+    /* TODO make changes corresponded to Frank issues
+    sprintf ( szCommand,
+              "SELECT a.attname "
+              "FROM pg_attribute a, pg_constraint c, pg_class cl "
+              "WHERE c.contype='p' AND c.conrelid=cl.oid "
+              "AND a.attnum = c.conkey[1] AND a.attrelid=cl.oid "
+              "AND cl.relname = '%s'",
+              pszTable );
+
+    hResult = PQexec(hPGConn, szCommand );
+
+    if ( hResult && PQntuples( hResult ) == 1 && PQgetisnull( hResult,0,0 ) == false )
+    {
+        sprintf( szPrimaryKey, "%s", PQgetvalue(hResult,0,0) );
+        CPLDebug( "OGR_PG", "Primary key name (FID): %s", szPrimaryKey );
+    }
+    else
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "%s", PQerrorMessage(hPGConn) );
+        CPLError( CE_Warning, CPLE_AppDefined,
+                  "Unable to detect table primary key. Use default 'ogc_fid'");
+    }*/
+
 /* -------------------------------------------------------------------- */
 /*      Fire off commands to get back the schema of the table.          */
 /* -------------------------------------------------------------------- */
-    poDS->FlushSoftTransaction();
     hResult = PQexec(hPGConn, "BEGIN");
 
     if( hResult && PQresultStatus(hResult) == PGRES_COMMAND_OK )
     {
         PQclear( hResult );
-        sprintf( szCommand, 
+        sprintf( szCommand,
                  "DECLARE mycursor CURSOR for "
                  "SELECT a.attname, t.typname, a.attlen,"
                  "       format_type(a.atttypid,a.atttypmod) "
                  "FROM pg_class c, pg_attribute a, pg_type t "
                  "WHERE c.relname = '%s' "
                  "AND a.attnum > 0 AND a.attrelid = c.oid "
-                 "AND a.atttypid = t.oid", 
+                 "AND a.atttypid = t.oid",
                  pszTable );
 
         hResult = PQexec(hPGConn, szCommand );
@@ -200,7 +277,7 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
 
     if( !hResult || PQresultStatus(hResult) != PGRES_TUPLES_OK )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "%s", PQerrorMessage(hPGConn) );
         return NULL;
     }
@@ -211,6 +288,8 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
     OGRFeatureDefn *poDefn = new OGRFeatureDefn( pszTable );
     int            iRecord;
 
+    poDefn->Reference();
+
     for( iRecord = 0; iRecord < PQntuples(hResult); iRecord++ )
     {
         const char      *pszType, *pszFormatType;
@@ -220,7 +299,7 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
         pszFormatType = PQgetvalue(hResult,iRecord,3);
 
         /* TODO: Add detection of other primary key to use as FID */
-        if( EQUAL(oField.GetNameRef(),"ogc_fid") )
+        if( EQUAL(oField.GetNameRef(),szPrimaryKey) )
         {
             bHasFid = TRUE;
             pszFIDColumn = CPLStrdup(oField.GetNameRef());
@@ -229,12 +308,13 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
         else if( EQUAL(pszType,"geometry") )
         {
             bHasPostGISGeometry = TRUE;
-            pszGeomColumn = CPLStrdup( oField.GetNameRef());
+            pszGeomColumn = CPLStrdup(oField.GetNameRef());
             continue;
         }
         else if( EQUAL(oField.GetNameRef(),"WKB_GEOMETRY") )
         {
             bHasWkb = TRUE;
+            pszGeomColumn = CPLStrdup(oField.GetNameRef());
             if( EQUAL(pszType,"OID") )
                 bWkbAsOid = TRUE;
             continue;
@@ -268,7 +348,7 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
             nWidth = atoi(pszFormatName + 8);
             if( pszPrecision != NULL )
                 nPrecision = atoi(pszPrecision+1);
-            
+
             if( nPrecision == 0 )
                 oField.SetType( OFTInteger );
             else
@@ -298,17 +378,17 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
         {
             oField.SetType( OFTReal );
         }
-        else if( EQUAL(pszType, "date") ) 
+        else if( EQUAL(pszType, "date") )
         {
             oField.SetType( OFTString );
             oField.SetWidth( 10 );
         }
-        else if( EQUAL(pszType, "time") ) 
+        else if( EQUAL(pszType, "time") )
         {
             oField.SetType( OFTString );
             oField.SetWidth( 8 );
         }
-        
+
         poDefn->AddFieldDefn( &oField );
     }
 
@@ -320,6 +400,50 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
     hResult = PQexec(hPGConn, "COMMIT");
     PQclear( hResult );
 
+    // get layer geometry type (for PostGIS dataset)
+    if ( bHasPostGISGeometry )
+    {
+        sprintf(szCommand,
+                "SELECT type, coord_dimension FROM geometry_columns WHERE f_table_name='%s'",
+                pszTable);
+
+        hResult = PQexec(hPGConn,szCommand);
+        if ( hResult && PQntuples(hResult) == 1 && !PQgetisnull(hResult,0,0) )
+        {
+            char * pszType = PQgetvalue(hResult,0,0);
+            OGRwkbGeometryType nGeomType = wkbUnknown;
+
+            nCoordDimension = MAX(2,MIN(3,atoi(PQgetvalue(hResult,0,1))));
+
+            // check only standard OGC geometry types
+            if ( EQUAL(pszType, "POINT") )
+                nGeomType = wkbPoint;
+            else if ( EQUAL(pszType,"LINESTRING"))
+                nGeomType = wkbLineString;
+            else if ( EQUAL(pszType,"POLYGON"))
+                nGeomType = wkbPolygon;
+            else if ( EQUAL(pszType,"MULTIPOINT"))
+                nGeomType = wkbMultiPoint;
+            else if ( EQUAL(pszType,"MULTILINESTRING"))
+                nGeomType = wkbMultiLineString;
+            else if ( EQUAL(pszType,"MULTIPOLYGON"))
+                nGeomType = wkbMultiPolygon;
+            else if ( EQUAL(pszType,"GEOMETRYCOLLECTION"))
+                nGeomType = wkbGeometryCollection;
+
+            if( nCoordDimension == 3 && nGeomType != wkbUnknown )
+                nGeomType = (OGRwkbGeometryType) (nGeomType | wkb25DBit);
+
+            CPLDebug("OGR_PG","Layer '%s' geometry type: %s:%s, Dim=%d",
+                     pszTable, pszType, OGRGeometryTypeToName(nGeomType),
+                     nCoordDimension );
+
+            poDefn->SetGeomType( nGeomType );
+
+            PQclear( hResult );
+        }
+    }
+
     return poDefn;
 }
 
@@ -360,10 +484,10 @@ void OGRPGTableLayer::BuildWhere()
         OGREnvelope  sEnvelope;
 
         m_poFilterGeom->getEnvelope( &sEnvelope );
-        sprintf( szWHERE, 
-                 "WHERE %s && GeometryFromText('BOX3D(%.12f %.12f, %.12f %.12f)'::box3d,%d) ",
-                 pszGeomColumn, 
-                 sEnvelope.MinX, sEnvelope.MinY, 
+        sprintf( szWHERE,
+                 "WHERE %s && SetSRID('BOX3D(%.12f %.12f, %.12f %.12f)'::box3d,%d) ",
+                 pszGeomColumn,
+                 sEnvelope.MinX, sEnvelope.MinY,
                  sEnvelope.MaxX, sEnvelope.MaxY,
                  nSRSId );
     }
@@ -394,13 +518,13 @@ void OGRPGTableLayer::BuildFullQueryStatement()
 
     char *pszFields = BuildFields();
 
-    pszQueryStatement = (char *) 
+    pszQueryStatement = (char *)
         CPLMalloc(strlen(pszFields)+strlen(pszWHERE)
                   +strlen(poFeatureDefn->GetName()) + 40);
     sprintf( pszQueryStatement,
-             "SELECT %s FROM \"%s\" %s", 
+             "SELECT %s FROM \"%s\" %s",
              pszFields, poFeatureDefn->GetName(), pszWHERE );
-    
+
     CPLFree( pszFields );
 }
 
@@ -452,12 +576,23 @@ char *OGRPGTableLayer::BuildFields()
 
         if( bHasPostGISGeometry )
         {
-            sprintf( pszFieldList+strlen(pszFieldList), 
-                     "AsText(\"%s\")", pszGeomColumn );
+            if ( poDS->bUseBinaryCursor )
+            {
+                nSize += 10;
+                sprintf( pszFieldList+strlen(pszFieldList),
+                         "AsBinary(\"%s\")", pszGeomColumn );
+            }
+            else
+            if ( poDS->sPostGISVersion.nMajor >= 1 )
+                sprintf( pszFieldList+strlen(pszFieldList),
+                        "AsEWKT(\"%s\")", pszGeomColumn );
+            else
+                sprintf( pszFieldList+strlen(pszFieldList),
+                        "AsText(\"%s\")", pszGeomColumn );
         }
         else
         {
-            sprintf( pszFieldList+strlen(pszFieldList), 
+            sprintf( pszFieldList+strlen(pszFieldList),
                      "\"%s\"", pszGeomColumn );
         }
     }
@@ -517,7 +652,7 @@ OGRErr OGRPGTableLayer::DeleteFeature( long nFID )
 /* -------------------------------------------------------------------- */
     if( !bHasFid )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "DeleteFeature(%d) failed.  Unable to delete features in tables without\n"
                   "a recognised FID column.",
                   nFID );
@@ -534,7 +669,7 @@ OGRErr OGRPGTableLayer::DeleteFeature( long nFID )
 
     sprintf( pszCommand, "DELETE FROM \"%s\" WHERE \"%s\" = %ld",
              poFeatureDefn->GetName(), pszFIDColumn, nFID );
-             
+
 /* -------------------------------------------------------------------- */
 /*      Execute the insert.                                             */
 /* -------------------------------------------------------------------- */
@@ -551,12 +686,12 @@ OGRErr OGRPGTableLayer::DeleteFeature( long nFID )
 
     if( PQresultStatus(hResult) != PGRES_COMMAND_OK )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
-                  "DeleteFeature() DELETE statement failed.\n%s", 
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "DeleteFeature() DELETE statement failed.\n%s",
                   PQerrorMessage(hPGConn) );
 
         PQclear( hResult );
-        
+
         poDS->SoftRollback();
 
         return OGRERR_FAILURE;
@@ -580,7 +715,7 @@ OGRErr OGRPGTableLayer::SetFeature( OGRFeature *poFeature )
 
     if( poFeature->GetFID() == OGRNullFID )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "FID required on features given to SetFeature()." );
         return OGRERR_FAILURE;
     }
@@ -623,7 +758,7 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
         strcat( pszCommand, "WKB_GEOMETRY " );
         bNeedComma = TRUE;
     }
-    
+
     if( bHasPostGISGeometry && poFeature->GetGeometryRef() != NULL )
     {
         strcat( pszCommand, pszGeomColumn );
@@ -662,19 +797,17 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
     {
         char    *pszWKT = NULL;
 
-        // Do we need to force nSRSId to be set?
-        if( nSRSId == -2 )
-            GetSpatialRef();
-
         if( poFeature->GetGeometryRef() != NULL )
         {
             OGRGeometry *poGeom = (OGRGeometry *) poFeature->GetGeometryRef();
 
             poGeom->closeRings();
+            poGeom->setCoordinateDimension( nCoordDimension );
+
             poGeom->exportToWkt( &pszWKT );
         }
-        
-        if( pszWKT != NULL 
+
+        if( pszWKT != NULL
             && strlen(pszCommand) + strlen(pszWKT) > nCommandBufSize - 10000 )
         {
             nCommandBufSize = strlen(pszCommand) + strlen(pszWKT) + 10000;
@@ -683,8 +816,12 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
 
         if( pszWKT != NULL )
         {
-            sprintf( pszCommand + strlen(pszCommand), 
-                     "GeometryFromText('%s'::TEXT,%d) ", pszWKT, nSRSId );
+            if( poDS->sPostGISVersion.nMajor >= 1 )
+                sprintf( pszCommand + strlen(pszCommand),
+                         "GeomFromEWKT('SRID=%d;%s'::TEXT) ", nSRSId, pszWKT );
+            else
+                sprintf( pszCommand + strlen(pszCommand),
+                         "GeometryFromText('%s'::TEXT,%d) ", pszWKT, nSRSId );
             OGRFree( pszWKT );
         }
         else
@@ -702,7 +839,7 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
 
         if( pszBytea != NULL )
         {
-            sprintf( pszCommand + strlen(pszCommand), 
+            sprintf( pszCommand + strlen(pszCommand),
                      "'%s'", pszBytea );
             CPLFree( pszBytea );
         }
@@ -715,7 +852,7 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
 
         if( oid != 0 )
         {
-            sprintf( pszCommand + strlen(pszCommand), 
+            sprintf( pszCommand + strlen(pszCommand),
                      "'%d' ", oid );
         }
         else
@@ -752,7 +889,7 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
         {
             int nCount, nOff = 0, j;
             const int *panItems = poFeature->GetFieldAsIntegerList(i,&nCount);
-            
+
             pszNeedToFree = (char *) CPLMalloc(nCount * 13 + 10);
             strcpy( pszNeedToFree, "{" );
             for( j = 0; j < nCount; j++ )
@@ -772,7 +909,7 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
         {
             int nCount, nOff = 0, j;
             const double *padfItems =poFeature->GetFieldAsDoubleList(i,&nCount);
-            
+
             pszNeedToFree = (char *) CPLMalloc(nCount * 40 + 10);
             strcpy( pszNeedToFree, "{" );
             for( j = 0; j < nCount; j++ )
@@ -788,7 +925,7 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
         }
 
         // Grow the command buffer?
-        if( strlen(pszStrValue) + strlen(pszCommand+nOffset) + nOffset 
+        if( strlen(pszStrValue) + strlen(pszCommand+nOffset) + nOffset
             > nCommandBufSize-50 )
         {
             nCommandBufSize = strlen(pszCommand) + strlen(pszStrValue) + 10000;
@@ -804,21 +941,21 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
             strcat( pszCommand+nOffset, "'" );
 
             nOffset += strlen(pszCommand+nOffset);
-            
+
             for( iChar = 0; pszStrValue[iChar] != '\0'; iChar++ )
             {
                 if( poFeatureDefn->GetFieldDefn(i)->GetType() != OFTIntegerList
                     && poFeatureDefn->GetFieldDefn(i)->GetType() != OFTRealList
-                    && poFeatureDefn->GetFieldDefn(i)->GetWidth() > 0 
+                    && poFeatureDefn->GetFieldDefn(i)->GetWidth() > 0
                     && iChar == poFeatureDefn->GetFieldDefn(i)->GetWidth() )
                 {
-                    CPLDebug( "PG", 
+                    CPLDebug( "PG",
                               "Truncated %s field value, it was too long.",
                               poFeatureDefn->GetFieldDefn(i)->GetNameRef() );
                     break;
                 }
 
-                if( pszStrValue[iChar] == '\\' 
+                if( pszStrValue[iChar] == '\\'
                     || pszStrValue[iChar] == '\'' )
                 {
                     pszCommand[nOffset++] = '\\';
@@ -850,14 +987,14 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
     {
         CPLDebug( "OGR_PG", "PQexec(%s)\n", pszCommand );
 
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "INSERT command for new feature failed.\n%s\nCommand: %s",
+                  PQerrorMessage(hPGConn), pszCommand);
+
         CPLFree( pszCommand );
 
-        CPLError( CE_Failure, CPLE_AppDefined, 
-                  "INSERT command for new feature failed.\n%s", 
-                  PQerrorMessage(hPGConn) );
-
         PQclear( hResult );
-        
+
         poDS->SoftRollback();
 
         return OGRERR_FAILURE;
@@ -883,7 +1020,7 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
 int OGRPGTableLayer::TestCapability( const char * pszCap )
 
 {
-    if( EQUAL(pszCap,OLCSequentialWrite) 
+    if( EQUAL(pszCap,OLCSequentialWrite)
              || EQUAL(pszCap,OLCRandomWrite) )
         return bUpdateAccess;
 
@@ -893,7 +1030,7 @@ int OGRPGTableLayer::TestCapability( const char * pszCap )
     else if( EQUAL(pszCap,OLCRandomRead) )
         return bHasFid;
 
-    else 
+    else
         return OGRPGLayer::TestCapability( pszCap );
 }
 
@@ -923,12 +1060,12 @@ OGRErr OGRPGTableLayer::CreateField( OGRFieldDefn *poFieldIn, int bApproxOK )
 
         if( EQUAL(oField.GetNameRef(),"oid") )
         {
-            CPLError( CE_Warning, CPLE_AppDefined, 
+            CPLError( CE_Warning, CPLE_AppDefined,
                       "Renaming field 'oid' to 'oid_' to avoid conflict with internal oid field." );
             oField.SetName( "oid_" );
         }
     }
-    
+
 /* -------------------------------------------------------------------- */
 /*      Work out the PostgreSQL type.                                   */
 /* -------------------------------------------------------------------- */
@@ -941,9 +1078,9 @@ OGRErr OGRPGTableLayer::CreateField( OGRFieldDefn *poFieldIn, int bApproxOK )
     }
     else if( oField.GetType() == OFTReal )
     {
-        if( oField.GetWidth() > 0 && oField.GetPrecision() > 0 
+        if( oField.GetWidth() > 0 && oField.GetPrecision() > 0
             && bPreservePrecision )
-            sprintf( szFieldType, "NUMERIC(%d,%d)", 
+            sprintf( szFieldType, "NUMERIC(%d,%d)",
                      oField.GetWidth(), oField.GetPrecision() );
         else
             strcpy( szFieldType, "FLOAT8" );
@@ -988,12 +1125,12 @@ OGRErr OGRPGTableLayer::CreateField( OGRFieldDefn *poFieldIn, int bApproxOK )
     hResult = PQexec(hPGConn, "BEGIN");
     PQclear( hResult );
 
-    sprintf( szCommand, "ALTER TABLE \"%s\" ADD COLUMN \"%s\" %s", 
+    sprintf( szCommand, "ALTER TABLE \"%s\" ADD COLUMN \"%s\" %s",
              poFeatureDefn->GetName(), oField.GetNameRef(), szFieldType );
     hResult = PQexec(hPGConn, szCommand);
     if( PQresultStatus(hResult) != PGRES_COMMAND_OK )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Failure, CPLE_AppDefined,
                   "%s\n%s", szCommand, PQerrorMessage(hPGConn) );
 
         PQclear( hResult );
@@ -1040,10 +1177,10 @@ OGRFeature *OGRPGTableLayer::GetFeature( long nFeatureId )
     poDS->FlushSoftTransaction();
     poDS->SoftStartTransaction();
 
-    sprintf( pszCommand, 
+    sprintf( pszCommand,
              "DECLARE getfeaturecursor CURSOR for "
-             "SELECT %s FROM \"%s\" WHERE %s = %ld", 
-             pszFieldList, poFeatureDefn->GetName(), pszFIDColumn, 
+             "SELECT %s FROM \"%s\" WHERE %s = %ld",
+             pszFieldList, poFeatureDefn->GetName(), pszFIDColumn,
              nFeatureId );
     CPLFree( pszFieldList );
 
@@ -1111,13 +1248,13 @@ int OGRPGTableLayer::GetFeatureCount( int bForce )
     hResult = PQexec(hPGConn, "BEGIN");
     PQclear( hResult );
 
-    sprintf( szCommand, 
+    sprintf( szCommand,
              "DECLARE countCursor CURSOR for "
              "SELECT count(*) FROM \"%s\" "
              "%s",
              poFeatureDefn->GetName(), pszWHERE );
 
-    CPLDebug( "OGR_PG", "PQexec(%s)\n", 
+    CPLDebug( "OGR_PG", "PQexec(%s)\n",
               szCommand );
 
     hResult = PQexec(hPGConn, szCommand);
@@ -1135,7 +1272,7 @@ int OGRPGTableLayer::GetFeatureCount( int bForce )
 
     hResult = PQexec(hPGConn, "COMMIT");
     PQclear( hResult );
-    
+
     return nCount;
 }
 
@@ -1160,14 +1297,14 @@ OGRSpatialReference *OGRPGTableLayer::GetSpatialRef()
 
         poDS->SoftStartTransaction();
 
-        sprintf( szCommand, 
+        sprintf( szCommand,
                  "SELECT srid FROM geometry_columns "
                  "WHERE f_table_name = '%s'",
                  poFeatureDefn->GetName() );
         hResult = PQexec(hPGConn, szCommand );
 
-        if( hResult 
-            && PQresultStatus(hResult) == PGRES_TUPLES_OK 
+        if( hResult
+            && PQresultStatus(hResult) == PGRES_TUPLES_OK
             && PQntuples(hResult) == 1 )
         {
             nSRSId = atoi(PQgetvalue(hResult,0,0));
@@ -1179,3 +1316,73 @@ OGRSpatialReference *OGRPGTableLayer::GetSpatialRef()
 
     return OGRPGLayer::GetSpatialRef();
 }
+
+/************************************************************************/
+/*                             GetExtent()                              */
+/*                                                                      */
+/*      For PostGIS use internal Extend(geometry) function              */
+/*      in other cases we use standard OGRLayer::GetExtent()            */
+/************************************************************************/
+
+OGRErr OGRPGTableLayer::GetExtent( OGREnvelope *psExtent, int bForce )
+{
+    if ( psExtent == NULL )
+        return OGRERR_FAILURE;
+
+    if ( bHasPostGISGeometry )
+    {
+        PGconn          *hPGConn = poDS->GetPGConn();
+        PGresult        *hResult;
+        char            szCommand[1024];
+
+        sprintf( szCommand, "SELECT Extent(\"%s\") FROM \"%s\"", pszGeomColumn, poFeatureDefn->GetName() );
+
+        hResult = PQexec( hPGConn, szCommand );
+        if( ! hResult || PQresultStatus(hResult) != PGRES_TUPLES_OK || PQgetisnull(hResult,0,0) )
+        {
+            PQclear( hResult );
+            CPLDebug("OGR_PG","Unable to get extent by PostGIS. Using standard OGRLayer method.");
+            return OGRPGLayer::GetExtent( psExtent, bForce );
+        }
+
+        char * pszBox = PQgetvalue(hResult,0,0);
+        char * ptr = pszBox;
+        char szVals[64*6+6];
+
+        while ( *ptr != '(' && ptr ) ptr++; ptr++;
+
+        strncpy(szVals,ptr,strstr(ptr,")") - ptr);
+        szVals[strstr(ptr,")") - ptr] = '\0';
+
+        char ** papszTokens = CSLTokenizeString2(szVals," ,",CSLT_HONOURSTRINGS);
+        int nTokenCnt = poDS->sPostGISVersion.nMajor >= 1 ? 4 : 6;
+
+        if ( CSLCount(papszTokens) != nTokenCnt )
+        {
+            CPLError( CE_Failure, CPLE_IllegalArg,
+                      "Bad extent representation: '%s'", pszBox);
+            CSLDestroy(papszTokens);
+            PQclear(hResult);
+            return OGRERR_FAILURE;
+        }
+
+        // Take X,Y coords
+        // For PostGis ver >= 1.0.0 -> Tokens: X1 Y1 X2 Y2 (nTokenCnt = 4)
+        // For PostGIS ver < 1.0.0 -> Tokens: X1 Y1 Z1 X2 Y2 Z2 (nTokenCnt = 6)
+        // =>   X2 index calculated as nTokenCnt/2
+        //      Y2 index caluclated as nTokenCnt/2+1
+
+        psExtent->MinX = CPLScanDouble(papszTokens[0],strlen(papszTokens[0]),"C");
+        psExtent->MinY = CPLScanDouble(papszTokens[1],strlen(papszTokens[1]),"C");
+        psExtent->MaxX = CPLScanDouble(papszTokens[nTokenCnt/2],strlen(papszTokens[nTokenCnt/2]),"C");
+        psExtent->MaxY = CPLScanDouble(papszTokens[nTokenCnt/2+1],strlen(papszTokens[nTokenCnt/2+1]),"C");
+
+        CSLDestroy(papszTokens);
+
+        PQclear( hResult );
+        return OGRERR_NONE;
+    }
+
+    return OGRLayer::GetExtent( psExtent, bForce );
+}
+
