@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrreclayer.cpp,v 1.6 2005/02/02 20:54:27 fwarmerdam Exp $
+ * $Id: ogrreclayer.cpp,v 1.7 2005/09/21 00:55:42 fwarmerdam Exp $
  *
  * Project:  EPIInfo .REC Reader
  * Purpose:  Implements OGRRECLayer class.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrreclayer.cpp,v $
+ * Revision 1.7  2005/09/21 00:55:42  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.6  2005/02/02 20:54:27  fwarmerdam
  * track m_nFeaturesRead
  *
@@ -52,7 +55,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrreclayer.cpp,v 1.6 2005/02/02 20:54:27 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrreclayer.cpp,v 1.7 2005/09/21 00:55:42 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                            OGRRECLayer()                             */
@@ -72,6 +75,7 @@ OGRRECLayer::OGRRECLayer( const char *pszLayerNameIn,
     nNextFID = 1;
 
     poFeatureDefn = new OGRFeatureDefn( pszLayerNameIn );
+    poFeatureDefn->Reference();
 
     nFieldCount = 0;
     panFieldOffset = (int *) CPLCalloc(sizeof(int),nFieldCountIn);
@@ -162,8 +166,13 @@ OGRRECLayer::~OGRRECLayer()
                   (int) m_nFeaturesRead, 
                   poFeatureDefn->GetName() );
     }
+    
+    if( fpREC != NULL )
+        VSIFClose( fpREC );
+
+    if( poFeatureDefn )
+        poFeatureDefn->Release();
 
-    delete poFeatureDefn;
     CPLFree( panFieldOffset );
     CPLFree( panFieldWidth );
 }
@@ -273,6 +282,8 @@ OGRFeature * OGRRECLayer::GetNextUnfilteredFeature()
     poFeature->SetFID( nNextFID++ );
     m_nFeaturesRead++;
 
+    CPLFree( pszRecord );
+
     return poFeature;
 }
 
