@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrs57datasource.cpp,v 1.26 2004/08/30 20:11:51 warmerda Exp $
+ * $Id: ogrs57datasource.cpp,v 1.27 2005/09/21 00:54:43 fwarmerdam Exp $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements OGRS57DataSource class
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrs57datasource.cpp,v $
+ * Revision 1.27  2005/09/21 00:54:43  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.26  2004/08/30 20:11:51  warmerda
  * keep the S57ClassRegistrar on the driver, not the datasource
  *
@@ -112,7 +115,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrs57datasource.cpp,v 1.26 2004/08/30 20:11:51 warmerda Exp $");
+CPL_CVSID("$Id: ogrs57datasource.cpp,v 1.27 2005/09/21 00:54:43 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                          OGRS57DataSource()                          */
@@ -171,7 +174,7 @@ OGRS57DataSource::~OGRS57DataSource()
 
     CSLDestroy( papszOptions );
 
-    delete poSpatialRef;
+    poSpatialRef->Release();
 
     if( poWriter != NULL )
     {
