@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrs57driver.cpp,v 1.7 2004/08/30 20:11:51 warmerda Exp $
+ * $Id: ogrs57driver.cpp,v 1.8 2005/07/30 17:23:05 fwarmerdam Exp $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements OGRS57Driver
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrs57driver.cpp,v $
+ * Revision 1.8  2005/07/30 17:23:05  fwarmerdam
+ * added profile to LoadInfo
+ *
  * Revision 1.7  2004/08/30 20:11:51  warmerda
  * keep the S57ClassRegistrar on the driver, not the datasource
  *
@@ -54,7 +57,7 @@
 #include "ogr_s57.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrs57driver.cpp,v 1.7 2004/08/30 20:11:51 warmerda Exp $");
+CPL_CVSID("$Id: ogrs57driver.cpp,v 1.8 2005/07/30 17:23:05 fwarmerdam Exp $");
 
 S57ClassRegistrar *OGRS57Driver::poRegistrar = NULL;
 
@@ -164,7 +167,7 @@ S57ClassRegistrar *OGRS57Driver::GetS57Registrar()
     {
         poRegistrar = new S57ClassRegistrar();
 
-        if( !poRegistrar->LoadInfo( NULL, FALSE ) )
+        if( !poRegistrar->LoadInfo( NULL, NULL, FALSE ) )
         {
             delete poRegistrar;
             poRegistrar = NULL;
