@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrs57layer.cpp,v 1.14 2005/02/22 12:53:12 fwarmerdam Exp $
+ * $Id: ogrs57layer.cpp,v 1.15 2005/09/21 00:54:43 fwarmerdam Exp $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements OGRS57Layer class.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: ogrs57layer.cpp,v $
+ * Revision 1.15  2005/09/21 00:54:43  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.14  2005/02/22 12:53:12  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -76,7 +79,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrs57layer.cpp,v 1.14 2005/02/22 12:53:12 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrs57layer.cpp,v 1.15 2005/09/21 00:54:43 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                            OGRS57Layer()                             */
@@ -128,7 +131,7 @@ OGRS57Layer::~OGRS57Layer()
                   poFeatureDefn->GetName() );
     }
 
-    delete poFeatureDefn;
+    poFeatureDefn->Release();
 }
 
 /************************************************************************/
