@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: s57classregistrar.cpp,v 1.9 2004/08/30 20:11:14 warmerda Exp $
+ * $Id: s57classregistrar.cpp,v 1.16 2005/08/30 17:57:34 fwarmerdam Exp $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements S57ClassRegistrar class for keeping track of
@@ -29,12 +29,52 @@
  ******************************************************************************
  *
  * $Log: s57classregistrar.cpp,v $
+ * Revision 1.16  2005/08/30 17:57:34  fwarmerdam
+ * allow directly set profile names too
+ *
+ * Revision 1.15  2005/08/30 17:55:50  fwarmerdam
+ * Changed profile names
+ *
+ * Revision 1.14  2005/07/30 18:26:59  fwarmerdam
+ * dont freak out about duplicate attributes
+ *
+ * Revision 1.13  2005/07/30 17:24:27  fwarmerdam
+ * Use CPLGetConfigOption for S57_CSV value.
+ *
+ * Revision 1.12  2005/07/30 17:22:11  fwarmerdam
+ * added preliminary "profile" support, and make class selection case
+ * sensitive.
+ *
+ * Revision 16.1  2005/06/21 18:29:33  hsaggu
+ * =S57 : Updates to the reader and first check-in of writer code from Frank.(PR#5223).<hss><geh>
+ *
+ * Revision 1.11  2005/05/03 16:11:53  fwarmerdam
+ * Fixed initialization of attr tables.
+ *
+ * Revision 1.10  2005/04/27 13:34:14  fwarmerdam
+ * Fixed a few memory leaks, including the ones from bug 840.
+ *
+ * Revision 16.0  2005/04/14 19:27:58  geh
+ * FME 2006 Beta baseline start. <geh>
+ *
  * Revision 1.9  2004/08/30 20:11:14  warmerda
  * various optimizations made
  *
+ * Revision 15.0  2004/04/26 20:09:44  geh
+ * FME 2005 Baseline just after making branch FME_2004_ICE_Branch. <geh>
+ *
+ * Revision 14.0  2003/11/07 23:21:02  cvs
+ * FME 2004 Baseline before branch/FME 2004ICE start point
+ *
  * Revision 1.8  2003/09/12 21:18:54  warmerda
  * open csv files in binary mode, not text mode
  *
+ * Revision 5.0  2003/04/05 02:03:15  geh
+ * 2003X2 Baseline before branch/ FME 2004 Start Point
+ *
+ * Revision 4.1  2003/02/19 22:40:29  ryan
+ * S57 - Latest version from Frank that does not fail to build with the newest OGR <rjp><dal>
+ *
  * Revision 1.7  2001/12/17 22:38:42  warmerda
  * restructure LoadInfo() to support in-code tables
  *
@@ -63,7 +103,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: s57classregistrar.cpp,v 1.9 2004/08/30 20:11:14 warmerda Exp $");
+CPL_CVSID("$Id: s57classregistrar.cpp,v 1.16 2005/08/30 17:57:34 fwarmerdam Exp $");
 
 
 #ifdef S57_BUILTIN_CLASSES
@@ -86,6 +126,12 @@ S57ClassRegistrar::S57ClassRegistrar()
     papszTempResult = NULL;
     papszNextLine = NULL;
     papapszClassesFields = NULL;
+    pachAttrClass = NULL;
+    pachAttrType = NULL;
+    panAttrIndex = NULL;
+    papszAttrNames = NULL;
+    papszAttrAcronym = NULL;
+    papapszAttrValues = NULL;
 }
 
 /************************************************************************/
@@ -95,15 +141,30 @@ S57ClassRegistrar::S57ClassRegistrar()
 S57ClassRegistrar::~S57ClassRegistrar()
 
 {
+    int i;
+
     CSLDestroy( papszClassesInfo );
     CSLDestroy( papszTempResult );
     
     if( papapszClassesFields != NULL )
     {
-        for( int i = 0; i < nClasses; i++ )
+        for( i = 0; i < nClasses; i++ )
             CSLDestroy( papapszClassesFields[i] );
         CPLFree( papapszClassesFields );
     }
+    if( papszAttrNames )
+    {
+        for( i = 0; i < MAX_ATTRIBUTES; i++ )
+        {
+            CPLFree( papszAttrNames[i] );
+            CPLFree( papszAttrAcronym[i] );
+        }
+        CPLFree( papszAttrNames );
+        CPLFree( papszAttrAcronym );
+    }
+    CPLFree( pachAttrType );
+    CPLFree( pachAttrClass );
+    CPLFree( panAttrIndex );
 }
 
 /************************************************************************/
@@ -183,18 +244,40 @@ const char *S57ClassRegistrar::ReadLine( FILE * fp )
 /************************************************************************/
 
 int S57ClassRegistrar::LoadInfo( const char * pszDirectory, 
+                                 const char * pszProfile,
                                  int bReportErr )
 
 {
     FILE        *fp;
+    char        szTargetFile[1024];
 
-    if( pszDirectory == NULL && getenv( "S57_CSV" ) != NULL )
-        pszDirectory = getenv( "S57_CSV" );
+    if( pszDirectory == NULL )
+        pszDirectory = CPLGetConfigOption("S57_CSV",NULL);
 
 /* ==================================================================== */
 /*      Read the s57objectclasses file.                                 */
 /* ==================================================================== */
-    if( !FindFile( "s57objectclasses.csv", pszDirectory, bReportErr, &fp ) )
+    if( pszProfile == NULL )
+        pszProfile = CPLGetConfigOption( "S57_PROFILE", "" );
+    
+    if( EQUAL(pszProfile, "Additional_Military_Layers") )
+    {
+       sprintf( szTargetFile, "s57objectclasses_%s.csv", "aml" );
+    }
+    else if ( EQUAL(pszProfile, "Inland_Waterways") )
+    {
+       sprintf( szTargetFile, "s57objectclasses_%s.csv", "iw" );
+    }
+    else if( strlen(pszProfile) > 0 )
+    {
+       sprintf( szTargetFile, "s57objectclasses_%s.csv", pszProfile );
+    }
+    else
+    {
+       strcpy( szTargetFile, "s57objectclasses.csv" );
+    }
+
+    if( !FindFile( szTargetFile, pszDirectory, bReportErr, &fp ) )
         return FALSE;
 
 /* -------------------------------------------------------------------- */
@@ -247,7 +330,21 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
 /* ==================================================================== */
 /*      Read the attributes list.                                       */
 /* ==================================================================== */
-    if( !FindFile( "s57attributes.csv", pszDirectory, bReportErr, &fp ) )
+
+    if( EQUAL(pszProfile, "Additional_Military_Layers") )
+    {
+        sprintf( szTargetFile, "s57attributes_%s.csv", "aml" );
+    }
+    else if ( EQUAL(pszProfile, "Inland_Waterways") )
+    {
+       sprintf( szTargetFile, "s57attributes_%s.csv", "iw" );
+    }
+    else
+    {
+       strcpy( szTargetFile, "s57attributes.csv" );
+    }
+       
+    if( !FindFile( szTargetFile, pszDirectory, bReportErr, &fp ) )
         return FALSE;
 
 /* -------------------------------------------------------------------- */
@@ -267,12 +364,12 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
 /*      Prepare arrays for the per-attribute information.               */
 /* -------------------------------------------------------------------- */
     nAttrMax = MAX_ATTRIBUTES-1;
-    papszAttrNames = (char **) CPLCalloc(sizeof(char *),nAttrMax);
-    papszAttrAcronym = (char **) CPLCalloc(sizeof(char *),nAttrMax);
-    papapszAttrValues = (char ***) CPLCalloc(sizeof(char **),nAttrMax);
-    pachAttrType = (char *) CPLCalloc(sizeof(char),nAttrMax);
-    pachAttrClass = (char *) CPLCalloc(sizeof(char),nAttrMax);
-    panAttrIndex = (int *) CPLCalloc(sizeof(int),nAttrMax);
+    papszAttrNames = (char **) CPLCalloc(sizeof(char *),MAX_ATTRIBUTES);
+    papszAttrAcronym = (char **) CPLCalloc(sizeof(char *),MAX_ATTRIBUTES);
+    //papapszAttrValues = (char ***) CPLCalloc(sizeof(char **),MAX_ATTRIBUTES);
+    pachAttrType = (char *) CPLCalloc(sizeof(char),MAX_ATTRIBUTES);
+    pachAttrClass = (char *) CPLCalloc(sizeof(char),MAX_ATTRIBUTES);
+    panAttrIndex = (int *) CPLCalloc(sizeof(int),MAX_ATTRIBUTES);
     
 /* -------------------------------------------------------------------- */
 /*      Read and form string list.                                      */
@@ -294,7 +391,8 @@ int S57ClassRegistrar::LoadInfo( const char * pszDirectory,
         if( iAttr < 0 || iAttr >= nAttrMax
             || papszAttrNames[iAttr] != NULL )
         {
-            CPLAssert( FALSE );
+            CPLDebug( "S57", "Duplicate definition for attribute %d:%s", 
+                      iAttr, papszTokens[2] );
             continue;
         }
         
@@ -407,7 +505,7 @@ int S57ClassRegistrar::SelectClass( const char *pszAcronym )
         if( !SelectClassByIndex( i ) )
             continue;
 
-        if( EQUAL(GetAcronym(),pszAcronym) )
+        if( strcmp(GetAcronym(),pszAcronym) == 0 )
             return TRUE;
     }
 
