@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: s57featuredefns.cpp,v 1.2 2003/11/17 20:10:46 warmerda Exp $
+ * $Id: s57featuredefns.cpp,v 1.4 2005/09/21 00:54:43 fwarmerdam Exp $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements methods to create OGRFeatureDefns for various
@@ -29,6 +29,12 @@
  ******************************************************************************
  *
  * $Log: s57featuredefns.cpp,v $
+ * Revision 1.4  2005/09/21 00:54:43  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.3  2005/07/30 18:26:38  fwarmerdam
+ * accept one character primitives like we get in AML classes
+ *
  * Revision 1.2  2003/11/17 20:10:46  warmerda
  * added support for writing FFPT linkages
  *
@@ -42,7 +48,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: s57featuredefns.cpp,v 1.2 2003/11/17 20:10:46 warmerda Exp $");
+CPL_CVSID("$Id: s57featuredefns.cpp,v 1.4 2005/09/21 00:54:43 fwarmerdam Exp $");
 
 
 /************************************************************************/
@@ -83,6 +89,7 @@ OGRFeatureDefn *S57GenerateGeomFeatureDefn( OGRwkbGeometryType eGType,
     else
         return NULL;
 
+    poFDefn->Reference();
     S57GenerateStandardAttributes( poFDefn, nOptionFlags );
 
     return poFDefn;
@@ -121,6 +128,8 @@ S57GenerateVectorPrimitiveFeatureDefn( int nRCNM, int nOptionFlags )
     else
         return NULL;
 
+    poFDefn->Reference();
+
 /* -------------------------------------------------------------------- */
 /*      Core vector primitive attributes                                */
 /* -------------------------------------------------------------------- */
@@ -203,6 +212,7 @@ OGRFeatureDefn *S57GenerateObjectClassDefn( S57ClassRegistrar *poCR,
 /*      acronym.                                                        */
 /* -------------------------------------------------------------------- */
     poFDefn = new OGRFeatureDefn( poCR->GetAcronym() );
+    poFDefn->Reference();
 
 /* -------------------------------------------------------------------- */
 /*      Try and establish the geometry type.  If more than one          */
@@ -217,7 +227,7 @@ OGRFeatureDefn *S57GenerateObjectClassDefn( S57ClassRegistrar *poCR,
     {
         // leave as unknown geometry type.
     }
-    else if( EQUAL(papszGeomPrim[0],"Point") )
+    else if( papszGeomPrim[0][0] == 'P' )
     {
         if( EQUAL(poCR->GetAcronym(),"SOUNDG") )
         {
@@ -229,11 +239,11 @@ OGRFeatureDefn *S57GenerateObjectClassDefn( S57ClassRegistrar *poCR,
         else
             poFDefn->SetGeomType( wkbPoint );
     }
-    else if( EQUAL(papszGeomPrim[0],"Area") )
+    else if( papszGeomPrim[0][0] == 'A' )
     {
         poFDefn->SetGeomType( wkbPolygon );
     }
-    else if( EQUAL(papszGeomPrim[0],"Line") )
+    else if( papszGeomPrim[0][0] == 'L' )
     {
         poFDefn->SetGeomType( wkbLineString );
     }
