@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: s57reader.cpp,v 1.49 2004/08/30 20:10:38 warmerda Exp $
+ * $Id: s57reader.cpp,v 1.53 2005/08/07 13:58:17 fwarmerdam Exp $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements S57Reader class.
@@ -28,6 +28,19 @@
  ******************************************************************************
  *
  * $Log: s57reader.cpp,v $
+ * Revision 1.53  2005/08/07 13:58:17  fwarmerdam
+ * avoid use of std::sprintf in favor of plain sprintf
+ *
+ * Revision 1.52  2005/08/06 04:41:42  fwarmerdam
+ * Incorporated support for finding update files using the catalog file
+ * as per bug 840.
+ *
+ * Revision 1.51  2005/08/06 04:23:51  fwarmerdam
+ * Bug 840: allow linestrings with zero vertices between end nodes.
+ *
+ * Revision 1.50  2005/06/23 18:22:48  fwarmerdam
+ * Fixed ReadNextFeature(poTarget=NULL) support.
+ *
  * Revision 1.49  2004/08/30 20:10:38  warmerda
  * implemented FetchLine() with optimized access
  *
@@ -183,7 +196,10 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: s57reader.cpp,v 1.49 2004/08/30 20:10:38 warmerda Exp $");
+#include <string>
+#include <fstream>
+
+CPL_CVSID("$Id: s57reader.cpp,v 1.53 2005/08/07 13:58:17 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                             S57Reader()                              */
@@ -236,6 +252,8 @@ S57Reader::~S57Reader()
     
     CPLFree( pszModuleName );
     CSLDestroy( papszOptions );
+
+    CPLFree( papoFDefnList );
 }
 
 /************************************************************************/
@@ -693,7 +711,7 @@ OGRFeature * S57Reader::ReadNextFeature( OGRFeatureDefn * poTarget )
             oFE_Index.SetClientInfoByIndex( nNextFEIndex, poFeatureDefn );
         }
 
-        if( poFeatureDefn != poTarget )
+        if( poFeatureDefn != poTarget && poTarget != NULL )
         {
             nNextFEIndex++;
             continue;
@@ -1356,12 +1374,13 @@ int S57Reader::FetchLine( DDFRecord *poSRecord,
         return TRUE;
     }
 
+/* -------------------------------------------------------------------- */
+/*      It is legitimate to have zero vertices for line segments        */
+/*      that just have the start and end node (bug 840).                */
+/* -------------------------------------------------------------------- */
     if( nVCount == 0 )
-    {
-        CPLDebug( "S57", "VCount is zero." );
-        return FALSE;
-    }
-
+        return TRUE;
+ 
 /* -------------------------------------------------------------------- */
 /*      Make sure out line is long enough to hold all the vertices      */
 /*      we will apply.                                                  */
@@ -1814,8 +1833,6 @@ void S57Reader::AssembleAreaGeometry( DDFRecord * poFRecord,
             if( !FetchLine( poSRecord, poLine->getNumPoints(), 1, poLine ) )
             {
                 CPLDebug( "S57", "FetchLine() failed in AssembleAreaGeometry()!" );
-                delete poLine;
-                continue;
             }
 
 
@@ -2470,26 +2487,79 @@ int S57Reader::FindAndApplyUpdates( const char * pszPath )
 
     for( iUpdate = 1; bSuccess; iUpdate++ )
     {
-        char    szExtension[4];
-        char    *pszUpdateFilename;
-        DDFModule oUpdateModule;
-
-        sprintf( szExtension, "%03d", iUpdate );
-        
-        pszUpdateFilename = CPLStrdup(CPLResetExtension(pszPath,szExtension));
-
-        bSuccess = oUpdateModule.Open( pszUpdateFilename, TRUE );
-
-        if( bSuccess )
-            CPLDebug( "S57", "Applying feature updates from %s.", 
-                      pszUpdateFilename );
-        CPLFree( pszUpdateFilename );
-
-        if( bSuccess )
+        //Creaing file extension
+        std::string extension;
+        std::string dirname;
+        if( 1 <= iUpdate &&  iUpdate < 10 )
         {
-            if( !ApplyUpdates( &oUpdateModule ) )
-                return FALSE;
+            char buf[2];
+            sprintf( buf, "%i", iUpdate );
+            extension.append("00");
+            extension.append(buf);
+            dirname.append(buf);
         }
+        else if( 10 <= iUpdate && iUpdate < 100 )
+        {
+            char buf[3];
+            sprintf( buf, "%i", iUpdate );
+            extension.append("0");
+            extension.append(buf);
+            dirname.append(buf);
+        }
+        else if( 100 <= iUpdate && iUpdate < 1000 )
+        {
+            char buf[4];
+            sprintf( buf, "%i", iUpdate );
+            extension.append(buf);
+            dirname.append(buf);
+        }
+
+        DDFModule oUpdateModule;
+          
+        //trying current dir first
+        char    *pszUpdateFilename = 
+            CPLStrdup(CPLResetExtension(pszPath,extension.c_str()));
+
+        std::ifstream file(pszUpdateFilename);
+        if( file )
+        {
+            file.close();
+            bSuccess = oUpdateModule.Open( pszUpdateFilename, TRUE );
+            if( bSuccess )
+                CPLDebug( "S57", "Applying feature updates from %s.", 
+                          pszUpdateFilename );
+            if( bSuccess )
+            {
+                if( !ApplyUpdates( &oUpdateModule ) )
+                    return FALSE;
+            }
+        }
+        else // file is store on Primar generated cd
+        {
+            file.close();
+            char* pszBaseFileDir = CPLStrdup(CPLGetDirname(pszPath));
+            char* pszFileDir = CPLStrdup(CPLGetDirname(pszBaseFileDir));
+            std::string remotefile(pszFileDir);
+            remotefile.append( "/" );
+            remotefile.append( dirname );
+            remotefile.append( "/" );
+            remotefile.append( CPLGetBasename(pszPath) );
+            remotefile.append( "." );
+            remotefile.append( extension );
+            bSuccess = oUpdateModule.Open( remotefile.c_str(), TRUE );
+	
+            if( bSuccess )
+                CPLDebug( "S57", "Applying feature updates from %s.", 
+                          remotefile.c_str() );
+            CPLFree( pszBaseFileDir );
+            CPLFree( pszFileDir );
+            if( bSuccess )
+            {
+                if( !ApplyUpdates( &oUpdateModule ) )
+                    return FALSE;
+            }
+        }//end for if-else
+        CPLFree( pszUpdateFilename );
     }
 
     return TRUE;
