@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrshapelayer.cpp,v 1.19 2005/02/22 12:51:56 fwarmerdam Exp $
+ * $Id: ogrshapelayer.cpp,v 1.21 2005/09/21 00:53:20 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRShapeLayer class.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogrshapelayer.cpp,v $
+ * Revision 1.21  2005/09/21 00:53:20  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.20  2005/09/10 22:32:34  fwarmerdam
+ * Make GetNextShape() more bulletproof if SHPReadOGRFeature() fails.
+ *
  * Revision 1.19  2005/02/22 12:51:56  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -92,7 +98,7 @@
 #include "ogrshape.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrshapelayer.cpp,v 1.19 2005/02/22 12:51:56 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrshapelayer.cpp,v 1.21 2005/09/21 00:53:20 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           OGRShapeLayer()                            */
@@ -150,10 +156,8 @@ OGRShapeLayer::~OGRShapeLayer()
 
     CPLFree( pszFullName );
 
-    delete poFeatureDefn;
-
-    if( poSRS != NULL )
-        delete poSRS;
+    poFeatureDefn->Release();
+    poSRS->Release();
 
     if( hDBF != NULL )
         DBFClose( hDBF );
@@ -353,8 +357,6 @@ OGRFeature *OGRShapeLayer::GetNextFeature()
 
             poFeature = SHPReadOGRFeature( hSHP, hDBF, poFeatureDefn, 
                                            panMatchingFIDs[iMatchingFID++] );
-            if( poFeature != NULL )
-                m_nFeaturesRead++;
         }
         else
         {
@@ -363,17 +365,20 @@ OGRFeature *OGRShapeLayer::GetNextFeature()
     
             poFeature = SHPReadOGRFeature( hSHP, hDBF, poFeatureDefn,
                                            iNextShapeId++ );
-            if( poFeature != NULL )
-                m_nFeaturesRead++;
         }
 
-        if( (m_poFilterGeom == NULL
-             || FilterGeometry( poFeature->GetGeometryRef() ) )
-            && (m_poAttrQuery == NULL
-                || m_poAttrQuery->Evaluate( poFeature )) )
-            return poFeature;
+        if( poFeature != NULL )
+        {
+            m_nFeaturesRead++;
 
-        delete poFeature;
+            if( (m_poFilterGeom == NULL
+                 || FilterGeometry( poFeature->GetGeometryRef() ) )
+                && (m_poAttrQuery == NULL
+                    || m_poAttrQuery->Evaluate( poFeature )) )
+                return poFeature;
+
+            delete poFeature;
+        }
     }        
 }
 
