@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: shpopen.c,v 1.45 2004/09/26 20:09:48 fwarmerdam Exp $
+ * $Id: shpopen.c,v 1.46 2005/02/11 17:17:46 fwarmerdam Exp $
  *
  * Project:  Shapelib
  * Purpose:  Implementation of core Shapefile read/write functions.
@@ -34,6 +34,9 @@
  ******************************************************************************
  *
  * $Log: shpopen.c,v $
+ * Revision 1.46  2005/02/11 17:17:46  fwarmerdam
+ * added panPartStart[0] validation
+ *
  * Revision 1.45  2004/09/26 20:09:48  fwarmerdam
  * const correctness changes
  *
@@ -183,8 +186,9 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 
-SHP_CVSID("$Id: shpopen.c,v 1.45 2004/09/26 20:09:48 fwarmerdam Exp $")
+SHP_CVSID("$Id: shpopen.c,v 1.46 2005/02/11 17:17:46 fwarmerdam Exp $")
 
 typedef unsigned char uchar;
 
@@ -955,11 +959,23 @@ SHPCreateObject( int nSHPType, int nShapeId, int nParts,
         for( i = 0; i < nParts; i++ )
         {
             psObject->panPartStart[i] = panPartStart[i];
+
             if( panPartType != NULL )
                 psObject->panPartType[i] = panPartType[i];
             else
                 psObject->panPartType[i] = SHPP_RING;
         }
+
+        if( psObject->panPartStart[0] != 0 )
+        {
+#ifdef USE_CPL
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "panPartStart[0] != 0, patching internally.  Please fix your code!\n" );
+#else
+            fprintf( stderr, "panPartStart[0] != 0, patching internally.  Please fix your code!\n" );
+#endif
+            psObject->panPartStart[0] = 0;
+        }
     }
 
 /* -------------------------------------------------------------------- */
