@@ -1,6 +1,6 @@
 /*-*-C++-*-*/
 /******************************************************************************
- * $Id: ogr_tiger.h,v 1.19 2005/02/22 12:50:22 fwarmerdam Exp $
+ * $Id: ogr_tiger.h,v 1.21 2005/04/06 16:05:29 fwarmerdam Exp $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Main declarations for Tiger translator.
@@ -29,6 +29,12 @@
  ******************************************************************************
  *
  * $Log: ogr_tiger.h,v $
+ * Revision 1.21  2005/04/06 16:05:29  fwarmerdam
+ * added spatialmetadata (RTM) support
+ *
+ * Revision 1.20  2005/04/06 15:04:23  fwarmerdam
+ * added TIGER2004 support
+ *
  * Revision 1.19  2005/02/22 12:50:22  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -135,6 +141,7 @@ typedef enum {
     TIGER_UA2000 = 10,
     TIGER_2002 = 11,
     TIGER_2003 = 12,
+    TIGER_2004 = 13,
     TIGER_Unknown
 } TigerVersion;
 
@@ -588,6 +595,26 @@ public:
   virtual OGRErr      CreateFeature( OGRFeature *poFeature );
 };
 
+/************************************************************************/
+/*                TigerSpatialMetadata (Type M records)                 */
+/************************************************************************/
+
+class TigerSpatialMetadata : public TigerFileBase
+{
+ private:
+  TigerRecordInfo    *psRTMInfo;
+
+public:
+                      TigerSpatialMetadata( OGRTigerDataSource *, const char * );
+  virtual            ~TigerSpatialMetadata();
+
+  virtual int         SetModule( const char * );
+
+  virtual OGRFeature *GetFeature( int );
+
+  virtual OGRErr      CreateFeature( OGRFeature *poFeature );
+};
+
 /************************************************************************/
 /*                   TigerPIP (Type P records)                          */
 /************************************************************************/
