@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrtigerdatasource.cpp,v 1.31 2004/09/22 16:17:06 fwarmerdam Exp $
+ * $Id: ogrtigerdatasource.cpp,v 1.33 2005/04/06 16:05:29 fwarmerdam Exp $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements OGRTigerDataSource class
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: ogrtigerdatasource.cpp,v $
+ * Revision 1.33  2005/04/06 16:05:29  fwarmerdam
+ * added spatialmetadata (RTM) support
+ *
+ * Revision 1.32  2005/04/06 15:04:23  fwarmerdam
+ * added TIGER2004 support
+ *
  * Revision 1.31  2004/09/22 16:17:06  fwarmerdam
  * Make GDT support work even if version number seems weird.
  *
@@ -148,7 +154,7 @@
 #include "cpl_string.h"
 #include <ctype.h>
 
-CPL_CVSID("$Id: ogrtigerdatasource.cpp,v 1.31 2004/09/22 16:17:06 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrtigerdatasource.cpp,v 1.33 2005/04/06 16:05:29 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                        TigerClassifyVersion()                        */
@@ -179,7 +185,8 @@ TigerVersion TigerClassifyVersion( int nVersionCode )
 ** ????    ????
 **
 ** 0206 to 0299   TIGER/Line Files, 2002
-** 0300 and up    TIGER/Line Files, 2003
+** 0300 to 0399   TIGER/Line Files, 2003
+** 0400+          TIGER/Line Files, 2004 - one sample is 0405
 ** ????
 */
 
@@ -221,8 +228,10 @@ TigerVersion TigerClassifyVersion( int nVersionCode )
         nVersion = TIGER_UA2000;
     else if( nVersionCode >=  206 /*0206*/ && nVersionCode < 300 /* 2003 */ )
         nVersion = TIGER_2002;
-    else if( nVersionCode >=  300 /*0300*/ )
+    else if( nVersionCode >=  300 /*0300*/ && nVersionCode <= 400 )
         nVersion = TIGER_2003;
+    else if( nVersionCode >=  400 )
+        nVersion = TIGER_2004;
 
     return nVersion;
 }
@@ -246,6 +255,7 @@ char * TigerVersionString( TigerVersion nVersion )
   if (nVersion == TIGER_UA2000) { return "TIGER_UA2000"; }
   if (nVersion == TIGER_2002) { return "TIGER_2002"; }
   if (nVersion == TIGER_2003) { return "TIGER_2003"; }
+  if (nVersion == TIGER_2004) { return "TIGER_2004"; }
   if (nVersion == TIGER_Unknown) { return "TIGER_Unknown"; }
   return "???";
 }
@@ -660,6 +670,11 @@ int OGRTigerDataSource::Open( const char * pszFilename, int bTestOpen,
                                  new TigerPolyChainLink( this,
                                                        papszModules[0]) ));
     
+    // RTM
+    AddLayer( new OGRTigerLayer( this,
+                                 new TigerSpatialMetadata( this,
+                                                           papszModules[0] ) ) );
+    
     // RTP
     AddLayer( new OGRTigerLayer( this,
                                  new TigerPIP( this,
@@ -970,6 +985,12 @@ OGRLayer *OGRTigerDataSource::CreateLayer( const char *pszLayerName,
                                      new TigerPolygonEconomic( this, NULL ) );
     }
 
+    else if( EQUAL(pszLayerName,"SpatialMetadata") )
+    {
+        poLayer = new OGRTigerLayer( this,
+                                     new TigerSpatialMetadata( this, NULL ) );
+    }
+
     else if( EQUAL(pszLayerName,"ZeroCellID") )
     {
         poLayer = new OGRTigerLayer( this,
