@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigercompletechain.cpp,v 1.20 2004/10/05 19:30:44 fwarmerdam Exp $
+ * $Id: tigercompletechain.cpp,v 1.21 2005/09/21 00:53:19 fwarmerdam Exp $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerCompleteChain, providing access to RT1 and
@@ -29,6 +29,9 @@
  ******************************************************************************
  *
  * $Log: tigercompletechain.cpp,v $
+ * Revision 1.21  2005/09/21 00:53:19  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.20  2004/10/05 19:30:44  fwarmerdam
  * Added bug fix for a "full" shape record at the end of the .RT2 file.
  * http://bugzilla.remotesensing.org/show_bug.cgi?id=628
@@ -97,7 +100,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigercompletechain.cpp,v 1.20 2004/10/05 19:30:44 fwarmerdam Exp $");
+CPL_CVSID("$Id: tigercompletechain.cpp,v 1.21 2005/09/21 00:53:19 fwarmerdam Exp $");
 
 static TigerFieldInfo rt1_2002_fields[] = {
   // fieldname    fmt  type OFTType      beg  end  len  bDefine bSet bWrite
@@ -286,6 +289,7 @@ TigerCompleteChain::TigerCompleteChain( OGRTigerDataSource * poDSIn,
 {
     poDS = poDSIn;
     poFeatureDefn = new OGRFeatureDefn( "CompleteChain" );
+    poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( wkbLineString );
 
 
