@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigerfilebase.cpp,v 1.17 2003/10/15 19:09:23 warmerda Exp $
+ * $Id: tigerfilebase.cpp,v 1.19 2005/09/21 00:53:19 fwarmerdam Exp $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerBaseFile class, providing common services to all
@@ -29,6 +29,13 @@
  ******************************************************************************
  *
  * $Log: tigerfilebase.cpp,v $
+ * Revision 1.19  2005/09/21 00:53:19  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
+ * Revision 1.18  2005/04/06 14:34:37  fwarmerdam
+ * Added TIGER_LFIELD_AS_STRING configuration option to force "L" numeric
+ * fields to be treated as strings to preserve leading zeros.
+ *
  * Revision 1.17  2003/10/15 19:09:23  warmerda
  * fix featuredefn memory leak
  *
@@ -87,8 +94,9 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 #include "cpl_error.h"
+#include "cpl_string.h"
 
-CPL_CVSID("$Id: tigerfilebase.cpp,v 1.17 2003/10/15 19:09:23 warmerda Exp $");
+CPL_CVSID("$Id: tigerfilebase.cpp,v 1.19 2005/09/21 00:53:19 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           TigerFileBase()                            */
@@ -118,7 +126,7 @@ TigerFileBase::~TigerFileBase()
 
     if( poFeatureDefn != NULL )
     {
-        delete poFeatureDefn;
+        poFeatureDefn->Release();
         poFeatureDefn = NULL;
     }
 
@@ -514,16 +522,26 @@ int TigerFileBase::SetWriteModule( const char *pszExtension, int nRecLen,
 void TigerFileBase::AddFieldDefns(TigerRecordInfo *psRTInfo,
                                   OGRFeatureDefn  *poFeatureDefn)
 {
-  OGRFieldDefn        oField("",OFTInteger);
-  int i;
-  for (i=0; i<psRTInfo->nFieldCount; ++i) {
-    if (psRTInfo->pasFields[i].bDefine) {
-      oField.Set( psRTInfo->pasFields[i].pszFieldName,
-                  psRTInfo->pasFields[i].OGRtype,
-                  psRTInfo->pasFields[i].nLen );
-      poFeatureDefn->AddFieldDefn( &oField );
+    OGRFieldDefn        oField("",OFTInteger);
+    int i, bLFieldHack;
+
+    bLFieldHack = 
+        CSLTestBoolean( CPLGetConfigOption( "TIGER_LFIELD_AS_STRING", "NO" ) );
+    
+    for (i=0; i<psRTInfo->nFieldCount; ++i) {
+        if (psRTInfo->pasFields[i].bDefine) {
+            OGRFieldType eFT = psRTInfo->pasFields[i].OGRtype;
+
+            if( bLFieldHack 
+                && psRTInfo->pasFields[i].cFmt == 'L' 
+                && psRTInfo->pasFields[i].cType == 'N' )
+                eFT = OFTString;
+
+            oField.Set( psRTInfo->pasFields[i].pszFieldName, eFT, 
+                        psRTInfo->pasFields[i].nLen );
+            poFeatureDefn->AddFieldDefn( &oField );
+        }
     }
-  }
 }
 
 /************************************************************************/
