@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigerkeyfeatures.cpp,v 1.10 2003/01/04 23:21:56 mbp Exp $
+ * $Id: tigerkeyfeatures.cpp,v 1.11 2005/09/21 00:53:19 fwarmerdam Exp $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerKeyFeatures, providing access to .RT9 files.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: tigerkeyfeatures.cpp,v $
+ * Revision 1.11  2005/09/21 00:53:19  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.10  2003/01/04 23:21:56  mbp
  * Minor bug fixes and field definition changes.  Cleaned
  * up and commented code written for TIGER 2002 support.
@@ -65,7 +68,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigerkeyfeatures.cpp,v 1.10 2003/01/04 23:21:56 mbp Exp $");
+CPL_CVSID("$Id: tigerkeyfeatures.cpp,v 1.11 2005/09/21 00:53:19 fwarmerdam Exp $");
 
 #define FILE_CODE "9"
 
@@ -105,6 +108,7 @@ TigerKeyFeatures::TigerKeyFeatures( OGRTigerDataSource * poDSIn,
 
     poDS = poDSIn;
     poFeatureDefn = new OGRFeatureDefn( "KeyFeatures" );
+    poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( wkbNone );
 
     psRT9Info = &rt9_info;
