@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigerzipplus4.cpp,v 1.8 2003/01/04 23:21:56 mbp Exp $
+ * $Id: tigerzipplus4.cpp,v 1.9 2005/09/21 00:53:19 fwarmerdam Exp $
  *
  * Project:  TIGER/Line Translator
  * Purpose:  Implements TigerZipPlus4, providing access to .RTZ files.
@@ -28,6 +28,9 @@
  ******************************************************************************
  *
  * $Log: tigerzipplus4.cpp,v $
+ * Revision 1.9  2005/09/21 00:53:19  fwarmerdam
+ * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
+ *
  * Revision 1.8  2003/01/04 23:21:56  mbp
  * Minor bug fixes and field definition changes.  Cleaned
  * up and commented code written for TIGER 2002 support.
@@ -59,7 +62,7 @@
 #include "ogr_tiger.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: tigerzipplus4.cpp,v 1.8 2003/01/04 23:21:56 mbp Exp $");
+CPL_CVSID("$Id: tigerzipplus4.cpp,v 1.9 2005/09/21 00:53:19 fwarmerdam Exp $");
 
 #define FILE_CODE       "Z"
 
@@ -90,6 +93,7 @@ TigerZipPlus4::TigerZipPlus4( OGRTigerDataSource * poDSIn,
 
     poDS = poDSIn;
     poFeatureDefn = new OGRFeatureDefn( "ZipPlus4" );
+    poFeatureDefn->Reference();
     poFeatureDefn->SetGeomType( wkbNone );
 
     psRTZInfo = &rtZ_info;
