@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_vrt.h,v 1.3 2005/02/22 12:50:10 fwarmerdam Exp $
+ * $Id: ogr_vrt.h,v 1.7 2005/09/05 19:35:10 fwarmerdam Exp $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for OGR/VRT driver.
@@ -28,6 +28,18 @@
  ******************************************************************************
  *
  * $Log: ogr_vrt.h,v $
+ * Revision 1.7  2005/09/05 19:35:10  fwarmerdam
+ * Added shape support
+ *
+ * Revision 1.6  2005/08/16 00:08:52  fwarmerdam
+ * Added bSrcLayerFromSQL
+ *
+ * Revision 1.5  2005/08/02 20:17:26  fwarmerdam
+ * pass attribute filter to sublayer
+ *
+ * Revision 1.4  2005/05/16 20:09:46  fwarmerdam
+ * added spatial query on x/y columns
+ *
  * Revision 1.3  2005/02/22 12:50:10  fwarmerdam
  * use OGRLayer base spatial filter support
  *
@@ -51,7 +63,8 @@ typedef enum {
     VGS_Direct,
     VGS_PointFromColumns, 
     VGS_WKT, 
-    VGS_WKB
+    VGS_WKB,
+    VGS_Shape
 } OGRVRTGeometryStyle;
 
 /************************************************************************/
@@ -66,11 +79,12 @@ class OGRVRTLayer : public OGRLayer
     OGRDataSource       *poSrcDS;
     OGRLayer            *poSrcLayer;
     int                 bNeedReset;
+    int                 bSrcLayerFromSQL;
 
     // Layer spatial reference system, and srid.
     OGRSpatialReference *poSRS;
 
-    char                *pszQuery;
+    char                *pszAttrFilter;
 
     int                 iFIDField; // -1 means pass through. 
 
@@ -87,6 +101,9 @@ class OGRVRTLayer : public OGRLayer
     int                *pabDirectCopy;
 
     OGRFeature         *TranslateFeature( OGRFeature * );
+    OGRErr              createFromShapeBin( GByte *, OGRGeometry **, int );
+
+    void                ResetSourceReading();
 
   public:
                         OGRVRTLayer();
@@ -106,7 +123,7 @@ class OGRVRTLayer : public OGRLayer
 
     virtual int         GetFeatureCount( int );
 
-//    virtual OGRErr      SetAttributeFilter( const char * );
+    virtual OGRErr      SetAttributeFilter( const char * );
 
     virtual int         TestCapability( const char * );
 };
