@@ -19,6 +19,18 @@
  ******************************************************************************
  *
  * $Log: swq.c,v $
+ * Revision 1.26  2005/08/03 13:31:36  fwarmerdam
+ * Added ILIKE, but LIKE and ILIKE are both still case insensitive.
+ *
+ * Revision 1.25  2005/07/13 18:02:58  mbrudka
+ * Removed serious bug in sql_realloc which sometimes corrupted heap.
+ *
+ * Revision 1.24  2005/03/17 04:59:20  fwarmerdam
+ * added local FORCE_CDECL definition
+ *
+ * Revision 1.23  2005/03/17 04:21:25  fwarmerdam
+ * use FORCE_CDECL for args to qsort()
+ *
  * Revision 1.22  2003/05/21 04:49:17  warmerda
  * avoid warnings
  *
@@ -99,6 +111,12 @@
 #define SWQ_FREE(x) free(x)
 #endif
 
+#ifdef _MSC_VER
+#  define FORCE_CDECL  __cdecl
+#else
+#  define FORCE_CDECL 
+#endif
+
 #ifndef TRUE
 #  define TRUE 1
 #endif
@@ -149,11 +167,11 @@ void *swq_realloc( void *old_mem, int old_size, int new_size )
 
     if( old_mem != NULL )
     {
-        memcpy( new_mem, old_mem, old_size );
+        memcpy( new_mem, old_mem, old_size < new_size ? old_size : new_size);
         SWQ_FREE( old_mem );
     }
-
-    memset( ((char *) new_mem) + old_size, 0, new_size - old_size );
+    if (old_size <= new_size )
+        memset( ((char *) new_mem) + old_size, 0, new_size - old_size );
 
     return new_mem;
 }
@@ -373,7 +391,8 @@ static swq_op swq_identify_op( char **tokens, int *tokens_consumed )
     if( strcasecmp(token,"NOT") == 0 )
     {
         if( tokens[*tokens_consumed+1] != NULL
-            && strcasecmp(tokens[*tokens_consumed+1],"LIKE") == 0 )
+            && (strcasecmp(tokens[*tokens_consumed+1],"LIKE") == 0 
+                || strcasecmp(tokens[*tokens_consumed+1],"ILIKE") == 0) )
         {
             *tokens_consumed += 1;
             return SWQ_NOTLIKE;
@@ -412,6 +431,9 @@ static swq_op swq_identify_op( char **tokens, int *tokens_consumed )
     if( strcasecmp(token,"LIKE") == 0 )
         return SWQ_LIKE;
 
+    if( strcasecmp(token,"ILIKE") == 0 )
+        return SWQ_LIKE;
+
     if( strcasecmp(token,"IN") == 0 )
         return SWQ_IN;
 
@@ -2146,7 +2168,7 @@ swq_select_summarize( swq_select *select_info,
 /*                      sort comparison functions.                      */
 /************************************************************************/
 
-static int swq_compare_int( const void *item1, const void *item2 )
+static int FORCE_CDECL swq_compare_int( const void *item1, const void *item2 )
 {
     int  v1, v2;
 
@@ -2156,7 +2178,7 @@ static int swq_compare_int( const void *item1, const void *item2 )
     return v1 - v2;
 }
 
-static int swq_compare_real( const void *item1, const void *item2 )
+static int FORCE_CDECL swq_compare_real( const void *item1, const void *item2 )
 {
     double  v1, v2;
 
@@ -2171,7 +2193,7 @@ static int swq_compare_real( const void *item1, const void *item2 )
         return 1;
 }
 
-static int swq_compare_string( const void *item1, const void *item2 )
+static int FORCE_CDECL swq_compare_string( const void *item1, const void *item2 )
 {
     return strcmp( *((const char **) item1), *((const char **) item2) );
 }
@@ -2186,7 +2208,7 @@ static int swq_compare_string( const void *item1, const void *item2 )
 const char *swq_select_finish_summarize( swq_select *select_info )
 
 {
-    int (*compare_func)(const void *, const void*);
+    int (FORCE_CDECL *compare_func)(const void *, const void*);
     int count;
     char **distinct_list;
 
