@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_csv.cpp,v 1.12 2004/11/22 16:01:05 fwarmerdam Exp $
+ * $Id: cpl_csv.cpp,v 1.14 2005/08/31 03:33:35 fwarmerdam Exp $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  CSV (comma separated value) file access.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: cpl_csv.cpp,v $
+ * Revision 1.14  2005/08/31 03:33:35  fwarmerdam
+ * dont bother trying to make szPath in CSVFileOverride() threadlocal
+ *
+ * Revision 1.13  2005/05/23 03:58:15  fwarmerdam
+ * make make static buffers threadlocal
+ *
  * Revision 1.12  2004/11/22 16:01:05  fwarmerdam
  * added GDAL_PREFIX
  *
@@ -75,7 +81,7 @@
 #include "cpl_csv.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: cpl_csv.cpp,v 1.12 2004/11/22 16:01:05 fwarmerdam Exp $");
+CPL_CVSID("$Id: cpl_csv.cpp,v 1.14 2005/08/31 03:33:35 fwarmerdam Exp $");
 
 CPL_C_START
 const char * GDALDefaultCSVFilename( const char *pszBasename );
@@ -107,7 +113,10 @@ typedef struct ctb {
     char        *pszRawData;
 } CSVTable;
 
-static CSVTable *psCSVTableList = NULL;
+/* It would likely be better to share this list between threads, but
+   that will require some rework. */
+
+static CPL_THREADLOCAL CSVTable *psCSVTableList = NULL;
 
 /************************************************************************/
 /*                             CSVAccess()                              */
@@ -903,10 +912,10 @@ const char *CSVGetField( const char * pszFilename,
 const char * GDALDefaultCSVFilename( const char *pszBasename )
 
 {
-    static char         szPath[512];
+    static CPL_THREADLOCAL char         szPath[512];
     FILE    *fp = NULL;
     const char *pszResult;
-    static int bFinderInitialized = FALSE;
+    static CPL_THREADLOCAL int bFinderInitialized = FALSE;
 
     pszResult = CPLFindFile( "epsg_csv", pszBasename );
 
