@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cpl_error.cpp,v 1.27 2003/05/27 20:44:16 warmerda Exp $
+ * $Id: cpl_error.cpp,v 1.32 2005/07/25 18:09:10 fwarmerdam Exp $
  *
  * Name:     cpl_error.cpp
  * Project:  CPL - Common Portability Library
@@ -29,6 +29,22 @@
  **********************************************************************
  *
  * $Log: cpl_error.cpp,v $
+ * Revision 1.32  2005/07/25 18:09:10  fwarmerdam
+ * Made pfnErrorHandler static.
+ *
+ * Revision 1.31  2005/07/25 17:56:34  fwarmerdam
+ * Altered so that CPLSetErrorHandler sets a handler global to all threads, but
+ * that the error handler stack takes precidence over it.
+ *
+ * Revision 1.30  2005/07/08 14:35:56  fwarmerdam
+ * restructured to use error context and TLS
+ *
+ * Revision 1.29  2005/05/23 03:58:51  fwarmerdam
+ * make handler stack threadlocal
+ *
+ * Revision 1.28  2005/04/04 15:23:31  fwarmerdam
+ * some functions now CPL_STDCALL
+ *
  * Revision 1.27  2003/05/27 20:44:16  warmerda
  * use VSI time services
  *
@@ -115,20 +131,14 @@
 #include "cpl_error.h"
 #include "cpl_vsi.h"
 #include "cpl_conv.h"
+#include "cpl_multiproc.h"
 
 #define TIMESTAMP_DEBUG
 
-CPL_CVSID("$Id: cpl_error.cpp,v 1.27 2003/05/27 20:44:16 warmerda Exp $");
+CPL_CVSID("$Id: cpl_error.cpp,v 1.32 2005/07/25 18:09:10 fwarmerdam Exp $");
 
-/* static buffer to store the last error message.  We'll assume that error
- * messages cannot be longer than 2000 chars... which is quite reasonable
- * (that's 25 lines of 80 chars!!!)
- */
-static char gszCPLLastErrMsg[2000] = "";
-static int  gnCPLLastErrNo = 0;
-static CPLErr geCPLLastErrType = CE_None;
-
-static CPLErrorHandler gpfnCPLErrorHandler = CPLDefaultErrorHandler;
+static void *hErrorMutex = NULL;
+static CPLErrorHandler pfnErrorHandler = CPLDefaultErrorHandler;
 
 typedef struct errHandler
 {
@@ -136,7 +146,34 @@ typedef struct errHandler
     CPLErrorHandler     pfnHandler;
 } CPLErrorHandlerNode;
 
-static CPLErrorHandlerNode * psHandlerStack = NULL;
+typedef struct {
+    char    szLastErrMsg[2000];
+    int     nLastErrNo;
+    CPLErr  eLastErrType;
+    
+    CPLErrorHandlerNode *psHandlerStack;
+} CPLErrorContext;
+
+/************************************************************************/
+/*                         CPLGetErrorContext()                         */
+/************************************************************************/
+
+static CPLErrorContext *CPLGetErrorContext()
+
+{
+    CPLErrorContext *psCtx = 
+        (CPLErrorContext *) CPLGetTLS( CTLS_ERRORCONTEXT );
+
+    if( psCtx == NULL )
+    {
+        psCtx = (CPLErrorContext *) CPLCalloc(sizeof(CPLErrorContext),1);
+        psCtx->eLastErrType = CE_None;
+        CPLSetTLS( CTLS_ERRORCONTEXT, psCtx, TRUE );
+    }
+
+    return psCtx;
+}
+
 
 /**********************************************************************
  *                          CPLError()
@@ -191,25 +228,39 @@ void    CPLError(CPLErr eErrClass, int err_no, const char *fmt, ...)
 
 void    CPLErrorV(CPLErr eErrClass, int err_no, const char *fmt, va_list args )
 {
+    CPLErrorContext *psCtx = CPLGetErrorContext();
+
     /* Expand the error message 
      */
 #if defined(HAVE_VSNPRINTF)
-    vsnprintf( gszCPLLastErrMsg, sizeof(gszCPLLastErrMsg), fmt, args );
+    vsnprintf( psCtx->szLastErrMsg, sizeof(psCtx->szLastErrMsg), fmt, args );
 #else
-    vsprintf(gszCPLLastErrMsg, fmt, args);
+    vsprintf( psCtx->szLastErrMsg, fmt, args);
 #endif
 
     /* If the user provided his own error handling function, then call
      * it, otherwise print the error to stderr and return.
      */
-    gnCPLLastErrNo = err_no;
-    geCPLLastErrType = eErrClass;
+    psCtx->nLastErrNo = err_no;
+    psCtx->eLastErrType = eErrClass;
 
     if( CPLGetConfigOption("CPL_LOG_ERRORS",NULL) != NULL )
-        CPLDebug( "CPLError", "%s", gszCPLLastErrMsg );
+        CPLDebug( "CPLError", "%s", psCtx->szLastErrMsg );
 
-    if( gpfnCPLErrorHandler )
-        gpfnCPLErrorHandler(eErrClass, err_no, gszCPLLastErrMsg);
+/* -------------------------------------------------------------------- */
+/*      Invoke the current error handler.                               */
+/* -------------------------------------------------------------------- */
+    if( psCtx->psHandlerStack != NULL )
+    {
+        psCtx->psHandlerStack->pfnHandler(eErrClass, err_no, 
+                                          psCtx->szLastErrMsg);
+    }
+    else
+    {
+        CPLMutexHolderD( &hErrorMutex );
+        if( pfnErrorHandler != NULL )
+            pfnErrorHandler(eErrClass, err_no, psCtx->szLastErrMsg);
+    }
 
     if( eErrClass == CE_Fatal )
         abort();
@@ -243,6 +294,7 @@ void    CPLErrorV(CPLErr eErrClass, int err_no, const char *fmt, va_list args )
 void CPLDebug( const char * pszCategory, const char * pszFormat, ... )
 
 {
+    CPLErrorContext *psCtx = CPLGetErrorContext();
     char        *pszMessage;
     va_list     args;
     const char  *pszDebug = CPLGetConfigOption("CPL_DEBUG",NULL);
@@ -318,11 +370,18 @@ void CPLDebug( const char * pszCategory, const char * pszFormat, ... )
     va_end(args);
 
 /* -------------------------------------------------------------------- */
-/*      If the user provided his own error handling function, then call */
-/*      it, otherwise print the error to stderr and return.             */
+/*      Invoke the current error handler.                               */
 /* -------------------------------------------------------------------- */
-    if( gpfnCPLErrorHandler )
-        gpfnCPLErrorHandler(CE_Debug, CPLE_None, pszMessage);
+    if( psCtx->psHandlerStack != NULL )
+    {
+        psCtx->psHandlerStack->pfnHandler( CE_Debug, CPLE_None, pszMessage );
+    }
+    else
+    {
+        CPLMutexHolderD( &hErrorMutex );
+        if( pfnErrorHandler != NULL )
+            pfnErrorHandler( CE_Debug, CPLE_None, pszMessage );
+    }
 
     VSIFree( pszMessage );
 }
@@ -338,11 +397,13 @@ void CPLDebug( const char * pszCategory, const char * pszFormat, ... )
  * from does not appear to be still in play with high level functions.
  */
 
-void    CPLErrorReset()
+void CPL_STDCALL CPLErrorReset()
 {
-    gnCPLLastErrNo = CPLE_None;
-    gszCPLLastErrMsg[0] = '\0';
-    geCPLLastErrType = CE_None;
+    CPLErrorContext *psCtx = CPLGetErrorContext();
+
+    psCtx->nLastErrNo = CPLE_None;
+    psCtx->szLastErrMsg[0] = '\0';
+    psCtx->eLastErrType = CE_None;
 }
 
 
@@ -359,9 +420,11 @@ void    CPLErrorReset()
  * if there are no posted errors.
  */
 
-int     CPLGetLastErrorNo()
+int CPL_STDCALL CPLGetLastErrorNo()
 {
-    return gnCPLLastErrNo;
+    CPLErrorContext *psCtx = CPLGetErrorContext();
+
+    return psCtx->nLastErrNo;
 }
 
 /**********************************************************************
@@ -377,9 +440,11 @@ int     CPLGetLastErrorNo()
  * if there are no posted errors.
  */
 
-CPLErr CPLGetLastErrorType()
+CPLErr CPL_STDCALL CPLGetLastErrorType()
 {
-    return geCPLLastErrType;
+    CPLErrorContext *psCtx = CPLGetErrorContext();
+
+    return psCtx->eLastErrType;
 }
 
 /**********************************************************************
@@ -397,16 +462,18 @@ CPLErr CPLGetLastErrorType()
  * message.
  */
 
-const char* CPLGetLastErrorMsg()
+const char* CPL_STDCALL CPLGetLastErrorMsg()
 {
-    return gszCPLLastErrMsg;
+    CPLErrorContext *psCtx = CPLGetErrorContext();
+
+    return psCtx->szLastErrMsg;
 }
 
 /************************************************************************/
 /*                       CPLDefaultErrorHandler()                       */
 /************************************************************************/
 
-void CPLDefaultErrorHandler( CPLErr eErrClass, int nError, 
+void CPL_STDCALL CPLDefaultErrorHandler( CPLErr eErrClass, int nError, 
                              const char * pszErrorMsg )
 
 {
@@ -440,7 +507,7 @@ void CPLDefaultErrorHandler( CPLErr eErrClass, int nError,
 /*                        CPLQuietErrorHandler()                        */
 /************************************************************************/
 
-void CPLQuietErrorHandler( CPLErr eErrClass , int nError, 
+void CPL_STDCALL CPLQuietErrorHandler( CPLErr eErrClass , int nError, 
                            const char * pszErrorMsg )
 
 {
@@ -452,7 +519,7 @@ void CPLQuietErrorHandler( CPLErr eErrClass , int nError,
 /*                       CPLLoggingErrorHandler()                       */
 /************************************************************************/
 
-void CPLLoggingErrorHandler( CPLErr eErrClass, int nError, 
+void CPL_STDCALL CPLLoggingErrorHandler( CPLErr eErrClass, int nError, 
                              const char * pszErrorMsg )
 
 {
@@ -551,48 +618,75 @@ void CPLLoggingErrorHandler( CPLErr eErrClass, int nError,
  * make any attempt to report the passed error or warning messages but
  * will process debug messages via CPLDefaultErrorHandler.
  *
+ * Note that error handlers set with CPLSetErrorHandler() apply to all
+ * threads in an application, while error handlers set with CPLPushErrorHandler
+ * are thread-local.  However, any error handlers pushed with 
+ * CPLPushErrorHandler (and not removed with CPLPopErrorHandler) take 
+ * precidence over the global error handlers set with CPLSetErrorHandler(). 
+ * Generally speaking CPLSetErrorHandler() would be used to set a desired
+ * global error handler, while CPLPushErrorHandler() would be used to install
+ * a temporary local error handler, such as CPLQuietErrorHandler() to suppress
+ * error reporting in a limited segment of code. 
+ *
  * @param pfnErrorHandler new error handler function.
  * @return returns the previously installed error handler.
  */ 
 
-CPLErrorHandler CPLSetErrorHandler( CPLErrorHandler pfnErrorHandler )
+CPLErrorHandler CPL_STDCALL 
+CPLSetErrorHandler( CPLErrorHandler pfnErrorHandlerNew )
 {
-    CPLErrorHandler     pfnOldHandler = gpfnCPLErrorHandler;
-    
-    gpfnCPLErrorHandler = pfnErrorHandler;
+    CPLErrorHandler     pfnOldHandler = pfnErrorHandler;
+    CPLErrorContext *psCtx = CPLGetErrorContext();
+
+    if( psCtx->psHandlerStack != NULL )
+    {
+        CPLDebug( "CPL", 
+                  "CPLSetErrorHandler() called with an error handler on\n"
+                  "the local stack.  New error handler will not be used immediately.\n" );
+    }
+
+
+    {
+        CPLMutexHolderD( &hErrorMutex );
+
+        pfnOldHandler = pfnErrorHandler;
+        
+        if( pfnErrorHandler == NULL )
+            pfnErrorHandler = CPLDefaultErrorHandler;
+        else
+            pfnErrorHandler = pfnErrorHandlerNew;
+    }
 
     return pfnOldHandler;
 }
 
-
-
 /************************************************************************/
 /*                        CPLPushErrorHandler()                         */
 /************************************************************************/
 
 /**
- * Assign new CPLError handler.
+ * Push a new CPLError handler.
  *
- * The old handler is "pushed down" onto a stack and can be easily
- * restored with CPLPopErrorHandler().  Otherwise this works similarly
- * to CPLSetErrorHandler() which contains more details on how error
- * handlers work.
+ * This pushes a new error handler on the thread-local error handler
+ * stack.  This handler will be used untill removed with CPLPopErrorHandler().
+ *
+ * The CPLSetErrorHandler() docs have further information on how 
+ * CPLError handlers work.
  *
  * @param pfnErrorHandler new error handler function.
  */
 
-void CPLPushErrorHandler( CPLErrorHandler pfnErrorHandler )
+void CPL_STDCALL CPLPushErrorHandler( CPLErrorHandler pfnErrorHandlerNew )
 
 {
+    CPLErrorContext *psCtx = CPLGetErrorContext();
     CPLErrorHandlerNode         *psNode;
 
     psNode = (CPLErrorHandlerNode *) VSIMalloc(sizeof(CPLErrorHandlerNode));
-    psNode->psNext = psHandlerStack;
-    psNode->pfnHandler = gpfnCPLErrorHandler;
+    psNode->psNext = psCtx->psHandlerStack;
+    psNode->pfnHandler = pfnErrorHandlerNew;
 
-    psHandlerStack = psNode;
-
-    CPLSetErrorHandler( pfnErrorHandler );
+    psCtx->psHandlerStack = psNode;
 }
 
 /************************************************************************/
@@ -600,21 +694,24 @@ void CPLPushErrorHandler( CPLErrorHandler pfnErrorHandler )
 /************************************************************************/
 
 /**
- * Restore old CPLError handler.
+ * Pop error handler off stack.
  *
- * Discards the current error handler, and restore the one in use before
- * the last CPLPushErrorHandler() call.
+ * Discards the current error handler on the error handler stack, and restores 
+ * the one in use before the last CPLPushErrorHandler() call.  This method
+ * has no effect if there are no error handlers on the current threads error
+ * handler stack. 
  */ 
 
-void CPLPopErrorHandler()
+void CPL_STDCALL CPLPopErrorHandler()
 
 {
-    if( psHandlerStack != NULL )
-    {
-        CPLErrorHandlerNode     *psNode = psHandlerStack;
+    CPLErrorContext *psCtx = CPLGetErrorContext();
 
-        psHandlerStack = psNode->psNext;
-        CPLSetErrorHandler( psNode->pfnHandler );
+    if( psCtx->psHandlerStack != NULL )
+    {
+        CPLErrorHandlerNode     *psNode = psCtx->psHandlerStack;
+
+        psCtx->psHandlerStack = psNode->psNext;
         VSIFree( psNode );
     }
 }
@@ -637,7 +734,7 @@ void CPLPopErrorHandler()
  * There is no reason for application code to call _CPLAssert() directly.
  */
 
-void _CPLAssert( const char * pszExpression, const char * pszFile,
+void CPL_STDCALL _CPLAssert( const char * pszExpression, const char * pszFile,
                  int iLine )
 
 {
