@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_findfile.cpp,v 1.8 2005/01/15 07:46:20 fwarmerdam Exp $
+ * $Id: cpl_findfile.cpp,v 1.10 2005/09/13 23:56:06 fwarmerdam Exp $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Generic data file location finder, with application hooking.
@@ -28,6 +28,12 @@
  ******************************************************************************
  *
  * $Log: cpl_findfile.cpp,v $
+ * Revision 1.10  2005/09/13 23:56:06  fwarmerdam
+ * improved cleanup logic
+ *
+ * Revision 1.9  2005/05/23 03:59:07  fwarmerdam
+ * make finder stuff threadlocal
+ *
  * Revision 1.8  2005/01/15 07:46:20  fwarmerdam
  * make cplpopfinderlocation safer for final cleanup
  *
@@ -57,12 +63,12 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: cpl_findfile.cpp,v 1.8 2005/01/15 07:46:20 fwarmerdam Exp $");
+CPL_CVSID("$Id: cpl_findfile.cpp,v 1.10 2005/09/13 23:56:06 fwarmerdam Exp $");
 
-static int bFinderInitialized = FALSE;
-static int nFileFinders = 0;
-static CPLFileFinder *papfnFinders = NULL;
-static char **papszFinderLocations = NULL;
+static CPL_THREADLOCAL int bFinderInitialized = FALSE;
+static CPL_THREADLOCAL int nFileFinders = 0;
+static CPL_THREADLOCAL CPLFileFinder *papfnFinders = NULL;
+static CPL_THREADLOCAL char **papszFinderLocations = NULL;
 
 /************************************************************************/
 /*                           CPLFinderInit()                            */
@@ -171,7 +177,7 @@ CPLFileFinder CPLPopFileFinder()
 {
     CPLFileFinder pfnReturn;
 
-    CPLFinderInit();
+//    CPLFinderInit();
 
     if( nFileFinders == 0 )
         return NULL;
