@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cpl_minixml.cpp,v 1.34 2005/03/09 17:07:25 fwarmerdam Exp $
+ * $Id: cpl_minixml.cpp,v 1.38 2005/09/11 19:30:12 fwarmerdam Exp $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Implementation of MiniXML Parser and handling.
@@ -37,6 +37,19 @@
  *   hostile source.
  *
  * $Log: cpl_minixml.cpp,v $
+ * Revision 1.38  2005/09/11 19:30:12  fwarmerdam
+ * Try to write in text mode through large file API.
+ *
+ * Revision 1.37  2005/09/11 19:14:54  fwarmerdam
+ * Use largefile API for virtualization support.  Note that XML files are
+ * no longer produced in machine-local text format.
+ *
+ * Revision 1.36  2005/05/22 08:17:07  fwarmerdam
+ * allow CPLAddXMLChild() to support siblings
+ *
+ * Revision 1.35  2005/05/13 18:17:48  fwarmerdam
+ * added CPLRemoveXMLChild
+ *
  * Revision 1.34  2005/03/09 17:07:25  fwarmerdam
  * added CPLSearchXMLNode
  *
@@ -150,7 +163,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: cpl_minixml.cpp,v 1.34 2005/03/09 17:07:25 fwarmerdam Exp $");
+CPL_CVSID("$Id: cpl_minixml.cpp,v 1.38 2005/09/11 19:30:12 fwarmerdam Exp $");
 
 typedef enum {
     TNone,
@@ -1360,9 +1373,6 @@ void CPLAddXMLChild( CPLXMLNode *psParent, CPLXMLNode *psChild )
 {
     CPLXMLNode *psSib;
 
-    CPLAssert( psChild->psNext == NULL );
-    psChild->psNext = NULL;
-
     if( psParent->psChild == NULL )
     {
         psParent->psChild = psChild;
@@ -1397,6 +1407,49 @@ void CPLAddXMLChild( CPLXMLNode *psParent, CPLXMLNode *psChild )
     psSib->psNext = psChild;
 }
 
+/************************************************************************/
+/*                           CPLAddXMLChild()                           */
+/************************************************************************/
+
+/**
+ * Remove child node from parent. 
+ *
+ * The passed child is removed from the child list of the passed parent,
+ * but the child is not destroyed.  The child retains ownership of it's
+ * own children, but is cleanly removed from the child list of the parent.
+ *
+ * @param psParent the node to the child is attached to.
+ *
+ * @param psChild the child to remove.
+ *
+ * @return TRUE on success or FALSE if the child was not found.
+ */
+
+int CPLRemoveXMLChild( CPLXMLNode *psParent, CPLXMLNode *psChild )
+
+{
+    CPLXMLNode *psLast = NULL, *psThis;
+
+    if( psParent == NULL )
+        return FALSE;
+
+    for( psThis = psParent->psChild; psThis != NULL; psThis = psThis->psNext )
+    {
+        if( psThis == psChild )
+        {
+            if( psLast == NULL )
+                psParent->psChild = psThis->psNext;
+            else 
+                psLast->psNext = psThis->psNext;
+
+            psThis->psNext = NULL;
+            return TRUE;
+        }
+    }
+
+    return FALSE;
+}
+
 /************************************************************************/
 /*                          CPLAddXMLSibling()                          */
 /************************************************************************/
@@ -1707,7 +1760,7 @@ CPLXMLNode *CPLParseXMLFile( const char *pszFilename )
 /* -------------------------------------------------------------------- */
 /*      Read the file.                                                  */
 /* -------------------------------------------------------------------- */
-    fp = VSIFOpen( pszFilename, "rb" );
+    fp = VSIFOpenL( pszFilename, "rb" );
     if( fp == NULL )
     {
         CPLError( CE_Failure, CPLE_OpenFailed, 
@@ -1715,9 +1768,9 @@ CPLXMLNode *CPLParseXMLFile( const char *pszFilename )
         return NULL;
     }
 
-    VSIFSeek( fp, 0, SEEK_END );
-    nLen = VSIFTell( fp );
-    VSIFSeek( fp, 0, SEEK_SET );
+    VSIFSeekL( fp, 0, SEEK_END );
+    nLen = VSIFTellL( fp );
+    VSIFSeekL( fp, 0, SEEK_SET );
     
     pszDoc = (char *) VSIMalloc(nLen+1);
     if( pszDoc == NULL )
@@ -1726,17 +1779,17 @@ CPLXMLNode *CPLParseXMLFile( const char *pszFilename )
                   "Out of memory allocating space for %d byte buffer in\n"
                   "CPLParseXMLFile(%.500s).", 
                   nLen+1, pszFilename );
-        VSIFClose( fp );
+        VSIFCloseL( fp );
         return NULL;
     }
-    if( (int) VSIFRead( pszDoc, 1, nLen, fp ) < nLen )
+    if( (int) VSIFReadL( pszDoc, 1, nLen, fp ) < nLen )
     {
         CPLError( CE_Failure, CPLE_FileIO, 
                   "VSIFRead() result short of expected %d bytes from %.500s.", 
                   nLen, pszFilename );
         pszDoc[0] = '\0';
     }
-    VSIFClose( fp );
+    VSIFCloseL( fp );
 
     pszDoc[nLen] = '\0';
 
@@ -1784,7 +1837,7 @@ int CPLSerializeXMLTreeToFile( CPLXMLNode *psTree, const char *pszFilename )
 /* -------------------------------------------------------------------- */
 /*      Create file.                                                    */
 /* -------------------------------------------------------------------- */
-    fp = VSIFOpen( pszFilename, "wt" );
+    fp = VSIFOpenL( pszFilename, "wt" );
     if( fp == NULL )
     {
         CPLError( CE_Failure, CPLE_OpenFailed, 
@@ -1795,12 +1848,12 @@ int CPLSerializeXMLTreeToFile( CPLXMLNode *psTree, const char *pszFilename )
 /* -------------------------------------------------------------------- */
 /*      Write file.                                                     */
 /* -------------------------------------------------------------------- */
-    if( (int) VSIFWrite( pszDoc, 1, nLength, fp ) != nLength )
+    if( (int) VSIFWriteL( pszDoc, 1, nLength, fp ) != nLength )
     {
         CPLError( CE_Failure, CPLE_FileIO, 
                   "Failed to write whole XML document (%.500s).",
                   pszFilename );
-        VSIFClose( fp );
+        VSIFCloseL( fp );
         CPLFree( pszDoc );
         return FALSE;
     }
@@ -1808,7 +1861,7 @@ int CPLSerializeXMLTreeToFile( CPLXMLNode *psTree, const char *pszFilename )
 /* -------------------------------------------------------------------- */
 /*      Cleanup                                                         */
 /* -------------------------------------------------------------------- */
-    VSIFClose( fp );
+    VSIFCloseL( fp );
     CPLFree( pszDoc );
 
     return TRUE;
@@ -1851,3 +1904,4 @@ void       CPL_DLL CPLCleanXMLElementName( char *pszTarget )
         }
     }
 }
+
