@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cpl_multiproc.cpp,v 1.4 2003/05/06 18:30:54 warmerda Exp $
+ * $Id: cpl_multiproc.cpp,v 1.16 2005/08/31 01:00:51 fwarmerdam Exp $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  CPL Multi-Threading, and process handling portability functions.
@@ -28,6 +28,42 @@
  **********************************************************************
  *
  * $Log: cpl_multiproc.cpp,v $
+ * Revision 1.16  2005/08/31 01:00:51  fwarmerdam
+ * Fixed assert limits.
+ *
+ * Revision 1.15  2005/08/24 21:51:06  fwarmerdam
+ * added CPLCleanupTLS
+ *
+ * Revision 1.14  2005/08/01 18:58:42  fwarmerdam
+ * Fixed problem with _NP mutex initializer.
+ *
+ * Revision 1.13  2005/07/31 02:14:49  fwarmerdam
+ * improved recursive mutex creation for pthreads, works on macosx now
+ *
+ * Revision 1.12  2005/07/18 15:34:11  fwarmerdam
+ * Fixed papTLSList sizing.
+ *
+ * Revision 1.11  2005/07/08 18:17:52  fwarmerdam
+ * complete TLS implementation for win32
+ *
+ * Revision 1.10  2005/07/08 16:30:25  fwarmerdam
+ * real implementation of TLS for pthreads
+ *
+ * Revision 1.9  2005/07/08 14:35:26  fwarmerdam
+ * preliminary TLS support
+ *
+ * Revision 1.8  2005/05/23 16:00:33  fwarmerdam
+ * Make sure that stub implementation of mutex support recursive holds.
+ *
+ * Revision 1.7  2005/05/23 06:40:40  fwarmerdam
+ * fixed flaw in CPLCreateOrAcquireMutex, added mutex holder
+ *
+ * Revision 1.6  2005/05/20 19:19:00  fwarmerdam
+ * added CPLCreateOrAcquireMutex()
+ *
+ * Revision 1.5  2005/04/26 20:52:10  fwarmerdam
+ * use a typedef type for thread mains (for Sun port)
+ *
  * Revision 1.4  2003/05/06 18:30:54  warmerda
  * fix unix createmutex to implicitly acquire it
  *
@@ -46,7 +82,127 @@
 #include "cpl_conv.h"
 #include <time.h>
 
-CPL_CVSID("$Id: cpl_multiproc.cpp,v 1.4 2003/05/06 18:30:54 warmerda Exp $");
+CPL_CVSID("$Id: cpl_multiproc.cpp,v 1.16 2005/08/31 01:00:51 fwarmerdam Exp $");
+
+#undef DEBUG
+
+/************************************************************************/
+/*                           CPLMutexHolder()                           */
+/************************************************************************/
+
+CPLMutexHolder::CPLMutexHolder( void **phMutex, double dfWaitInSeconds,
+                                const char *pszFileIn, 
+                                int nLineIn )
+
+{
+    pszFile = pszFileIn;
+    nLine = nLineIn;
+
+#ifdef DEBUG
+    CPLDebug( "MH", "Request %p for pid %d at %d/%s", 
+              *phMutex, CPLGetPID(), nLine, pszFile );
+#endif
+
+    if( !CPLCreateOrAcquireMutex( phMutex, dfWaitInSeconds ) )
+    {
+        CPLDebug( "CPLMutexHolder", "failed to acquire mutex!" );
+        hMutex = NULL;
+    }
+    else
+    {
+#ifdef DEBUG
+        CPLDebug( "MH", "Acquired %p for pid %d at %d/%s", 
+                  *phMutex, CPLGetPID(), nLine, pszFile );
+#endif
+
+        hMutex = *phMutex;
+    }
+}
+
+/************************************************************************/
+/*                          ~CPLMutexHolder()                           */
+/************************************************************************/
+
+CPLMutexHolder::~CPLMutexHolder()
+
+{
+    if( hMutex != NULL )
+    {
+#ifdef DEBUG
+        CPLDebug( "MH", "Release %p for pid %d at %d/%s", 
+                  hMutex, CPLGetPID(), nLine, pszFile );
+#endif
+        CPLReleaseMutex( hMutex );
+    }
+}
+
+
+/************************************************************************/
+/*                      CPLCreateOrAcquireMutex()                       */
+/************************************************************************/
+
+int CPLCreateOrAcquireMutex( void **phMutex, double dfWaitInSeconds )
+
+{
+    static void *hCOAMutex = NULL;
+
+    /*
+    ** ironically, creation of this initial mutex is not threadsafe
+    ** even though we use it to ensure that creation of other mutexes
+    ** is threadsafe. 
+    */
+    if( hCOAMutex == NULL )
+    {
+        hCOAMutex = CPLCreateMutex();
+    }
+    else
+    {
+        CPLAcquireMutex( hCOAMutex, dfWaitInSeconds );
+    }
+
+    if( *phMutex == NULL )
+    {
+        *phMutex = CPLCreateMutex();
+        CPLReleaseMutex( hCOAMutex );
+        return TRUE;
+    }
+    else
+    {
+        CPLReleaseMutex( hCOAMutex );
+
+        int bSuccess = CPLAcquireMutex( *phMutex, dfWaitInSeconds );
+        
+        return bSuccess;
+    }
+}
+
+/************************************************************************/
+/*                        CPLCleanupTLSList()                           */
+/*                                                                      */
+/*      Free resources associated with a TLS vector (implementation     */
+/*      independent).                                                   */
+/************************************************************************/
+
+static void CPLCleanupTLSList( void **papTLSList )
+
+{
+    int i;
+
+//    printf( "CPLCleanupTLSList(%p)\n", papTLSList );
+    
+    if( papTLSList == NULL )
+        return;
+
+    for( i = 0; i < CTLS_MAX; i++ )
+    {
+        if( papTLSList[i] != NULL && papTLSList[i+CTLS_MAX] != NULL )
+        {
+            CPLFree( papTLSList[i] );
+        }
+    }
+
+    CPLFree( papTLSList );
+}
 
 #ifdef CPL_MULTIPROC_STUB
 /************************************************************************/
@@ -98,11 +254,6 @@ int CPLAcquireMutex( void *hMutex, double dfWaitInSeconds )
     CPLAssert( pabyMutex[1] == 'r' && pabyMutex[2] == 'e' 
                && pabyMutex[3] == 'd' );
 
-    if( pabyMutex[0] != 0 )
-        CPLDebug( "CPLMultiProc", 
-                  "CPLAcquireMutex() called on mutex with %d as ref count!",
-                  pabyMutex[0] );
-
     pabyMutex[0] += 1;
 
     return TRUE;
@@ -120,7 +271,7 @@ void CPLReleaseMutex( void *hMutex )
     CPLAssert( pabyMutex[1] == 'r' && pabyMutex[2] == 'e' 
                && pabyMutex[3] == 'd' );
 
-    if( pabyMutex[0] != 1 )
+    if( pabyMutex[0] < 1 )
         CPLDebug( "CPLMultiProc", 
                   "CPLReleaseMutex() called on mutex with %d as ref count!",
                   pabyMutex[0] );
@@ -234,7 +385,7 @@ int CPLGetPID()
 /*                          CPLCreateThread();                          */
 /************************************************************************/
 
-int CPLCreateThread( void (*pfnMain)(void *), void *pArg )
+int CPLCreateThread( CPLThreadFunc pfnMain, void *pArg )
 
 {
     return -1;
@@ -260,6 +411,32 @@ void CPLSleep( double dfWaitInSeconds )
     }
 }
 
+/************************************************************************/
+/*                           CPLGetTLSList()                            */
+/************************************************************************/
+
+static void **papTLSList = NULL;
+
+static void **CPLGetTLSList()
+
+{
+    if( papTLSList == NULL )
+        papTLSList = (void **) CPLCalloc(sizeof(void*),CTLS_MAX*2);
+
+    return papTLSList;
+}
+
+/************************************************************************/
+/*                           CPLCleanupTLS()                            */
+/************************************************************************/
+
+void CPLCleanupTLS()
+
+{
+    CPLCleanupTLSList( papTLSList );
+    papTLSList = NULL;
+}
+
 #endif /* def CPL_MULTIPROC_STUB */
 
 #ifdef CPL_MULTIPROC_WIN32
@@ -409,7 +586,7 @@ int CPLGetPID()
 
 typedef struct {
     void *pAppData;
-    void (*pfnMain)(void *);
+    CPLThreadFunc pfnMain;
 } CPLStdCallThreadInfo;
 
 static DWORD WINAPI CPLStdCallThreadJacket( void *pData )
@@ -432,7 +609,7 @@ static DWORD WINAPI CPLStdCallThreadJacket( void *pData )
 /*      to supply that.                                                 */
 /************************************************************************/
 
-int CPLCreateThread( void (*pfnMain)(void *), void *pThreadArg )
+int CPLCreateThread( CPLThreadFunc pfnMain, void *pThreadArg )
 
 {
     HANDLE hThread;
@@ -464,6 +641,65 @@ void CPLSleep( double dfWaitInSeconds )
     Sleep( (DWORD) (dfWaitInSeconds * 1000.0) );
 }
 
+static int           bTLSKeySetup = FALSE;
+static DWORD         nTLSKey;
+
+/************************************************************************/
+/*                           CPLGetTLSList()                            */
+/************************************************************************/
+
+static void **CPLGetTLSList()
+
+{
+    void **papTLSList;
+
+    if( !bTLSKeySetup )
+    {
+        nTLSKey = TlsAlloc();
+        if( nTLSKey == TLS_OUT_OF_INDEXES )
+        {
+            CPLError( CE_Fatal, CPLE_AppDefined, 
+                      "TlsAlloc() failed!" );
+        }
+        bTLSKeySetup = TRUE;
+    }
+
+    papTLSList = (void **) TlsGetValue( nTLSKey );
+    if( papTLSList == NULL )
+    {
+        papTLSList = (void **) CPLCalloc(sizeof(void*),CTLS_MAX*2);
+        if( TlsSetValue( nTLSKey, papTLSList ) == 0 )
+        {
+            CPLError( CE_Fatal, CPLE_AppDefined, 
+                      "TlsSetValue() failed!" );
+        }
+    }
+
+    return papTLSList;
+}
+
+/************************************************************************/
+/*                           CPLCleanupTLS()                            */
+/************************************************************************/
+
+void CPLCleanupTLS()
+
+{
+    void **papTLSList;
+    int i;
+
+    if( !bTLSKeySetup )
+        return;
+
+    papTLSList = (void **) TlsGetValue( nTLSKey );
+    if( papTLSList == NULL )
+        return;
+
+    TlsSetValue( nTLSKey, NULL );
+
+    CPLCleanupTLSList( papTLSList );
+}
+
 #endif /* def CPL_MULTIPROC_WIN32 */
 
 #ifdef CPL_MULTIPROC_PTHREAD
@@ -497,10 +733,22 @@ void *CPLCreateMutex()
 
 {
     pthread_mutex_t *hMutex;
-    pthread_mutex_t hMutexSrc = PTHREAD_MUTEX_INITIALIZER;
 
     hMutex = (pthread_mutex_t *) malloc(sizeof(pthread_mutex_t));
-    *hMutex = hMutexSrc;
+
+#if defined(PTHREAD_MUTEX_RECURSIVE)
+    {
+        pthread_mutexattr_t  attr;
+        pthread_mutexattr_init( &attr );
+        pthread_mutexattr_settype( &attr, PTHREAD_MUTEX_RECURSIVE );
+        pthread_mutex_init( hMutex, &attr );
+    }
+#elif defined(PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP)
+    pthread_mutex_t tmp_mutex = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
+    *hMutex = tmp_mutex;
+#else
+#error "Recursive mutexes apparently unsupported, configure --without-threads" 
+#endif
 
     // mutexes are implicitly acquired when created.
     CPLAcquireMutex( hMutex, 0.0 );
@@ -515,8 +763,20 @@ void *CPLCreateMutex()
 int CPLAcquireMutex( void *hMutexIn, double dfWaitInSeconds )
 
 {
+    int err;
+
     /* we need to add timeout support */
-    pthread_mutex_lock( (pthread_mutex_t *) hMutexIn );
+    err =  pthread_mutex_lock( (pthread_mutex_t *) hMutexIn );
+    
+    if( err != 0 )
+    {
+        if( err == EDEADLK )
+            CPLDebug( "CPLAcquireMutex", "Error = %d/EDEADLK", err );
+        else
+            CPLDebug( "CPLAcquireMutex", "Error = %d", err );
+
+        return FALSE;
+    }
 
     return TRUE;
 }
@@ -580,7 +840,7 @@ int CPLGetPID()
 
 typedef struct {
     void *pAppData;
-    void (*pfnMain)(void *);
+    CPLThreadFunc pfnMain;
     pthread_t hThread;
 } CPLStdCallThreadInfo;
 
@@ -604,7 +864,7 @@ static void *CPLStdCallThreadJacket( void *pData )
 /*      to supply that.                                                 */
 /************************************************************************/
 
-int CPLCreateThread( void (*pfnMain)(void *), void *pThreadArg )
+int CPLCreateThread( CPLThreadFunc pfnMain, void *pThreadArg )
 
 {
     
@@ -641,5 +901,92 @@ void CPLSleep( double dfWaitInSeconds )
     nanosleep( &sRequest, &sRemain );
 }
 
+static int           bTLSKeySetup = FALSE;
+static pthread_key_t oTLSKey;
+
+/************************************************************************/
+/*                           CPLCleanupTLS()                            */
+/************************************************************************/
+
+void CPLCleanupTLS()
+
+{
+    void **papTLSList;
+
+    if( !bTLSKeySetup )
+        return;
+
+    papTLSList = (void **) pthread_getspecific( oTLSKey );
+    if( papTLSList == NULL )
+        return;
+
+    pthread_setspecific( oTLSKey, NULL );
+
+    CPLCleanupTLSList( papTLSList );
+}
+
+/************************************************************************/
+/*                           CPLGetTLSList()                            */
+/************************************************************************/
+
+static void **CPLGetTLSList()
+
+{
+    void **papTLSList;
+
+    if( !bTLSKeySetup )
+    {
+        if( pthread_key_create( &oTLSKey, 
+                                (void (*)(void*)) CPLCleanupTLSList ) != 0 )
+        {
+            CPLError( CE_Fatal, CPLE_AppDefined, 
+                      "pthread_key_create() failed!" );
+        }
+        bTLSKeySetup = TRUE;
+    }
+
+    papTLSList = (void **) pthread_getspecific( oTLSKey );
+    if( papTLSList == NULL )
+    {
+        papTLSList = (void **) CPLCalloc(sizeof(void*),CTLS_MAX*2);
+        if( pthread_setspecific( oTLSKey, papTLSList ) != 0 )
+        {
+            CPLError( CE_Fatal, CPLE_AppDefined, 
+                      "pthread_setspecific() failed!" );
+        }
+    }
+
+    return papTLSList;
+}
+
 #endif /* def CPL_MULTIPROC_PTHREAD */
 
+/************************************************************************/
+/*                             CPLGetTLS()                              */
+/************************************************************************/
+
+void *CPLGetTLS( int nIndex )
+
+{
+    void **papTLSList = CPLGetTLSList();
+
+    CPLAssert( nIndex >= 0 && nIndex < CTLS_MAX );
+
+    return papTLSList[nIndex];
+}
+
+/************************************************************************/
+/*                             CPLSetTLS()                              */
+/************************************************************************/
+
+void CPLSetTLS( int nIndex, void *pData, int bFreeOnExit )
+
+{
+    void **papTLSList = CPLGetTLSList();
+
+    CPLAssert( nIndex >= 0 && nIndex < CTLS_MAX );
+
+    papTLSList[nIndex] = pData;
+    papTLSList[CTLS_MAX + nIndex] = (void *) bFreeOnExit;
+}
+
