@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_odbc.cpp,v 1.19 2005/01/12 07:51:29 fwarmerdam Exp $
+ * $Id: cpl_odbc.cpp,v 1.25 2005/09/24 04:58:16 fwarmerdam Exp $
  *
  * Project:  OGR ODBC Driver
  * Purpose:  Declarations for ODBC Access Cover API.
@@ -28,6 +28,25 @@
  ******************************************************************************
  *
  * $Log: cpl_odbc.cpp,v $
+ * Revision 1.25  2005/09/24 04:58:16  fwarmerdam
+ * Support SQLDriverConnect (pass opts) in EstablishSession
+ *
+ * Revision 1.24  2005/09/05 20:18:43  fwarmerdam
+ * added binary column support
+ *
+ * Revision 1.23  2005/08/31 03:32:41  fwarmerdam
+ * GetTypeName now returns CPLString
+ *
+ * Revision 1.22  2005/08/07 14:05:44  fwarmerdam
+ * added connect/disconnect debug statements
+ *
+ * Revision 1.21  2005/06/29 01:01:01  ssoule
+ * Changed return type of CPLODBCStatement::GetTypeName from const char * to
+ * std::string.
+ *
+ * Revision 1.20  2005/05/23 03:56:44  fwarmerdam
+ * make make static buffers threadlocal
+ *
  * Revision 1.19  2005/01/12 07:51:29  fwarmerdam
  * Fix some argument types needed for ODBC 3.52. In particular SQLDescribeCol
  * and SQLGetData require SQLLEN and SQLULEN instead of SQLINTEGER and
@@ -99,7 +118,7 @@
 #include "cpl_vsi.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: cpl_odbc.cpp,v 1.19 2005/01/12 07:51:29 fwarmerdam Exp $");
+CPL_CVSID("$Id: cpl_odbc.cpp,v 1.25 2005/09/24 04:58:16 fwarmerdam Exp $");
 
 /************************************************************************/
 /*                           CPLODBCSession()                           */
@@ -132,6 +151,7 @@ int CPLODBCSession::CloseSession()
 {
     if( m_hDBC!=NULL ) 
     {
+        CPLDebug( "ODBC", "SQLDisconnect()" );
         SQLDisconnect( m_hDBC );
         SQLFreeConnect( m_hDBC );
         m_hDBC = NULL;
@@ -216,10 +236,32 @@ int CPLODBCSession::EstablishSession( const char *pszDSN,
     if( pszPassword == NULL )
         pszPassword = "";
 
-    if( Failed( SQLConnect( m_hDBC, (SQLCHAR *) pszDSN, SQL_NTS, 
-                            (SQLCHAR *) pszUserid, SQL_NTS, 
-                            (SQLCHAR *) pszPassword, SQL_NTS ) ) )
+    int bFailed;
+    if( strstr(pszDSN,"=") != NULL )
     {
+        char szOutConnString[1024];
+        SQLSMALLINT nOutConnStringLen;
+
+        CPLDebug( "ODBC", "SQLDriverConnect(%s)", pszDSN );
+        bFailed = Failed(
+            SQLDriverConnect( m_hDBC, NULL, 
+                              (SQLCHAR *) pszDSN, strlen(pszDSN), 
+                              (SQLCHAR *) szOutConnString, 
+                              sizeof(szOutConnString), 
+                              &nOutConnStringLen, SQL_DRIVER_NOPROMPT ) );
+    }
+    else
+    {
+        CPLDebug( "ODBC", "SQLConnect(%s)", pszDSN );
+        bFailed = Failed(
+            SQLConnect( m_hDBC, (SQLCHAR *) pszDSN, SQL_NTS, 
+                        (SQLCHAR *) pszUserid, SQL_NTS, 
+                        (SQLCHAR *) pszPassword, SQL_NTS ) );
+    }
+
+    if( bFailed )
+    {
+        CPLDebug( "ODBC", "... failed: %s", GetLastError() );
         CloseSession();
         return FALSE;
     }
@@ -275,6 +317,7 @@ CPLODBCStatement::CPLODBCStatement( CPLODBCSession *poSession )
     m_panColNullable = NULL;
 
     m_papszColValues = NULL;
+    m_panColValueLengths = NULL;
 
     m_pszStatement = NULL;
     m_nStatementMax = 0;
@@ -356,6 +399,7 @@ int CPLODBCStatement::CollectResultsInfo()
 /* -------------------------------------------------------------------- */
     m_papszColNames = (char **) VSICalloc(sizeof(char *),(m_nColCount+1));
     m_papszColValues = (char **) VSICalloc(sizeof(char *),(m_nColCount+1));
+    m_panColValueLengths = (int *) VSICalloc(sizeof(int),(m_nColCount+1));
 
     m_panColType = (short *) VSICalloc(sizeof(short),m_nColCount);
     m_panColSize = (_SQLULEN *) VSICalloc(sizeof(_SQLULEN),m_nColCount);
@@ -572,33 +616,52 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
     
     for( iCol = 0; iCol < m_nColCount; iCol++ )
     {
-        char szWrkData[256];
+        char szWrkData[512];
         _SQLLEN cbDataLen;
         int nRetCode;
+        int nFetchType;
+
+        if( m_panColType[iCol] == SQL_BINARY 
+            || m_panColType[iCol] == SQL_VARBINARY 
+            || m_panColType[iCol] == SQL_LONGVARBINARY )
+        {
+            nFetchType = SQL_C_BINARY;
+        }
+        else
+            nFetchType = SQL_C_CHAR;
 
         szWrkData[0] = '\0';
         szWrkData[sizeof(szWrkData)-1] = '\0';
 
-        nRetCode = SQLGetData( m_hStmt, (SQLUSMALLINT) iCol+1, SQL_C_CHAR,
+        nRetCode = SQLGetData( m_hStmt, (SQLUSMALLINT) iCol+1, nFetchType,
                                szWrkData, sizeof(szWrkData)-1, 
                                &cbDataLen );
         if( Failed( nRetCode ) )
             return FALSE;
 
         if( cbDataLen == SQL_NULL_DATA )
+        {
             m_papszColValues[iCol] = NULL;
+            m_panColValueLengths[iCol] = 0;
+        }
 
         // assume big result: should check for state=SQLSATE 01004.
         else if( nRetCode == SQL_SUCCESS_WITH_INFO  ) 
         {
-            m_papszColValues[iCol] = strdup(szWrkData);
+            if( cbDataLen > (int) (sizeof(szWrkData)-1) )
+                cbDataLen = (int) (sizeof(szWrkData)-1);
+
+            m_papszColValues[iCol] = (char *) CPLMalloc(cbDataLen+1);
+            memcpy( m_papszColValues[iCol], szWrkData, cbDataLen );
+            m_papszColValues[iCol][cbDataLen] = '\0';
+            m_panColValueLengths[iCol] = cbDataLen;
 
             while( TRUE )
             {
                 int nChunkLen;
 
                 nRetCode = SQLGetData( m_hStmt, (SQLUSMALLINT) iCol+1, 
-                                       SQL_C_CHAR,
+                                       nFetchType,
                                        szWrkData, sizeof(szWrkData)-1, 
                                        &cbDataLen );
                 if( nRetCode == SQL_NO_DATA )
@@ -616,17 +679,23 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
 
                 m_papszColValues[iCol] = (char *) 
                     CPLRealloc( m_papszColValues[iCol], 
-                                strlen(m_papszColValues[iCol]) + nChunkLen+1 );
-                strcat( m_papszColValues[iCol], szWrkData );
+                                m_panColValueLengths[iCol] + nChunkLen + 1 );
+                memcpy( m_papszColValues[iCol] + m_panColValueLengths[iCol], 
+                        szWrkData, nChunkLen );
+                m_panColValueLengths[iCol] += nChunkLen;
+                m_papszColValues[iCol][m_panColValueLengths[iCol]] = '\0';
             }
         }
         else
         {
-            m_papszColValues[iCol] = strdup( szWrkData );
+            m_panColValueLengths[iCol] = cbDataLen;
+            m_papszColValues[iCol] = (char *) CPLMalloc(cbDataLen+1);
+            memcpy( m_papszColValues[iCol], szWrkData, cbDataLen );
+            m_papszColValues[iCol][cbDataLen] = '\0';
         }
 
         // Trim white space off end, if there is any.
-        if( m_papszColValues[iCol] != NULL )
+        if( nFetchType == SQL_C_CHAR && m_papszColValues[iCol] != NULL )
         {
             char *pszTarget = m_papszColValues[iCol];
             int iEnd = strlen(pszTarget) - 1;
@@ -702,6 +771,21 @@ const char *CPLODBCStatement::GetColData( const char *pszColName,
         return GetColData( iCol, pszDefault );
 }
 
+/************************************************************************/
+/*                          GetColDataLength()                          */
+/************************************************************************/
+
+int CPLODBCStatement::GetColDataLength( int iCol )
+
+{
+    if( iCol < 0 || iCol >= m_nColCount )
+        return 0;
+    else if( m_papszColValues[iCol] != NULL )
+        return m_panColValueLengths[iCol];
+    else
+        return 0;
+}
+
 /************************************************************************/
 /*                              GetColId()                              */
 /************************************************************************/
@@ -1041,8 +1125,6 @@ int CPLODBCStatement::GetColumns( const char *pszTable,
 /* -------------------------------------------------------------------- */
     int iCol;
 
-    CPLDebug( "ODBC", "GetColumn() - m_nColCount=%d", m_nColCount ); 
-
     for( iCol = 0; iCol < m_nColCount; iCol++ )
     {
         char szWrkData[8193];
@@ -1203,7 +1285,8 @@ void CPLODBCStatement::DumpResult( FILE *fp, int bShowSchema )
             else
                 fprintf( fp, " Size:%5d", GetColSize(iCol) );
 
-            fprintf( fp, " Type:%s", GetTypeName( GetColType(iCol) ) );
+            CPLString osType = GetTypeName( GetColType(iCol) );
+            fprintf( fp, " Type:%s", osType.c_str() );
             if( GetColNullable(iCol) )
                 fprintf( fp, " NULLABLE" );
             fprintf( fp, "\n" );
@@ -1233,15 +1316,15 @@ void CPLODBCStatement::DumpResult( FILE *fp, int bShowSchema )
 /**
  * Get name for SQL column type.
  *
- * Returns a pointer to an internal static string name for the 
- * indicate type code (as returned from CPLODBCStatement::GetColType().
+ * Returns a string name for the indicated type code (as returned
+ * from CPLODBCStatement::GetColType()).
  *
  * @param nTypeCode the SQL_ code, such as SQL_CHAR.
  *
  * @return internal string, "UNKNOWN" if code not recognised. 
  */
 
-const char *CPLODBCStatement::GetTypeName( int nTypeCode )
+CPLString CPLODBCStatement::GetTypeName( int nTypeCode )
 
 {
     switch( nTypeCode )
@@ -1287,9 +1370,8 @@ const char *CPLODBCStatement::GetTypeName( int nTypeCode )
         return "TIMESTAMP";
 
       default:
-          static char szType[100];
-
-          sprintf( szType, "UNKNOWN:%d", nTypeCode );
-          return szType;
+        CPLString osResult;
+        osResult.Printf( "UNKNOWN:%d", nTypeCode );
+        return osResult;
     }
 }
