@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_port.h,v 1.40 2005/03/11 14:59:07 fwarmerdam Exp $
+ * $Id: cpl_port.h,v 1.45 2005/06/15 09:47:40 dron Exp $
  *
  * Project:  CPL - Common Portability Library
  * Author:   Frank Warmerdam, warmerdam@pobox.com
@@ -42,6 +42,21 @@
  ******************************************************************************
  *
  * $Log: cpl_port.h,v $
+ * Revision 1.45  2005/06/15 09:47:40  dron
+ * Fixed typo.
+ *
+ * Revision 1.44  2005/06/15 09:11:58  dron
+ * Added CPLIsEqual() macro.
+ *
+ * Revision 1.43  2005/05/23 03:57:08  fwarmerdam
+ * added default definition of CPL_THREADLOCAL
+ *
+ * Revision 1.42  2005/04/04 15:22:36  fwarmerdam
+ * added CPL_STDCALL declaration
+ *
+ * Revision 1.41  2005/03/17 04:20:24  fwarmerdam
+ * added FORCE_CDECL
+ *
  * Revision 1.40  2005/03/11 14:59:07  fwarmerdam
  * Default to assuming nothing is infinite if isinf() macro not defined.
  * Per http://bugzilla.remotesensing.org/show_bug.cgi?id=795
@@ -250,6 +265,19 @@ typedef unsigned long    GUIntBig;
 #endif
 #endif
 
+#ifndef CPL_STDCALL
+#if defined(_MSC_VER) && !defined(CPL_DISABLE_STDCALL)
+#  define CPL_STDCALL     __stdcall
+#else
+#  define CPL_STDCALL
+#endif
+#endif
+
+#ifdef _MSC_VER
+#  define FORCE_CDECL  __cdecl
+#else
+#  define FORCE_CDECL 
+#endif
 
 #ifndef NULL
 #  define NULL  0
@@ -272,6 +300,15 @@ typedef unsigned long    GUIntBig;
 #  define ABS(x)        ((x<0) ? (-1*(x)) : x)
 #endif
 
+/* -------------------------------------------------------------------- */
+/*      Macro to test equality of two floating point values.            */
+/*      We use fabs() function instead of ABS() macro to avoid side     */
+/*      effects.                                                        */
+/* -------------------------------------------------------------------- */
+#ifndef CPLIsEqual
+#  define CPLIsEqual(x,y) (fabs(fabs(x) - fabs(y)) < 0.0000000000001 ? 1 : 0)
+#endif
+
 #ifndef EQUAL
 #ifdef WIN32
 #  define EQUALN(a,b,n)           (strnicmp(a,b,n)==0)
@@ -288,6 +325,10 @@ int strncasecmp(char * str1, char * str2, int len);
 char * strdup (char *instr);
 #endif
 
+#ifndef CPL_THREADLOCAL 
+#  define CPL_THREADLOCAL 
+#endif
+
 /* -------------------------------------------------------------------- */
 /*      Handle isnan() and isinf().  Note that isinf() and isnan()      */
 /*      are supposed to be macros according to C99.  Some systems       */
