@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cpl_string.cpp,v 1.41 2004/09/17 21:26:28 fwarmerdam Exp $
+ * $Id: cpl_string.cpp,v 1.50 2005/09/30 19:11:40 fwarmerdam Exp $
  *
  * Name:     cpl_string.cpp
  * Project:  CPL - Common Portability Library
@@ -44,6 +44,33 @@
  *   without vsnprintf(). 
  *
  * $Log: cpl_string.cpp,v $
+ * Revision 1.50  2005/09/30 19:11:40  fwarmerdam
+ * fixed hextobinary conversion, bug 935
+ *
+ * Revision 1.49  2005/09/14 19:21:17  fwarmerdam
+ * binary pointer is const in binarytohex
+ *
+ * Revision 1.48  2005/09/13 15:07:23  dron
+ * Initialize counters in CPLHexToBinary().
+ *
+ * Revision 1.47  2005/09/11 21:09:27  fwarmerdam
+ * use large file API in CSLLoad
+ *
+ * Revision 1.46  2005/09/05 20:19:08  fwarmerdam
+ * fixed binarytohex function
+ *
+ * Revision 1.45  2005/08/31 03:31:15  fwarmerdam
+ * added binarytohex/hextobinary
+ *
+ * Revision 1.44  2005/08/04 19:41:33  fwarmerdam
+ * support setting null value in CSLSetNameValue
+ *
+ * Revision 1.43  2005/05/23 03:59:44  fwarmerdam
+ * make cplsprintf buffer threadlocal
+ *
+ * Revision 1.42  2005/04/04 15:23:31  fwarmerdam
+ * some functions now CPL_STDCALL
+ *
  * Revision 1.41  2004/09/17 21:26:28  fwarmerdam
  * Yikes ... CPLEscapeString() was badly broken for BackslashEscapable.
  *
@@ -177,7 +204,7 @@
 #include "cpl_string.h"
 #include "cpl_vsi.h"
 
-CPL_CVSID("$Id: cpl_string.cpp,v 1.41 2004/09/17 21:26:28 fwarmerdam Exp $");
+CPL_CVSID("$Id: cpl_string.cpp,v 1.50 2005/09/30 19:11:40 fwarmerdam Exp $");
 
 /*=====================================================================
                     StringList manipulation functions.
@@ -267,7 +294,7 @@ const char * CSLGetField( char ** papszStrList, int iField )
  *
  * Free all memory used by a StringList.
  **********************************************************************/
-void CSLDestroy(char **papszStrList)
+void CPL_STDCALL CSLDestroy(char **papszStrList)
 {
     char **papszPtr;
 
@@ -329,21 +356,21 @@ char **CSLLoad(const char *pszFname)
     const char  *pszLine;
     char        **papszStrList=NULL;
 
-    fp = VSIFOpen(pszFname, "rb");
+    fp = VSIFOpenL(pszFname, "rb");
 
     if (fp)
     {
-        while(!VSIFEof(fp))
+        while(!VSIFEofL(fp))
         {
-            if ( (pszLine = CPLReadLine(fp)) != NULL )
+            if ( (pszLine = CPLReadLineL(fp)) != NULL )
             {
                 papszStrList = CSLAddString(papszStrList, pszLine);
             }
         }
 
-        VSIFClose(fp);
+        VSIFCloseL(fp);
 
-        CPLReadLine( NULL );
+        CPLReadLineL( NULL );
     }
     else
     {
@@ -809,8 +836,8 @@ char ** CSLTokenizeString2( const char * pszString,
  */
 #define CPLSPrintf_BUF_SIZE 8000
 #define CPLSPrintf_BUF_Count 10
-static char gszCPLSPrintfBuffer[CPLSPrintf_BUF_Count][CPLSPrintf_BUF_SIZE];
-static int gnCPLSPrintfBuffer = 0;
+static CPL_THREADLOCAL char gszCPLSPrintfBuffer[CPLSPrintf_BUF_Count][CPLSPrintf_BUF_SIZE];
+static CPL_THREADLOCAL int gnCPLSPrintfBuffer = 0;
 
 const char *CPLSPrintf(char *fmt, ...)
 {
@@ -1101,7 +1128,8 @@ char **CSLAddNameValue(char **papszStrList,
  * @param pszName the name to be assigned a value.  This should be a well
  * formed token (no spaces or very special characters). 
  * @param pszValue the value to assign to the name.  This should not contain
- * any newlines (CR or LF) but is otherwise pretty much unconstrained.
+ * any newlines (CR or LF) but is otherwise pretty much unconstrained.  If
+ * NULL any corresponding value will be removed.
  *
  * @return modified stringlist.
  */
@@ -1130,8 +1158,28 @@ char **CSLSetNameValue(char **papszList,
             cSep = (*papszPtr)[nLen];
 
             CPLFree(*papszPtr);
-            *papszPtr = (char *) CPLMalloc(strlen(pszName)+strlen(pszValue)+2);
-            sprintf( *papszPtr, "%s%c%s", pszName, cSep, pszValue );
+
+            /* 
+             * If the value is NULL, remove this entry completely/
+             */
+            if( pszValue == NULL )
+            {
+                while( papszPtr[1] != NULL )
+                {
+                    *papszPtr = papszPtr[1];
+                    papszPtr++;
+                }
+                *papszPtr = NULL;
+            }
+
+            /*
+             * Otherwise replace with new value.
+             */
+            else
+            {
+                *papszPtr = (char *) CPLMalloc(strlen(pszName)+strlen(pszValue)+2);
+                sprintf( *papszPtr, "%s%c%s", pszName, cSep, pszValue );
+            }
             return papszList;
         }
         papszPtr++;
@@ -1552,3 +1600,92 @@ char *CPLUnescapeString( const char *pszInput, int *pnLength, int nScheme )
 
     return pszOutput;
 }
+
+/************************************************************************/
+/*                           CPLBinaryToHex()                           */
+/************************************************************************/
+
+/**
+ * Binary to hexadecimal translation.
+ *
+ * @param nBytes number of bytes of binary data in pabyData.
+ * @param pabyData array of data bytes to translate. 
+ * 
+ * @return hexadecimal translation, zero terminated.  Free with CPLFree().
+ */
+
+char *CPLBinaryToHex( int nBytes, const GByte *pabyData )
+
+{
+    char *pszHex = (char *) CPLMalloc(nBytes * 2 + 1 );
+    int i;
+    static const char achHex[] = "0123456789ABCDEF";
+
+    pszHex[nBytes*2] = '\0';
+
+    for( i = 0; i < nBytes; i++ )
+    {
+        int nLow = pabyData[i] & 0x0f;
+        int nHigh = (pabyData[i] & 0xf0) >> 4;
+
+        pszHex[i*2] = achHex[nHigh];
+        pszHex[i*2+1] = achHex[nLow];
+    }
+
+    return pszHex;
+}
+
+
+/************************************************************************/
+/*                           CPLHexToBinary()                           */
+/************************************************************************/
+
+/**
+ * Hexadecimal to binary translation
+ *
+ * @param 
+
+*/
+
+GByte *CPLHexToBinary( const char *pszHex, int *pnBytes )
+
+{
+    int iSrc = 0, iDst = 0, nHexLen = strlen(pszHex);
+
+    GByte *pabyWKB;
+
+    pabyWKB = (GByte *) CPLMalloc(nHexLen / 2 + 2);
+
+    while( pszHex[iSrc] != '\0' )
+    {
+        if( pszHex[iSrc] >= '0' && pszHex[iSrc] <= '9' )
+            pabyWKB[iDst] = pszHex[iSrc] - '0';
+        else if( pszHex[iSrc] >= 'A' && pszHex[iSrc] <= 'F' )
+            pabyWKB[iDst] = pszHex[iSrc] - 'A' + 10;
+        else if( pszHex[iSrc] >= 'a' && pszHex[iSrc] <= 'f' )
+            pabyWKB[iDst] = pszHex[iSrc] - 'a' + 10;
+        else 
+            break;
+
+        pabyWKB[iDst] *= 16;
+
+        iSrc++;
+
+        if( pszHex[iSrc] >= '0' && pszHex[iSrc] <= '9' )
+            pabyWKB[iDst] += pszHex[iSrc] - '0';
+        else if( pszHex[iSrc] >= 'A' && pszHex[iSrc] <= 'F' )
+            pabyWKB[iDst] += pszHex[iSrc] - 'A' + 10;
+        else if( pszHex[iSrc] >= 'a' && pszHex[iSrc] <= 'f' )
+            pabyWKB[iDst] += pszHex[iSrc] - 'a' + 10;
+        else
+            break;
+
+        iSrc++;
+        iDst++;
+    }
+    
+    pabyWKB[iDst] = 0;
+    *pnBytes = iDst;
+
+    return pabyWKB;
+}
