@@ -30,6 +30,9 @@
  *    instance validation of access strings to fopen().
  * 
  * $Log: cpl_vsisimple.cpp,v $
+ * Revision 1.18  2005/09/11 18:01:28  fwarmerdam
+ * preliminary implementatin of fully virtualized large file api
+ *
  * Revision 1.17  2003/09/10 19:44:36  warmerda
  * added VSIStrerrno()
  *
@@ -86,7 +89,7 @@
 #include "cpl_vsi.h"
 #include "cpl_error.h"
 
-CPL_CVSID("$Id: cpl_vsisimple.cpp,v 1.17 2003/09/10 19:44:36 warmerda Exp $");
+CPL_CVSID("$Id: cpl_vsisimple.cpp,v 1.18 2005/09/11 18:01:28 fwarmerdam Exp $");
 
 /* for stat() */
 
@@ -367,43 +370,6 @@ int VSIStat( const char * pszFilename, VSIStatBuf * pStatBuf )
 #endif
 }
 
-/************************************************************************/
-/*                              VSIMkdir()                              */
-/************************************************************************/
-
-int VSIMkdir( const char *pszPathname, long mode )
-
-{
-#ifdef WIN32
-    (void) mode;
-    return mkdir( pszPathname );
-#elif defined(macos_pre10)
-    return -1;
-#else
-    return mkdir( pszPathname, mode );
-#endif
-}
-
-/************************************************************************/
-/*                             VSIUnlink()                              */
-/*************************a***********************************************/
-
-int VSIUnlink( const char * pszFilename )
-
-{
-    return unlink( pszFilename );
-}
-
-/************************************************************************/
-/*                              VSIRmdir()                              */
-/************************************************************************/
-
-int VSIRmdir( const char * pszFilename )
-
-{
-    return rmdir( pszFilename );
-}
-
 /************************************************************************/
 /*                              VSITime()                               */
 /************************************************************************/
