@@ -1,5 +1,5 @@
 #******************************************************************************
-#  $Id: ogr.py,v 1.49 2005/03/02 17:42:46 hobu Exp $
+#  $Id: ogr.py,v 1.53 2005/09/18 15:26:00 fwarmerdam Exp $
 # 
 #  Project:  OpenGIS Simple Features Reference Implementation
 #  Purpose:  OGR Python Shadow Class Implementations
@@ -28,6 +28,18 @@
 #******************************************************************************
 # 
 # $Log: ogr.py,v $
+# Revision 1.53  2005/09/18 15:26:00  fwarmerdam
+# added Distance method.
+#
+# Revision 1.52  2005/07/20 02:34:52  fwarmerdam
+# fixed up AddPoint to default z
+#
+# Revision 1.51  2005/07/20 01:46:51  fwarmerdam
+# ogr coordinate dimension upgrades
+#
+# Revision 1.50  2005/05/05 22:45:33  fwarmerdam
+# Fixed Empty call ... only one argument.
+#
 # Revision 1.49  2005/03/02 17:42:46  hobu
 # First pass at a __setattr__ for Feature
 #
@@ -1054,6 +1066,9 @@ class Geometry:
     def GetCoordinateDimension( self ):
         return _gdal.OGR_G_GetCoordinateDimension( self._o )
     
+    def SetCoordinateDimension( self, new_dimension ):
+        return _gdal.OGR_G_SetCoordinateDimension( self._o, new_dimension )
+    
     def WkbSize( self ):
         return _gdal.OGR_G_WkbSize( self._o )
 
@@ -1126,8 +1141,8 @@ class Geometry:
     def Overlaps( self, other_geom ):
         return _gdal.OGR_G_Overlaps( self._o, other_geom._o )
 
-    def Empty( self, other_geom ):
-        return _gdal.OGR_G_Empty( self._o, other_geom._o )
+    def Empty( self ):
+        return _gdal.OGR_G_Empty( self._o )
 
     def GetArea( self ):
         return _gdal.OGR_G_GetArea( self._o )
@@ -1158,9 +1173,15 @@ class Geometry:
     def SetPoint( self, i, x, y, z = 0):
         return _gdal.OGR_G_SetPoint( self._o, i, x, y, z )
 
-    def AddPoint( self, x, y, z = 0 ):
+    def SetPoint_2D( self, i, x, y):
+        return _gdal.OGR_G_SetPoint_2D( self._o, i, x, y )
+
+    def AddPoint( self, x, y, z=0 ):
         return _gdal.OGR_G_AddPoint( self._o, x, y, z )
 
+    def AddPoint_2D( self, x, y ):
+        return _gdal.OGR_G_AddPoint_2D( self._o, x, y )
+
     def GetGeometryCount( self ):
         return _gdal.OGR_G_GetGeometryCount( self._o )
 
@@ -1229,6 +1250,8 @@ class Geometry:
         else:
             return None
         
+    def Distance( self, other ):
+        return _gdal.OGR_G_Distance( self._o, other._o )
 
 def BuildPolygonFromEdges( edges, bBestEffort=0, bAutoClose=0, Tolerance=0 ):
     _o = _gdal.OGRBuildPolygonFromEdges( edges._o, bBestEffort, bAutoClose,
